@@ -2,15 +2,12 @@
 #include "CPyCppyy.h"
 #include "CallContext.h"
 
-
-//- data _____________________________________________________________________
-namespace CPyCppyy {
-
-    CallContext::ECallFlags CallContext::sMemoryPolicy = CallContext::kUseStrict;
-// this is just a data holder for linking; actual value is set in CPyCppyyModule.cxx
-    CallContext::ECallFlags CallContext::sSignalPolicy = CallContext::kNone;
-
-} // namespace CPyCppyy
+//-----------------------------------------------------------------------------
+uint32_t &CPyCppyy::CallContext::GlobalPolicyFlags()
+{
+   static uint32_t flags = 0;
+   return flags;
+}
 
 //-----------------------------------------------------------------------------
 void CPyCppyy::CallContext::AddTemporary(PyObject* pyobj) {
@@ -38,24 +35,13 @@ void CPyCppyy::CallContext::Cleanup() {
 }
 
 //-----------------------------------------------------------------------------
-bool CPyCppyy::CallContext::SetMemoryPolicy(ECallFlags e)
+bool CPyCppyy::CallContext::SetGlobalPolicy(ECallFlags toggleFlag, bool enabled)
 {
-// Set the global memory policy, which affects object ownership when objects
-// are passed as function arguments.
-    if (kUseHeuristics == e || e == kUseStrict) {
-        sMemoryPolicy = e;
-        return true;
-    }
-    return false;
-}
-
-//-----------------------------------------------------------------------------
-bool CPyCppyy::CallContext::SetGlobalSignalPolicy(bool setProtected)
-{
-// Set the global signal policy, which determines whether a jmp address
-// should be saved to return to after a C++ segfault.
-    bool old = sSignalPolicy == kProtected;
-    sSignalPolicy = setProtected ? kProtected : kNone;
+    auto &flags = GlobalPolicyFlags();
+    bool old = flags & toggleFlag;
+    if (enabled)
+        flags |= toggleFlag;
+    else
+        flags &= ~toggleFlag;
     return old;
 }
-
