@@ -15,9 +15,7 @@ CppyyTestData::CppyyTestData() : m_const_int(17), m_owns_arrays(false)
     m_wchar    = L'D';
     m_char16   = u'\u00df';
     m_char32   = U'\u00df';
-#if __cplusplus > 201402L
     m_byte     = (std::byte)'d';
-#endif
     m_int8     =  -9;
     m_uint8    =   9;
     m_short    = -11;
@@ -42,9 +40,7 @@ CppyyTestData::CppyyTestData() : m_const_int(17), m_owns_arrays(false)
     m_bool_array2     = new bool[N];
     m_schar_array2    = new signed char[N];
     m_uchar_array2    = new unsigned char[N];
-#if __cplusplus > 201402L
     m_byte_array2     = new std::byte[N];
-#endif
     m_int8_array2     = new int8_t[N];
     m_uint8_array2    = new uint8_t[N];
     m_short_array2    = new short[N];
@@ -64,10 +60,8 @@ CppyyTestData::CppyyTestData() : m_const_int(17), m_owns_arrays(false)
         m_bool_array2[i]     =  bool((i+1)%2);
         m_uchar_array[i]     =   1u*i;
         m_uchar_array2[i]    =   2u*i;
-#if __cplusplus > 201402L
         m_byte_array[i]      =   (std::byte)(3u*i);
         m_byte_array2[i]     =   (std::byte)(4u*i);
-#endif
         m_int8_array[i]      = - 1*i;
         m_int8_array2[i]     = - 2*i;
         m_uint8_array[i]     =   3u*i;
@@ -110,9 +104,7 @@ void CppyyTestData::destroy_arrays() {
     if (m_owns_arrays == true) {
         delete[] m_bool_array2;
         delete[] m_uchar_array2;
-#if __cplusplus > 201402L
         delete[] m_byte_array2;
-#endif
         delete[] m_int8_array2;
         delete[] m_uint8_array2;
         delete[] m_short_array2;
@@ -139,9 +131,7 @@ unsigned char        CppyyTestData::get_uchar()    { return m_uchar; }
 wchar_t              CppyyTestData::get_wchar()    { return m_wchar; }
 char16_t             CppyyTestData::get_char16()   { return m_char16; }
 char32_t             CppyyTestData::get_char32()   { return m_char32; }
-#if __cplusplus > 201402L
 std::byte            CppyyTestData::get_byte()     { return m_byte; }
-#endif
 int8_t               CppyyTestData::get_int8()     { return m_int8; }
 uint8_t              CppyyTestData::get_uint8()    { return m_uint8; }
 short                CppyyTestData::get_short()    { return m_short; }
@@ -170,10 +160,8 @@ signed char*    CppyyTestData::get_schar_array()   { return m_schar_array; }
 signed char*    CppyyTestData::get_schar_array2()  { return m_schar_array2; }
 unsigned char*  CppyyTestData::get_uchar_array()   { return m_uchar_array; }
 unsigned char*  CppyyTestData::get_uchar_array2()  { return m_uchar_array2; }
-#if __cplusplus > 201402L
 std::byte*      CppyyTestData::get_byte_array()    { return m_byte_array; }
 std::byte*      CppyyTestData::get_byte_array2()   { return m_byte_array2; }
-#endif
 int8_t*         CppyyTestData::get_int8_array()    { return m_int8_array; }
 int8_t*         CppyyTestData::get_int8_array2()   { return m_int8_array2; }
 uint8_t*        CppyyTestData::get_uint8_array()   { return m_uint8_array; }
@@ -215,9 +203,7 @@ const unsigned char&        CppyyTestData::get_uchar_cr()    { return m_uchar; }
 const wchar_t&              CppyyTestData::get_wchar_cr()    { return m_wchar; }
 const char16_t&             CppyyTestData::get_char16_cr()   { return m_char16; }
 const char32_t&             CppyyTestData::get_char32_cr()   { return m_char32; }
-#if __cplusplus > 201402L
 const std::byte&            CppyyTestData::get_byte_cr()     { return m_byte; }
-#endif
 const int8_t&               CppyyTestData::get_int8_cr()     { return m_int8; }
 const uint8_t&              CppyyTestData::get_uint8_cr()    { return m_uint8; }
 const short&                CppyyTestData::get_short_cr()    { return m_short; }
@@ -246,9 +232,7 @@ unsigned char&        CppyyTestData::get_uchar_r()    { return m_uchar; }
 wchar_t&              CppyyTestData::get_wchar_r()    { return m_wchar; }
 char16_t&             CppyyTestData::get_char16_r()   { return m_char16; }
 char32_t&             CppyyTestData::get_char32_r()   { return m_char32; }
-#if __cplusplus > 201402L
 std::byte&            CppyyTestData::get_byte_r()     { return m_byte; }
-#endif
 int8_t&               CppyyTestData::get_int8_r()     { return m_int8; }
 uint8_t&              CppyyTestData::get_uint8_r()    { return m_uint8; }
 short&                CppyyTestData::get_short_r()    { return m_short; }
@@ -277,9 +261,7 @@ void CppyyTestData::set_uchar(unsigned char uc)            { m_uchar    = uc; }
 void CppyyTestData::set_wchar(wchar_t wc)                  { m_wchar    = wc; }
 void CppyyTestData::set_char16(char16_t c16)               { m_char16   = c16; }
 void CppyyTestData::set_char32(char32_t c32)               { m_char32   = c32; }
-#if __cplusplus > 201402L
 void CppyyTestData::set_byte(std::byte b)                  { m_byte     = b; }
-#endif
 void CppyyTestData::set_int8(int8_t s8)                    { m_int8     = s8; }
 void CppyyTestData::set_uint8(uint8_t u8)                  { m_uint8    = u8; }
 void CppyyTestData::set_short(short s)                     { m_short    = s; }
@@ -321,9 +303,7 @@ void CppyyTestData::set_uchar_cr(const unsigned char& uc)        { m_uchar    =
 void CppyyTestData::set_wchar_cr(const wchar_t& wc)              { m_wchar    = wc; }
 void CppyyTestData::set_char16_cr(const char16_t& c16)           { m_char16   = c16; }
 void CppyyTestData::set_char32_cr(const char32_t& c32)           { m_char32   = c32; }
-#if __cplusplus > 201402L
 void CppyyTestData::set_byte_cr(const std::byte& b)              { m_byte     = b; }
-#endif
 void CppyyTestData::set_int8_cr(const int8_t& s8)                { m_int8     = s8; }
 void CppyyTestData::set_uint8_cr(const uint8_t& u8)              { m_uint8    = u8; }
 void CppyyTestData::set_short_cr(const short& s)                 { m_short    = s; }
@@ -352,9 +332,7 @@ void CppyyTestData::set_char16_r(char16_t& c16)           { c16 = u'\u6c24'; }
 void CppyyTestData::set_char32_r(char32_t& c32)           { c32 = U'\U0001f34e'; }
 void CppyyTestData::set_schar_r(signed char& sc)          { sc = 'c'; }
 void CppyyTestData::set_uchar_r(unsigned char& uc)        { uc = 'd'; }
-#if __cplusplus > 201402L
 void CppyyTestData::set_byte_r(std::byte& b)              { b = (std::byte)'e'; }
-#endif
 void CppyyTestData::set_short_r(short& s)                 { s = -1; }
 void CppyyTestData::set_ushort_r(unsigned short& us)      { us = 2; }
 void CppyyTestData::set_int_r(int& i)                     { i = -3; }
@@ -375,9 +353,7 @@ void CppyyTestData::set_char16_p(char16_t* c16)           { *c16 = u'\u6c24'; }
 void CppyyTestData::set_char32_p(char32_t* c32)           { *c32 = U'\U0001f34e'; }
 void CppyyTestData::set_schar_p(signed char* sc)          { *sc = 'c'; }
 void CppyyTestData::set_uchar_p(unsigned char* uc)        { *uc = 'd'; }
-#if __cplusplus > 201402L
 void CppyyTestData::set_byte_p(std::byte* b)              { *b = (std::byte)'e'; }
-#endif
 void CppyyTestData::set_int8_p(int8_t* i8)                { *i8 = -27; }
 void CppyyTestData::set_uint8_p(uint8_t* ui8)             { *ui8 = 28; }
 void CppyyTestData::set_short_p(short* s)                 { *s = -1; }
@@ -421,12 +397,10 @@ void CppyyTestData::set_uchar_ppa(unsigned char** uc) {
     (*uc) = new unsigned char[3];
     (*uc)[0] = 'k'; (*uc)[1] = 'l'; (*uc)[2] = 'm';
 }
-#if __cplusplus > 201402L
 void CppyyTestData::set_byte_ppa(std::byte** b) {
     (*b) = new std::byte[3];
     (*b)[0] = (std::byte)'n'; (*b)[1] = (std::byte)'o'; (*b)[2] = (std::byte)'p';
 }
-#endif
 void CppyyTestData::set_int8_ppa(int8_t** i8) {
     (*i8) = new int8_t[3];
     (*i8)[0] = -27; (*i8)[1] = -28; (*i8)[2] = -29;
@@ -531,9 +505,7 @@ void CppyyTestData::set_uchar_rv(unsigned char&& uc)        { m_uchar    = uc; }
 void CppyyTestData::set_wchar_rv(wchar_t&& wc)              { m_wchar    = wc; }
 void CppyyTestData::set_char16_rv(char16_t&& c16)           { m_char16   = c16; }
 void CppyyTestData::set_char32_rv(char32_t&& c32)           { m_char32   = c32; }
-#if __cplusplus > 201402L
 void CppyyTestData::set_byte_rv(std::byte&& b)              { m_byte     = b; }
-#endif
 void CppyyTestData::set_int8_rv(int8_t&& s8)                { m_int8     = s8; }
 void CppyyTestData::set_uint8_rv(uint8_t&& u8)              { m_uint8    = u8; }
 void CppyyTestData::set_short_rv(short&& s)                 { m_short    = s; }
@@ -575,9 +547,7 @@ unsigned char        CppyyTestData::s_uchar    = 'u';
 wchar_t              CppyyTestData::s_wchar    = L'U';
 char16_t             CppyyTestData::s_char16   = u'\u6c29';
 char32_t             CppyyTestData::s_char32   = U'\U0001f34b';
-#if __cplusplus > 201402L
 std::byte            CppyyTestData::s_byte     = (std::byte)'b';
-#endif
 int8_t               CppyyTestData::s_int8     = - 87;
 uint8_t              CppyyTestData::s_uint8    =   87;
 short                CppyyTestData::s_short    = -101;
@@ -637,9 +607,7 @@ unsigned char      g_uchar    = 'u';
 wchar_t            g_wchar    = L'U';
 char16_t           g_char16   = u'\u6c21';
 char32_t           g_char32   = U'\u6c21';
-#if __cplusplus > 201402L
 std::byte          g_byte     = (std::byte)'x';
-#endif
 int8_t             g_int8     =  -66;
 uint8_t            g_uint8    =   66;
 short              g_short    =  -88;
@@ -713,12 +681,10 @@ int64_t sum_uc_data(unsigned char* data, int size)
     return total;
 }
 
-#if __cplusplus > 201402L
 int64_t sum_byte_data(std::byte* data, int size)
 {
     return sum_uc_data((unsigned char*)data, size);
 }
-#endif
 
 
 //= function pointer passing ================================================
