@@ -1,12 +1,9 @@
-import py
-from pytest import raises
-from .support import setup_make
+import py, pytest, os
+from pytest import raises, mark
+from support import setup_make, IS_MAC
 
-currpath = py.path.local(__file__).dirpath()
-test_dct = str(currpath.join("example01Dict"))
-
-def setup_module(mod):
-    setup_make("example01")
+currpath = os.getcwd()
+test_dct = currpath + "/libexample01Dict"
 
 
 class TestACLASSLOADER:
@@ -14,6 +11,7 @@ class TestACLASSLOADER:
     def setup_class(cls):
         import cppyy
 
+    @mark.xfail(condition=IS_MAC, reason="Fails on OSX")
     def test01_class_autoloading(self):
         """Test whether a class can be found through .rootmap."""
         import cppyy
@@ -22,3 +20,7 @@ class TestACLASSLOADER:
         cl2 = cppyy.gbl.example01
         assert cl2
         assert example01_class is cl2
+
+
+if __name__ == "__main__":
+    exit(pytest.main(args=['-sv', '-ra', __file__]))
