@@ -1,12 +1,9 @@
-import py
+import py, pytest, os
 from pytest import raises
-from .support import setup_make
+from support import setup_make
 
-currpath = py.path.local(__file__).dirpath()
-test_dct = str(currpath.join("conversionsDict"))
-
-def setup_module(mod):
-    setup_make("conversions")
+currpath = os.getcwd()
+test_dct = currpath + "/libconversionsDict"
 
 
 class TestCONVERSIONS:
@@ -125,3 +122,7 @@ class TestCONVERSIONS:
         assert     ns.Test1()
         assert     ns.Test2(True)
         assert not ns.Test2(False)
+
+
+if __name__ == "__main__":
+    exit(pytest.main(args=['-sv', '-ra', __file__]))
