@@ -1,12 +1,10 @@
-import py, sys
+import py, sys, pytest, os
 from pytest import mark, raises, skip
-from .support import setup_make, pylong, pyunicode, IS_WINDOWS, ispypy
+from support import setup_make, pylong, pyunicode, IS_WINDOWS, ispypy
 
-currpath = py.path.local(__file__).dirpath()
-test_dct = str(currpath.join("datatypesDict"))
 
-def setup_module(mod):
-    setup_make("datatypes")
+currpath = os.getcwd()
+test_dct = currpath + "/libdatatypesDict"
 
 
 class TestLOWLEVEL:
@@ -17,9 +15,6 @@ class TestLOWLEVEL:
         cls.datatypes = cppyy.load_reflection_info(cls.test_dct)
         cls.N = cppyy.gbl.N
 
-        at_least_17 = 201402 < cppyy.gbl.gInterpreter.ProcessLine("__cplusplus;")
-        cls.has_nested_namespace = at_least_17
-
     def test00_import_all(self):
         """Validity of `from cppyy.ll import *`"""
 
@@ -499,9 +494,6 @@ class TestLOWLEVEL:
     def test15_templated_arrays_gmpxx(self):
         """Use of gmpxx array types in templates"""
 
-        if not self.has_nested_namespace:
-            return
-
         import cppyy
 
         try:
@@ -754,3 +746,7 @@ class TestMULTIDIMARRAYS:
         for i, v in enumerate(("s1", "s23", "s456")):
             assert len(ns.str_array[i]) == 7
             assert ns.str_array[i].as_string() == v
+
+
+if __name__ == "__main__":
+    exit(pytest.main(args=['-sv', '-ra', __file__]))
