@@ -1,12 +1,10 @@
-import py
-from pytest import raises, skip
-from .support import setup_make, pylong, maxvalue, IS_WINDOWS
+import py, pytest, os
+from pytest import raises, skip, mark
+from support import setup_make, pylong, maxvalue, IS_WINDOWS, IS_MAC, no_root_errors
 
-currpath = py.path.local(__file__).dirpath()
-test_dct = str(currpath.join("operatorsDict"))
 
-def setup_module(mod):
-    setup_make("operators")
+currpath = os.getcwd()
+test_dct = currpath + "/liboperatorsDict"
 
 
 class TestOPERATORS:
@@ -224,12 +222,16 @@ class TestOPERATORS:
             assert m[1]    == 74
             assert m(1,2)  == 74
 
+    @mark.xfail(reason="Compilation of unused call wrappers emits errors")
     def test09_templated_operator(self):
         """Templated operator<()"""
 
         from cppyy.gbl import TOIClass
 
-        assert (TOIClass() < 1)
+        # We don't want to see compile errors for overloads that were tried
+        # but didn't succeed
+        with no_root_errors():
+            assert (TOIClass() < 1)
 
     def test10_r_non_associative(self):
         """Use of radd/rmul with non-associative types"""
@@ -336,6 +338,7 @@ class TestOPERATORS:
         b = ns.Bar()
         assert b[42] == 42
 
+    @mark.xfail(condition=IS_MAC, reason="Fails on OSX")
     def test15_class_and_global_mix(self):
         """Iterator methods have both class and global overloads"""
 
@@ -384,3 +387,26 @@ class TestOPERATORS:
 
         assert     ns.AGe(5) >= ns.AGe(4)
         assert not ns.AGe(4) >= ns.AGe(5)
+
+    def test17_arrow_operator_recursion(self):
+        """operator->() returning same type should not recurse"""
+
+        import cppyy
+
+        cppyy.cppdef(r"""\
+        namespace Recursion {
+        class MCPCollection {
+        public:
+          MCPCollection() = default;
+          MCPCollection* operator->() { return this; }
+        }; }""")
+
+        ns = cppyy.gbl.Recursion
+
+        coll = ns.MCPCollection()
+        with raises(AttributeError):
+            coll.non_existing_method
+
+
+if __name__ == "__main__":
+    exit(pytest.main(args=['-sv', '-ra', __file__]))
