@@ -1,14 +1,12 @@
 # -*- coding: UTF-8 -*-
-import py, sys
-from pytest import raises, skip
-from .support import setup_make, pylong, pyunicode, maxvalue, ispypy
+import py, sys, pytest, os
+from pytest import mark, raises, skip
+from support import setup_make, pylong, pyunicode, maxvalue, ispypy, no_root_errors
 
-currpath = py.path.local(__file__).dirpath()
-test_dct = str(currpath.join("stltypesDict"))
-
-def setup_module(mod):
-    setup_make("stltypes")
+currpath = os.getcwd()
+test_dct = currpath + "/libstltypesDict"
 
+global_n = 5
 
 # after CPython's Lib/test/seq_tests.py
 def iterfunc(seqn):
@@ -197,7 +195,7 @@ class TestSTLVECTOR:
         cls.test_dct = test_dct
         import cppyy
         cls.stltypes = cppyy.load_reflection_info(cls.test_dct)
-        cls.N = cppyy.gbl.N
+        cls.N = global_n
 
     def test01_builtin_type_vector_types(self):
         """Test access to std::vector<int>/std::vector<double>"""
@@ -287,7 +285,7 @@ class TestSTLVECTOR:
         v.__destruct__()
 
     def test03_empty_vector_type(self):
-        """Test behavior of empty std::vector<int>"""
+        """Test behavior of empty std::vector<>"""
 
         import cppyy
 
@@ -297,6 +295,9 @@ class TestSTLVECTOR:
             pass
         v.__destruct__()
 
+        for x in cppyy.gbl.std.vector["std::string*"]():
+            pass
+
     def test04_vector_iteration(self):
         """Test iteration over an std::vector<int>"""
 
@@ -492,6 +493,7 @@ class TestSTLVECTOR:
         ll4[1] = 'a'
         raises(TypeError, a.vector_pair, ll4)
 
+    @mark.skip()
     def test12_vector_lifeline(self):
         """Check lifeline setting on vectors of objects"""
 
@@ -614,6 +616,7 @@ class TestSTLVECTOR:
         v = cppyy.gbl.std.vector(l)
         assert list(l) == l
 
+    @mark.xfail
     def test18_array_interface(self):
         """Test usage of __array__ from numpy"""
 
@@ -789,6 +792,58 @@ class TestSTLVECTOR:
         for f, d in zip(x, v):
             assert f == d
 
+    def test24_byte_vectors(self):
+        """Vectors of "byte" types should return low level views"""
+
+        import cppyy
+        import cppyy.types
+
+        vector = cppyy.gbl.std.vector
+
+        for ctype in ('unsigned char', 'signed char', 'int8_t', 'uint8_t'):
+            vc = vector[ctype](range(10))
+            data = vc.data()
+
+            assert type(data) == cppyy.types.LowLevelView
+            assert len(data) == 10
+
+            for i, d in enumerate(data):
+                assert d == i
+
+        for ctype in ('signed char', 'int8_t'):
+            vc = vector[ctype](range(-5, 5, 1))
+            data = vc.data()
+
+            assert type(data) == cppyy.types.LowLevelView
+            assert len(data) == 10
+
+            for i, d in zip(range(-5, 5, 1), data):
+                assert d == i
+
+    def test24_vector_to_span(self):
+        """Vectors should convert to std::span without errors"""
+
+        import cppyy
+
+        cppyy.cppdef("""
+        double calc_cumsum(std::size_t n, std::span<double> v)
+        {
+            double sum = 0.0;
+            for (int i = 0; i < n; i += 1) {
+                sum += v[i];
+            }
+            return sum;
+        }
+        """)
+
+        l = list(range(4))
+        v = cppyy.gbl.std.vector["double"](l)
+
+        with no_root_errors():
+            result = cppyy.gbl.calc_cumsum(len(v), v)
+
+        assert result == sum(l)
+
 
 class TestSTLSTRING:
     def setup_class(cls):
@@ -829,6 +884,7 @@ class TestSTLSTRING:
 
             raises(TypeError, c.get_string2, "temp string")
 
+    @mark.xfail()
     def test02_string_data_access(self):
         """Test access to std::string object data members"""
 
@@ -961,6 +1017,7 @@ class TestSTLSTRING:
         assert d[x] == 0
         assert d['x'] == 0
 
+    @mark.xfail()
     def test08_string_operators(self):
         """Mixing of C++ and Python types in global operators"""
 
@@ -1050,6 +1107,7 @@ class TestSTLSTRING:
         assert s.rfind('c')  < 0
         assert s.rfind('c') == s.npos
 
+    @mark.xfail()
     def test10_string_in_repr_and_str_bytes(self):
         """Special cases for __str__/__repr__"""
 
@@ -1639,9 +1697,7 @@ class TestSTLSTRING_VIEW:
         """Usage of std::string_view as formal argument"""
 
         import cppyy
-        if cppyy.gbl.gInterpreter.ProcessLine("__cplusplus;") <= 201402:
-            # string_view exists as of C++17
-            return
+
         countit = cppyy.gbl.StringViewTest.count
         countit_cr = cppyy.gbl.StringViewTest.count_cr
 
@@ -1698,7 +1754,7 @@ class TestSTLDEQUE:
         cls.test_dct = test_dct
         import cppyy
         cls.stltypes = cppyy.load_reflection_info(cls.test_dct)
-        cls.N = cppyy.gbl.N
+        cls.N = global_n
 
     def test01_deque_byvalue_regression(self):
         """Return by value of a deque used to crash"""
@@ -1725,7 +1781,7 @@ class TestSTLSET:
         cls.test_dct = test_dct
         import cppyy
         cls.stltypes = cppyy.load_reflection_info(cls.test_dct)
-        cls.N = cppyy.gbl.N
+        cls.N = global_n
 
     def test01_set_iteration(self):
         """Iterate over a set"""
@@ -1812,13 +1868,20 @@ class TestSTLSET:
         for i in range(100):
             assert not (2**30 in S)
 
+        assert '__contains__' in cppyy.gbl.std.string.__dict__
+        my_string = cppyy.gbl.std.string("hello world")
+
+        assert "hello" in my_string
+        assert "world" in my_string
+        assert "bye" not in my_string
+
 
 class TestSTLTUPLE:
     def setup_class(cls):
         cls.test_dct = test_dct
         import cppyy
         cls.stltypes = cppyy.load_reflection_info(cls.test_dct)
-        cls.N = cppyy.gbl.N
+        cls.N = global_n
 
     def test01_tuple_creation_and_access(self):
         """Create tuples and access their elements"""
@@ -1863,6 +1926,7 @@ class TestSTLTUPLE:
         t = std.make_tuple("aap", 42, 5.)
         assert std.tuple_size(type(t)).value == 3
 
+    @mark.xfail()
     def test03_tuple_iter(self):
         """Pack/unpack tuples"""
 
@@ -1877,6 +1941,7 @@ class TestSTLTUPLE:
         assert b == '2'
         assert c == 5.
 
+    @mark.xfail()
     def test04_tuple_lifeline(self):
         """Tuple memory management"""
 
@@ -1908,7 +1973,7 @@ class TestSTLPAIR:
         cls.test_dct = test_dct
         import cppyy
         cls.stltypes = cppyy.load_reflection_info(cls.test_dct)
-        cls.N = cppyy.gbl.N
+        cls.N = global_n
 
     def test01_pair_pack_unpack(self):
         """Pack/unpack pairs"""
@@ -2080,3 +2145,7 @@ class TestSTLEXCEPTION:
 
         gc.collect()
         assert cppyy.gbl.GetMyErrorCount() == 0
+
+
+if __name__ == "__main__":
+    exit(pytest.main(args=['-sv', '-ra', __file__]))
