@@ -1,12 +1,10 @@
-import py
-from pytest import raises
-from .support import setup_make
+import py, pytest, os
+from pytest import mark, raises
+from support import setup_make
 
-currpath = py.path.local(__file__).dirpath()
-test_dct = str(currpath.join("std_streamsDict"))
 
-def setup_module(mod):
-    setup_make("std_streams")
+currpath = os.getcwd()
+test_dct = currpath + "/libstd_streamsDict"
 
 
 class TestSTDStreams:
@@ -48,6 +46,7 @@ class TestSTDStreams:
         cppyy.gbl.stringstream_base.pass_through_base(s)
         assert s.str() == "TEST STRING"
 
+    @mark.xfail()
     def test04_naming_of_ostringstream(self):
         """Naming consistency of ostringstream"""
 
@@ -73,3 +72,7 @@ class TestSTDStreams:
         assert cl0 == cl1
         assert cl1 == cl2
         assert cl2 == cl0
+
+
+if __name__ == "__main__":
+    exit(pytest.main(args=['-sv', '-ra', __file__]))
