@@ -2,6 +2,7 @@
 Tests to verify that TH1 and derived histograms conform to the UHI Indexing interfaces (setting, accessing and slicing).
 """
 
+import numpy as np
 import pytest
 import ROOT
 from ROOT._pythonization._uhi import _get_axis, _get_processed_slices, _overflow, _shape, _underflow
@@ -12,7 +13,7 @@ def _special_setting(hist):
     # For these classes, SetBinContent works differently than for other classes,
     # as in it does not set the bin content to the specified value, but does some other calculations
     # for that, these classes will be tested differently
-    return isinstance(hist, (ROOT.TProfile, ROOT.TProfile2D, ROOT.TProfile2Poly, ROOT.TProfile3D, ROOT.TH1K))
+    return isinstance(hist, (ROOT.TProfile, ROOT.TProfile2D, ROOT.TProfile2Poly, ROOT.TProfile3D))
 
 
 def _get_index_for_dimension(hist, index):
@@ -299,9 +300,9 @@ class TestTH1Indexing:
         # Check if slicing over everything preserves the statistics
         sliced_hist_full = hist_setup[...]
 
-        assert hist_setup.GetEffectiveEntries() == sliced_hist_full.GetEffectiveEntries()
-        assert sliced_hist_full.GetEntries() == sliced_hist_full.GetEffectiveEntries()
-        assert hist_setup.Integral() == sliced_hist_full.Integral()
+        assert hist_setup.GetEffectiveEntries() == pytest.approx(sliced_hist_full.GetEffectiveEntries())
+        assert sliced_hist_full.GetEntries() == pytest.approx(sliced_hist_full.GetEffectiveEntries())
+        assert hist_setup.Integral() == pytest.approx(sliced_hist_full.Integral())
 
         # Check if slicing over a range updates the statistics
         dim = hist_setup.GetDimension()
@@ -310,24 +311,68 @@ class TestTH1Indexing:
         sliced_hist = hist_setup[slice_indices]
 
         assert hist_setup.Integral() == sliced_hist.Integral()
-        assert hist_setup.GetEffectiveEntries() == sliced_hist.GetEffectiveEntries()
-        assert sliced_hist.GetEntries() == sliced_hist.GetEffectiveEntries()
+        assert hist_setup.GetEffectiveEntries() == pytest.approx(sliced_hist.GetEffectiveEntries())
+        assert sliced_hist.GetEntries() == pytest.approx(sliced_hist.GetEffectiveEntries())
         assert hist_setup.GetStdDev() == pytest.approx(sliced_hist.GetStdDev(), rel=10e-5)
         assert hist_setup.GetMean() == pytest.approx(sliced_hist.GetMean(), rel=10e-5)
 
-    def test_equality(self, hist_setup):
+    def test_list_iter(self, hist_setup):
+        import numpy as np
+
         if _special_setting(hist_setup):
             pytest.skip("Setting cannot be tested here")
 
-        hist_copy_ptr = hist_setup
-        assert hist_setup == hist_copy_ptr
+        expected = np.full(_shape(hist_setup), 3, dtype=np.int64)
+        hist_setup[...] = expected
+        assert list(hist_setup) == expected.flatten().tolist()
 
-        hist_copy = hist_setup.Clone()
-        assert hist_setup != hist_copy
 
-        hist_full_slice = hist_setup[...]
-        assert hist_setup != hist_full_slice
+class TestInfiniteEdges:
+    def setup_method(self):
+        # create a 2D histogram with an infinite upper edge on the Y axis
+        xedges = np.array([0.0, 1.0, 2.0], dtype="float64")
+        yedges = np.array([0.0, 1.0, 2.0, np.inf], dtype="float64")
+        self.h_inf = ROOT.TH2D("h_inf", "h_inf", len(xedges) - 1, xedges, len(yedges) - 1, yedges)
+
+        for i in range(1, self.h_inf.GetNbinsX() + 1):
+            for j in range(1, self.h_inf.GetNbinsY() + 1):
+                self.h_inf.SetBinContent(i, j, 10 * i + j)
+
+    def test_uhi_projection_preserves_content(self):
+        """check that UHI projection behaves like standard projection"""
+        # projection on X axis
+        proj_x_ref = self.h_inf.ProjectionX()
+        proj_x_uhi = self.h_inf[:, ROOT.uhi.sum]
+        ref_values = proj_x_ref.values()
+        uhi_values = proj_x_uhi.values()
+
+        assert np.allclose(uhi_values, ref_values)
+
+        # projection on Y axis
+        proj_y_ref = self.h_inf.ProjectionY()
+        proj_y_uhi = self.h_inf[ROOT.uhi.sum, :]
+        ref_values = proj_y_ref.values()
+        uhi_values = proj_y_uhi.values()
+
+        assert np.allclose(uhi_values, ref_values)
+
+
+class TestHistogramKind:
+    def test_histogram_kind_no_weights(self):
+        h = ROOT.TH1D("h", "h", 100, -10, 10)
+        h.Fill(5)
+        assert h.kind == "COUNT"
+
+    def test_histogram_kind_with_weights(self):
+        h_weighted = ROOT.TH1D("h_weighted", "h_weighted", 100, -10, 10)
+        h_weighted.Fill(5, 2.0)
+        assert h_weighted.kind == "COUNT"
+
+    def test_profile_histogram_kind(self):
+        h_profile = ROOT.TProfile("h_profile", "h_profile", 100, -10, 10)
+        h_profile.Fill(5, 3.0)
+        assert h_profile.kind == "MEAN"
 
 
 if __name__ == "__main__":
-    pytest.main(args=[__file__])
+    raise SystemExit(pytest.main(args=[__file__]))
