@@ -74,7 +74,6 @@ Bool_t TIdleTimer::Notify()
 }
 
 
-ClassImp(TApplication);
 
 static void CallEndOfProcessCleanups()
 {
@@ -111,8 +110,7 @@ TApplication::TApplication() :
 /// line options recognized by TApplication are described in the GetOptions()
 /// method. The recognized options are removed from the argument array.
 /// The original list of argument options can be retrieved via the Argc()
-/// and Argv() methods. The appClassName "proofserv" is reserved for the
-/// PROOF system. The "options" and "numOptions" arguments are not used,
+/// and Argv() methods. The "options" and "numOptions" arguments are not used,
 /// except if you want to by-pass the argv processing by GetOptions()
 /// in which case you should specify numOptions<0. All options will
 /// still be available via the Argv() method for later use.
@@ -179,6 +177,10 @@ TApplication::TApplication(const char *appClassName, Int_t *argc, char **argv,
    if (fArgv)
       gSystem->SetProgname(fArgv[0]);
 
+   // Alternative to '-b' command line switch (i.e. for pyROOT)
+   if (gSystem->Getenv("ROOT_BATCH"))
+      MakeBatch();
+
    // Tell TSystem the TApplication has been created
    gSystem->NotifyApplicationCreated();
 
@@ -990,8 +992,8 @@ TString TApplication::GetSetup()
                                          gROOT->GetGitBranch(),
                                          gROOT->GetGitCommit()));
    }
-   lines.emplace_back(TString::Format("With %s",
-                                      gSystem->GetBuildCompilerVersionStr()));
+   lines.emplace_back(TString::Format("With %s std%ld",
+                                      gSystem->GetBuildCompilerVersionStr(), __cplusplus));
    lines.emplace_back("Binary directory: "+ gROOT->GetBinDir());
    lines.emplace_back("```");
    TString setup = "";
@@ -1339,6 +1341,7 @@ void TApplication::Help(const char *line)
          Printf("   Ctrl+D           : quit ROOT (if empty line)");
          Printf("   Ctrl+C           : send kSigInt interrupt signal");
          Printf("   Ctrl+Z           : send kSigStop pause job signal");
+         Printf("   Ctrl+\\           : send kSigQuit quit job signal");
 
          Printf("   Arrow_Down       : navigate downwards in command history [Ctrl+N]");
          Printf("   Arrow_Up         : navigate upwards in command history [Ctrl+P]");
