@@ -69,6 +69,7 @@ General purpose message signal
 #include "RQ_OBJECT.h"
 #include "TVirtualMutex.h"
 #include "RConfigure.h"
+#include "strlcpy.h"
 
 void *gTQSender; // A pointer to the object that sent the last signal.
                  // Getting access to the sender might be practical
@@ -77,10 +78,6 @@ void *gTQSender; // A pointer to the object that sent the last signal.
 Bool_t TQObject::fgAllSignalsBlocked = kFALSE;
 
 
-ClassImpQ(TQObject)
-ClassImpQ(TQObjSender)
-ClassImpQ(TQClass)
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Removes "const" words and blanks from full (with prototype)
 /// method name and resolve any typedefs in the method signature.
@@ -101,11 +98,11 @@ TString TQObject::CompressName(const char *method_name)
       return res;
 
    {
-      static TVirtualMutex *  lock = 0;
+      static TVirtualMutex *  lock = nullptr;
       R__LOCKGUARD2(lock);
 
-      static TPMERegexp *constRe = 0, *wspaceRe = 0;
-      if (constRe == 0) {
+      static TPMERegexp *constRe = nullptr, *wspaceRe = nullptr;
+      if (constRe == nullptr) {
          constRe  = new TPMERegexp("(?<=\\(|\\s|,|&|\\*)const(?=\\s|,|\\)|&|\\*)", "go");
          wspaceRe = new TPMERegexp("\\s+(?=([^\"]*\"[^\"]*\")*[^\"]*$)", "go");
       }
@@ -152,7 +149,7 @@ TMethod *GetMethodWithPrototype(TClass *cl, const char *method,
 {
    nargs = 0;
 
-   if (!gInterpreter || cl == 0) return 0;
+   if (!gInterpreter || cl == nullptr) return nullptr;
 
    TMethod *m = cl->GetMethodWithPrototype(method,proto);
    if (m) nargs = m->GetNargs();
@@ -164,7 +161,7 @@ TMethod *GetMethodWithPrototype(TClass *cl, const char *method,
 
 static TMethod *GetMethod(TClass *cl, const char *method, const char *params)
 {
-   if (!gInterpreter || cl == 0) return 0;
+   if (!gInterpreter || cl == nullptr) return nullptr;
    return cl->GetMethod(method,params);
 }
 
@@ -179,8 +176,9 @@ Int_t TQObject::CheckConnectArgs(TQObject *sender,
                                  TClass *sender_class, const char *signal,
                                  TClass *receiver_class, const char *slot)
 {
-   char *signal_method = new char[strlen(signal)+1];
-   if (signal_method) strcpy(signal_method, signal);
+   auto len = strlen(signal)+1;
+   char *signal_method = new char[len];
+   if (signal_method) strlcpy(signal_method, signal, len);
 
    char *signal_proto;
    char *tmp;
@@ -236,11 +234,12 @@ Int_t TQObject::CheckConnectArgs(TQObject *sender,
    // cleaning
    delete [] signal_method;
 
-   char *slot_method = new char[strlen(slot)+1];
-   if (slot_method) strcpy(slot_method, slot);
+   auto len2 = strlen(slot)+1;
+   char *slot_method = new char[len2];
+   if (slot_method) strlcpy(slot_method, slot, len2);
 
    char *slot_proto;
-   char *slot_params = 0;
+   char *slot_params = nullptr;
 
    if ((slot_proto = strchr(slot_method,'('))) {
 
@@ -254,10 +253,10 @@ Int_t TQObject::CheckConnectArgs(TQObject *sender,
    if (!slot_proto) slot_proto = (char*)"";     // avoid zero strings
    if ((slot_params = strchr(slot_proto,'='))) *slot_params = ' ';
 
-   TFunction *slotMethod = 0;
+   TFunction *slotMethod = nullptr;
    if (!receiver_class) {
       // case of slot_method is compiled/intrepreted function
-      slotMethod = gROOT->GetGlobalFunction(slot_method,0,kFALSE);
+      slotMethod = gROOT->GetGlobalFunction(slot_method,nullptr,kFALSE);
    } else {
       slotMethod  = !slot_params ?
                           GetMethodWithPrototype(receiver_class,
@@ -311,11 +310,11 @@ private:
 public:
    TQConnectionList(const char *name, Int_t nsigargs) : TList()
       { fName = name; fSignalArgs = nsigargs; }
-   virtual ~TQConnectionList();
+   ~TQConnectionList() override;
 
-   Bool_t Disconnect(void *receiver=0, const char *slot_name=0);
+   Bool_t Disconnect(void *receiver = nullptr, const char *slot_name = nullptr);
    Int_t  GetNargs() const { return fSignalArgs; }
-   void   ls(Option_t *option = "") const;
+   void   ls(Option_t *option = "") const override;
 };
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -340,7 +339,7 @@ TQConnectionList::~TQConnectionList()
 
 Bool_t TQConnectionList::Disconnect(void *receiver, const char *slot_name)
 {
-   TQConnection *connection = 0;
+   TQConnection *connection = nullptr;
    Bool_t return_value = kFALSE;
 
    TObjLink *lnk = FirstLink();
@@ -389,8 +388,8 @@ void TQConnectionList::ls(Option_t *option) const
 
 TQObject::TQObject()
 {
-   fListOfSignals     = 0;
-   fListOfConnections = 0;
+   fListOfSignals     = nullptr;
+   fListOfConnections = nullptr;
    fSignalsBlocked    = kFALSE;
 }
 
@@ -431,11 +430,11 @@ TQObject::~TQObject()
 
 TList *TQObject::GetListOfClassSignals() const
 {
-   TQClass *qcl = 0;
+   TQClass *qcl = nullptr;
 
    qcl = dynamic_cast<TQClass*>(IsA());
 
-   return qcl ? qcl->fListOfSignals : 0; //!!
+   return qcl ? qcl->fListOfSignals : nullptr; //!!
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -528,7 +527,7 @@ Bool_t TQObject::HasConnection(const char *signal_name) const
 
    TString signal = CompressName(signal_name);
 
-   return (fListOfSignals->FindObject(signal) != 0);
+   return (fListOfSignals->FindObject(signal) != nullptr);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -588,7 +587,7 @@ Bool_t TQObject::ConnectToClass(TQObject *sender,
    }
 
    TIter next(clist);
-   TQConnection *connection = 0;
+   TQConnection *connection = nullptr;
 
    while ((connection = (TQConnection*)next())) {
       if (!strcmp(slot_name,connection->GetName()) &&
@@ -631,7 +630,7 @@ Bool_t TQObject::ConnectToClass(const char *class_name,
 
    // check consistency of signal/slot methods/args
    Int_t nsigargs;
-   if ((nsigargs = CheckConnectArgs(0, sender, signal_name, cl, slot_name)) == -1)
+   if ((nsigargs = CheckConnectArgs(nullptr, sender, signal_name, cl, slot_name)) == -1)
       return kFALSE;
 
    if (!slist)
@@ -644,7 +643,7 @@ Bool_t TQObject::ConnectToClass(const char *class_name,
       slist->Add(clist);
    }
 
-   TQConnection *connection = 0;
+   TQConnection *connection = nullptr;
    TIter next(clist);
 
    while ((connection = (TQConnection*)next())) {
@@ -721,7 +720,7 @@ Bool_t TQObject::Connect(TQObject *sender,
 
    // check consistency of signal/slot methods/args
    Int_t nsigargs;
-   if ((nsigargs = CheckConnectArgs(sender, sender->IsA(), signal_name, 0, slot_name)) == -1)
+   if ((nsigargs = CheckConnectArgs(sender, sender->IsA(), signal_name, nullptr, slot_name)) == -1)
       return kFALSE;
 
    if (!sender->fListOfSignals) sender->fListOfSignals = new THashList();
@@ -734,7 +733,7 @@ Bool_t TQObject::Connect(TQObject *sender,
       sender->fListOfSignals->Add(clist);
    }
 
-   TQConnection *connection = 0;
+   TQConnection *connection = nullptr;
    TIter next(clist);
 
    while ((connection = (TQConnection*)next())) {
@@ -817,7 +816,7 @@ Bool_t TQObject::Connect(const char *class_name,
 
    // check consistency of signal/slot methods/args
    Int_t nsigargs;
-   if ((nsigargs = CheckConnectArgs(0, sender, signal_name, 0, slot_name)) == -1)
+   if ((nsigargs = CheckConnectArgs(nullptr, sender, signal_name, nullptr, slot_name)) == -1)
       return kFALSE;
 
    if (!slist) {
@@ -832,7 +831,7 @@ Bool_t TQObject::Connect(const char *class_name,
       slist->Add(clist);
    }
 
-   TQConnection *connection = 0;
+   TQConnection *connection = nullptr;
    TIter next(clist);
 
    while ((connection = (TQConnection*)next())) {
@@ -873,7 +872,7 @@ Bool_t TQObject::Connect(const char *signal,
    TString slot_name   = CompressName(slot);
 
    // check consistency of signal/slot methods/args
-   TClass *cl = 0;
+   TClass *cl = nullptr;
    if (receiver_class)
       cl = TClass::GetClass(receiver_class);
    Int_t nsigargs;
@@ -891,7 +890,7 @@ Bool_t TQObject::Connect(const char *signal,
    }
 
    TIter next(clist);
-   TQConnection *connection = 0;
+   TQConnection *connection = nullptr;
 
    while ((connection = (TQConnection*)next())) {
       if (!strcmp(slot_name,connection->GetName()) &&
@@ -967,7 +966,7 @@ Bool_t TQObject::Disconnect(TQObject *sender,
    TString signal_name = CompressName(signal);
    TString slot_name   = CompressName(slot);
 
-   TQConnectionList *slist = 0;
+   TQConnectionList *slist = nullptr;
    TIter next_signal(sender->GetListOfSignals());
 
    while ((slist = (TQConnectionList*)next_signal()))   {
