@@ -20,7 +20,7 @@ an URL. The supported url format is:
 ~~~
 */
 
-#include <stdlib.h>
+#include <cstdlib>
 #include "TUrl.h"
 #include "THashList.h"
 #include "TObjArray.h"
@@ -54,7 +54,6 @@ namespace {
 }
 #endif
 
-ClassImp(TUrl);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Parse url character string and split in its different subcomponents.
@@ -62,15 +61,14 @@ ClassImp(TUrl);
 /// ~~~ {.cpp}
 /// url: [proto://][user[:passwd]@]host[:port]/file.ext[?options][#anchor]
 /// ~~~
-/// Known protocols: http, root, proof, ftp, news and any special protocols
+/// Known protocols: http, root, ftp, news and any special protocols
 /// defined in the rootrc Url.Special key.
 /// The default protocol is "http", unless defaultIsFile is true in which
 /// case the url is assumed to be of type "file".
 /// If a passwd contains a @ it must be escaped by a \\, e.g.
 /// "pip@" becomes "pip\\@".
 ///
-/// Default ports: http=80, root=1094, proof=1093, ftp=20, news=119.
-/// Port #1093 has been assigned by IANA (www.iana.org) to proofd.
+/// Default ports: http=80, root=1094, ftp=20, news=119.
 /// Port #1094 has been assigned by IANA (www.iana.org) to rootd.
 
 TUrl::TUrl(const char *url, Bool_t defaultIsFile)
@@ -96,15 +94,14 @@ TUrl::~TUrl()
 ///~~~ {.cpp}
 /// url: [proto://][user[:passwd]@]host[:port]/file.ext[?options][#anchor]
 ///~~~
-/// Known protocols: http, root, proof, ftp, news and any special protocols
+/// Known protocols: http, root, ftp, news and any special protocols
 /// defined in the rootrc Url.Special key.
 /// The default protocol is "http", unless defaultIsFile is true in which
 /// case the url is assumed to be of type "file".
 /// If a passwd contains a @ it must be escaped by a \\, e.g.
 /// "pip@" becomes "pip\\@".
 ///
-/// Default ports: http=80, root=1094, proof=1093, ftp=20, news=119.
-/// Port #1093 has been assigned by IANA (www.iana.org) to proofd.
+/// Default ports: http=80, root=1094, ftp=20, news=119.
 /// Port #1094 has been assigned by IANA (www.iana.org) to rootd.
 
 void TUrl::SetUrl(const char *url, Bool_t defaultIsFile)
@@ -131,13 +128,20 @@ void TUrl::SetUrl(const char *url, Bool_t defaultIsFile)
    fHostFQ     = "";
 
    // if url starts with a / consider it as a file url
-   if (url[0] == '/')
+   if (url[0] == '/') {
       defaultIsFile = kTRUE;
+      // ROOT-5430: if url starts with two slashes but
+      // not three slashes, just remove the first of them
+      if (strlen(url) > 2 && url[1] == '/' && url[2] != '/') {
+         url = &url[1];
+      }
+   }
 
    // Find protocol
    char *s, sav;
 
    TString surl = url;
+
    char *u, *u0 = Strip(defaultIsFile && surl.EndsWith(":/") ? TString(surl(0,surl.Length()-2)).Data() : url);
 tryfile:
    u = u0;
@@ -420,7 +424,6 @@ const char *TUrl::GetUrl(Bool_t withDeflt) const
 
       Bool_t deflt = kFALSE;
       if ((!fProtocol.CompareTo("http")  && fPort == 80)   ||
-          (fProtocol.BeginsWith("proof") && fPort == 1093) ||
           (fProtocol.BeginsWith("root")  && fPort == 1094) ||
           (!fProtocol.CompareTo("ftp")   && fPort == 20)   ||
           (!fProtocol.CompareTo("news")  && fPort == 119)  ||
@@ -529,8 +532,6 @@ void TUrl::SetProtocol(const char *proto, Bool_t setDefaultPort)
          fPort = 80;
       else if (!fProtocol.CompareTo("https"))
          fPort = 443;
-      else if (fProtocol.BeginsWith("proof"))  // can also be proofs or proofk
-         fPort = 1093;
       else if (fProtocol.BeginsWith("root"))   // can also be roots or rootk
          fPort = 1094;
       else if (!fProtocol.CompareTo("ftp"))
