@@ -3,10 +3,13 @@
 
 #pragma extra_include "deque";
 
+#pragma create TClass deque<unsigned short>;
+#pragma create TClass deque<unsigned int>;
+#pragma create TClass deque<unsigned long>;
+#pragma create TClass deque<short>;
 #pragma create TClass deque<int>;
 #pragma create TClass deque<long>;
 #pragma create TClass deque<float>;
 #pragma create TClass deque<double>;
 #pragma create TClass deque<void*>;
 #pragma create TClass deque<char*>;
-
