@@ -1,9 +1,4 @@
-// For backward compatibility only
-#ifdef __CINT__
-#include <multimap>
-#else
 #include <map>
-#endif
 #include <string>
 
 #pragma create TClass multimap<char*,int>;
