@@ -13,12 +13,14 @@
 \ingroup Containers
 Iterator abstract base class. This base class provides the interface
 for collection iterators.
+\note See TList documentation for examples of derived classes and on how
+to loop with these iterators, and TCollection documentation for more
+modern alternatives that dynamically cast the derived class.
 */
 
 #include "TIterator.h"
 #include "TError.h"
 
-ClassImp(TIterator);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Compare two iterator objects.
