@@ -26,7 +26,6 @@ Both key and value must inherit from TObject.
 #include "TBrowser.h"
 #include "TRegexp.h"
 
-ClassImp(TMap);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// TMap ctor. See THashTable for a description of the arguments.
@@ -465,7 +464,6 @@ void TPair::Browse(TBrowser *b)
 Iterator of map.
 */
 
-ClassImp(TMapIter);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Create a map iterator. Use dir to specify the desired iteration direction.
