@@ -24,6 +24,7 @@
 
 #include <iostream>
 #include <sstream> // class ostringstream
+#include "llvm/ADT/StringExtras.h"
 
 #include "SelectionRules.h"
 
@@ -57,8 +58,6 @@ inline static bool IsElementPresent(const std::vector<const T*> &v, T *el){
 using namespace ROOT;
 using namespace clang;
 
-extern cling::Interpreter *gInterp;
-
 const char* RScanner::fgClangDeclKey = "ClangDecl"; // property key used for connection with Clang objects
 const char* RScanner::fgClangFuncKey = "ClangFunc"; // property key for demangled names
 
@@ -552,6 +551,7 @@ int RScanner::AddAnnotatedRecordDecl(const ClassSelectionRule* selected,
       return 1;
    }
 
+   // clang-format off
    if (has_attr_name) {
       fSelectedClasses.emplace_back(selected->GetIndex() + indexOffset,
                                     req_type,
@@ -562,6 +562,7 @@ int RScanner::AddAnnotatedRecordDecl(const ClassSelectionRule* selected,
                                     selected->RequestNoInputOperator(),
                                     selected->RequestOnlyTClass(),
                                     selected->RequestedVersionNumber(),
+                                    selected->RequestedRNTupleSerializationMode(),
                                     fInterpreter,
                                     fNormCtxt);
    } else {
@@ -572,9 +573,11 @@ int RScanner::AddAnnotatedRecordDecl(const ClassSelectionRule* selected,
                                     selected->RequestNoInputOperator(),
                                     selected->RequestOnlyTClass(),
                                     selected->RequestedVersionNumber(),
+                                    selected->RequestedRNTupleSerializationMode(),
                                     fInterpreter,
                                     fNormCtxt);
    }
+   // clang-format on
 
    if (fVerboseLevel > 0) {
       std::string qual_name;
@@ -1054,6 +1057,8 @@ void RScanner::Scan(const clang::ASTContext &C)
       std::cout<<"File name detected"<<std::endl;
    }
 
+   cling::Interpreter::PushTransactionRAII RAII(const_cast<cling::Interpreter *>(&fInterpreter));
+
    if (fScanType == EScanType::kTwoPasses)
       TraverseDecl(C.getTranslationUnitDecl());
 
