@@ -55,7 +55,7 @@ TProtoClass::TProtoClass(TClass* cl):
    TNamed(*cl), fBase(cl->GetListOfBases()),
    fEnums(cl->GetListOfEnums()), fSizeof(cl->Size()), fCheckSum(cl->fCheckSum),
    fCanSplit(cl->fCanSplit), fStreamerType(cl->fStreamerType), fProperty(cl->fProperty),
-   fClassProperty(cl->fClassProperty)
+   fClassProperty(cl->fClassProperty), fOwner(false)
 {
    if (cl->Property() & kIsNamespace){
       //fData=new TListOfDataMembers();
@@ -168,15 +168,20 @@ TProtoClass::~TProtoClass()
 
 void TProtoClass::Delete(Option_t* opt /*= ""*/)
 {
-   if (fBase) fBase->Delete(opt);
-   delete fBase; fBase = nullptr;
+   if (fOwner && fBase) {
+      fBase->Delete(opt);
+      delete fBase; fBase = nullptr;
+   }
 
-   for (auto dm: fData)
-      delete dm;
+   if (fOwner)
+      for (auto dm: fData)
+         delete dm;
    fData.clear();
 
-   if (fEnums) fEnums->Delete(opt);
-   delete fEnums; fEnums = nullptr;
+   if (fOwner && fEnums) {
+      fEnums->Delete(opt);
+      delete fEnums; fEnums = nullptr;
+   }
 
    if (gErrorIgnoreLevel==-2) printf("Delete the protoClass %s \n",GetName());
 }
@@ -262,6 +267,7 @@ Bool_t TProtoClass::FillTClass(TClass* cl) {
    cl->fName  = this->fName;
    cl->fTitle = this->fTitle;
    cl->fBase = fBase;
+   fBase = nullptr;
 
    // fill list of data members in TClass
    //if (cl->fData) { cl->fData->Delete(); delete cl->fData;  }
@@ -378,7 +384,7 @@ Bool_t TProtoClass::FillTClass(TClass* cl) {
          //TProtoRealData* prd = (TProtoRealData*)element;
          // pass a previous real data only if depth
 
-         if (TRealData* rd = element.CreateRealData(currentRDClass, cl,prevRealData, prevLevel)) {
+         if (TRealData* rd = element.CreateRealData(currentRDClass, cl,prevRealData, prevLevel, element.TestFlag(TProtoRealData::kIsTransient))) {
             if (first) {
                //LM: need to do here because somehow fRealData is destroyed when calling TClass::GetListOfDataMembers()
                if (cl->fRealData) {
@@ -461,15 +467,23 @@ TProtoClass::TProtoRealData::~TProtoRealData()
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Create a TRealData from this, with its data member coming from dmClass.
+/// \brief Create a TRealData from this, with its data member coming from dmClass.
 /// find data member from protoclass
+///
+/// \return the created TRealData
+///
+/// \param [in] dmClass Class where the data member is declared
+/// \param [in] parent  Parent class
+/// \param [in] prevData the previous 'real' data member (might be part of another class)
+/// \param [in] prevLevel nesting level
+/// \param [in] quiet Whether we should not warn about missing information (usually set to true for transient members)
 
 TRealData* TProtoClass::TProtoRealData::CreateRealData(TClass* dmClass,
-                                                       TClass* parent, TRealData *prevData, int prevLevel) const
+                                                       TClass* parent, TRealData *prevData, int prevLevel, bool quiet) const
 {
 
    //TDataMember* dm = (TDataMember*)dmClass->GetListOfDataMembers()->FindObject(fName);
-   TDataMember* dm = TProtoClass::FindDataMember(dmClass, fDMIndex);
+   TDataMember* dm = TProtoClass::FindDataMember(dmClass, fDMIndex, quiet);
 
    if (!dm && dmClass->GetState()!=TClass::kForwardDeclared && !dmClass->fIsSyntheticPair) {
       ::Error("CreateRealData",
@@ -557,8 +571,16 @@ Int_t TProtoClass::DataMemberIndex(TClass * cl, const char * name)
    return -1;
 }
 ////////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////
+/// \brief Find the requested TDataMember
+///
+/// \return the requested TDataMember if found
+///
+/// \param [in] cl TClass to search for the data member
+/// \param [in] index Numerical index of the object's data member
+/// \param [in] quiet Whether we should not warn about missing information (usually set to true for transient members)
 
-TDataMember * TProtoClass::FindDataMember(TClass * cl, Int_t index)
+TDataMember * TProtoClass::FindDataMember(TClass * cl, Int_t index, bool quiet)
 {
    TList * dmList = cl->GetListOfDataMembers(false);
 
@@ -572,7 +594,7 @@ TDataMember * TProtoClass::FindDataMember(TClass * cl, Int_t index)
          return dm;
       i++;
    }
-   if (cl->GetState()!=TClass::kForwardDeclared && !cl->fIsSyntheticPair)
+   if (cl->GetState()!=TClass::kForwardDeclared && !cl->fIsSyntheticPair && !quiet)
       ::Error("TProtoClass::FindDataMember","data member with index %d is not found in class %s",index,cl->GetName());
    return nullptr;
 }
