@@ -37,6 +37,8 @@ but the type metadata comes from the Clang C++ compiler, not CINT.
 #include "clang/AST/PrettyPrinter.h"
 #include "clang/Frontend/CompilerInstance.h"
 
+#include <CppInterOp/CppInterOp.h>
+
 #include <cstdio>
 #include <string>
 
@@ -128,6 +130,8 @@ long TClingTypeInfo::Property() const
    property = TClingDeclInfo::Property(property, QT);
    const clang::TagType *tagQT = llvm::dyn_cast<clang::TagType>(QT.getTypePtr());
    if (tagQT) {
+      // getDecl and isAbstract can trigger deserialization
+      cling::Interpreter::PushTransactionRAII RAII(fInterp);
       // Note: Now we have class, enum, struct, union only.
       const clang::TagDecl *TD = llvm::dyn_cast<clang::TagDecl>(tagQT->getDecl());
       if (!TD)
@@ -172,8 +176,6 @@ long TClingTypeInfo::Property() const
          else if (CRD->isUnion()) {
             property |= kIsUnion;
          }
-         // isAbstract can trigger deserialization
-         cling::Interpreter::PushTransactionRAII RAII(fInterp);
          if (CRD->isThisDeclarationADefinition() && CRD->isAbstract()) {
             property |= kIsAbstract;
          }
@@ -235,21 +237,7 @@ int TClingTypeInfo::Size() const
    if (!IsValid()) {
       return 1;
    }
-   if (fQualType->isDependentType()) {
-      // Dependent on a template parameter, we do not know what it is yet.
-      return 0;
-   }
-   if (const clang::RecordType *RT = fQualType->getAs<clang::RecordType>()) {
-      if (!RT->getDecl()->getDefinition()) {
-         // This is a forward-declared class.
-         return 0;
-      }
-   }
-   clang::ASTContext &Context = fInterp->getCI()->getASTContext();
-   // Note: This is an int64_t.
-   clang::CharUnits::QuantityType Quantity =
-      Context.getTypeSizeInChars(fQualType).getQuantity();
-   return static_cast<int>(Quantity);
+   return Cpp::GetSizeOfType(QualTypePtr());
 }
 
 ////////////////////////////////////////////////////////////////////////////////
