@@ -22,9 +22,8 @@
 #include "TThread.h"
 
 #include <process.h>
-#include <errno.h>
+#include <cerrno>
 
-ClassImp(TWin32Thread);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Win32 threads -- spawn new thread (like pthread_create).
