@@ -18,7 +18,6 @@
 #include "TGeoMatrix.h"
 #include "TMath.h"
 
-ClassImp(TGeoArb8);
 
 /** \class TGeoArb8
 \ingroup Trapezoids
@@ -1384,7 +1383,6 @@ void TGeoArb8::Safety_v(const Double_t *points, const Bool_t *inside, Double_t *
       safe[i] = Safety(&points[3 * i], inside[i]);
 }
 
-ClassImp(TGeoTrap);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default ctor
@@ -1972,7 +1970,6 @@ void TGeoTrap::Safety_v(const Double_t *points, const Bool_t *inside, Double_t *
       safe[i] = Safety(&points[3 * i], inside[i]);
 }
 
-ClassImp(TGeoGtra);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default ctor
