@@ -43,7 +43,7 @@ Begin_Macro
    gGeoManager->SetNsegments(30);
    top->Draw();
    TView *view = gPad->GetView();
-   view->ShowAxis();
+   if (view) view->ShowAxis();
 }
 End_Macro
 */
@@ -77,7 +77,7 @@ Begin_Macro
    gGeoManager->SetNsegments(30);
    top->Draw();
    TView *view = gPad->GetView();
-   view->ShowAxis();
+   if (view) view->ShowAxis();
 }
 End_Macro
 */
@@ -92,7 +92,6 @@ End_Macro
 #include "TBuffer3DTypes.h"
 #include "TMath.h"
 
-ClassImp(TGeoCone);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default constructor
@@ -188,7 +187,7 @@ void TGeoCone::ComputeBBox()
 ////////////////////////////////////////////////////////////////////////////////
 /// Compute normal to closest surface from POINT.
 
-void TGeoCone::ComputeNormal(const Double_t *point, const Double_t *dir, Double_t *norm)
+void TGeoCone::ComputeNormal(const Double_t *point, const Double_t *dir, Double_t *norm) const
 {
    Double_t safr, safe, phi;
    memset(norm, 0, 3 * sizeof(Double_t));
@@ -1254,7 +1253,6 @@ void TGeoCone::Safety_v(const Double_t *points, const Bool_t *inside, Double_t *
       safe[i] = Safety(&points[3 * i], inside[i]);
 }
 
-ClassImp(TGeoConeSeg);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default constructor
@@ -1434,7 +1432,7 @@ void TGeoConeSeg::ComputeBBox()
 ////////////////////////////////////////////////////////////////////////////////
 /// Compute normal to closest surface from POINT.
 
-void TGeoConeSeg::ComputeNormal(const Double_t *point, const Double_t *dir, Double_t *norm)
+void TGeoConeSeg::ComputeNormal(const Double_t *point, const Double_t *dir, Double_t *norm) const
 {
    Double_t saf[3];
    Double_t ro1 = 0.5 * (fRmin1 + fRmin2);
