@@ -223,7 +223,6 @@ const Int_t kN9 = 9 * sizeof(Double_t);
 
 // statics and globals
 
-ClassImp(TGeoMatrix);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// dummy constructor
@@ -569,7 +568,6 @@ basically an array of 3 doubles matching the positions 12, 13
 and 14 in the homogenous matrix description.
 */
 
-ClassImp(TGeoTranslation);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default constructor
@@ -852,7 +850,6 @@ Class describing rotations. A rotation is a 3*3 array
 Column vectors has to be orthogonal unit vectors.
 */
 
-ClassImp(TGeoRotation);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default constructor.
@@ -1437,7 +1434,6 @@ array of 3 doubles (sx, sy, sz) multiplying elements 0, 5 and 10
 of the homogenous matrix. A scale is normalized : sx*sy*sz = 1
 */
 
-ClassImp(TGeoScale);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// default constructor
@@ -1674,7 +1670,6 @@ Class describing rotation + translation. Most frequently used in the description
 of TGeoNode 's
 */
 
-ClassImp(TGeoCombiTrans);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// dummy ctor
@@ -2222,7 +2217,6 @@ const Double_t *TGeoCombiTrans::GetRotationMatrix() const
 Most general transformation, holding a translation, a rotation and a scale
 */
 
-ClassImp(TGeoGenTrans);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// dummy ctor
@@ -2336,7 +2330,6 @@ and returns pointers to static null translation and identity
 transformations for rotation and scale
 */
 
-ClassImp(TGeoIdentity);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// dummy ctor
@@ -2376,7 +2369,6 @@ is generally used to pile-up local transformations starting from
 the top level physical node, down to the current node.
 */
 
-ClassImp(TGeoHMatrix);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// dummy ctor
