@@ -79,6 +79,7 @@ painting a node on a pad will take the corresponding volume attributes.
 #include "TGeoShape.h"
 #include "TGeoVolume.h"
 #include "TVirtualGeoPainter.h"
+#include "TVirtualGeoChecker.h"
 #include "TGeoVoxelFinder.h"
 #include "TGeoNode.h"
 #include "TMath.h"
@@ -87,20 +88,19 @@ painting a node on a pad will take the corresponding volume attributes.
 
 // statics and globals
 
-ClassImp(TGeoNode);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default constructor
 
 TGeoNode::TGeoNode()
 {
-   fVolume = 0;
-   fMother = 0;
+   fVolume = nullptr;
+   fMother = nullptr;
    fNumber = 0;
    fNovlp = 0;
-   fOverlaps = 0;
-   fUserExtension = 0;
-   fFWExtension = 0;
+   fOverlaps = nullptr;
+   fUserExtension = nullptr;
+   fFWExtension = nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -116,12 +116,12 @@ TGeoNode::TGeoNode(const TGeoVolume *vol)
    if (fVolume->IsAdded())
       fVolume->SetReplicated();
    fVolume->SetAdded();
-   fMother = 0;
+   fMother = nullptr;
    fNumber = 0;
    fNovlp = 0;
-   fOverlaps = 0;
-   fUserExtension = 0;
-   fFWExtension = 0;
+   fOverlaps = nullptr;
+   fUserExtension = nullptr;
+   fFWExtension = nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -133,11 +133,11 @@ TGeoNode::~TGeoNode()
       delete[] fOverlaps;
    if (fUserExtension) {
       fUserExtension->Release();
-      fUserExtension = 0;
+      fUserExtension = nullptr;
    }
    if (fFWExtension) {
       fFWExtension->Release();
-      fFWExtension = 0;
+      fFWExtension = nullptr;
    }
 }
 
@@ -212,7 +212,7 @@ void TGeoNode::CheckOverlaps(Double_t ovlp, Option_t *option)
       Info("CheckOverlaps", "=== NOTE: Extrusions NOT checked with sampling option ! ===");
    }
    timer->Start();
-   geom->GetGeomPainter()->OpProgress(fVolume->GetName(), icheck, ncheck, timer, kFALSE);
+   geom->GetGeomChecker()->OpProgress(fVolume->GetName(), icheck, ncheck, timer, kFALSE);
    fVolume->CheckOverlaps(ovlp, option);
    icheck++;
    TGeoIterator next(fVolume);
@@ -226,7 +226,7 @@ void TGeoNode::CheckOverlaps(Double_t ovlp, Option_t *option)
       icheck++;
       if (!node->GetVolume()->IsSelected()) {
          msg = TString::Format("found %d overlaps", overlaps->GetEntriesFast());
-         geom->GetGeomPainter()->OpProgress(node->GetVolume()->GetName(), icheck, ncheck, timer, kFALSE, msg);
+         geom->GetGeomChecker()->OpProgress(node->GetVolume()->GetName(), icheck, ncheck, timer, kFALSE, msg);
          node->GetVolume()->SelectVolume(kFALSE);
          node->GetVolume()->CheckOverlaps(ovlp, option);
       }
@@ -240,7 +240,7 @@ void TGeoNode::CheckOverlaps(Double_t ovlp, Option_t *option)
       obj = (TNamed *)overlaps->At(i);
       obj->SetName(TString::Format("ov%05d", i));
    }
-   geom->GetGeomPainter()->OpProgress("Check overlaps:", icheck, ncheck, timer, kTRUE);
+   geom->GetGeomChecker()->OpProgress("Check overlaps:", icheck, ncheck, timer, kTRUE);
    Info("CheckOverlaps", "Number of illegal overlaps/extrusions : %d\n", novlps);
    delete timer;
 }
@@ -281,10 +281,10 @@ void TGeoNode::ExecuteEvent(Int_t event, Int_t px, Int_t py)
 char *TGeoNode::GetObjectInfo(Int_t px, Int_t py) const
 {
    if (!fVolume)
-      return 0;
+      return nullptr;
    TVirtualGeoPainter *painter = fVolume->GetGeoManager()->GetPainter();
    if (!painter)
-      return 0;
+      return nullptr;
    return (char *)painter->GetVolumeInfo(fVolume, px, py);
 }
 
@@ -534,7 +534,7 @@ TGeoExtension *TGeoNode::GrabUserExtension() const
 {
    if (fUserExtension)
       return fUserExtension->Grab();
-   return 0;
+   return nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -546,7 +546,7 @@ TGeoExtension *TGeoNode::GrabFWExtension() const
 {
    if (fFWExtension)
       return fFWExtension->Grab();
-   return 0;
+   return nullptr;
 }
 ////////////////////////////////////////////////////////////////////////////////
 /// Check the overlab between the bounding box of the node overlaps with the one
@@ -741,14 +741,13 @@ void TGeoNode::VisibleDaughters(Bool_t vis)
 A node containing local transformation.
 */
 
-ClassImp(TGeoNodeMatrix);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default constructor
 
 TGeoNodeMatrix::TGeoNodeMatrix()
 {
-   fMatrix = 0;
+   fMatrix = nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -838,7 +837,6 @@ void TGeoNodeMatrix::SetMatrix(const TGeoMatrix *matrix)
 Node containing an offset.
 */
 
-ClassImp(TGeoNodeOffset);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default constructor
@@ -848,7 +846,7 @@ TGeoNodeOffset::TGeoNodeOffset()
    TObject::SetBit(kGeoNodeOffset);
    fOffset = 0;
    fIndex = 0;
-   fFinder = 0;
+   fFinder = nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -859,7 +857,7 @@ TGeoNodeOffset::TGeoNodeOffset(const TGeoVolume *vol, Int_t index, Double_t offs
    TObject::SetBit(kGeoNodeOffset);
    fOffset = offset;
    fIndex = index;
-   fFinder = 0;
+   fFinder = nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -982,8 +980,6 @@ We want to find out a volume named "MyVol" in the hierarchy of TOP volume.
 \ingroup Geometry_classes
 */
 
-ClassImp(TGeoIteratorPlugin);
-ClassImp(TGeoIterator);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Geometry iterator for a branch starting with a TOP node.
@@ -998,7 +994,7 @@ TGeoIterator::TGeoIterator(TGeoVolume *top)
    fArray = new Int_t[30];
    fMatrix = new TGeoHMatrix();
    fTopName = fTop->GetName();
-   fPlugin = 0;
+   fPlugin = nullptr;
    fPluginAutoexec = kFALSE;
 }
 
@@ -1065,14 +1061,14 @@ TGeoIterator &TGeoIterator::operator=(const TGeoIterator &iter)
 TGeoNode *TGeoIterator::Next()
 {
    if (fMustStop)
-      return 0;
-   TGeoNode *mother = 0;
-   TGeoNode *next = 0;
+      return nullptr;
+   TGeoNode *mother = nullptr;
+   TGeoNode *next = nullptr;
    Int_t i;
    Int_t nd = fTop->GetNdaughters();
    if (!nd) {
       fMustStop = kTRUE;
-      return 0;
+      return nullptr;
    }
    if (!fLevel) {
       fArray[++fLevel] = 0;
@@ -1119,7 +1115,7 @@ TGeoNode *TGeoIterator::Next()
                return fTop->GetNode(fArray[fLevel]);
             }
             fMustStop = kTRUE;
-            return 0;
+            return nullptr;
          } else {
             nd = next->GetNdaughters();
             if (fArray[fLevel] < nd - 1) {
@@ -1146,7 +1142,7 @@ TGeoNode *TGeoIterator::Next()
       }
    }
    fMustStop = kTRUE;
-   return 0;
+   return nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1180,7 +1176,7 @@ const TGeoMatrix *TGeoIterator::GetCurrentMatrix() const
 TGeoNode *TGeoIterator::GetNode(Int_t level) const
 {
    if (!level || level > fLevel)
-      return 0;
+      return nullptr;
    TGeoNode *node = fTop->GetNode(fArray[1]);
    for (Int_t i = 2; i < level + 1; i++)
       node = node->GetDaughter(fArray[i]);
@@ -1252,7 +1248,7 @@ void TGeoIterator::Skip()
       // cd up and pick next
       while (next) {
          next = GetNode(fLevel - 1);
-         nd = (next == 0) ? fTop->GetNdaughters() : next->GetNdaughters();
+         nd = (next == nullptr) ? fTop->GetNdaughters() : next->GetNdaughters();
          if (fArray[fLevel] < nd - 1) {
             ++fArray[fLevel];
             return;
@@ -1266,7 +1262,7 @@ void TGeoIterator::Skip()
       break;
    case 1: // one level search
       next = GetNode(fLevel - 1);
-      nd = (next == 0) ? fTop->GetNdaughters() : next->GetNdaughters();
+      nd = (next == nullptr) ? fTop->GetNdaughters() : next->GetNdaughters();
       if (fArray[fLevel] < nd - 1) {
          ++fArray[fLevel];
          return;
