@@ -37,7 +37,7 @@ Begin_Macro
    gGeoManager->SetNsegments(30);
    top->Draw();
    TView *view = gPad->GetView();
-   view->ShowAxis();
+   if (view) view->ShowAxis();
 }
 End_Macro
 
@@ -59,7 +59,6 @@ End_Macro
 #include "TBuffer3DTypes.h"
 #include "TMath.h"
 
-ClassImp(TGeoSphere);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default constructor
@@ -229,7 +228,7 @@ void TGeoSphere::ComputeBBox()
 ////////////////////////////////////////////////////////////////////////////////
 /// Compute normal to closest surface from POINT.
 
-void TGeoSphere::ComputeNormal(const Double_t *point, const Double_t *dir, Double_t *norm)
+void TGeoSphere::ComputeNormal(const Double_t *point, const Double_t *dir, Double_t *norm) const
 {
    Double_t rxy2 = point[0] * point[0] + point[1] * point[1];
    Double_t r2 = rxy2 + point[2] * point[2];
