@@ -31,7 +31,6 @@ Full description with examples and pictures
 #include "TGeoManager.h"
 #include "TGeoStateInfo.h"
 
-ClassImp(TGeoVoxelFinder);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default constructor
@@ -2258,9 +2257,10 @@ void TGeoVoxelFinder::SortAll(Option_t *)
    delete[] extra_right;
 
    if ((!fPriority[0]) && (!fPriority[1]) && (!fPriority[2])) {
-      SetInvalid();
-      if (nd > 1)
+      if (nd > 1) {
+         SetInvalid();
          Error("SortAll", "Volume %s: Cannot make slices on any axis", fVolume->GetName());
+      }
    }
 }
 
