@@ -34,7 +34,6 @@ Editor for a TGeoTube.
 #include "TGLabel.h"
 #include "TGDoubleSlider.h"
 
-ClassImp(TGeoTubeEditor);
 
 enum ETGeoTubeWid { kTUBE_NAME, kTUBE_RMIN, kTUBE_RMAX, kTUBE_Z, kTUBE_APPLY, kTUBE_UNDO };
 
@@ -298,7 +297,6 @@ Editor for a TGeoTubeSeg.
 
 */
 
-ClassImp(TGeoTubeSegEditor);
 
 enum ETGeoTubeSegWid { kTUBESEG_PHI1, kTUBESEG_PHI2, kTUBESEG_PHI };
 
@@ -537,7 +535,6 @@ Editor for a TGeoCtub.
 
 */
 
-ClassImp(TGeoCtubEditor);
 
 enum ETGeoCtubSegWid { kCTUB_THLO, kCTUB_PHLO, kCTUB_THHI, kCTUB_PHHI };
 
