@@ -36,7 +36,6 @@
 OpenGL renderer class for TEveCaloLego.
 */
 
-ClassImp(TEveCaloLegoGL);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Constructor.
@@ -47,10 +46,10 @@ TEveCaloLegoGL::TEveCaloLegoGL() :
    fGridColor(-1),
    fFontColor(-1),
 
-   fEtaAxis(0),
-   fPhiAxis(0),
-   fZAxis(0),
-   fM(0),
+   fEtaAxis(nullptr),
+   fPhiAxis(nullptr),
+   fZAxis(nullptr),
+   fM(nullptr),
    fDLCacheOK(kFALSE),
    fMaxVal(0),
    fValToPixel(0),
