@@ -20,7 +20,6 @@
 Editor for TEveGeoNode class.
 */
 
-ClassImp(TEveGeoNodeEditor);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Constructor.
@@ -122,7 +121,6 @@ void TEveGeoNodeEditor::DoVizVolumeDaughters()
 Editor for TEveGeoTopNode class.
 */
 
-ClassImp(TEveGeoTopNodeEditor);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Constructor.
