@@ -35,7 +35,6 @@
 #include "TGL5DPainter.h"
 #include "TGLUtil.h"
 
-ClassImp(TGLHistPainter);
 
 /** \class TGLHistPainter
 \ingroup opengl
@@ -101,7 +100,7 @@ The following types of plots are provided:
    The supported option is:
 
   - `"GLCOL" :` TH3 is drawn using semi-transparent colored boxes.
-  See `$ROOTSYS/tutorials/gl/glvox1.C`.
+  See glvox1.C .
 
 
 #### `TF3` (implicit function) - (`TGLTF3Painter`)
@@ -111,7 +110,7 @@ The following types of plots are provided:
 
 
 #### Parametric surfaces - (`TGLParametricPlot`)
-  `$ROOTSYS/tutorials/gl/glparametric.C` shows how to create parametric equations and
+  glparametric.C shows how to create parametric equations and
   visualize the surface.
 
 
