@@ -43,7 +43,6 @@ Implements GUI for axis attributes.
 #include "TVirtualPad.h"
 #include "TStyle.h"
 
-ClassImp(TAxisEditor);
 
 enum EAxisWid {
    kCOL_AXIS,
@@ -250,7 +249,7 @@ void TAxisEditor::ConnectSignals2Slots()
    (fDiv2->GetNumberEntry())->Connect("ReturnPressed()", "TAxisEditor", this, "DoDivisions()");
    fDiv1->Connect("ValueSet(Long_t)", "TAxisEditor", this, "DoDivisions()");
    (fDiv1->GetNumberEntry())->Connect("ReturnPressed()", "TAxisEditor", this, "DoDivisions()");
-   fTitle->Connect("TextChanged(const char *)", "TAxisEditor", this, "DoTitle(const char *)");;
+   fTitle->Connect("TextChanged(const char *)", "TAxisEditor", this, "DoTitle(const char *)");
    fTitleColor->Connect("ColorSelected(Pixel_t)", "TAxisEditor", this, "DoTitleColor(Pixel_t)");
    fTitleSize->Connect("ValueSet(Long_t)", "TAxisEditor", this, "DoTitleSize()");
    (fTitleSize->GetNumberEntry())->Connect("ReturnPressed()", "TAxisEditor", this, "DoTitleSize()");
