@@ -10,33 +10,63 @@
  *************************************************************************/
 
 
-//////////////////////////////////////////////////////////////////////////
-//                                                                      //
-// TGedPatternFrame, TGedPatternSelector, TGedPatternPopup              //
-// and TGedPatternColor                                                 //
-//                                                                      //
-// The TGedPatternFrame is a small frame with border showing            //
-// a specific pattern (fill style.                                      //
-//                                                                      //
-// The TGedPatternSelector is a composite frame with TGedPatternFrames  //
-// of all diferent styles                                               //
-//                                                                      //
-// The TGedPatternPopup is a popup containing a TGedPatternSelector.    //
-//                                                                      //
-// The TGedPatternSelect widget is a button with pattern area with      //
-// a little down arrow. When clicked on the arrow the                   //
-// TGedPatternPopup pops up.                                            //
-//                                                                      //
-// Selecting a pattern in this widget will generate the event:          //
-// kC_PATTERNSEL, kPAT_SELCHANGED, widget id, style.                    //
-//                                                                      //
-// and the signal:                                                      //
-// PatternSelected(Style_t pattern)                                     //
-//                                                                      //
-// TGedSelect is button that shows popup window when clicked.           //
-// TGedPopup is a popup window.                                         //
-//                                                                      //
-//////////////////////////////////////////////////////////////////////////
+/** \class TGedPatternFrame,
+    \ingroup ged
+
+The TGedPatternFrame is a small frame with border showing
+a specific pattern (fill style.
+
+*/
+
+
+/** \class TGedPatternSelector,
+    \ingroup ged
+
+The TGedPatternSelector is a composite frame with TGedPatternFrames
+of all diferent styles
+
+*/
+
+
+/** \class TGedPatternPopup
+    \ingroup ged
+
+The TGedPatternPopup is a popup containing a TGedPatternSelector.
+
+*/
+
+
+/** \class  TGedPatternSelect
+    \ingroup ged
+
+is a button with pattern area with
+a little down arrow. When clicked on the arrow the
+TGedPatternPopup pops up.
+
+Selecting a pattern in this widget will generate the event:
+kC_PATTERNSEL, kPAT_SELCHANGED, widget id, style.
+
+and the signal:
+PatternSelected(Style_t pattern)
+
+*/
+
+
+/** \class TGedSelect
+    \ingroup ged
+
+is button that shows popup window when clicked.
+
+*/
+
+
+/** \class TGedPopup
+    \ingroup ged
+
+is a popup window.
+
+*/
+
 
 #include "TGResourcePool.h"
 #include "TGedPatternSelect.h"
@@ -49,12 +79,6 @@
 
 #include <iostream>
 
-ClassImp(TGedPopup);
-ClassImp(TGedSelect);
-ClassImp(TGedPatternFrame);
-ClassImp(TGedPatternSelector);
-ClassImp(TGedPatternPopup);
-ClassImp(TGedPatternSelect);
 
 TGGC* TGedPatternFrame::fgGC = nullptr;
 
@@ -248,7 +272,7 @@ void TGedPatternSelector::SetActive(Int_t newat)
 ////////////////////////////////////////////////////////////////////////////////
 /// Process message generated by pattern popup window.
 
-Bool_t TGedPatternSelector::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2)
+Bool_t TGedPatternSelector::ProcessMessage(Longptr_t msg, Longptr_t parm1, Longptr_t parm2)
 {
    switch (GET_MSG(msg)) {
       case kC_PATTERNSEL:
@@ -359,7 +383,7 @@ Bool_t TGedPopup::HandleButton(Event_t *event)
 ////////////////////////////////////////////////////////////////////////////////
 /// Process messages generated by popup window.
 
-Bool_t TGedPopup::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2)
+Bool_t TGedPopup::ProcessMessage(Longptr_t msg, Longptr_t parm1, Longptr_t parm2)
 {
    switch (GET_MSG(msg)) {
       case kC_POPUP:
@@ -404,7 +428,7 @@ TGedPatternPopup::~TGedPatternPopup()
 ////////////////////////////////////////////////////////////////////////////////
 /// Process messages generated by pattern popup window.
 
-Bool_t TGedPatternPopup::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2)
+Bool_t TGedPatternPopup::ProcessMessage(Longptr_t msg, Longptr_t parm1, Longptr_t parm2)
 {
    switch (GET_MSG(msg)) {
       case kC_PATTERNSEL:
@@ -607,12 +631,12 @@ TGedPatternSelect::TGedPatternSelect(const TGWindow *p, Style_t pattern, Int_t i
 ////////////////////////////////////////////////////////////////////////////////
 /// Process message according to the user input.
 
-Bool_t TGedPatternSelect::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2)
+Bool_t TGedPatternSelect::ProcessMessage(Longptr_t msg, Longptr_t parm1, Longptr_t parm2)
 {
    if (GET_MSG(msg) == kC_PATTERNSEL && GET_SUBMSG(msg) == kPAT_SELCHANGED)
    {
       SetPattern(parm2);
-      parm1 = (Long_t)fWidgetId;
+      parm1 = (Longptr_t)fWidgetId;
       SendMessage(fMsgWindow, MK_MSG(kC_PATTERNSEL, kPAT_SELCHANGED),
                   parm1, parm2);
    }
