@@ -105,7 +105,7 @@ private:
    TGClient  *fClient;   // connection to display server
 public:
    TGInputHandler(TGClient *c, Int_t fd) : TFileHandler(fd, 1) { fClient = c; }
-   Bool_t Notify();
+   Bool_t Notify() override;
    // Important: don't override ReadNotify()
 };
 
@@ -118,7 +118,6 @@ Bool_t TGInputHandler::Notify()
 }
 
 
-ClassImp(TGClient);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Create a connection with the display sever on host dpyName and setup
@@ -291,6 +290,15 @@ const TGPicture *TGClient::GetPicture(const char *name)
    return fPicturePool->GetPicture(name);
 }
 
+////////////////////////////////////////////////////////////////////////////////
+/// @copydoc TGPicturePool::GetPictureOrEmpty(const char*)
+
+const TGPicture *TGClient::GetPictureOrEmpty(const char *name)
+{
+   return fPicturePool->GetPictureOrEmpty(name);
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////
 /// Get picture with specified size from pool (picture will be scaled if
 /// necessary). Picture must be freed using TGClient::FreePicture(). If
@@ -460,13 +468,13 @@ Pixel_t TGClient::GetHilite(Pixel_t base_color) const
    GetColorByName("white", white_p.fPixel);
    gVirtualX->QueryColor(attributes.fColormap, white_p);
 
-   color.fRed   = TMath::Max((UShort_t)(white_p.fRed/5),   color.fRed);
-   color.fGreen = TMath::Max((UShort_t)(white_p.fGreen/5), color.fGreen);
-   color.fBlue  = TMath::Max((UShort_t)(white_p.fBlue/5),  color.fBlue);
+   color.fRed   = std::max((UShort_t)(white_p.fRed/5),   color.fRed);
+   color.fGreen = std::max((UShort_t)(white_p.fGreen/5), color.fGreen);
+   color.fBlue  = std::max((UShort_t)(white_p.fBlue/5),  color.fBlue);
 
-   color.fRed   = (UShort_t)TMath::Min((Int_t)white_p.fRed,   (Int_t)(color.fRed*140)/100);
-   color.fGreen = (UShort_t)TMath::Min((Int_t)white_p.fGreen, (Int_t)(color.fGreen*140)/100);
-   color.fBlue  = (UShort_t)TMath::Min((Int_t)white_p.fBlue,  (Int_t)(color.fBlue*140)/100);
+   color.fRed   = (UShort_t)std::min((Int_t)white_p.fRed,   (Int_t)(color.fRed*140)/100);
+   color.fGreen = (UShort_t)std::min((Int_t)white_p.fGreen, (Int_t)(color.fGreen*140)/100);
+   color.fBlue  = (UShort_t)std::min((Int_t)white_p.fBlue,  (Int_t)(color.fBlue*140)/100);
 
    if (!gVirtualX->AllocColor(attributes.fColormap, color))
       Error("GetHilite", "couldn't allocate hilight color");
