@@ -61,9 +61,6 @@ It can be displayed with respect to any origin.
 #include <iostream>
 
 
-ClassImp(TGFont);
-ClassImp(TGFontPool);
-ClassImp(TGTextLayout);
 
 #define FONT_FAMILY     0
 #define FONT_SIZE       1
