@@ -30,9 +30,6 @@ a web browser with sensitive regions (MAP HTML tag).
 #include "TVirtualX.h"
 
 
-ClassImp(TGRegion);
-ClassImp(TGRegionWithId);
-ClassImpQ(TGImageMap)
 
 
 TGRegionWithId  *gCurrentRegion; // current region
