@@ -35,7 +35,6 @@ The property editor
 #include "TGSplitter.h"
 #include "TGTableLayout.h"
 
-ClassImp(TGuiBldEditor);
 
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -55,7 +54,7 @@ private:
 
 public:
    TGuiBldBorderFrame(const TGWindow *p, TGuiBldEditor *editor);
-   virtual ~TGuiBldBorderFrame() { }
+   ~TGuiBldBorderFrame() override { }
 
    void  ChangeSelected(TGFrame*);
 };
