@@ -37,8 +37,8 @@
 #include <process.h>
 #else
 #include <unistd.h>
-#include <stdlib.h>
-#include <signal.h>
+#include <cstdlib>
+#include <csignal>
 #include <spawn.h>
 #ifdef R__MACOSX
 #include <sys/wait.h>
@@ -263,6 +263,7 @@ static void DummyTimeOutHandler(int /* Sig */) {}
 
 //////////////////////////////////////////////////////////////////////////////////////////////////
 /// Display given URL in web browser
+/// \note See more details related to webdisplay on RWebWindowsManager::ShowWindow
 
 std::unique_ptr<RWebDisplayHandle>
 RWebDisplayHandle::BrowserCreator::Display(const RWebDisplayArgs &args)
@@ -296,6 +297,13 @@ RWebDisplayHandle::BrowserCreator::Display(const RWebDisplayArgs &args)
 
    ProcessGeometry(exec, args);
 
+   std::string extra = args.GetExtraArgs();
+   if (!extra.empty()) {
+      auto p = exec.find("$url");
+      if (p != std::string::npos)
+         exec.insert(p, extra + " ");
+   }
+
    std::string rmdir = MakeProfile(exec, args.IsBatchMode() || args.IsHeadless());
 
    std::string tmpfile;
@@ -304,7 +312,7 @@ RWebDisplayHandle::BrowserCreator::Display(const RWebDisplayArgs &args)
    if (((url.find("token=") != std::string::npos) || (url.find("key=") != std::string::npos)) && !args.IsBatchMode() && !args.IsHeadless()) {
       TString filebase = "root_start_";
 
-      auto f = TemporaryFile(filebase, IsSnapChromium() ? 1 : 0, ".html");
+      auto f = TemporaryFile(filebase, IsSnapBrowser() ? 1 : 0, ".html");
 
       bool ferr = false;
 
@@ -686,11 +694,6 @@ void RWebDisplayHandle::ChromeCreator::ProcessGeometry(std::string &exec, const
                                            std::to_string(args.GetY() >= 0 ? args.GetY() : 0));
    }
 
-   if (!args.GetExtraArgs().empty()) {
-      if (!geometry.empty()) geometry.append(" ");
-      geometry.append(args.GetExtraArgs());
-   }
-
    exec = std::regex_replace(exec, std::regex("\\$geometry"), geometry);
 }
 
@@ -712,6 +715,9 @@ std::string RWebDisplayHandle::ChromeCreator::MakeProfile(std::string &exec, boo
       TRandom3 rnd;
       rnd.SetSeed(0);
       profile_arg = gSystem->TempDirectory();
+      if ((profile_arg.compare(0, 4, "/tmp") == 0) && IsSnapBrowser())
+         profile_arg = gSystem->GetHomeDirectory();
+
 #ifdef _MSC_VER
       char slash = '\\';
 #else
@@ -744,6 +750,7 @@ RWebDisplayHandle::FirefoxCreator::FirefoxCreator() : BrowserCreator(true)
    TestProg("/Applications/Firefox.app/Contents/MacOS/firefox");
 #endif
 #ifdef R__LINUX
+   TestProg("/snap/bin/firefox");
    TestProg("/usr/bin/firefox");
    TestProg("/usr/bin/firefox-bin");
 #endif
@@ -794,6 +801,8 @@ std::string RWebDisplayHandle::FirefoxCreator::MakeProfile(std::string &exec, bo
       TRandom3 rnd;
       rnd.SetSeed(0);
       std::string profile_dir = gSystem->TempDirectory();
+      if ((profile_dir.compare(0, 4, "/tmp") == 0) && IsSnapBrowser())
+         profile_dir = gSystem->GetHomeDirectory();
 
 #ifdef _MSC_VER
       char slash = '\\';
@@ -1221,7 +1230,7 @@ bool RWebDisplayHandle::ProduceImages(const std::vector<std::string> &fnames, co
    if (isChrome) {
       use_home_dir = chrome_tmp_workaround;
       auto &h1 = FindCreator("chrome", "ChromeCreator");
-      if (h1 && h1->IsActive() && h1->IsSnapChromium() && (use_home_dir == 0))
+      if (h1 && h1->IsActive() && h1->IsSnapBrowser() && (use_home_dir == 0))
          use_home_dir = 1;
    }
 
@@ -1387,9 +1396,13 @@ try_again:
 
    auto handle = RWebDisplayHandle::Display(args);
 
-   if (!handle) {
-      R__LOG_DEBUG(0, WebGUILog()) << "Cannot start " << args.GetBrowserName() << " to produce image " << fdebug;
-      return false;
+   // ensure file is created by browser draw
+   if (use_browser_draw && handle) {
+      Int_t batch_timeout = gEnv->GetValue("WebGui.BatchTimeout", 30) * 10;
+      while (gSystem->AccessPathName(wait_file_name.Data()) && (--batch_timeout > 0)) {
+         gSystem->ProcessEvents();
+         gSystem->Sleep(100);
+      }
    }
 
    // delete temporary HTML file
@@ -1400,12 +1413,18 @@ try_again:
          gSystem->Unlink(html_name.Data());
    }
 
-   if (!wait_file_name.IsNull() && gSystem->AccessPathName(wait_file_name.Data())) {
-      R__LOG_ERROR(WebGUILog()) << "Fail to produce image " << fdebug;
+   if (!handle) {
+      R__LOG_DEBUG(0, WebGUILog()) << "Cannot start " << args.GetBrowserName() << " to produce image " << fdebug;
       return false;
    }
 
    if (use_browser_draw) {
+
+      if (gSystem->AccessPathName(wait_file_name.Data())) {
+         R__LOG_ERROR(WebGUILog()) << "Fail to produce image " << fdebug;
+         return false;
+      }
+
       if (fmts[0] == "s.pdf")
          ::Info("ProduceImages", "PDF file %s with %d pages has been created", fnames[0].c_str(), (int) jsons.size());
       else {
