@@ -12,9 +12,10 @@
 #include "TMath.h"
 #include "THLimitsFinder.h"
 
-THLimitsFinder *THLimitsFinder::fgLimitsFinder = 0;
+#include <algorithm>
+
+THLimitsFinder *THLimitsFinder::fgLimitsFinder = nullptr;
 
-ClassImp(THLimitsFinder);
 
 /** \class THLimitsFinder
     \ingroup Histograms
@@ -381,8 +382,9 @@ void THLimitsFinder::OptimizeLimits(Int_t nbins, Int_t &newbins, Double_t &xmin,
       xmin = -1;
       xmax = 1;
    } else {
-      xmin    = binlow;
-      xmax    = binhigh;
+      const auto delta = 0.01 * (xmax - xmin);
+      xmin = std::min(binlow, xmin - delta);
+      xmax = std::max(binhigh, xmax + delta);
    }
    if (isInteger) {
       Long64_t ixmin = Long64_t(xmin);
