@@ -418,7 +418,6 @@ of the chunks holds an array of THnSparseCompactBinCoord and the content
 THnSparse) and passed in at construction time.
 */
 
-ClassImp(THnSparseArrayChunk);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// (Default) initialize a chunk. Takes ownership of cont (~THnSparseArrayChunk deletes it),
@@ -536,14 +535,14 @@ be retrieved by THnSparse::GetBinContent(linidx, (Int_t*)coord).
 After the call, coord will contain the bin coordinate of each axis for the bin
 with linear index linidx. A possible call would be
 
-   std::cout << hs.GetBinContent(0, coord);
-   std::cout <<" is the content of bin [x = " << coord[0] "
-        << " | y = " << coord[1] << "]" << std::endl;
+    std::cout << hs.GetBinContent(0, coord);
+    std::cout <<" is the content of bin [x = " << coord[0] "
+         << " | y = " << coord[1] << "]" << std::endl;
 
 ## Efficiency
 TH1 and TH2 are generally faster than THnSparse for one and two dimensional
 distributions. THnSparse becomes competitive for a sparsely filled TH3
-with large numbers of bins per dimension. The tutorial sparsehist.C
+with large numbers of bins per dimension. The tutorial \ref hist103_THnSparse_hist.C
 shows the turning point. On a AMD64 with 8GB memory, THnSparse "wins"
 starting with a TH3 with 30 bins per dimension. Using a THnSparse for a
 one-dimensional histogram is only reasonable if it has a huge number of bins.
@@ -581,7 +580,6 @@ retrieve the matching bin.
 */
 
 
-ClassImp(THnSparse);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Construct an empty THnSparse.
@@ -611,6 +609,65 @@ THnSparse::THnSparse(const char* name, const char* title, Int_t dim,
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+/// Construct a THnSparse with chunksize as the size of the chunks.
+/// "axes" is a vector of TAxis, its size sets the number of dimensions.
+/// This method is convenient for passing the Axes on the fly:
+/// auto h0= new THnSparseI{"h0", "", {{100, -50., 50.}, {50, -1000., 1000.}}};
+
+THnSparse::THnSparse(const char* name, const char* title,
+                     const std::vector<TAxis>& axes,
+                     Int_t chunksize):
+THnBase(name, title, axes),
+  fChunkSize(chunksize), fFilledBins(0), fCompactCoord(nullptr)
+{
+  const size_t dim=axes.size();
+  auto nbins=new Int_t[dim];
+  for (size_t i=0; i<dim; i++)
+    nbins[i]=axes.at(i).GetNbins();
+  fCompactCoord = new THnSparseCompactBinCoord(dim, nbins);
+  fBinContent.SetOwner();
+  delete[] nbins;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+/// Construct a THnSparse with dim dimensions and unequal binning.
+/// nbins and std::vector xbins are used to describe bin edges for each dimension.
+/// chunksize represents the size of the chunks.
+
+THnSparse::THnSparse(const char *name, const char *title, Int_t dim, const Int_t *nbins,
+                     const std::vector<std::vector<double>> &xbins, Int_t chunksize)
+   : THnBase(name, title, dim, nbins, xbins), fChunkSize(chunksize), fFilledBins(0), fCompactCoord(nullptr)
+{
+   fCompactCoord = new THnSparseCompactBinCoord(dim, nbins);
+   fBinContent.SetOwner();
+}
+
+////////////////////////////////////////////////////////////////////////////////
+/// Construct a THnSparse as a copy of "other"
+
+THnSparse::THnSparse(const THnSparse &other)
+   : THnBase(other),
+     fChunkSize(other.fChunkSize),
+     fFilledBins(other.fFilledBins),
+     fBins(other.fBins),
+     fBinsContinued(other.fBinsContinued),
+     fCompactCoord(nullptr)
+{
+
+   TObjArray *copiedContent = (TObjArray *)other.fBinContent.Clone();
+   fBinContent = *copiedContent;
+   copiedContent->SetOwner(kFALSE);
+   delete copiedContent;
+   fBinContent.SetOwner(kTRUE);
+
+   Int_t dim = other.GetNdimensions();
+   std::vector<Int_t> nbins(dim);
+   for (Int_t i = 0; i < dim; i++)
+      nbins[i] = other.GetAxis(i)->GetNbins();
+
+   fCompactCoord = new THnSparseCompactBinCoord(dim, nbins.data());
+}
+
 /// Destruct a THnSparse
 
 THnSparse::~THnSparse() {
