@@ -124,7 +124,8 @@ namespace cling {
                        bool prepend = false) {
        if (!dir.empty()) {
           for (auto & item : m_SearchPaths)
-            if (dir.equals(item.Path)) return;
+            if (dir == item.Path)
+              return;
           auto pos = prepend ? m_SearchPaths.begin() : m_SearchPaths.end();
           m_SearchPaths.insert(pos, SearchPathInfo{dir.str(), isUser});
        }
