@@ -61,14 +61,18 @@ using namespace cling;
 
 namespace {
   static constexpr unsigned CxxStdCompiledWith() {
+    // The value of __cplusplus in GCC < 14 is 202100L when -std=c++2b or
+    // -std=c++23 is specified, thus we relax the check to 202100L.
+#if __cplusplus >= 202100L
+    return 23;
+#elif __cplusplus >  201703L
+    return 20;
+#elif __cplusplus > 201402L
+    return 17;
     // The value of __cplusplus in GCC < 5.0 (e.g. 4.9.3) when
     // either -std=c++1y or -std=c++14 is specified is 201300L, which fails
     // the test for C++14 or more (201402L) as previously specified.
     // I would claim that the check should be relaxed to:
-#if __cplusplus >  201703L
-    return 20;
-#elif __cplusplus > 201402L
-    return 17;
 #elif __cplusplus > 201103L || (defined(_WIN32) && _MSC_VER >= 1900)
     return 14;
 #elif __cplusplus >= 201103L
@@ -110,6 +114,23 @@ namespace {
                                        llvm::SmallVectorImpl<char>& Buf,
                                        AdditionalArgList& Args,
                                        bool Verbose) {
+    // For power-users: Let's see if the path is available as a predefined env
+    // variable to save repeated system calls when ROOT/CLING is initialized for
+    // each process in a many process system. CLING_CPPSYSINCL should contain
+    // paths separated by a ":" and otherwise contain the same paths as returned
+    // from the CppInclQuery further below.
+    auto PrefCppSystemIncl = std::getenv("CLING_CPPSYSINCL");
+    if (PrefCppSystemIncl != nullptr) {
+      llvm::StringRef PathsString(PrefCppSystemIncl);
+      llvm::SmallVector<StringRef, 10> Paths;
+      PathsString.split(Paths, ":");
+      for (auto& P : Paths) {
+        P = P.trim();
+        Args.addArgument("-cxx-isystem", P.str());
+      }
+      return;
+    }
+
     std::string CppInclQuery("LC_ALL=C ");
     CppInclQuery.append(Compiler);
 
@@ -398,6 +419,15 @@ namespace {
 
     Opts.IncrementalExtensions = 1;
 
+    //
+    // Tell the parser to parse without RecoveryExpr nodes.
+    // This restores old (LLVM 18) behavior: error -> ExprError (invalid),
+    // without emitting a <recovery-expr>() that later asserts:
+    // Assertion `!Init->isValueDependent()' failed.
+    // https://github.com/llvm/llvm-project/commit/fd87d765c0455265aea4595a3741a96b4c078fbc
+    //
+    Opts.RecoveryAST = 0;
+
     Opts.Exceptions = 1;
     if (Opts.CPlusPlus) {
       Opts.CXXExceptions = 1;
@@ -576,7 +606,9 @@ namespace {
 
     // We can't use "assert.h" because it is defined in the resource dir, too.
 #ifdef _WIN32
+    #ifdef CLING_SUPPORT_VC
     llvm::SmallString<256> vcIncLoc(getIncludePathForHeader(HS, "vcruntime.h"));
+    #endif
     llvm::SmallString<256> servIncLoc(getIncludePathForHeader(HS, "windows.h"));
 #endif
     llvm::SmallString<128> cIncLoc(getIncludePathForHeader(HS, "time.h"));
@@ -589,7 +621,9 @@ namespace {
     llvm::SmallString<256> boostIncLoc(getIncludePathForHeader(HS, "boost/version.hpp"));
     llvm::SmallString<256> tinyxml2IncLoc(getIncludePathForHeader(HS, "tinyxml2.h"));
     llvm::SmallString<256> cudaIncLoc(getIncludePathForHeader(HS, "cuda.h"));
+    #ifdef CLING_SUPPORT_VC
     llvm::SmallString<256> vcVcIncLoc(getIncludePathForHeader(HS, "Vc/Vc"));
+    #endif
     llvm::SmallString<256> clingIncLoc(getIncludePathForHeader(HS,
                                         "cling/Interpreter/RuntimeUniverse.h"));
 
@@ -675,10 +709,12 @@ namespace {
     std::string MOverlay;
 
 #ifdef _WIN32
+    #ifdef CLING_SUPPORT_VC
     maybeAppendOverlayEntry(vcIncLoc.str(), "vcruntime.modulemap",
                             clingIncLoc.str().str(), MOverlay,
                             /*RegisterModuleMap=*/ true,
                             /*AllowModulemapOverride=*/ false);
+    #endif
     maybeAppendOverlayEntry(servIncLoc.str(), "services_msvc.modulemap",
                             clingIncLoc.str().str(), MOverlay,
                             /*RegisterModuleMap=*/ true,
@@ -727,11 +763,13 @@ namespace {
                               clingIncLoc.str().str(), MOverlay,
                               /*RegisterModuleMap=*/ true,
                               /*AllowModulemapOverride=*/ false);
+    #ifdef CLING_SUPPORT_VC
     if (!vcVcIncLoc.empty())
       maybeAppendOverlayEntry(vcVcIncLoc.str(), "vc.modulemap",
                               clingIncLoc.str().str(), MOverlay,
                               /*RegisterModuleMap=*/ true,
                               /*AllowModulemapOverride=*/ false);
+    #endif
     if (!boostIncLoc.empty()) {
       // Add the modulemap in the include/boost folder not in include.
       llvm::sys::path::append(boostIncLoc, "boost");
@@ -774,7 +812,7 @@ namespace {
     clang::HeaderSearchOptions& HSOpts = CI.getHeaderSearchOpts();
     // Register prebuilt module paths where we will lookup module files.
     addPrebuiltModulePaths(HSOpts,
-                           getPathsFromEnv(getenv("CLING_PREBUILT_MODULE_PATH")));
+                           getPathsFromEnv(std::getenv("CLING_PREBUILT_MODULE_PATH")));
 
     // Register all modulemaps necessary for cling to run. If we have specified
     // -fno-implicit-module-maps then we have to add them explicitly to the list
@@ -837,6 +875,9 @@ namespace {
     PPOpts.addMacroDef("__CLING__GNUC_MINOR__=" ClingStringify(__GNUC_MINOR__));
 #elif defined(_MSC_VER)
     PPOpts.addMacroDef("__CLING__MSVC__=" ClingStringify(_MSC_VER));
+#if defined(_WIN64) && defined(_DEBUG)
+    PPOpts.addMacroDef("_ITERATOR_DEBUG_LEVEL=0");
+#endif
 #endif
 
 // https://gcc.gnu.org/onlinedocs/libstdc++/manual/using_dual_abi.html
@@ -942,6 +983,8 @@ namespace {
     // Sanity check that clang delivered the language standard requested
     if (CompilerOpts.DefaultLanguage(&LangOpts)) {
       switch (CxxStdCompiledWith()) {
+        case 23: assert(LangOpts.CPlusPlus23 && "Language version mismatch");
+          LLVM_FALLTHROUGH;
         case 20: assert(LangOpts.CPlusPlus20 && "Language version mismatch");
           LLVM_FALLTHROUGH;
         case 17: assert(LangOpts.CPlusPlus17 && "Language version mismatch");
@@ -1046,7 +1089,8 @@ namespace {
           Out.indent(2) << "Module map file: " << ModuleMapPath << "\n";
         }
 
-        bool ReadLanguageOptions(const LangOptions &LangOpts, bool /*Complain*/,
+        bool ReadLanguageOptions(const LangOptions &LangOpts,
+                                 StringRef ModuleFilename, bool /*Complain*/,
                                  bool /*AllowCompatibleDifferences*/) override {
           Out.indent(2) << "Language options:\n";
 #define LANGOPT(Name, Bits, Default, Description)                       \
@@ -1070,6 +1114,7 @@ namespace {
         }
 
         bool ReadTargetOptions(const TargetOptions &TargetOpts,
+                               StringRef ModuleFilename,
                                bool /*Complain*/,
                                bool /*AllowCompatibleDifferences*/) override {
           Out.indent(2) << "Target options:\n";
@@ -1089,6 +1134,7 @@ namespace {
         }
 
         bool ReadDiagnosticOptions(IntrusiveRefCntPtr<DiagnosticOptions> DiagOpts,
+                                   StringRef ModuleFilename,
                                    bool /*Complain*/) override {
           Out.indent(2) << "Diagnostic options:\n";
 #define DIAGOPT(Name, Bits, Default) DUMP_BOOLEAN(DiagOpts->Name, #Name);
@@ -1108,6 +1154,7 @@ namespace {
         }
 
         bool ReadHeaderSearchOptions(const HeaderSearchOptions &HSOpts,
+                                     StringRef ModuleFilename,
                                      StringRef SpecificModuleCachePath,
                                      bool /*Complain*/) override {
           Out.indent(2) << "Header search options:\n";
@@ -1130,6 +1177,7 @@ namespace {
 
         bool
         ReadPreprocessorOptions(const PreprocessorOptions& PPOpts,
+                                StringRef /*ModuleFilename*/,
                                 bool /*ReadMacros*/, bool /*Complain*/,
                                 std::string& /*SuggestedPredefines*/) override {
           Out.indent(2) << "Preprocessor options:\n";
@@ -1240,11 +1288,10 @@ namespace {
 
   static CompilerInstance*
   createCIImpl(std::unique_ptr<llvm::MemoryBuffer> Buffer,
-               const CompilerOptions& COpts,
-               const char* LLVMDir,
+               const CompilerOptions& COpts, const char* LLVMDir,
                std::unique_ptr<clang::ASTConsumer> customConsumer,
                const CIFactory::ModuleFileExtensions& moduleExtensions,
-               bool OnlyLex, bool HasInput = false) {
+               bool OnlyLex, bool HasInput = false, bool AutoComplete = false) {
     // Follow clang -v convention of printing version on first line
     if (COpts.Verbose)
       cling::log() << "cling version " << ClingStringify(CLING_VERSION) << '\n';
@@ -1344,6 +1391,7 @@ namespace {
       // and by enforcing the std version now cling is telling clang what to
       // do, rather than after clang has dedcuded a default.
       switch (CxxStdCompiledWith()) {
+        case 23: argvCompile.emplace_back("-std=c++23"); break;
         case 20: argvCompile.emplace_back("-std=c++20"); break;
         case 17: argvCompile.emplace_back("-std=c++17"); break;
         case 14: argvCompile.emplace_back("-std=c++14"); break;
@@ -1367,11 +1415,6 @@ namespace {
       argvCompile.push_back("-fno-omit-frame-pointer");
 #endif
 
-#ifdef __cpp_sized_deallocation
-      // Propagate the setting of the compiler to the interpreter
-      argvCompile.push_back("-fsized-deallocation");
-#endif
-
     // Disable optimizations and keep frame pointer when debugging, overriding
     // other optimization options that might be in argv
     if (debuggingEnabled) {
@@ -1379,6 +1422,11 @@ namespace {
       argvCompile.push_back("-fno-omit-frame-pointer");
     }
 
+#ifdef CLING_WITH_ADAPTIVECPP
+    argvCompile.push_back("-D__ACPP_ENABLE_LLVM_SSCP_TARGET__");
+    argvCompile.push_back("-Xclang");
+    argvCompile.push_back("-disable-O0-optnone");
+#endif
     // Add host specific includes, -resource-dir if necessary, and -isysroot
     std::string ClingBin = GetExecutablePath(argv[0]);
     AddHostArguments(ClingBin, argvCompile, LLVMDir, COpts);
@@ -1395,15 +1443,16 @@ namespace {
   #endif
     }
 #endif
-
-    if (!COpts.HasOutput || !HasInput) {
-      // suppress the warning "argument unused during compilation: -c" of the
-      // device interpreter instance
-      if (!COpts.CUDADevice)
-        argvCompile.push_back("-c");
+    if ((!COpts.HasOutput || !HasInput) && !AutoComplete) {
       argvCompile.push_back("-");
     }
 
+    if (AutoComplete) {
+      // Put a dummy C++ file on to ensure there's at least one compile job for
+      // the driver to construct.
+      argvCompile.push_back("<<< cling interactive line includer >>>");
+    }
+
     auto InvocationPtr = std::make_shared<clang::CompilerInvocation>();
 
     // The compiler invocation is the owner of the diagnostic options.
@@ -1464,8 +1513,13 @@ namespace {
     // Chain in -verify checker, if requested.
     if (DiagOpts.VerifyDiagnostics)
       Diags->setClient(new clang::VerifyDiagnosticConsumer(*Diags));
+
+    IntrusiveRefCntPtr<llvm::vfs::OverlayFileSystem> Overlay =
+        new llvm::vfs::OverlayFileSystem(llvm::vfs::getRealFileSystem());
+    auto FileMgr = CI->createFileManager(Overlay);
+    llvm::vfs::FileSystem &VFS = FileMgr->getVirtualFileSystem();
     // Configure our handling of diagnostics.
-    ProcessWarningOptions(*Diags, DiagOpts);
+    ProcessWarningOptions(*Diags, DiagOpts, VFS);
 
     if (COpts.HasOutput && !OnlyLex) {
       ActionScan scan(clang::driver::Action::PrecompileJobClass,
@@ -1478,13 +1532,10 @@ namespace {
       if (!SetupCompiler(CI.get(), COpts))
         return nullptr;
 
-      ProcessWarningOptions(*Diags, DiagOpts);
+      ProcessWarningOptions(*Diags, DiagOpts, VFS);
       return CI.release();
     }
 
-    IntrusiveRefCntPtr<llvm::vfs::OverlayFileSystem> Overlay =
-        new llvm::vfs::OverlayFileSystem(llvm::vfs::getRealFileSystem());
-    CI->createFileManager(Overlay);
     clang::CompilerInvocation& Invocation = CI->getInvocation();
     std::string& PCHFile = Invocation.getPreprocessorOpts().ImplicitPCHInclude;
     bool InitLang = true, InitTarget = true;
@@ -1501,6 +1552,7 @@ namespace {
             m_Invocation(I), m_ReadLang(false), m_ReadTarget(false) {}
 
           bool ReadLanguageOptions(const LangOptions &LangOpts,
+                                   StringRef ModuleFilename,
                                    bool /*Complain*/,
                                    bool /*AllowCompatibleDifferences*/) override {
             m_Invocation.getLangOpts() = LangOpts;
@@ -1508,6 +1560,7 @@ namespace {
             return false;
           }
           bool ReadTargetOptions(const TargetOptions &TargetOpts,
+                                 StringRef ModuleFilename,
                                  bool /*Complain*/,
                                  bool /*AllowCompatibleDifferences*/) override {
             m_Invocation.getTargetOpts() = TargetOpts;
@@ -1515,7 +1568,9 @@ namespace {
             return false;
           }
           bool ReadPreprocessorOptions(
-              const PreprocessorOptions& PPOpts, bool /*ReadMacros*/,
+              const PreprocessorOptions& PPOpts,
+              StringRef /*ModuleFilename*/,
+              bool /*ReadMacros*/,
               bool /*Complain*/,
               std::string& /*SuggestedPredefines*/) override {
             // Import selected options, e.g. don't overwrite ImplicitPCHInclude.
@@ -1589,7 +1644,7 @@ namespace {
     // name), clang will call $PWD "." which is terrible if we ever change
     // directories (see ROOT-7114). By asking for $PWD (and not ".") it will
     // be registered as $PWD instead, which is stable even after chdirs.
-    FM.getDirectory(platform::GetCwd());
+    llvm::consumeError(FM.getDirectoryRef(platform::GetCwd()).takeError());
 
     // Build the virtual file, Give it a name that's likely not to ever
     // be #included (so we won't get a clash in clang's cache).
@@ -1600,13 +1655,13 @@ namespace {
     SM->setFileIsTransient(FE);
     FileID MainFileID = SM->createFileID(FE, SourceLocation(), SrcMgr::C_User);
     SM->setMainFileID(MainFileID);
-    const SrcMgr::SLocEntry& MainFileSLocE = SM->getSLocEntry(MainFileID);
-    const SrcMgr::FileInfo& MainFileFI = MainFileSLocE.getFile();
-    SrcMgr::ContentCache& MainFileCC
-      = const_cast<SrcMgr::ContentCache&>(MainFileFI.getContentCache());
     if (!Buffer)
       Buffer = llvm::MemoryBuffer::getMemBuffer("/*CLING DEFAULT MEMBUF*/;\n");
-    MainFileCC.setBuffer(std::move(Buffer));
+
+    // Adapted from upstream clang/lib/Interpreter/Interpreter.cpp
+    // FIXME: Merge with CompilerInstance::ExecuteAction.
+    llvm::MemoryBuffer* MB = Buffer.release();
+    CI->getPreprocessorOpts().addRemappedFile(Filename, MB);
 
     // Create TargetInfo for the other side of CUDA and OpenMP compilation.
     if ((CI->getLangOpts().CUDA || CI->getLangOpts().OpenMPIsTargetDevice) &&
@@ -1618,7 +1673,7 @@ namespace {
     }
 
     // Set up the preprocessor
-    auto TUKind = COpts.ModuleName.empty() ? TU_Complete : TU_Module;
+    auto TUKind = COpts.ModuleName.empty() ? TU_Complete : TU_ClangModule;
     CI->createPreprocessor(TUKind);
 
     // With modules, we now start adding prebuilt module paths to the CI.
@@ -1774,16 +1829,17 @@ namespace {
 
 namespace cling {
 
-CompilerInstance*
-CIFactory::createCI(llvm::StringRef Code, const InvocationOptions& Opts,
-                    const char* LLVMDir,
-                    std::unique_ptr<clang::ASTConsumer> consumer,
-                    const ModuleFileExtensions& moduleExtensions) {
-  return createCIImpl(llvm::MemoryBuffer::getMemBuffer(Code), Opts.CompilerOpts,
-                      LLVMDir, std::move(consumer), moduleExtensions,
-                      false /*OnlyLex*/,
-                      !Opts.IsInteractive());
-}
+  CompilerInstance*
+  CIFactory::createCI(llvm::StringRef Code, const InvocationOptions& Opts,
+                      const char* LLVMDir,
+                      std::unique_ptr<clang::ASTConsumer> consumer,
+                      const ModuleFileExtensions& moduleExtensions,
+                      bool AutoComplete /*false*/) {
+    return createCIImpl(llvm::MemoryBuffer::getMemBuffer(Code),
+                        Opts.CompilerOpts, LLVMDir, std::move(consumer),
+                        moduleExtensions, false /*OnlyLex*/,
+                        !Opts.IsInteractive(), AutoComplete);
+  }
 
 CompilerInstance* CIFactory::createCI(
     MemBufPtr_t Buffer, int argc, const char* const* argv, const char* LLVMDir,
