@@ -57,7 +57,6 @@ the sub-file starts. The members can be accessed in any order.
 #include "TObjArray.h"
 
 
-ClassImp(TZIPFile);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default ctor.
@@ -99,17 +98,17 @@ Long64_t TZIPFile::FindEndHeader()
 {
    const Int_t kBUFSIZE = 1024;
    Long64_t    size = fFile->GetSize();
-   Long64_t    limit = TMath::Min(size, Long64_t(kMAX_VAR_LEN));
+   Long64_t    limit = std::min(size, Long64_t(kMAX_VAR_LEN));
    char        buf[kBUFSIZE+4];
 
    // Note, this works correctly even if the signature straddles read
    // boundaries since we always read an overlapped area of four
    // bytes on the next read
    for (Long64_t offset = 4; offset < limit; ) {
-      offset = TMath::Min(offset + kBUFSIZE, limit);
+      offset = std::min(offset + kBUFSIZE, limit);
 
       Long64_t pos = size - offset;
-      Int_t    n = TMath::Min(kBUFSIZE+4, Int_t(offset));
+      Int_t    n = std::min(kBUFSIZE+4, Int_t(offset));
 
       fFile->Seek(pos);
       if (fFile->ReadBuffer(buf, n)) {
@@ -573,7 +572,6 @@ void TZIPFile::Print(Option_t *) const
 }
 
 
-ClassImp(TZIPMember);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default ctor.
