@@ -48,9 +48,9 @@ There are limitations for complex objects like TTree, which can not be converted
 #include "RZip.h"
 #include "snprintf.h"
 
+#include <limits>
 #include <memory>
 
-ClassImp(TBufferXML);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Creates buffer object to serialize/deserialize data to/from xml.
