@@ -18,7 +18,6 @@ import os
 import sys
 from time import sleep
 from itertools import zip_longest
-import cppyy
 
 def fileno(file_or_fd):
     """
@@ -198,7 +197,7 @@ def isDirectoryKey(key):
     """
     classname = key.GetClassName()
     cl = ROOT.gROOT.GetClass(classname)
-    if cl == cppyy.nullptr:
+    if not cl:
         logging.warning("Unknown class to ROOT: " + classname)
         return False
     return cl.InheritsFrom(ROOT.TDirectory.Class())
@@ -210,7 +209,7 @@ def isTreeKey(key):
     """
     classname = key.GetClassName()
     cl = ROOT.gROOT.GetClass(classname)
-    if cl == cppyy.nullptr:
+    if not cl:
         logging.warning("Unknown class to ROOT: " + classname)
         return False
     return cl.InheritsFrom(ROOT.TTree.Class())
@@ -222,7 +221,7 @@ def isTHnSparseKey(key):
     """
     classname = key.GetClassName()
     cl = ROOT.gROOT.GetClass(classname)
-    if cl == cppyy.nullptr:
+    if not cl:
         logging.warning("Unknown class to ROOT: " + classname)
         return False
     return cl.InheritsFrom(ROOT.THnSparse.Class())
@@ -557,7 +556,7 @@ def getSourceDestListOptDict(parser, wildcards=True):
 # Several functions shared by rootcp, rootmv and rootrm
 
 TARGET_ERROR = "target '{0}' is not a directory"
-OMITTING_ERROR = "omitting {0} '{1}'. Did you forget to specify the -r option for a recursive copy?"
+OMITTING_ERROR = "{0} '{1}' will be copied but not its subdirectories (if any). Use the -r option if you need a recursive copy."
 OVERWRITE_ERROR = "cannot overwrite non-directory '{0}' with directory '{1}'"
 
 
@@ -578,13 +577,12 @@ def copyRootObject(sourceFile, sourcePathSplit, destFile, destPathSplit, oneSour
         logging.warning(TARGET_ERROR.format(destPathSplit[-1]))
         retcode += 1
     # Entire ROOT file or directory in input omitting "-r" option
-    # OMITTING_FILE_ERROR or OMITTING_DIRECTORY_ERROR
     if not recursiveOption:
         if sourcePathSplit == []:
             logging.warning(OMITTING_ERROR.format("file", sourceFile.GetName()))
             retcode += 1
         elif isDirectory(sourceFile, sourcePathSplit):
-            logging.warning(OMITTING_DIRECTORY_ERROR.format("directory", sourcePathSplit[-1]))
+            logging.warning(OMITTING_ERROR.format("directory", sourcePathSplit[-1]))
             retcode += 1
     # Run copyRootObjectRecursive function with the wish
     # to follow the unix copy behaviour
@@ -653,8 +651,7 @@ def copyRootObjectRecursive(sourceFile, sourcePathSplit, destFile, destPathSplit
     to an other file or directory (destFile,destPathSplit)
     - Has the will to be unix-like
     - that's a recursive function
-    - Python adaptation of a root input/output tutorial :
-      $ROOTSYS/tutorials/io/copyFiles.C
+    - Python adaptation of a root input/output tutorial : copyFiles.C
     """
     retcode = 0
     replaceOption = replace
@@ -698,16 +695,10 @@ def copyRootObjectRecursive(sourceFile, sourcePathSplit, destFile, destPathSplit
             if replaceOption and isExisting(destFile, destPathSplit + [setName]):
                 changeDirectory(destFile, destPathSplit)
                 otherObj = getFromDirectory(setName)
-                if not otherObj == obj:
-                    retcodeTemp = deleteObject(destFile, destPathSplit + [setName])
-                    if retcodeTemp:
-                        retcode += retcodeTemp
-                        continue
-                    else:
-                        if isinstance(obj, ROOT.TNamed):
-                            obj.SetName(setName)
-                        changeDirectory(destFile, destPathSplit)
-                        obj.Write()
+                retcodeTemp = deleteObject(destFile, destPathSplit + [setName])
+                if retcodeTemp:
+                    retcode += retcodeTemp
+                    continue
                 else:
                     if isinstance(obj, ROOT.TNamed):
                         obj.SetName(setName)
@@ -792,40 +783,6 @@ REPLACE_HELP = "replace object if already existing"
 # End of help strings
 ##########
 
-##########
-# ROOTBROWSE
-
-
-def _openBrowser(rootFile=None):
-    browser = ROOT.TBrowser()
-    if ROOT.gSystem.InheritsFrom("TMacOSXSystem") or browser.IsWeb():
-        print("Press ctrl+c to exit.")
-        try:
-            while True:
-                if ROOT.gROOT.IsInterrupted() or ROOT.gSystem.ProcessEvents():
-                    break
-                sleep(0.01)
-        except (KeyboardInterrupt, SystemExit):
-            pass
-    else:
-        input("Press enter to exit.")
-
-
-def rootBrowse(fileName=None):
-    if fileName:
-        rootFile = openROOTFile(fileName)
-        if not rootFile:
-            return 1
-        _openBrowser(rootFile)
-        rootFile.Close()
-    else:
-        _openBrowser()
-    return 0
-
-
-# End of ROOTBROWSE
-##########
-
 ##########
 # ROOTCP
 
@@ -985,465 +942,6 @@ def rootEventselector(
 # End of ROOTEVENTSELECTOR
 ##########
 
-##########
-# ROOTLS
-
-# Ansi characters
-ANSI_BOLD = "\x1B[1m"
-ANSI_BLUE = "\x1B[34m"
-ANSI_GREEN = "\x1B[32m"
-ANSI_END = "\x1B[0m"
-
-# Needed for column width calculation
-ANSI_BOLD_LENGTH = len(ANSI_BOLD + ANSI_END)
-ANSI_BLUE_LENGTH = len(ANSI_BLUE + ANSI_END)
-ANSI_GREEN_LENGTH = len(ANSI_GREEN + ANSI_END)
-
-# Terminal and platform booleans
-IS_TERMINAL = sys.stdout.isatty()
-IS_WIN32 = sys.platform == "win32"
-
-
-def isSpecial(ansiCode, string):
-    """Use ansi code on 'string' if the output is the
-    terminal of a not Windows platform"""
-    if IS_TERMINAL and not IS_WIN32:
-        return ansiCode + string + ANSI_END
-    else:
-        return string
-
-
-def write(string, indent=0, end=""):
-    """Use sys.stdout.write to write the string with an indentation
-    equal to indent and specifying the end character"""
-    sys.stdout.write(" " * indent + string + end)
-
-
-TREE_TEMPLATE = "{0:{nameWidth}}" + "{1:{titleWidth}}{2:{memoryWidth}}"
-
-
-def _recursiveTreePrinter(tree, indent):
-    """Print recursively tree informations"""
-    listOfBranches = tree.GetListOfBranches()
-    if len(listOfBranches) > 0:  # Width informations
-        maxCharName = max([len(branch.GetName()) for branch in listOfBranches])
-        maxCharTitle = max([len(branch.GetTitle()) for branch in listOfBranches])
-        dic = {"nameWidth": maxCharName + 2, "titleWidth": maxCharTitle + 4, "memoryWidth": 1}
-    for branch in listOfBranches:  # Print loop
-        rec = [branch.GetName(), '"' + branch.GetTitle() + '"', str(branch.GetTotBytes())]
-        write(TREE_TEMPLATE.format(*rec, **dic), indent, end="\n")
-        _recursiveTreePrinter(branch, indent + 2)
-    write("")
-
-
-def _prepareTime(time):
-    """Get time in the proper shape
-    ex : 174512 for 17h 45m 12s
-    ex : 094023 for 09h 40m 23s"""
-    time = str(time)
-    time = "000000" + time
-    time = time[len(time) - 6 :]
-    return time
-
-
-MONTH = {
-    1: "Jan",
-    2: "Feb",
-    3: "Mar",
-    4: "Apr",
-    5: "May",
-    6: "Jun",
-    7: "Jul",
-    8: "Aug",
-    9: "Sep",
-    10: "Oct",
-    11: "Nov",
-    12: "Dec",
-}
-LONG_TEMPLATE = (
-    isSpecial(ANSI_BOLD, "{0:{classWidth}}") + "{1:{timeWidth}}" + "{2:{nameWidth}}{3:{titleWidth}}{4:{cycleWidth}}"
-)
-
-
-def _printClusters(tree, indent):
-    clusterStart = 0
-    nTotClusters = 0
-    clusterIter = tree.GetClusterIterator(0)
-    clusterStart = clusterIter()
-    write(isSpecial(ANSI_BOLD, "Cluster INCLUSIVE ranges:\n"), indent)
-    while clusterStart < tree.GetEntries():
-        # here we list the inclusive ranges, therefore we have a -1
-        clustLine = " - # %d: [%d, %d]\n" % (nTotClusters, clusterStart, clusterIter.GetNextEntry() - 1)
-        write(clustLine, indent)
-        nTotClusters += 1
-        clusterStart = clusterIter()
-    write(isSpecial(ANSI_BOLD, "The total number of clusters is %d\n" % nTotClusters), indent)
-
-
-def _rootLsPrintLongLs(keyList, indent, treeListing, recursiveListing):
-    """Prints a list of `TKey`s and some information.
-
-    The information of each key is printed with the following pattern:
-
-    TKeyClassName  {date time pattern} TKeyName;TKeyCycle  TKeyTitle  {optional: [current/backup cycle]}
-
-    An example:
-
-    ```
-    $ rootls -l https://root.cern/files/tutorials/hsimple.root
-    TProfile  Jun 30 23:59 2018 hprof;1  "Profile of pz versus px"
-    TH1F      Jun 30 23:59 2018 hpx;1    "This is the px distribution"
-    TH2F      Jun 30 23:59 2018 hpxpy;1  "py vs px"
-    TNtuple   Jun 30 23:59 2018 ntuple;1 "Demo ntuple"
-    ```
-    """
-    # Early return if the keyList is empty
-    if not keyList:
-        return
-
-    maxCharClass = max([len(key.GetClassName()) for key in keyList])
-    maxCharTime = 12
-    maxCharName = max([len(key.GetName()) for key in keyList])
-    dic = {
-        "classWidth": maxCharClass + 2,
-        "timeWidth": maxCharTime + 2,
-        "nameWidth": maxCharName + 2,
-        "titleWidth": 1,
-        "cycleWidth": 1,
-    }
-
-    # Input keyList is a THashList. Convert it to a Python list to make it work
-    # with zip_longest later
-    keyList = [key for key in keyList]
-    # Mimic the logic used in TDirectoryFile::ls(Option_t *option)
-    # For any key in the list, we need to grab the previous one and the next one.
-    # To do this, we use the iterator returned by zip_longest. The three input
-    # lists to zip_longest can be visualized as follows:
-    #
-    # a = ["key_1","key_2","key_3"]
-    # a_lagright = [None] + a[:-1]
-    # a_lagleft = a[1:]
-    # list(zip_longest(a_lagright, a, a_lagleft))
-    # [(None, 'key_1', 'key_2'), ('key_1', 'key_2', 'key_3'), ('key_2', 'key_3', None)]
-    #
-    # So that for any key, we can have a correct reference to the previous and
-    # following keys of `keyList`. The first key has no previous key and the last
-    # key has no following key, so the respective elements of the zip_longest
-    # iterator are `None`.
-    for previouskey, currentkey, nextkey in zip_longest([None] + keyList[:-1], keyList, keyList[1:]):
-        # If this key is the first one in the list, or if it has a different
-        # name than the previous one, it means that it's the first object of
-        # that kind in the list.
-        if previouskey is None or currentkey.GetName() != previouskey.GetName():
-            # Then we check the following key. If the current key is not
-            # the last key in the list and if the following key has the same
-            # name, then it means it's another cycle of the same object.
-            # Thus, it's gonna be a backup cycle of the same object.
-            # Otherwise, it's just a key with one cycle so we don't need
-            # to print information two distinguish between different cycles
-            # of the same key.
-            if nextkey is not None and currentkey.GetName() == nextkey.GetName():
-                cyclestr = "[current cycle]"
-            else:
-                cyclestr = ""
-        else:
-            # This key is a subsequent cycle of a previous key
-            cyclestr = "[backup cycle]"
-
-        datime = currentkey.GetDatime()
-        time = datime.GetTime()
-        date = datime.GetDate()
-        year = datime.GetYear()
-        time = _prepareTime(time)
-
-        rec = [
-            currentkey.GetClassName(),
-            MONTH[int(str(date)[4:6])] + " " + str(date)[6:] + " " + time[:2] + ":" + time[2:4] + " " + str(year) + " ",
-            currentkey.GetName() + ";" + str(currentkey.GetCycle()),
-            ' "' + currentkey.GetTitle() + '"',
-            " " + cyclestr,
-        ]
-
-        write(LONG_TEMPLATE.format(*rec, **dic), indent, end="\n")
-        if treeListing and isTreeKey(currentkey):
-            tree = currentkey.ReadObj()
-            _recursiveTreePrinter(tree, indent + 2)
-            tree = tree.GetTree()
-            _printClusters(tree, indent + 2)
-        if treeListing and isTHnSparseKey(currentkey):
-            hs = currentkey.ReadObj()
-            hs.Print("all")
-        if (
-            recursiveListing
-            and isDirectoryKey(currentkey)
-            and len(ROOT.gDirectory.Get(currentkey.GetName()).GetListOfKeys())
-        ):
-            subDir = ROOT.gDirectory.Get(currentkey.GetName())
-            subkeyList = subDir.GetListOfKeys()
-            subDir.cd()
-            _rootLsPrintLongLs(subkeyList, indent + 2, treeListing, recursiveListing)
-            subDir.cd("../")
-
-
-##
-# The code of the getTerminalSize function can be found here :
-# https://gist.github.com/jtriley/1108174
-# Thanks jtriley !!
-
-import os
-import shlex
-import struct
-import platform
-import subprocess
-
-
-def getTerminalSize():
-    """getTerminalSize()
-    - get width and height of console
-    - works on linux,os x,windows,cygwin(windows)
-    originally retrieved from:
-    http://stackoverflow.com/questions/566746/how-to-get-console-window-width-in-python"""
-    current_os = platform.system()
-    tuple_xy = None
-    if current_os == "Windows":
-        tuple_xy = _get_terminal_size_windows()
-        if tuple_xy is None:
-            tuple_xy = _get_terminal_size_tput()
-            # needed for window's python in cygwin's xterm!
-    if current_os in ["Linux", "Darwin"] or current_os.startswith("CYGWIN"):
-        tuple_xy = _get_terminal_size_linux()
-    if tuple_xy is None:
-        # print "default"
-        # _get_terminal_size_windows() or _get_terminal_size_tput don't work
-        tuple_xy = (80, 25)  # default value
-    return tuple_xy
-
-
-def _get_terminal_size_windows():
-    try:
-        from ctypes import windll, create_string_buffer
-
-        # stdin handle is -10
-        # stdout handle is -11
-        # stderr handle is -12
-        h = windll.kernel32.GetStdHandle(-12)
-        csbi = create_string_buffer(22)
-        res = windll.kernel32.GetConsoleScreenBufferInfo(h, csbi)
-        if res:
-            (bufx, bufy, curx, cury, wattr, left, top, right, bottom, maxx, maxy) = struct.unpack(
-                "hhhhHhhhhhh", csbi.raw
-            )
-            sizex = right - left + 1
-            sizey = bottom - top + 1
-            return sizex, sizey
-    except:
-        pass
-
-
-def _get_terminal_size_tput():
-    # get terminal width
-    # src: http://stackoverflow.com/questions/263890/how-do-i-find-the-width-height-of-a-terminal-window
-    try:
-        cols = int(subprocess.check_call(shlex.split("tput cols")))
-        rows = int(subprocess.check_call(shlex.split("tput lines")))
-        return (cols, rows)
-    except:
-        pass
-
-
-def _get_terminal_size_linux():
-    def ioctl_GWINSZ(fd):
-        try:
-            import fcntl
-            import termios
-
-            cr = struct.unpack("hh", fcntl.ioctl(fd, termios.TIOCGWINSZ, "1234"))
-            return cr
-        except:
-            pass
-
-    cr = ioctl_GWINSZ(0) or ioctl_GWINSZ(1) or ioctl_GWINSZ(2)
-    if not cr:
-        try:
-            fd = os.open(os.ctermid(), os.O_RDONLY)
-            cr = ioctl_GWINSZ(fd)
-            os.close(fd)
-        except:
-            pass
-    if not cr:
-        try:
-            cr = (os.environ["LINES"], os.environ["COLUMNS"])
-        except:
-            return None
-    return int(cr[1]), int(cr[0])
-
-
-# End of getTerminalSize code
-##
-
-
-def _rootLsPrintSimpleLs(keyList, indent, oneColumn, recursiveListing):
-    """Print list of strings in columns
-    - blue for directories
-    - green for trees"""
-    # This code is adapted from the pprint_list function here :
-    # http://stackoverflow.com/questions/25026556/output-list-like-ls
-    # Thanks hawkjo !!
-    if len(keyList) == 0:
-        return
-    (term_width, term_height) = getTerminalSize()
-    term_width = term_width - indent
-    min_chars_between = 2
-    min_element_width = min(len(key.GetName()) for key in keyList) + min_chars_between
-    max_element_width = max(len(key.GetName()) for key in keyList) + min_chars_between
-    if max_element_width >= term_width:
-        ncol, col_widths = 1, [1]
-    else:
-        # Start with max possible number of columns and reduce until it fits
-        ncol = 1 if oneColumn else min(len(keyList), term_width // min_element_width)
-        while True:
-            col_widths = [
-                max(len(key.GetName()) + min_chars_between for j, key in enumerate(keyList) if j % ncol == i)
-                for i in range(ncol)
-            ]
-            if sum(col_widths) <= term_width:
-                break
-            else:
-                ncol -= 1
-
-    def print_simple_ls_recursive(recursiveListing, key, indent, oneColumn):
-        if recursiveListing and len(ROOT.gDirectory.Get(key.GetName()).GetListOfKeys()):
-            subDir = ROOT.gDirectory.Get(key.GetName())
-            subkeyList = subDir.GetListOfKeys()
-            subDir.cd()
-            write("\n")
-            _rootLsPrintSimpleLs(subkeyList, indent + 2, oneColumn, recursiveListing)
-            subDir.cd("../")
-
-    for i, key in enumerate(keyList):
-        if i % ncol == 0:
-            write("", indent)  # indentation
-        # Don't add spaces after the last element of the line or of the list
-        if (i + 1) % ncol != 0 and i != len(keyList) - 1:
-            if not IS_TERMINAL:
-                write(key.GetName().ljust(col_widths[i % ncol]))
-            elif isDirectoryKey(key):
-                write(isSpecial(ANSI_BLUE, key.GetName()).ljust(col_widths[i % ncol] + ANSI_BLUE_LENGTH))
-                print_simple_ls_recursive(recursiveListing, key, indent, oneColumn)
-            elif isTreeKey(key):
-                write(isSpecial(ANSI_GREEN, key.GetName()).ljust(col_widths[i % ncol] + ANSI_GREEN_LENGTH))
-            else:
-                write(key.GetName().ljust(col_widths[i % ncol]))
-        else:  # No spaces after the last element of the line or of the list
-            if not IS_TERMINAL:
-                write(key.GetName())
-            elif isDirectoryKey(key):
-                write(isSpecial(ANSI_BLUE, key.GetName()))
-                print_simple_ls_recursive(recursiveListing, key, indent, oneColumn)
-            elif isTreeKey(key):
-                write(isSpecial(ANSI_GREEN, key.GetName()))
-            else:
-                write(key.GetName())
-            write("\n")
-
-
-def _rootLsPrint(keyList, indent, oneColumn, longListing, treeListing, recursiveListing):
-    """Print informations given by keyList with a rootLs
-    style chosen with the options"""
-    if longListing or treeListing:
-        _rootLsPrintLongLs(keyList, indent, treeListing, recursiveListing)
-    else:
-        _rootLsPrintSimpleLs(keyList, indent, oneColumn, recursiveListing)
-
-
-def _rootLsProcessFile(
-    fileName, pathSplitList, manySources, indent, oneColumn, longListing, treeListing, recursiveListing
-):
-    """rootls main routine for one file looping over paths in the file
-
-    sorts out directories and key, and loops over all paths, then forwards to
-    (_rootLsPrintLongLs or _rootLsPrintSimpleLs) - split in _rootLsPrint
-
-    args:
-       oneColumn   (bool):
-       longListing (bool):
-       treeListing (bool):
-       recursiveListing(bool):
-       indent       (int): how many columns the printout should be indented globally
-       manySources (bool): if more than one file is printed
-       fileName     (str): the root file name
-       pathSplitList: a list of subdirectories,
-                       each being represented as a list itself with a string per level
-                       e.g.
-                       [['Method_BDT','BDT']]
-    Returns:
-        retcode (int): 0 in case of success, 1 if the file could not be opened
-    """
-    retcode = 0
-    rootFile = openROOTFile(fileName)
-    if not rootFile:
-        return 1
-
-    keyList, dirList = keyClassSplitter(rootFile, pathSplitList)
-    # keyList lists the TKey objects from pathSplitList
-    # dirList is 'just the pathSplitList' for what aren't TKeys
-    if manySources:
-        write("{0} :".format(fileName) + "\n")
-    _rootLsPrint(keyList, indent, oneColumn, longListing, treeListing, recursiveListing)
-
-    # Loop on the directories
-    manyPathSplits = len(pathSplitList) > 1
-    indentDir = 2 if manyPathSplits else 0
-    for pathSplit in dirList:
-        keyList = getKeyList(rootFile, pathSplit)
-        keyListSort(keyList)
-        if manyPathSplits:
-            write("{0} :".format("/".join(pathSplit)), indent, end="\n")
-        _rootLsPrint(keyList, indent + indentDir, oneColumn, longListing, treeListing, recursiveListing)
-
-    rootFile.Close()
-    return retcode
-
-
-def rootLs(sourceList, oneColumn=False, longListing=False, treeListing=False, recursiveListing=False):
-    """rootls main routine for an arbitrary number of files
-
-    args:
-       oneColumn   (bool):
-       longListing (bool):
-       treeListing (bool):
-       recursiveListing(bool):
-       sourceList: a list of tuples with one list element per file
-                   the first tuple entry being the root file,
-                   the second a list of subdirectories,
-                       each being represented as a list itself with a string per level
-                   e.g.
-                   rootls tutorial/tmva/TMVA.root:Method_BDT/BDT turns into
-                   [('tutorials/tmva/TMVA.root', [['Method_BDT','BDT']])]
-
-    returns:
-       retcode (int): 0 in case of success
-    """
-    # Check arguments
-    if sourceList == []:
-        return 1
-    # sort sourceList according to filenames
-    tupleListSort(sourceList)
-
-    # Loop on the ROOT files
-    retcode = 0
-    manySources = len(sourceList) > 1
-    indent = 2 if manySources else 0
-    for fileName, pathSplitList in sourceList:
-        retcode += _rootLsProcessFile(
-            fileName, pathSplitList, manySources, indent, oneColumn, longListing, treeListing, recursiveListing
-        )
-    return retcode
-
-
-# End of ROOTLS
-##########
-
 ##########
 # ROOTMKDIR
 
@@ -1560,13 +1058,24 @@ def rootMv(sourceList, destFileName, destPathSplit, compress=None, interactive=F
 # ROOTPRINT
 
 
-def _keyListExtended(rootFile, pathSplitList):
+def _keyListExtended(rootFile, pathSplitList, recursive = False):
+    prefixList = []
     keyList, dirList = keyClassSplitter(rootFile, pathSplitList)
     for pathSplit in dirList:
         keyList.extend(getKeyList(rootFile, pathSplit))
+    subList = [key for key in keyList if isDirectoryKey(key)]
     keyList = [key for key in keyList if not isDirectoryKey(key)]
-    keyListSort(keyList)
-    return keyList
+    prefixList = ["" for key in keyList]
+    if recursive:
+        for subdir in subList:
+            subkeyList, subprefixList = _keyListExtended(ROOT.gDirectory.Get(subdir.GetName()), pathSplitList, recursive)
+            keyList.extend(subkeyList)
+            prefixList.extend([subdir.GetName() + "_" + prefix for prefix in subprefixList])
+    if recursive:
+        keyList, prefixList = (list(t) for t in zip(*sorted(zip(keyList, prefixList), key=lambda x: x[0].GetName().lower())))
+    else:
+        keyListSort(keyList)
+    return keyList, prefixList
 
 
 def rootPrint(
@@ -1579,6 +1088,7 @@ def rootPrint(
     sizeOption=None,
     styleOption=None,
     verboseOption=False,
+    recursiveOption=False,
 ):
     # Check arguments
     if sourceList == []:
@@ -1658,8 +1168,8 @@ def rootPrint(
             continue
         openRootFiles.append(rootFile)
         # Fill the key list (almost the same as in root)
-        keyList = _keyListExtended(rootFile, pathSplitList)
-        for key in keyList:
+        keyList, prefixList = _keyListExtended(rootFile, pathSplitList, recursiveOption)
+        for k, key in enumerate(keyList):
             if isTreeKey(key):
                 pass
             else:
@@ -1678,8 +1188,9 @@ def rootPrint(
                         canvas.Clear()
                         canvas.Divide(x, y)
                 else:
+                    prefix = prefixList[k]
                     if not outputOption:
-                        outputFileName = key.GetName() + "." + formatOption
+                        outputFileName = prefix + key.GetName() + "." + formatOption
                         if directoryOption:
                             outputFileName = os.path.join(directoryOption, outputFileName)
                     if outputOption or formatOption == "pdf":
