@@ -22,7 +22,7 @@ href="https://www.gnu.org/software/gsl/manual/html_node/Unix-random-number-gener
 that it should not be used because its period is only 2**31, i.e. approximately 2 billion events, that can be generated
 in just few seconds.
 
-To generate random numbers, one should use the derived class, which  are :
+To generate random numbers, one should use one of the derived classes, which are:
 - TRandom3: it is based on the "Mersenne Twister generator",
 it is fast and a very long period of about \f$10^{6000}\f$. However it fails some of the most stringent tests of the
 <a href="http://simul.iro.umontreal.ca/testu01/tu01.html">TestU01 suite</a>.
@@ -33,14 +33,14 @@ the ROOT generator, *gRandom*.
 576 bits. This new implementation is built on the idea and the original code of Alexei Sibidanov, described in his
 <a href="https://arxiv.org/abs/1705.03123">paper </a>. It generates random numbers with 52 bit precision (double
 precision) and it has an higher luxury level than the original Ranlux generator (`p = 2048` instead of `p=794`).
-- ::TRandomMixMax: Generator based on the family of the MIXMAX matrix generators (see the
+- ::TRandomMixMax : Generator based on the family of the MIXMAX matrix generators (see the
 <a href="https://mixmax.hepforge.org">MIXMAX HEPFORGE Web page</a> and the documentation of the class
 ROOT::Math::MixMaxEngine for more information), that are base on the Asanov dynamical C systems. This generator has a
 state of N=240 64 bit integers, proof random properties, it provides 61 random bits and it has a very large period
 (\f$10^{4839}\f$). Furthermore, it provides the capability to be seeded with the guarantee that, for each given
 different seed, a different sequence of random numbers will be generated. The only drawback is that the seeding time is
 time consuming, of the order of 0.1 ms, while the time to generate a number is few ns (more than 10000 faster).
-- ::TRandomMixMax17: Another MixMax generator, but with a smaller state, N=17, and this results in a smaller entropy
+- ::TRandomMixMax17 : Another MixMax generator, but with a smaller state, N=17, and this results in a smaller entropy
 than the generator with N=240. However, it has the same seeding capabilities, with a much faster seeding time (about 200
 times less than TRandomMixMax240 and comparable to TRandom3).
 - ::TRandomMixMax256 : A variant of the MIXMAX generators, based on a state of N=256, and described in the
@@ -85,7 +85,7 @@ for the random numbers obtained using a macbookpro 2.6 GHz Intel Core i7 CPU:
 -   ::TRandom1          80   ns/call
 -   ::TRandomRanlux48  250  ns/call
 
-The following methods are provided to generate random numbers disctributed according to some basic distributions:
+The following methods are provided to generate random numbers distributed according to some basic distributions:
 
 - Exp(Double_t tau)
 - Integer(UInt_t imax)
@@ -177,8 +177,8 @@ A TRandom object may be written to a Root file
 #include "TDirectory.h"
 #include "Math/QuantFuncMathCore.h"
 #include "TUUID.h"
+#include "TError.h"
 
-ClassImp(TRandom);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default constructor. For seed see SetSeed().
@@ -250,7 +250,7 @@ void TRandom::Circle(Double_t &x, Double_t &y, Double_t r)
 
 Double_t TRandom::Exp(Double_t tau)
 {
-   Double_t x = Rndm();              // uniform on ] 0, 1 ]
+   Double_t x = Rndm();              // uniform on ]0, 1[
    Double_t t = -tau * TMath::Log( x ); // convert to exponential distribution
    return t;
 }
@@ -331,7 +331,7 @@ Double_t TRandom::Gaus(Double_t mean, Double_t sigma)
          }
       }
 
-      while (1) {
+      while (true) {
          x = Rndm();
          y = kYm * Rndm();
          z = kX0 - kS*x - y;
@@ -348,7 +348,7 @@ Double_t TRandom::Gaus(Double_t mean, Double_t sigma)
             if (rn*rn<4*(kB-log(x))) {
                result = rn; break; }
       }
-   } while(0);
+   } while(false);
 
    return mean + sigma * result;
 }
@@ -395,19 +395,20 @@ Double_t TRandom::Landau(Double_t mu, Double_t sigma)
 /// the exponential.
 /// For higher values use a rejection method comparing with a Lorentzian
 /// distribution, as suggested by several authors.
-/// This routine since is returning 32 bits integer will not work for values
-/// larger than 2*10**9.
-/// One should then use the Trandom::PoissonD for such large values.
+/// This routine returns now an unsigned 64 bit integer
+/// For large values, larger than 1.84e+19, we print an error message
+/// advising to use the Trandom::PoissonD for such large values,
+/// and return the max value UINT64_MAX
 
-Int_t TRandom::Poisson(Double_t mean)
+ULong64_t TRandom::Poisson(Double_t mean)
 {
-   Int_t n;
+   ULong64_t n;
    if (mean <= 0) return 0;
    if (mean < 25) {
       Double_t expmean = TMath::Exp(-mean);
       Double_t pir = 1;
       n = -1;
-      while(1) {
+      while(true) {
          n++;
          pir *= Rndm();
          if (pir <= expmean) break;
@@ -434,12 +435,17 @@ Int_t TRandom::Poisson(Double_t mean)
          t = 0.9*(1.0 + y*y)* TMath::Exp(em*alxm - TMath::LnGamma(em + 1.0) - g);
       } while( Rndm() > t );
 
-      return static_cast<Int_t> (em);
+      return static_cast<ULong64_t>(em);
 
    }
    else {
-      // use Gaussian approximation vor very large values
-      n = Int_t(Gaus(0,1)*TMath::Sqrt(mean) + mean +0.5);
+      // use Gaussian approximation for very large values
+      Double_t x = Gaus(0, 1) * TMath::Sqrt(mean) + mean + 0.5;
+      if (x > TMath::Limits<ULong64_t>::Max()) {
+         Error("Poisson", "Overflow in return value. Use PoissonD instead.");
+         return TMath::Limits<ULong64_t>::Max();
+      }
+      n = static_cast<ULong64_t>(x);
       return n;
    }
 }
@@ -459,7 +465,7 @@ Double_t TRandom::PoissonD(Double_t mean)
       Double_t expmean = TMath::Exp(-mean);
       Double_t pir = 1;
       n = -1;
-      while(1) {
+      while(true) {
          n++;
          pir *= Rndm();
          if (pir <= expmean) break;
@@ -489,7 +495,7 @@ Double_t TRandom::PoissonD(Double_t mean)
       return em;
 
    } else {
-      // use Gaussian approximation vor very large values
+      // use Gaussian approximation for very large values
       return Gaus(0,1)*TMath::Sqrt(mean) + mean +0.5;
    }
 }
@@ -570,7 +576,7 @@ Double_t TRandom::Rndm( )
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Return an array of n random numbers uniformly distributed in ]0,1].
+/// Return an array of n random numbers uniformly distributed in ]0,1[.
 
 void TRandom::RndmArray(Int_t n, Double_t *array)
 {
@@ -583,7 +589,7 @@ void TRandom::RndmArray(Int_t n, Double_t *array)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Return an array of n random numbers uniformly distributed in ]0,1].
+/// Return an array of n random numbers uniformly distributed in ]0,1[.
 
 void TRandom::RndmArray(Int_t n, Float_t *array)
 {
@@ -618,7 +624,10 @@ void TRandom::SetSeed(ULong_t seed)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Get the random generator seed.
+/// \brief Get the random generator seed.
+///
+/// \warning Might not be the initial seed!
+///
 /// Note that this function returns the given seed only when using
 /// as random generator engine TRandom itself, which is an LCG generator
 /// and it has as seed (state) only one 32 bit word.
