@@ -24,9 +24,9 @@
 #include "TBuffer.h"
 #include "TROOT.h"
 #include "strlcpy.h"
-#include "snprintf.h"
 
-ClassImp(TLinearFitter);
+#include <string>
+
 
 
 std::map<TString,TFormula*> TLinearFitter::fgFormulaMap;
@@ -226,22 +226,6 @@ fitters and doesn't require to set the initial values of parameters.
 ///run the function StoreData(kFALSE) after constructor
 
 TLinearFitter::TLinearFitter() :
-TVirtualFitter(),
-   fParams(),
-   fParCovar(),
-   fTValues(),
-   fDesign(),
-   fDesignTemp(),
-   fDesignTemp2(),
-   fDesignTemp3(),
-   fAtb(),
-   fAtbTemp(),
-   fAtbTemp2(),
-   fAtbTemp3(),
-   fFunctions(),
-   fY(),
-   fX(),
-   fE(),
    fVal()
 {
    fChisquare =0;
@@ -359,61 +343,6 @@ TLinearFitter::TLinearFitter(TFormula *function, Option_t *opt)
    SetFormula(function);
 }
 
-////////////////////////////////////////////////////////////////////////////////
-/// Copy ctor
-
-TLinearFitter::TLinearFitter(const TLinearFitter& tlf) :
-   TVirtualFitter(tlf),
-   fParams(tlf.fParams),
-   fParCovar(tlf.fParCovar),
-   fTValues(tlf.fTValues),
-   fParSign(tlf.fParSign),
-   fDesign(tlf.fDesign),
-   fDesignTemp(tlf.fDesignTemp),
-   fDesignTemp2(tlf.fDesignTemp2),
-   fDesignTemp3(tlf.fDesignTemp3),
-   fAtb(tlf.fAtb),
-   fAtbTemp(tlf.fAtbTemp),
-   fAtbTemp2(tlf.fAtbTemp2),
-   fAtbTemp3(tlf.fAtbTemp3),
-   fFunctions( * (TObjArray *)tlf.fFunctions.Clone()),
-   fY(tlf.fY),
-   fY2(tlf.fY2),
-   fY2Temp(tlf.fY2Temp),
-   fX(tlf.fX),
-   fE(tlf.fE),
-   fInputFunction(tlf.fInputFunction),
-   fVal(),
-   fNpoints(tlf.fNpoints),
-   fNfunctions(tlf.fNfunctions),
-   fFormulaSize(tlf.fFormulaSize),
-   fNdim(tlf.fNdim),
-   fNfixed(tlf.fNfixed),
-   fSpecial(tlf.fSpecial),
-   fFormula(nullptr),
-   fIsSet(tlf.fIsSet),
-   fStoreData(tlf.fStoreData),
-   fChisquare(tlf.fChisquare),
-   fH(tlf.fH),
-   fRobust(tlf.fRobust),
-   fFitsample(tlf.fFitsample),
-   fFixedParams(nullptr)
-{
-   // make a deep  copy of managed objects
-   // fFormula, fFixedParams and fFunctions
-
-   if ( tlf.fFixedParams && fNfixed > 0 ) {
-      fFixedParams=new Bool_t[fNfixed];
-      for(Int_t i=0; i<fNfixed; ++i)
-         fFixedParams[i]=tlf.fFixedParams[i];
-   }
-   if (tlf.fFormula) {
-      fFormula = new char[fFormulaSize+1];
-      strlcpy(fFormula,tlf.fFormula,fFormulaSize+1);
-   }
-
-}
-
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Linear fitter cleanup.
@@ -435,75 +364,6 @@ TLinearFitter::~TLinearFitter()
 
 }
 
-////////////////////////////////////////////////////////////////////////////////
-/// Assignment operator
-
-TLinearFitter& TLinearFitter::operator=(const TLinearFitter& tlf)
-{
-   if(this!=&tlf) {
-
-      TVirtualFitter::operator=(tlf);
-      fParams.ResizeTo(tlf.fParams);      fParams=tlf.fParams;
-      fParCovar.ResizeTo(tlf.fParCovar);  fParCovar=tlf.fParCovar;
-      fTValues.ResizeTo(tlf.fTValues);    fTValues=tlf.fTValues;
-      fParSign.ResizeTo(tlf.fParSign);    fParSign=tlf.fParSign;
-      fDesign.ResizeTo(tlf.fDesign);                fDesign=tlf.fDesign;
-      fDesignTemp.ResizeTo(tlf.fDesignTemp);        fDesignTemp=tlf.fDesignTemp;
-      fDesignTemp2.ResizeTo(tlf.fDesignTemp2);      fDesignTemp2=tlf.fDesignTemp2;
-      fDesignTemp3.ResizeTo(tlf.fDesignTemp3);      fDesignTemp3=tlf.fDesignTemp3;
-
-      fAtb.ResizeTo(tlf.fAtb);            fAtb=tlf.fAtb;
-      fAtbTemp.ResizeTo(tlf.fAtbTemp);    fAtbTemp=tlf.fAtbTemp;
-      fAtbTemp2.ResizeTo(tlf.fAtbTemp2);    fAtbTemp2=tlf.fAtbTemp2;
-      fAtbTemp3.ResizeTo(tlf.fAtbTemp3);    fAtbTemp3=tlf.fAtbTemp3;
-
-      // use clear instead of delete
-      fFunctions.Clear();
-      fFunctions= *(TObjArray*) tlf.fFunctions.Clone();
-
-      fY.ResizeTo(tlf.fY);    fY = tlf.fY;
-      fX.ResizeTo(tlf.fX);    fX = tlf.fX;
-      fE.ResizeTo(tlf.fE);    fE = tlf.fE;
-
-      fY2 = tlf.fY2;
-      fY2Temp = tlf.fY2Temp;
-      for(Int_t i = 0; i < 1000; i++) fVal[i] = tlf.fVal[i];
-
-      if(fInputFunction) { delete fInputFunction; fInputFunction = nullptr; }
-      if(tlf.fInputFunction) fInputFunction = new TFormula(*tlf.fInputFunction);
-
-      fNpoints=tlf.fNpoints;
-      fNfunctions=tlf.fNfunctions;
-      fFormulaSize=tlf.fFormulaSize;
-      fNdim=tlf.fNdim;
-      fNfixed=tlf.fNfixed;
-      fSpecial=tlf.fSpecial;
-
-      if(fFormula) { delete [] fFormula; fFormula = nullptr; }
-      if (tlf.fFormula) {
-         fFormula = new char[fFormulaSize+1];
-         strlcpy(fFormula,tlf.fFormula,fFormulaSize+1);
-      }
-
-      fIsSet=tlf.fIsSet;
-      fStoreData=tlf.fStoreData;
-      fChisquare=tlf.fChisquare;
-
-      fH=tlf.fH;
-      fRobust=tlf.fRobust;
-      fFitsample=tlf.fFitsample;
-
-      if(fFixedParams) { delete [] fFixedParams; fFixedParams = nullptr; }
-      if ( tlf.fFixedParams && fNfixed > 0 ) {
-         fFixedParams=new Bool_t[fNfixed];
-         for(Int_t i=0; i< fNfixed; ++i)
-            fFixedParams[i]=tlf.fFixedParams[i];
-      }
-
-   }
-   return *this;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 ///Add another linear fitter to this linear fitter. Points and Design matrices
 ///are added, but the previous fitting results (if any) are deleted.
@@ -1580,12 +1440,10 @@ void TLinearFitter::SetFormula(const char *formula)
       fFunctions.Expand(fNfunctions);
 
       //check if the old notation of xi is used somewhere instead of x[i]
-      char pattern[12];
-      char replacement[14];
       for (i=0; i<fNdim; i++){
-         snprintf(pattern,sizeof(pattern), "x%d", i);
-         snprintf(replacement,sizeof(replacement), "x[%d]", i);
-         sstring = sstring.ReplaceAll(pattern, Int_t(i/10)+2, replacement, Int_t(i/10)+4);
+         std::string pattern = "x" + std::to_string(i);
+         std::string replacement = "x[" + std::to_string(i) + "]";
+         sstring = sstring.ReplaceAll(pattern.c_str(), replacement.c_str());
       }
 
       //fill the array of functions
