@@ -2,6 +2,7 @@
 // Author: Jan Conrad
 
 /** \class TRolke
+    \legacy{TRolke, Consider switching to RooStats.}
     \ingroup Physics
  This class computes confidence intervals for the rate of a Poisson
  process in the presence of uncertain background and/or efficiency.
@@ -32,7 +33,7 @@
    from sidebands (or MC), and
    the signal efficiency was determined from Monte Carlo
 
-2: SetPoissonBkgGaussEff(x,y,em,sde,tau)
+2: SetPoissonBkgGaussEff(x,y,em,tau,sde)
 ~~~
    Background: Poisson
    Efficiency: Gaussian
@@ -166,7 +167,6 @@
 #include "TMath.h"
 #include <iostream>
 
-ClassImp(TRolke);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Constructor with optional Confidence Level argument.
@@ -452,7 +452,7 @@ bool TRolke::GetSensitivity(Double_t& low, Double_t& high, Double_t pPrecision)
 
    int loop_x = 0;
 
-   while (1) {
+   while (true) {
       ComputeInterval(loop_x, f_y, f_z, f_bm, f_em, f_e, f_mid, f_sde, f_sdb, f_tau, f_b, f_m);
       weight = TMath::PoissonI(loop_x, background);
       low += fLowerLimit * weight;
@@ -485,7 +485,7 @@ bool TRolke::GetLimitsQuantile(Double_t& low, Double_t& high, Int_t& out_x, Doub
    Double_t weightSum = 0;
    Int_t loop_x = 0;
 
-   while (1) {
+   while (true) {
       weight = TMath::PoissonI(loop_x, background);
       weightSum += weight;
       if (weightSum >= integral) {
@@ -579,7 +579,7 @@ bool TRolke::GetCriticalNumber(Int_t& ncrit, Int_t maxtry)
 void TRolke::SetSwitch(bool bnd) {
    if(fNumWarningsDeprecated1<2){
       std::cerr << "*******************************************" <<std::endl;
-      std::cerr << "TRolke - Warning: 'SetSwitch' is depricated and may be removed from future releases:" <<std::endl;
+      std::cerr << "TRolke - Warning: 'SetSwitch' is deprecated and may be removed from future releases:" <<std::endl;
       std::cerr << " - Use 'SetBounding' instead "<<std::endl;
       std::cerr << "*******************************************" <<std::endl;
       fNumWarningsDeprecated1++;
@@ -637,7 +637,7 @@ void TRolke::Print(Option_t*) const {
 Double_t TRolke::CalculateInterval(Int_t x, Int_t y, Int_t z, Double_t bm, Double_t em, Double_t e, Int_t mid, Double_t sde, Double_t sdb, Double_t tau, Double_t b, Int_t m){
    if (fNumWarningsDeprecated2<2 ) {
       std::cerr << "*******************************************" <<std::endl;
-      std::cerr << "TRolke - Warning: 'CalculateInterval' is depricated and may be removed from future releases:" <<std::endl;
+      std::cerr << "TRolke - Warning: 'CalculateInterval' is deprecated and may be removed from future releases:" <<std::endl;
       std::cerr << " - Use e.g. 'SetGaussBkgGaussEff' and 'GetLimits' instead (read the docs in Rolke.cxx )"<<std::endl;
       std::cerr << "*******************************************" <<std::endl;
       fNumWarningsDeprecated2++;
