@@ -50,7 +50,6 @@ Provides access to different ROOT collections and containers
 like TTree, TCanvas, TFile, ...
 */
 
-ClassImp(TRootSnifferFull);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// constructor
@@ -795,7 +794,7 @@ Bool_t TRootSnifferFull::ProduceExe(const std::string &path, const std::string &
       if (method)
          call->Execute(obj_ptr, &txt);
       else
-         call->Execute(0, &txt); // here 0 is artificial, there is no proper signature
+         call->Execute(&txt);
       if (txt != nullptr) {
          if (resbuf)
             resbuf->WriteString(txt);
