@@ -48,7 +48,7 @@ started either via inetd or by hand from the command line (no need
 to be super user).
 **/
 
-#include <errno.h>
+#include <cerrno>
 
 #include "Bytes.h"
 #include "NetErrors.h"
@@ -64,8 +64,6 @@ to be super user).
 
 // fgClientProtocol is now in TAuthenticate
 
-ClassImp(TNetFile);
-ClassImp(TNetSystem);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Create a TNetFile object. This is actually done inside Create(), so
