@@ -14,19 +14,19 @@
 // rpdconn                                                              //
 //                                                                      //
 // This header file contains the definition of some utility classes     //
-// used for process communication between xproofd, rootd, proofexecv.   //
+// used for process communication between rootd.                        //
 //                                                                      //
 //////////////////////////////////////////////////////////////////////////
 
 #include "rpdconn.h"
 
 #include <arpa/inet.h>
-#include <errno.h>
+#include <cerrno>
 #include <netdb.h>
 #include <poll.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/types.h>
