@@ -20,6 +20,7 @@
 #include <RooBernstein.h>
 #include <RooBifurGauss.h>
 #include <RooCBShape.h>
+#include <RooCategory.h>
 #include <RooChebychev.h>
 #include <RooConstVar.h>
 #include <RooConstraintSum.h>
@@ -37,6 +38,7 @@
 #include <RooHistPdf.h>
 #include <RooLandau.h>
 #include <RooLognormal.h>
+#include <RooMultiPdf.h>
 #include <RooMultiVarGaussian.h>
 #include <RooParamHistFunc.h>
 #include <RooPoisson.h>
@@ -65,6 +67,14 @@ namespace Experimental {
 
 namespace {
 
+// Return a stringy-field version of the value, formatted to maximum precision.
+std::string doubleToString(double val)
+{
+   std::stringstream ss;
+   ss << std::setprecision(std::numeric_limits<double>::max_digits10) << val;
+   return ss.str();
+}
+
 std::string mathFunc(std::string const &name)
 {
    return "RooFit::Detail::MathFuncs::" + name;
@@ -241,6 +251,56 @@ void codegenImpl(RooMultiVarGaussian &arg, CodegenContext &ctx)
                  ctx.buildCall(mathFunc("multiVarGaussian"), arg.xVec().size(), arg.xVec(), arg.muVec(), covISpan));
 }
 
+void codegenImpl(RooMultiPdf &arg, CodegenContext &ctx)
+{
+   int numPdfs = arg.getNumPdfs();
+
+   // MathFunc call
+
+   // The value of this number should be discussed. Beyound a certain number of
+   // indices MathFunc call becomes more efficient.
+   if (numPdfs > 2) {
+      ctx.addResult(&arg, ctx.buildCall(mathFunc("multipdf"), arg.indexCategory(), arg.getPdfList()));
+
+      std::cout << "MathFunc call used\n";
+
+   } else {
+
+      // Ternary nested expression
+      std::string indexExpr = ctx.getResult(arg.indexCategory());
+
+      // int numPdfs = arg.getNumPdfs();
+      std::string expr;
+
+      for (int i = 0; i < numPdfs; ++i) {
+         RooAbsPdf *pdf = arg.getPdf(i);
+         std::string pdfExpr = ctx.getResult(*pdf);
+
+         expr += "(" + indexExpr + " == " + std::to_string(i) + " ? (" + pdfExpr + ") : ";
+      }
+
+      expr += "0.0";
+      expr += std::string(numPdfs, ')'); // Close all ternary operators
+
+      ctx.addResult(&arg, expr);
+      std::cout << "Ternary expression call used \n";
+   }
+}
+
+// RooCategory index added.
+void codegenImpl(RooCategory &arg, CodegenContext &ctx)
+{
+   int idx = ctx.observableIndexOf(arg);
+   if (idx < 0) {
+
+      idx = 1;
+      ctx.addVecObs(arg.GetName(), idx);
+   }
+
+   std::string result = std::to_string(arg.getCurrentIndex());
+   ctx.addResult(&arg, result);
+}
+
 void codegenImpl(RooAddition &arg, CodegenContext &ctx)
 {
    if (arg.list().empty()) {
@@ -303,15 +363,7 @@ void codegenImpl(RooChebychev &arg, CodegenContext &ctx)
 
 void codegenImpl(RooConstVar &arg, CodegenContext &ctx)
 {
-   // Just return a stringy-field version of the const value.
-   // Formats to the maximum precision.
-   constexpr auto max_precision{std::numeric_limits<double>::digits10 + 1};
-   std::stringstream ss;
-   ss.precision(max_precision);
-   // Just use toString to make sure we do not output 'inf'.
-   // This is really ugly for large numbers...
-   ss << std::fixed << RooNumber::toString(arg.getVal());
-   ctx.addResult(&arg, ss.str());
+   ctx.addResult(&arg, doubleToString(arg.getVal()));
 }
 
 void codegenImpl(RooConstraintSum &arg, CodegenContext &ctx)
@@ -329,7 +381,10 @@ void codegenImpl(RooFormulaVar &arg, CodegenContext &ctx)
    arg.getVal(); // to trigger the creation of the TFormula
    std::string funcName = arg.getUniqueFuncName();
    ctx.collectFunction(funcName);
-   ctx.addResult(&arg, ctx.buildCall(funcName, arg.dependents()));
+   // We have to force the array type to be "double" because that's what the
+   // declared function wrapped by the TFormula expects.
+   auto inputVar = ctx.buildArg(arg.dependents(), /*arrayType=*/"double");
+   ctx.addResult(&arg, funcName + "(" + inputVar + ")");
 }
 
 void codegenImpl(RooEffProd &arg, CodegenContext &ctx)
@@ -372,7 +427,10 @@ void codegenImpl(RooGenericPdf &arg, CodegenContext &ctx)
    arg.getVal(); // to trigger the creation of the TFormula
    std::string funcName = arg.getUniqueFuncName();
    ctx.collectFunction(funcName);
-   ctx.addResult(&arg, ctx.buildCall(funcName, arg.dependents()));
+   // We have to force the array type to be "double" because that's what the
+   // declared function wrapped by the TFormula expects.
+   auto inputVar = ctx.buildArg(arg.dependents(), /*arrayType=*/"double");
+   ctx.addResult(&arg, funcName + "(" + inputVar + ")");
 }
 
 void codegenImpl(RooHistFunc &arg, CodegenContext &ctx)
@@ -450,15 +508,16 @@ void codegenImpl(RooParamHistFunc &arg, CodegenContext &ctx)
 {
    std::string const &idx = arg.dataHist().calculateTreeIndexForCodeSquash(ctx, arg.xList());
    std::string arrName = ctx.buildArg(arg.paramList());
-   std::string result = arrName + "[" + idx + "]";
+   std::stringstream result;
+   result << arrName << "[" << idx << "]";
    if (arg.relParam()) {
       // get weight[idx] * binv[idx]. Here we get the bin volume for the first element as we assume the distribution to
       // be binned uniformly.
       double binV = arg.dataHist().binVolume(0);
       std::string weightArr = arg.dataHist().declWeightArrayForCodeSquash(ctx, false);
-      result += " * *(" + weightArr + " + " + idx + ") * " + std::to_string(binV);
+      result << " * *(" << weightArr << " + " << idx + ") * " << doubleToString(binV);
    }
-   ctx.addResult(&arg, result);
+   ctx.addResult(&arg, result.str());
 }
 
 void codegenImpl(RooPoisson &arg, CodegenContext &ctx)
@@ -570,6 +629,7 @@ void codegenImpl(RooRealIntegral &arg, CodegenContext &ctx)
       << "   const int n = 1000; // number of sampling points\n"
       << "   double d = " << intVar.getMax(arg.intRange()) << " - " << intVar.getMin(arg.intRange()) << ";\n"
       << "   double eps = d / n;\n"
+      << "   #pragma clad checkpoint loop\n"
       << "   for (int i = 0; i < n; ++i) {\n"
       << "      " << obsName << "[0] = " << intVar.getMin(arg.intRange()) << " + eps * i;\n"
       << "      double tmpA = " << funcName << "(params, " << obsName << ", xlArr);\n"
@@ -599,15 +659,7 @@ void codegenImpl(RooRealVar &arg, CodegenContext &ctx)
    if (!arg.isConstant()) {
       ctx.addResult(&arg, arg.GetName());
    }
-   // Just return a formatted version of the const value.
-   // Formats to the maximum precision.
-   constexpr auto max_precision{std::numeric_limits<double>::digits10 + 1};
-   std::stringstream ss;
-   ss.precision(max_precision);
-   // Just use toString to make sure we do not output 'inf'.
-   // This is really ugly for large numbers...
-   ss << std::fixed << RooNumber::toString(arg.getVal());
-   ctx.addResult(&arg, ss.str());
+   ctx.addResult(&arg, doubleToString(arg.getVal()));
 }
 
 void codegenImpl(RooRecursiveFraction &arg, CodegenContext &ctx)
@@ -761,7 +813,7 @@ std::string rooHistIntegralTranslateImpl(int code, RooAbsArg const &arg, RooData
                                     << std::endl;
       return "";
    }
-   return std::to_string(dataHist.sum(histFuncMode));
+   return doubleToString(dataHist.sum(histFuncMode));
 }
 
 } // namespace
@@ -802,7 +854,7 @@ std::string codegenIntegralImpl(RooMultiVarGaussian &arg, int code, const char *
       throw std::runtime_error(errorMsg.str().c_str());
    }
 
-   return std::to_string(arg.analyticalIntegral(code, rangeName));
+   return doubleToString(arg.analyticalIntegral(code, rangeName));
 }
 
 std::string codegenIntegralImpl(RooPoisson &arg, int code, const char *rangeName, CodegenContext &ctx)
@@ -854,7 +906,7 @@ std::string codegenIntegralImpl(RooUniform &arg, int code, const char *rangeName
 {
    // The integral of a uniform distribution is static, so we can just hardcode
    // the result in a string.
-   return std::to_string(arg.analyticalIntegral(code, rangeName));
+   return doubleToString(arg.analyticalIntegral(code, rangeName));
 }
 
 } // namespace Experimental
