@@ -33,7 +33,6 @@ The parameterization here is physics driven and differs from the ROOT::Math::log
 
 #include <Math/PdfFuncMathCore.h>
 
-ClassImp(RooLognormal);
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -82,12 +81,6 @@ double RooLognormal::evaluate() const
    return ROOT::Math::lognormal_pdf(x, ln_m0, ln_k);
 }
 
-void RooLognormal::translate(RooFit::Detail::CodeSquashContext &ctx) const
-{
-   std::string funcName = _useStandardParametrization ? "logNormalEvaluateStandard" : "logNormal";
-   ctx.addResult(this, ctx.buildCall("RooFit::Detail::MathFuncs::" + funcName, x, k, m0));
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Compute multiple values of Lognormal distribution.
 void RooLognormal::doEval(RooFit::EvalContext &ctx) const
@@ -116,13 +109,6 @@ double RooLognormal::analyticalIntegral(Int_t /*code*/, const char *rangeName) c
    return 0.5 * (RooMath::erf(scaledMax / (root2 * ln_k)) - RooMath::erf(scaledMin / (root2 * ln_k)));
 }
 
-std::string RooLognormal::buildCallToAnalyticIntegral(int /*code*/, const char *rangeName,
-                                                      RooFit::Detail::CodeSquashContext &ctx) const
-{
-   std::string funcName = _useStandardParametrization ? "logNormalIntegralStandard" : "logNormalIntegral";
-   return ctx.buildCall("RooFit::Detail::MathFuncs::" + funcName, x.min(rangeName), x.max(rangeName), m0, k);
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 
 Int_t RooLognormal::getGenerator(const RooArgSet &directVars, RooArgSet &generateVars, bool /*staticInitOK*/) const
