@@ -20,20 +20,17 @@
 Flat p.d.f. in N dimensions
 **/
 
-#include "RooAbsReal.h"
 #include "RooArgSet.h"
 #include "RooRealVar.h"
 #include "RooUniform.h"
-#include "RunContext.h"
 
 
-ClassImp(RooUniform);
 
 ////////////////////////////////////////////////////////////////////////////////
 
 RooUniform::RooUniform(const char *name, const char *title, const RooArgSet& _x) :
   RooAbsPdf(name,title),
-  x("x","Observables",this,kTRUE,kFALSE)
+  x("x","Observables",this,true,false)
 {
   x.add(_x) ;
 }
@@ -47,48 +44,26 @@ RooUniform::RooUniform(const RooUniform& other, const char* name) :
 
 ////////////////////////////////////////////////////////////////////////////////
 
-Double_t RooUniform::evaluate() const
+double RooUniform::evaluate() const
 {
   return 1 ;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-///Compute multiple values of the uniform distribution (effectively return a span with ones)
-RooSpan<double> RooUniform::evaluateSpan(RooBatchCompute::RunContext& evalData, const RooArgSet* /*normSet*/) const
-{
-  size_t nEvents = 1;
-  for (auto elm : x) {
-    size_t nEventsCurrent = static_cast<const RooAbsReal*>(elm)->getValues(evalData).size();
-    if(nEventsCurrent != 1 && nEvents != 1 && nEventsCurrent != nEvents) {
-      auto errorMsg = std::string("RooUniform::evaluateSpan(): number of entries for input variables does not match")
-                      + "in RooUniform with name \"" + GetName() + "\".";
-      coutE(FastEvaluations) << errorMsg << std::endl ;
-      throw std::runtime_error(errorMsg);
-    }
-    nEvents = std::max(nEvents, nEventsCurrent);
-  }
-  RooSpan<double> values = evalData.makeBatch(this, nEvents);
-  for (size_t i=0; i<nEvents; i++) {
-    values[i] = 1.0;
-  }
-  return values;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Advertise analytical integral
 
 Int_t RooUniform::getAnalyticalIntegral(RooArgSet& allVars, RooArgSet& analVars, const char* /*rangeName*/) const
 {
-  Int_t nx = x.getSize() ;
+  Int_t nx = x.size() ;
   if (nx>31) {
     // Warn that analytical integration is only provided for the first 31 observables
-    coutW(Integration) << "RooUniform::getAnalyticalIntegral(" << GetName() << ") WARNING: p.d.f. has " << x.getSize()
+    coutW(Integration) << "RooUniform::getAnalyticalIntegral(" << GetName() << ") WARNING: p.d.f. has " << x.size()
              << " observables, analytical integration is only implemented for the first 31 observables" << std::endl ;
     nx=31 ;
   }
 
   Int_t code(0) ;
-  for (int i=0 ; i<x.getSize() ; i++) {
+  for (std::size_t i=0 ; i<x.size() ; i++) {
     if (allVars.find(x.at(i)->GetName())) {
       code |= (1<<i) ;
       analVars.add(*allVars.find(x.at(i)->GetName())) ;
@@ -100,12 +75,12 @@ Int_t RooUniform::getAnalyticalIntegral(RooArgSet& allVars, RooArgSet& analVars,
 ////////////////////////////////////////////////////////////////////////////////
 /// Implement analytical integral
 
-Double_t RooUniform::analyticalIntegral(Int_t code, const char* rangeName) const
+double RooUniform::analyticalIntegral(Int_t code, const char* rangeName) const
 {
-  Double_t ret(1) ;
+  double ret(1) ;
   for (int i=0 ; i<32 ; i++) {
     if (code&(1<<i)) {
-      RooAbsRealLValue* var = (RooAbsRealLValue*)x.at(i) ;
+      RooAbsRealLValue* var = static_cast<RooAbsRealLValue*>(x.at(i)) ;
       ret *= (var->getMax(rangeName) - var->getMin(rangeName)) ;
     }
   }
@@ -115,25 +90,24 @@ Double_t RooUniform::analyticalIntegral(Int_t code, const char* rangeName) const
 ////////////////////////////////////////////////////////////////////////////////
 /// Advertise internal generator
 
-Int_t RooUniform::getGenerator(const RooArgSet& directVars, RooArgSet &generateVars, Bool_t /*staticInitOK*/) const
+Int_t RooUniform::getGenerator(const RooArgSet& directVars, RooArgSet &generateVars, bool /*staticInitOK*/) const
 {
-  Int_t nx = x.getSize() ;
+  Int_t nx = x.size() ;
   if (nx>31) {
     // Warn that analytical integration is only provided for the first 31 observables
-    coutW(Integration) << "RooUniform::getGenerator(" << GetName() << ") WARNING: p.d.f. has " << x.getSize()
+    coutW(Integration) << "RooUniform::getGenerator(" << GetName() << ") WARNING: p.d.f. has " << x.size()
              << " observables, internal integrator is only implemented for the first 31 observables" << std::endl ;
     nx=31 ;
   }
 
   Int_t code(0) ;
-  for (int i=0 ; i<x.getSize() ; i++) {
+  for (std::size_t i=0 ; i<x.size() ; i++) {
     if (directVars.find(x.at(i)->GetName())) {
       code |= (1<<i) ;
       generateVars.add(*directVars.find(x.at(i)->GetName())) ;
     }
   }
   return code ;
-  return 0 ;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -143,13 +117,13 @@ void RooUniform::generateEvent(Int_t code)
 {
   // Fast-track handling of one-observable case
   if (code==1) {
-    ((RooAbsRealLValue*)x.at(0))->randomize() ;
+    (static_cast<RooAbsRealLValue*>(x.at(0)))->randomize() ;
     return ;
   }
 
   for (int i=0 ; i<32 ; i++) {
     if (code&(1<<i)) {
-      RooAbsRealLValue* var = (RooAbsRealLValue*)x.at(i) ;
+      RooAbsRealLValue* var = static_cast<RooAbsRealLValue*>(x.at(i)) ;
       var->randomize() ;
     }
   }
