@@ -1,3 +1,5 @@
+/// \cond ROOFIT_INTERNAL
+
 /*
  * Project: RooFit
  * Authors:
@@ -19,6 +21,8 @@ std::unique_ptr<RooAbsReal> createConstraintTerm(std::string const &name, RooAbs
                                                  RooArgSet const *constrainedParameters,
                                                  RooArgSet const *externalConstraints,
                                                  RooArgSet const *globalObservables, const char *globalObservablesTag,
-                                                 bool takeGlobalObservablesFromData, bool removeConstraintsFromPdf);
+                                                 bool takeGlobalObservablesFromData);
 
 #endif
+
+/// \endcond
