@@ -1,3 +1,5 @@
+/// \cond ROOFIT_INTERNAL
+
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
@@ -56,9 +58,6 @@ combined in the main thread.
 
 using std::endl, std::ostream;
 
-ClassImp(RooAbsTestStatistic);
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Create a test statistic from the given function and the data.
 /// \param[in] name Name of the test statistic
@@ -310,7 +309,7 @@ bool RooAbsTestStatistic::redirectServersHook(const RooAbsCollection& newServerL
     for (Int_t i = 0; i < _nCPU; ++i) {
       if (_mpfeArray[i]) {
    _mpfeArray[i]->recursiveRedirectServers(newServerList,mustReplaceAll,nameChange);
-//    cout << "redirecting servers on " << _mpfeArray[i]->GetName() << endl;
+//    std::cout << "redirecting servers on " << _mpfeArray[i]->GetName() << std::endl;
       }
     }
   }
@@ -327,13 +326,13 @@ void RooAbsTestStatistic::printCompactTreeHook(ostream& os, const char* indent)
 {
   if (SimMaster == _gofOpMode) {
     // Forward to slaves
-    os << indent << "RooAbsTestStatistic begin GOF contents" << endl ;
+    os << indent << "RooAbsTestStatistic begin GOF contents" << std::endl ;
     for (std::size_t i = 0; i < _gofArray.size(); ++i) {
       TString indent2(indent);
       indent2 += "[" + std::to_string(i) + "] ";
       _gofArray[i]->printCompactTreeHook(os,indent2);
     }
-    os << indent << "RooAbsTestStatistic end GOF contents" << endl;
+    os << indent << "RooAbsTestStatistic end GOF contents" << std::endl;
   } else if (MPMaster == _gofOpMode) {
     // WVE implement this
   }
@@ -418,7 +417,7 @@ void RooAbsTestStatistic::initMPMode(RooAbsReal* real, RooAbsData* data, const R
     gof->SetName(Form("%s_GOF%d",GetName(),i));
     gof->SetTitle(Form("%s_GOF%d",GetTitle(),i));
 
-    ccoutD(Eval) << "RooAbsTestStatistic::initMPMode: starting remote server process #" << i << endl;
+    ccoutD(Eval) << "RooAbsTestStatistic::initMPMode: starting remote server process #" << i << std::endl;
     _mpfeArray[i] = new RooRealMPFE(Form("%s_%zx_MPFE%d",GetName(),reinterpret_cast<size_t>(this),i),Form("%s_%zx_MPFE%d",GetTitle(),reinterpret_cast<size_t>(this),i),*gof,false);
     //_mpfeArray[i]->setVerbose(true,true);
     _mpfeArray[i]->initialize();
@@ -427,8 +426,8 @@ void RooAbsTestStatistic::initMPMode(RooAbsReal* real, RooAbsData* data, const R
     }
   }
   _mpfeArray[_nCPU - 1]->addOwnedComponents(*gof);
-  coutI(Eval) << "RooAbsTestStatistic::initMPMode: started " << _nCPU << " remote server process." << endl;
-  //cout << "initMPMode --- done" << endl ;
+  coutI(Eval) << "RooAbsTestStatistic::initMPMode: started " << _nCPU << " remote server process." << std::endl;
+  //cout << "initMPMode --- done" << std::endl ;
   return ;
 }
 
@@ -447,7 +446,7 @@ void RooAbsTestStatistic::initSimMode(RooSimultaneous* simpdf, RooAbsData* data,
 
   RooAbsCategoryLValue& simCat = const_cast<RooAbsCategoryLValue&>(simpdf->indexCat());
 
-  std::unique_ptr<TList> dsetList{const_cast<RooAbsData*>(data)->split(*simpdf,processEmptyDataSets())};
+  std::vector<std::unique_ptr<RooAbsData>> dsetList{const_cast<RooAbsData*>(data)->split(*simpdf,processEmptyDataSets())};
 
   // Create array of regular fit contexts, containing subset of data and single fitCat PDF
   for (const auto& catState : simCat) {
@@ -467,11 +466,14 @@ void RooAbsTestStatistic::initSimMode(RooSimultaneous* simpdf, RooAbsData* data,
 
     // Retrieve the PDF for this simCat state
     RooAbsPdf* pdf = simpdf->getPdf(catName.c_str());
-    RooAbsData* dset = static_cast<RooAbsData*>(dsetList->FindObject(catName.c_str()));
+    auto found = std::find_if(dsetList.begin(), dsetList.end(), [&](auto const &item) {
+      return catName == item->GetName();
+    });
+    RooAbsData *dset = found != dsetList.end() ? found->get() : nullptr;
 
     if (pdf && dset && (0. != dset->sumEntries() || processEmptyDataSets())) {
       ccoutI(Fitting) << "RooAbsTestStatistic::initSimMode: creating slave calculator #" << _gofArray.size() << " for state " << catName
-          << " (" << dset->numEntries() << " dataset entries)" << endl;
+          << " (" << dset->numEntries() << " dataset entries)" << std::endl;
 
 
       // *** START HERE
@@ -517,9 +519,7 @@ void RooAbsTestStatistic::initSimMode(RooSimultaneous* simpdf, RooAbsData* data,
   for(auto& gof : _gofArray) {
     gof->setSimCount(_gofArray.size());
   }
-  coutI(Fitting) << "RooAbsTestStatistic::initSimMode: created " << _gofArray.size() << " slave calculators." << endl;
-
-  dsetList->Delete(); // delete the content.
+  coutI(Fitting) << "RooAbsTestStatistic::initSimMode: created " << _gofArray.size() << " slave calculators." << std::endl;
 }
 
 
@@ -553,24 +553,24 @@ bool RooAbsTestStatistic::setData(RooAbsData& indata, bool cloneData)
         gof->setDataSlave(indata, cloneData);
       }
     } else {
-      std::unique_ptr<TList> dlist{indata.split(*static_cast<RooSimultaneous*>(_func), processEmptyDataSets())};
-      if (!dlist) {
-        coutE(Fitting) << "RooAbsTestStatistic::initSimMode(" << GetName() << ") ERROR: index category of simultaneous pdf is missing in dataset, aborting" << endl;
-        throw std::runtime_error("RooAbsTestStatistic::initSimMode() ERROR, index category of simultaneous pdf is missing in dataset, aborting");
-      }
+      std::vector<std::unique_ptr<RooAbsData>> dlist{indata.split(*static_cast<RooSimultaneous*>(_func), processEmptyDataSets())};
 
       for(auto& gof : _gofArray) {
-        if (auto compData = static_cast<RooAbsData*>(dlist->FindObject(gof->GetName()))) {
+        auto found = std::find_if(dlist.begin(), dlist.end(), [&](auto const &item) {
+          return strcmp(gof->GetName(), item->GetName()) == 0;
+        });
+        RooAbsData *compData = found != dlist.end() ? found->get() : nullptr;
+        if (compData) {
           gof->setDataSlave(*compData,false,true);
         } else {
-          coutE(DataHandling) << "RooAbsTestStatistic::setData(" << GetName() << ") ERROR: Cannot find component data for state " << gof->GetName() << endl;
+          coutE(DataHandling) << "RooAbsTestStatistic::setData(" << GetName() << ") ERROR: Cannot find component data for state " << gof->GetName() << std::endl;
         }
       }
     }
     break;
   case MPMaster:
     // Not supported
-    coutF(DataHandling) << "RooAbsTestStatistic::setData(" << GetName() << ") FATAL: setData() is not supported in multi-processor mode" << endl;
+    coutF(DataHandling) << "RooAbsTestStatistic::setData(" << GetName() << ") FATAL: setData() is not supported in multi-processor mode" << std::endl;
     throw std::runtime_error("RooAbsTestStatistic::setData is not supported in MPMaster mode");
     break;
   }
@@ -614,3 +614,5 @@ void RooAbsTestStatistic::enableOffsetting(bool flag)
 
 double RooAbsTestStatistic::getCarry() const
 { return _evalCarry; }
+
+/// \endcond
