@@ -107,7 +107,7 @@ RooAdaptiveIntegratorND::~RooAdaptiveIntegratorND()
 {
   delete _integrator ;
   if (_nError>_nWarn) {
-    oocoutW(nullptr, NumIntegration) << "RooAdaptiveIntegratorND::dtor(" << _intName
+    oocoutW(nullptr, NumericIntegration) << "RooAdaptiveIntegratorND::dtor(" << _intName
            << ") WARNING: Number of suppressed warningings about integral evaluations where target precision was not reached is " << _nError-_nWarn << std::endl;
   }
 
@@ -168,11 +168,11 @@ double RooAdaptiveIntegratorND::integral(const double* /*yvec*/)
   if (_integrator->Status()==1) {
     _nError++ ;
     if (_nError<=_nWarn) {
-      oocoutW(nullptr, NumIntegration) << "RooAdaptiveIntegratorND::integral(" << integrand()->getName() << ") WARNING: target rel. precision not reached due to nEval limit of "
+      oocoutW(nullptr, NumericIntegration) << "RooAdaptiveIntegratorND::integral(" << integrand()->getName() << ") WARNING: target rel. precision not reached due to nEval limit of "
              << _nmax << ", estimated rel. precision is " << Form("%3.1e",_integrator->RelError()) << std::endl ;
     }
     if (_nError==_nWarn) {
-      oocoutW(nullptr, NumIntegration) << "RooAdaptiveIntegratorND::integral(" << integrand()->getName()
+      oocoutW(nullptr, NumericIntegration) << "RooAdaptiveIntegratorND::integral(" << integrand()->getName()
              << ") Further warnings on target precision are suppressed conform specification in integrator specification" << std::endl ;
     }
   }
