@@ -45,7 +45,7 @@ RooEffProd::RooEffProd(const char *name, const char *title,
 RooEffProd::RooEffProd(const RooEffProd& other, const char* name) :
   RooAbsPdf(other, name),
   _pdf("pdf",this,other._pdf),
-  _eff("acc",this,other._eff)
+  _eff("eff",this,other._eff)
 {
 }
 
