@@ -18,7 +18,9 @@
 
 #include <RooGlobalFunc.h>
 
+#include <RooAbsData.h>
 #include <RooAbsPdf.h>
+#include <RooBatchCompute.h>
 #include <RooCategory.h>
 #include <RooDataHist.h>
 #include <RooDataSet.h>
@@ -30,7 +32,8 @@
 #include <RooRealConstant.h>
 #include <RooRealVar.h>
 
-#include <TColor.h>
+#include "RooEvaluatorWrapper.h"
+
 #include <TH1.h>
 #include <TInterpreter.h>
 
@@ -38,6 +41,23 @@
 
 namespace RooFit {
 
+/// Get the global choice for the RooBatchCompute library that RooFit will load.
+/// \see RooFit::setBatchCompute().
+std::string getBatchCompute()
+{
+   return RooBatchCompute::getBatchComputeChoice();
+}
+
+/// Globally select the RooBatchCompute CPU implementation that will be loaded
+/// in RooFit.
+/// Supported options are "auto" (default), "avx512", "avx2", "avx", "sse", "generic".
+/// \note It is not possible to change the selection after RooFit has already
+/// loaded a library (which is usually triggered by likelihood creation or fitting).
+void setBatchCompute(std::string const &value)
+{
+   return RooBatchCompute::setBatchComputeChoice(value);
+}
+
 // anonymous namespace for helper functions for the implementation of the global functions
 namespace {
 
@@ -85,19 +105,6 @@ int interpretString(std::string const &s)
    return gInterpreter->ProcessLine(s.c_str());
 }
 
-Color_t interpretColorString(std::string const &color)
-{
-   using Map = std::unordered_map<std::string, Color_t>;
-   // Color dictionary to define matplotlib conventions
-   static Map colorMap{{"r", kRed},   {"b", kBlue},  {"g", kGreen},   {"y", kYellow},
-                       {"w", kWhite}, {"k", kBlack}, {"m", kMagenta}, {"c", kCyan}};
-   auto found = colorMap.find(color);
-   if (found != colorMap.end())
-      return found->second;
-   // Lookup color from static color map otherwise
-   return TColor::GetColorByName(color.c_str());
-}
-
 Style_t interpretLineStyleString(std::string const &style)
 {
    using Map = std::unordered_map<std::string, Style_t>;
@@ -124,6 +131,13 @@ RooCmdArg ParallelDescentOptions(bool enable, int splitStrategy, int numSplits)
    return RooCmdArg("ParallelDescentOptions", enable, numSplits, splitStrategy, 0, nullptr, nullptr, nullptr, nullptr);
 }
 
+void writeCodegenDebugMacro(RooAbsReal const &absReal, std::string const &name)
+{
+   if (auto *wrapper = dynamic_cast<RooEvaluatorWrapper const *>(&absReal)) {
+      wrapper->writeDebugMacro(name);
+   }
+}
+
 } // namespace Experimental
 
 // RooAbsReal::plotOn arguments
@@ -212,30 +226,9 @@ RooCmdArg VLines()
 {
    return RooCmdArg("VLines", 1, 0, 0, 0, nullptr, nullptr, nullptr, nullptr);
 }
-RooCmdArg LineColor(Color_t color)
+RooCmdArg LineColor(TColorNumber color)
 {
-   return RooCmdArg("LineColor", color);
-}
-
-/// The `color` string argument argument will be evaluated with the ROOT
-/// interpreter to get the actual ROOT color enum value, like `ROOT.kRed`.
-/// Here is what you can do with it:
-///
-///   1. Pass a string with the enum value name instead, e.g.:
-/// ~~~ {.cxx}
-/// pdf.plotOn(frame, LineColor("kRed"))
-/// ~~~
-///   2. Pass a string with the corresponding single-character color code following the matplotlib convention:
-/// ~~~ {.cxx}
-/// pdf.plotOn(frame, LineColor("r"))
-/// ~~~
-///   3. Pass a string with the enum value name instead followed by some manipulation of the enum value:
-/// ~~~ {.cxx}
-/// pdf.plotOn(frame, LineColor("kRed+1"))
-/// ~~~
-RooCmdArg LineColor(std::string const &color)
-{
-   return LineColor(interpretColorString(color));
+   return RooCmdArg("LineColor", color.number());
 }
 RooCmdArg LineStyle(Style_t style)
 {
@@ -249,13 +242,9 @@ RooCmdArg LineWidth(Width_t width)
 {
    return RooCmdArg("LineWidth", width);
 }
-RooCmdArg FillColor(Color_t color)
+RooCmdArg FillColor(TColorNumber color)
 {
-   return RooCmdArg("FillColor", color);
-}
-RooCmdArg FillColor(std::string const &color)
-{
-   return RooCmdArg("FillColor", interpretColorString(color));
+   return RooCmdArg("FillColor", color.number());
 }
 RooCmdArg FillStyle(Style_t style)
 {
@@ -349,13 +338,9 @@ RooCmdArg MarkerSize(Size_t size)
 {
    return RooCmdArg("MarkerSize", 0, 0, size, 0, nullptr, nullptr, nullptr, nullptr);
 }
-RooCmdArg MarkerColor(Color_t color)
+RooCmdArg MarkerColor(TColorNumber color)
 {
-   return RooCmdArg("MarkerColor", color, 0, 0, 0, nullptr, nullptr, nullptr, nullptr);
-}
-RooCmdArg MarkerColor(std::string const &color)
-{
-   return MarkerColor(interpretColorString(color));
+   return RooCmdArg("MarkerColor", color.number());
 }
 RooCmdArg CutRange(const char *rangeName)
 {
@@ -469,7 +454,11 @@ RooCmdArg Extended(bool flag)
 }
 RooCmdArg DataError(Int_t etype)
 {
-   return RooCmdArg("DataError", (Int_t)etype, 0, 0, 0, nullptr, nullptr, nullptr, nullptr);
+   return RooCmdArg("DataError", etype);
+}
+RooCmdArg DataError(std::string const &etype)
+{
+   return DataError(RooAbsData::errorTypeFromString(etype));
 }
 RooCmdArg NumCPU(Int_t nCPU, Int_t interleave)
 {
@@ -571,7 +560,16 @@ RooCmdArg EventRange(Int_t nStart, Int_t nStop)
 
 // RooAbsPdf::fitTo arguments
 
-EvalBackend::EvalBackend(EvalBackend::Value value) : RooCmdArg{"EvalBackend", static_cast<int>(value)} {}
+EvalBackend::EvalBackend(EvalBackend::Value value) : RooCmdArg{"EvalBackend", static_cast<int>(value)}
+{
+#ifndef ROOFIT_CLAD
+   if (value == Value::Codegen || value == Value::CodegenNoGrad) {
+      oocoutE(nullptr, InputArguments)
+         << "RooFit was built without clad. Codegen backends are unavailable. Falling back to default.\n";
+      setInt(0, static_cast<int>(defaultValue()));
+   }
+#endif
+}
 EvalBackend::EvalBackend(std::string const &name) : EvalBackend{toValue(name)} {}
 EvalBackend::Value EvalBackend::toValue(std::string const &name)
 {
@@ -1022,9 +1020,9 @@ RooCmdArg Prefix(bool flag)
 {
    return RooCmdArg("Prefix", flag, 0, 0, 0, nullptr, nullptr, nullptr, nullptr);
 }
-RooCmdArg Color(Color_t color)
+RooCmdArg Color(TColorNumber color)
 {
-   return RooCmdArg("Color", color, 0, 0, 0, nullptr, nullptr, nullptr, nullptr);
+   return RooCmdArg("Color", color.number());
 }
 
 // RooWorkspace::import() arguments
