@@ -1,3 +1,5 @@
+/// \cond ROOFIT_INTERNAL
+
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
@@ -27,6 +29,7 @@
 #include "RooMsgService.h"
 #include "RooMinimizer.h"
 #include "RooNaNPacker.h"
+#include "RooCategory.h"
 
 #include "Math/Functor.h"
 #include "TMatrixDSym.h"
@@ -34,17 +37,19 @@
 #include <fstream>
 #include <iomanip>
 
-using std::cout, std::endl, std::setprecision;
+using std::setprecision;
 
 namespace {
 
 // Helper function that wraps RooAbsArg::getParameters and directly returns the
 // output RooArgSet. To be used in the initializer list of the RooMinimizerFcn
-// constructor.
-RooArgSet getParameters(RooAbsReal const &funct)
+// constructor. In the case of figuring out all parameters for the minimizer,
+// we don't want to strip disconnected parameters, becuase which parameters are
+// disconnected can change between minimization runs.
+RooArgSet getAllParameters(RooAbsReal const &funct)
 {
    RooArgSet out;
-   funct.getParameters(nullptr, out);
+   funct.getParameters(nullptr, out, /*stripDisconnected*/ false);
    return out;
 }
 
@@ -52,7 +57,7 @@ RooArgSet getParameters(RooAbsReal const &funct)
 
 // use reference wrapper for the Functor, such that the functor points to this RooMinimizerFcn by reference.
 RooMinimizerFcn::RooMinimizerFcn(RooAbsReal *funct, RooMinimizer *context)
-   : RooAbsMinimizerFcn(getParameters(*funct), context), _funct(funct)
+   : RooAbsMinimizerFcn(getAllParameters(*funct), context), _funct(funct)
 {
    unsigned int nDim = getNDim();
 
@@ -89,11 +94,11 @@ double RooMinimizerFcn::operator()(const double *x) const
 
    // Optional logging
    if (_logfile)
-      (*_logfile) << setprecision(15) << fvalue << setprecision(4) << endl;
+      (*_logfile) << setprecision(15) << fvalue << setprecision(4) << std::endl;
    if (cfg().verbose) {
-      cout << "\nprevFCN" << (_funct->isOffsetting() ? "-offset" : "") << " = " << setprecision(10) << fvalue
-           << setprecision(4) << "  ";
-      cout.flush();
+      std::cout << "\nprevFCN" << (_funct->isOffsetting() ? "-offset" : "") << " = " << setprecision(10) << fvalue
+                << setprecision(4) << "  ";
+      std::cout.flush();
    }
 
    finishDoEval();
@@ -145,3 +150,38 @@ void RooMinimizerFcn::setOffsetting(bool flag)
 {
    _funct->enableOffsetting(flag);
 }
+
+RooArgSet RooMinimizerFcn::freezeDisconnectedParameters() const
+{
+
+   RooArgSet paramsDisconnected;
+   RooArgSet paramsConnected;
+
+   _funct->getParameters(nullptr, paramsDisconnected, /*stripDisconnected*/ false);
+   _funct->getParameters(nullptr, paramsConnected, /*stripDisconnected*/ true);
+
+   paramsDisconnected.remove(paramsConnected, true, true);
+
+   RooArgSet changedSet;
+
+   for (RooAbsArg *a : paramsDisconnected) {
+      auto *v = dynamic_cast<RooRealVar *>(a);
+      auto *cv = dynamic_cast<RooCategory *>(a);
+      if (v && !v->isConstant()) {
+         v->setConstant();
+         changedSet.add(*v);
+      } else if (cv && !cv->isConstant()) {
+         cv->setConstant();
+         changedSet.add(*cv);
+      }
+   }
+
+   return changedSet;
+}
+
+void RooMinimizerFcn::initMinimizer(ROOT::Math::Minimizer &minim)
+{
+   minim.SetFunction(*_multiGenFcn);
+}
+
+/// \endcond
