@@ -14,7 +14,7 @@
 \class RooProfileLL
 \ingroup Roofitcore
 
-Class RooProfileLL implements the profile likelihood estimator for
+Implements the profile likelihood estimator for
 a given likelihood and set of parameters of interest. The value return by
 RooProfileLL is the input likelihood nll minimized w.r.t all nuisance parameters
 (which are all parameters except for those listed in the constructor) minus
@@ -30,22 +30,6 @@ as a MIGRAD minimization step is executed for each function evaluation
 #include "RooMsgService.h"
 #include "RooRealVar.h"
 
-using namespace std ;
-
-ClassImp(RooProfileLL);
-
-
-////////////////////////////////////////////////////////////////////////////////
-/// Default constructor
-/// Should only be used by proof.
-
- RooProfileLL::RooProfileLL() :
-   RooAbsReal("RooProfileLL","RooProfileLL"),
-   _nll(),
-   _obs("paramOfInterest","Parameters of interest",this),
-   _par("nuisanceParam","Nuisance parameters",this,false,false)
-{
-}
 
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -71,16 +55,13 @@ RooProfileLL::RooProfileLL(const char *name, const char *title,
 ////////////////////////////////////////////////////////////////////////////////
 /// Copy constructor
 
-RooProfileLL::RooProfileLL(const RooProfileLL& other, const char* name) :
-  RooAbsReal(other,name),
-  _nll("nll",this,other._nll),
-  _obs("obs",this,other._obs),
-  _par("par",this,other._par),
-  _startFromMin(other._startFromMin),
-  _absMinValid(false),
-  _absMin(0),
-  _paramFixed(other._paramFixed),
-  _neval(0)
+RooProfileLL::RooProfileLL(const RooProfileLL &other, const char *name)
+   : RooAbsReal(other, name),
+     _nll("input", this, other._nll),
+     _obs(this, other._obs),
+     _par(this, other._par),
+     _startFromMin(other._startFromMin),
+     _paramFixed(other._paramFixed)
 {
   _paramAbsMin.addClone(other._paramAbsMin) ;
   _obsAbsMin.addClone(other._obsAbsMin) ;
@@ -125,7 +106,7 @@ RooFit::OwningPtr<RooAbsReal> RooProfileLL::createProfile(const RooArgSet& param
 
 void RooProfileLL::initializeMinimizer() const
 {
-  coutI(Minimization) << "RooProfileLL::evaluate(" << GetName() << ") Creating instance of MINUIT" << endl ;
+  coutI(Minimization) << "RooProfileLL::evaluate(" << GetName() << ") Creating instance of MINUIT" << std::endl;
 
   bool smode = RooMsgService::instance().silentMode() ;
   RooMsgService::instance().setSilentMode(true) ;
@@ -196,7 +177,7 @@ void RooProfileLL::validateAbsMin() const
       if (_paramFixed[par->GetName()] != par->isConstant()) {
    cxcoutI(Minimization) << "RooProfileLL::evaluate(" << GetName() << ") constant status of parameter " << par->GetName() << " has changed from "
             << (_paramFixed[par->GetName()]?"fixed":"floating") << " to " << (par->isConstant()?"fixed":"floating")
-            << ", recalculating absolute minimum" << endl ;
+            << ", recalculating absolute minimum" << std::endl;
    _absMinValid = false ;
    break ;
       }
@@ -207,7 +188,7 @@ void RooProfileLL::validateAbsMin() const
   // If we don't have the absolute minimum w.r.t all observables, calculate that first
   if (!_absMinValid) {
 
-    cxcoutI(Minimization) << "RooProfileLL::evaluate(" << GetName() << ") determining minimum likelihood for current configurations w.r.t all observable" << endl ;
+    cxcoutI(Minimization) << "RooProfileLL::evaluate(" << GetName() << ") determining minimum likelihood for current configurations w.r.t all observable" << std::endl;
 
 
     if (!_minimizer) {
@@ -219,10 +200,10 @@ void RooProfileLL::validateAbsMin() const
     _obs.snapshot(obsStart, false) ;
 
     // Start from previous global minimum
-    if (_paramAbsMin.getSize()>0) {
+    if (!_paramAbsMin.empty()) {
       const_cast<RooSetProxy&>(_par).assignValueOnly(_paramAbsMin) ;
     }
-    if (_obsAbsMin.getSize()>0) {
+    if (!_obsAbsMin.empty()) {
       const_cast<RooSetProxy&>(_obs).assignValueOnly(_obsAbsMin) ;
     }
 
@@ -239,7 +220,7 @@ void RooProfileLL::validateAbsMin() const
     _paramAbsMin.removeAll() ;
 
     // Only store non-constant parameters here!
-    _paramAbsMin.addClone(*std::unique_ptr<RooArgSet>{static_cast<RooArgSet*>(_par.selectByAttrib("Constant",false))});
+    _paramAbsMin.addClone(*std::unique_ptr<RooArgSet>{_par.selectByAttrib("Constant", false)});
 
     _obsAbsMin.addClone(_obs) ;
 
@@ -257,7 +238,7 @@ void RooProfileLL::validateAbsMin() const
                                << static_cast<RooAbsReal const*>(arg)->getVal() ;
         first=false ;
       }
-      ccxcoutI(Minimization) << ")" << endl ;
+      ccxcoutI(Minimization) << ")" << std::endl;
     }
 
     // Restore original parameter values
@@ -276,5 +257,3 @@ bool RooProfileLL::redirectServersHook(const RooAbsCollection& newServerList, bo
   _minimizer.reset();
   return RooAbsReal::redirectServersHook(newServerList, mustReplaceAll, nameChange, isRecursive);
 }
-
-
