@@ -1,3 +1,5 @@
+/// \cond ROOFIT_INTERNAL
+
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
@@ -49,10 +51,6 @@ Object          | function
 
 #include <cmath>
 
-ClassImp(RooUnitTest);
-
-using namespace std;
-
 TDirectory* RooUnitTest::gMemDir = nullptr;
 
 
@@ -64,20 +62,12 @@ RooUnitTest::RooUnitTest(const char* name, TFile* refFile, bool writeRef, Int_t
 }
 
 
-
-////////////////////////////////////////////////////////////////////////////////
-
-RooUnitTest::~RooUnitTest()
-{
-}
-
-
 ////////////////////////////////////////////////////////////////////////////////
 
 void RooUnitTest::regPlot(RooPlot* frame, const char* refName)
 {
   if (_refFile) {
-    string refNameStr(refName) ;
+    std::string refNameStr(refName) ;
     frame->SetName(refName) ;
     // Since ROOT 6.28, the RooPlot doesn't clone the plot variable by default
     // anymore. This is a problem for registering the RooPlots, because they
@@ -97,8 +87,7 @@ void RooUnitTest::regPlot(RooPlot* frame, const char* refName)
 void RooUnitTest::regResult(std::unique_ptr<RooFitResult> r, const char* refName)
 {
   if (_refFile) {
-    string refNameStr(refName) ;
-    _regResults.push_back(make_pair(r.release(),refNameStr)) ;
+    _regResults.emplace_back(r.release(),refName);
   }
 }
 
@@ -108,8 +97,7 @@ void RooUnitTest::regResult(std::unique_ptr<RooFitResult> r, const char* refName
 void RooUnitTest::regValue(double d, const char* refName)
 {
   if (_refFile) {
-    string refNameStr(refName) ;
-    _regValues.push_back(make_pair(d,refNameStr)) ;
+    _regValues.emplace_back(d,refName);
   }
 }
 
@@ -119,8 +107,7 @@ void RooUnitTest::regValue(double d, const char* refName)
 void RooUnitTest::regTable(RooTable* t, const char* refName)
 {
   if (_refFile) {
-    string refNameStr(refName) ;
-    _regTables.push_back(make_pair(t,refNameStr)) ;
+    _regTables.emplace_back(t,refName) ;
   } else {
     delete t ;
   }
@@ -132,8 +119,7 @@ void RooUnitTest::regTable(RooTable* t, const char* refName)
 void RooUnitTest::regWS(RooWorkspace* ws, const char* refName)
 {
   if (_refFile) {
-    string refNameStr(refName) ;
-    _regWS.push_back(make_pair(ws,refNameStr)) ;
+    _regWS.emplace_back(ws,refName) ;
   } else {
     delete ws ;
   }
@@ -145,8 +131,7 @@ void RooUnitTest::regWS(RooWorkspace* ws, const char* refName)
 void RooUnitTest::regTH(TH1* th, const char* refName)
 {
   if (_refFile) {
-    string refNameStr(refName) ;
-    _regTH.push_back(make_pair(th,refNameStr)) ;
+    _regTH.emplace_back(th,refName) ;
   } else {
     delete th ;
   }
@@ -160,8 +145,8 @@ RooWorkspace* RooUnitTest::getWS(const char* refName)
   RooWorkspace* ws = dynamic_cast<RooWorkspace*>(_refFile->Get(refName)) ;
   if (!ws) {
     if(_verb >= 0) std::cout << "RooUnitTest ERROR: cannot retrieve RooWorkspace " << refName
-                             << " from reference file, skipping " << endl ;
-    return 0 ;
+                             << " from reference file, skipping " << std::endl ;
+    return nullptr ;
   }
 
   return ws ;
@@ -202,7 +187,7 @@ bool RooUnitTest::areTHidentical(TH1* htest, TH1* href)
 
     for (Int_t i=0 ; i<ntest ; i++) {
       if (std::abs(htest->GetBinContent(i)-href->GetBinContent(i))>htol()) {
-        if(_verb >= 0) std::cout << "htest[" << i << "] = " << htest->GetBinContent(i) << " href[" << i << "] = " << href->GetBinContent(i) << endl;
+        if(_verb >= 0) std::cout << "htest[" << i << "] = " << htest->GetBinContent(i) << " href[" << i << "] = " << href->GetBinContent(i) << std::endl;
       }
     }
 
@@ -220,7 +205,7 @@ bool RooUnitTest::runCompTests()
 {
   bool ret = true ;
 
-  list<pair<RooPlot*, string> >::iterator iter = _regPlots.begin() ;
+  auto iter = _regPlots.begin() ;
   while (iter!=_regPlots.end()) {
 
     if (!_write) {
@@ -238,12 +223,12 @@ bool RooUnitTest::runCompTests()
       }
 
       if (_verb > 0) {
-   cout << "comparing RooPlot " << iter->first << " to benchmark " << iter->second << " = " << bmark << endl ;
-   cout << "reference: " ; iter->first->Print() ;
-   cout << "benchmark: " ; bmark->Print() ;
+   std::cout << "comparing RooPlot " << iter->first << " to benchmark " << iter->second << " = " << bmark << std::endl ;
+   std::cout << "reference: " ; iter->first->Print() ;
+   std::cout << "benchmark: " ; bmark->Print() ;
       }
 
-      RooPlot* compPlot = _debug ? iter->first->emptyClone(Form("%s_comparison",iter->first->GetName())) : 0 ;
+      RooPlot* compPlot = _debug ? iter->first->emptyClone(Form("%s_comparison",iter->first->GetName())) : nullptr ;
       bool anyFail=false ;
 
       Stat_t nItems = iter->first->numItems() ;
@@ -270,12 +255,12 @@ bool RooUnitTest::runCompTests()
                                  <<   " fails comparison with counterpart in reference RooPlot " << bmark->GetName() << std::endl;
 
        if (compPlot) {
-         compPlot->addPlotable((RooHist*)testHist->Clone(),"P") ;
+         compPlot->addPlotable(static_cast<RooHist*>(testHist->Clone()),"P") ;
          compPlot->getAttLine()->SetLineColor(kRed) ;
          compPlot->getAttMarker()->SetMarkerColor(kRed) ;
          compPlot->getAttLine()->SetLineWidth(1) ;
 
-         compPlot->addPlotable((RooHist*)refHist->Clone(),"P") ;
+         compPlot->addPlotable(static_cast<RooHist*>(refHist->Clone()),"P") ;
          compPlot->getAttLine()->SetLineColor(kBlue) ;
          compPlot->getAttMarker()->SetMarkerColor(kBlue) ;
          compPlot->getAttLine()->SetLineWidth(1) ;
@@ -292,12 +277,12 @@ bool RooUnitTest::runCompTests()
                                  <<   " fails comparison with counterpart in reference RooPlot " << bmark->GetName() << std::endl;
 
        if (compPlot) {
-         compPlot->addPlotable((RooCurve*)testCurve->Clone()) ;
+         compPlot->addPlotable(static_cast<RooCurve*>(testCurve->Clone())) ;
          compPlot->getAttLine()->SetLineColor(kRed) ;
          compPlot->getAttLine()->SetLineWidth(1) ;
          compPlot->getAttLine()->SetLineStyle(kSolid) ;
 
-         compPlot->addPlotable((RooCurve*)refCurve->Clone()) ;
+         compPlot->addPlotable(static_cast<RooCurve*>(refCurve->Clone())) ;
          compPlot->getAttLine()->SetLineColor(kBlue) ;
          compPlot->getAttLine()->SetLineWidth(1) ;
          compPlot->getAttLine()->SetLineStyle(kDashed) ;
@@ -312,7 +297,7 @@ bool RooUnitTest::runCompTests()
       }
 
       if (anyFail && compPlot) {
-   cout << "RooUnitTest INFO: writing comparison plot " << compPlot->GetName() << " of failed test to RooUnitTest_DEBUG.root" << endl ;
+   std::cout << "RooUnitTest INFO: writing comparison plot " << compPlot->GetName() << " of failed test to RooUnitTest_DEBUG.root" << std::endl ;
    TFile fdbg("RooUnitTest_DEBUG.root","UPDATE") ;
    compPlot->Write() ;
    fdbg.Close() ;
@@ -327,7 +312,7 @@ bool RooUnitTest::runCompTests()
 
       // Writing mode
 
-      std::cout <<"RooUnitTest: Writing reference RooPlot " << iter->first << " as benchmark " << iter->second << endl ;
+      std::cout <<"RooUnitTest: Writing reference RooPlot " << iter->first << " as benchmark " << iter->second << std::endl ;
       _refFile->cd() ;
       iter->first->Write(iter->second.c_str()) ;
       gMemDir->cd() ;
@@ -337,7 +322,7 @@ bool RooUnitTest::runCompTests()
   }
 
 
-  list<pair<RooFitResult*, string> >::iterator iter2 = _regResults.begin() ;
+  auto iter2 = _regResults.begin() ;
   while (iter2!=_regResults.end()) {
 
     if (!_write) {
@@ -355,13 +340,15 @@ bool RooUnitTest::runCompTests()
       }
 
       if (_verb > 0) {
-   cout << "comparing RooFitResult " << iter2->first << " to benchmark " << iter2->second << " = " << bmark << endl ;
+   std::cout << "comparing RooFitResult " << iter2->first << " to benchmark " << iter2->second << " = " << bmark << std::endl ;
       }
 
       if (!iter2->first->isIdentical(*bmark,fptol(),fctol(),_verb >= 0)) {
-        if(_verb >= 0) cout << "RooUnitTest ERROR: comparison of object " << iter2->first->ClassName() << "::" << iter2->first->GetName()
-             << " from result " << iter2->second
-             <<   " fails comparison with counterpart in reference RooFitResult " << bmark->GetName() << std::endl;
+   if (_verb >= 0) {
+     std::cout << "RooUnitTest ERROR: comparison of object " << iter2->first->ClassName()
+               << "::" << iter2->first->GetName() << " from result " << iter2->second
+               << " fails comparison with counterpart in reference RooFitResult " << bmark->GetName() << std::endl;
+   }
         ret = false ;
       }
 
@@ -373,7 +360,7 @@ bool RooUnitTest::runCompTests()
 
       // Writing mode
 
-      std::cout <<"RooUnitTest: Writing reference RooFitResult " << iter2->first << " as benchmark " << iter2->second << endl ;
+      std::cout <<"RooUnitTest: Writing reference RooFitResult " << iter2->first << " as benchmark " << iter2->second << std::endl ;
       _refFile->cd() ;
       iter2->first->Write(iter2->second.c_str()) ;
       gMemDir->cd() ;
@@ -382,7 +369,7 @@ bool RooUnitTest::runCompTests()
     ++iter2 ;
   }
 
-  list<pair<double, string> >::iterator iter3 = _regValues.begin() ;
+  auto iter3 = _regValues.begin() ;
   while (iter3!=_regValues.end()) {
 
     if (!_write) {
@@ -399,11 +386,11 @@ bool RooUnitTest::runCompTests()
       }
 
       if (_verb > 0) {
-   cout << "comparing value " << iter3->first << " to benchmark " << iter3->second << " = " << (double)(*ref) << endl ;
+   std::cout << "comparing value " << iter3->first << " to benchmark " << iter3->second << " = " << (double)(*ref) << std::endl ;
       }
 
       if (std::abs(iter3->first - (double)(*ref))>vtol() ) {
-        if(_verb >= 0) cout << "RooUnitTest ERROR: comparison of value " << iter3->first <<   " fails comparison with reference " << ref->GetName() << endl ;
+        if(_verb >= 0) std::cout << "RooUnitTest ERROR: comparison of value " << iter3->first <<   " fails comparison with reference " << ref->GetName() << std::endl ;
         ret = false ;
       }
 
@@ -412,7 +399,7 @@ bool RooUnitTest::runCompTests()
 
       // Writing mode
 
-      std::cout <<"RooUnitTest: Writing reference double " << iter3->first << " as benchmark " << iter3->second << endl ;
+      std::cout <<"RooUnitTest: Writing reference double " << iter3->first << " as benchmark " << iter3->second << std::endl ;
       _refFile->cd() ;
       RooDouble* rd = new RooDouble(iter3->first) ;
       rd->Write(iter3->second.c_str()) ;
@@ -423,7 +410,7 @@ bool RooUnitTest::runCompTests()
   }
 
 
-  list<pair<RooTable*, string> >::iterator iter4 = _regTables.begin() ;
+  auto iter4 = _regTables.begin() ;
   while (iter4!=_regTables.end()) {
 
     if (!_write) {
@@ -433,19 +420,19 @@ bool RooUnitTest::runCompTests()
      // Retrieve benchmark
       RooTable* bmark = dynamic_cast<RooTable*>(_refFile->Get(iter4->second.c_str())) ;
       if (!bmark) {
-        if(_verb >= 0) cout << "RooUnitTest ERROR: cannot retrieve RooTable " << iter4->second << " from reference file, skipping " << endl ;
+        if(_verb >= 0) std::cout << "RooUnitTest ERROR: cannot retrieve RooTable " << iter4->second << " from reference file, skipping " << std::endl ;
         ++iter4 ;
         ret = false ;
         continue ;
       }
 
       if (_verb > 0) {
-   cout << "comparing RooTable " << iter4->first << " to benchmark " << iter4->second << " = " << bmark << endl ;
+   std::cout << "comparing RooTable " << iter4->first << " to benchmark " << iter4->second << " = " << bmark << std::endl ;
       }
 
       if (!iter4->first->isIdentical(*bmark, _verb >= 0)) {
         if(_verb >= 0) std::cout << "RooUnitTest ERROR: comparison of object " << iter4->first->ClassName() << "::" << iter4->first->GetName()
-            <<   " fails comparison with counterpart in reference RooTable " << bmark->GetName() << endl ;
+            <<   " fails comparison with counterpart in reference RooTable " << bmark->GetName() << std::endl ;
         if (_verb > 0) {
           iter4->first->Print("V");
           bmark->Print("V");
@@ -461,7 +448,7 @@ bool RooUnitTest::runCompTests()
 
       // Writing mode
 
-      std::cout <<"RooUnitTest: Writing reference RooTable " << iter4->first << " as benchmark " << iter4->second << endl ;
+      std::cout <<"RooUnitTest: Writing reference RooTable " << iter4->first << " as benchmark " << iter4->second << std::endl ;
       _refFile->cd() ;
       iter4->first->Write(iter4->second.c_str()) ;
       gMemDir->cd() ;
@@ -471,14 +458,14 @@ bool RooUnitTest::runCompTests()
   }
 
 
-  list<pair<RooWorkspace*, string> >::iterator iter5 = _regWS.begin() ;
+  auto iter5 = _regWS.begin() ;
   while (iter5!=_regWS.end()) {
 
     if (_write) {
 
       // Writing mode
 
-      std::cout <<"RooUnitTest: Writing reference RooWorkspace " << iter5->first << " as benchmark " << iter5->second << endl ;
+      std::cout <<"RooUnitTest: Writing reference RooWorkspace " << iter5->first << " as benchmark " << iter5->second << std::endl ;
       _refFile->cd() ;
       iter5->first->Write(iter5->second.c_str()) ;
       gMemDir->cd() ;
@@ -488,7 +475,7 @@ bool RooUnitTest::runCompTests()
   }
 
   /////////////////
-  list<pair<TH1*, string> >::iterator iter6 = _regTH.begin() ;
+  auto iter6 = _regTH.begin() ;
   while (iter6!=_regTH.end()) {
 
     if (!_write) {
@@ -498,25 +485,25 @@ bool RooUnitTest::runCompTests()
      // Retrieve benchmark
       TH1* bmark = dynamic_cast<TH1*>(_refFile->Get(iter6->second.c_str())) ;
       if (!bmark) {
-        if(_verb >= 0) cout << "RooUnitTest ERROR: cannot retrieve TH1 " << iter6->second << " from reference file, skipping " << endl ;
+        if(_verb >= 0) std::cout << "RooUnitTest ERROR: cannot retrieve TH1 " << iter6->second << " from reference file, skipping " << std::endl ;
         ++iter6 ;
         ret = false ;
         continue ;
       }
 
       if (_verb > 0) {
-   cout << "comparing TH1 " << iter6->first << " to benchmark " << iter6->second << " = " << bmark << endl ;
+   std::cout << "comparing TH1 " << iter6->first << " to benchmark " << iter6->second << " = " << bmark << std::endl ;
       }
 
       if (!areTHidentical(iter6->first,bmark)) {
    // coverity[NULL_RETURNS]
-   if(_verb >= 0) cout << "RooUnitTest ERROR: comparison of object " << iter6->first->ClassName() << "::" << iter6->first->GetName()
-        <<   " fails comparison with counterpart in reference TH1 " << bmark->GetName() << endl ;
+   if(_verb >= 0) std::cout << "RooUnitTest ERROR: comparison of object " << iter6->first->ClassName() << "::" << iter6->first->GetName()
+        <<   " fails comparison with counterpart in reference TH1 " << bmark->GetName() << std::endl ;
 
 
       if (_debug) {
-   cout << "RooUnitTest INFO: writing THx " << iter6->first->GetName() << " and " << bmark->GetName()
-        << " of failed test to RooUnitTest_DEBUG.root" << endl ;
+   std::cout << "RooUnitTest INFO: writing THx " << iter6->first->GetName() << " and " << bmark->GetName()
+        << " of failed test to RooUnitTest_DEBUG.root" << std::endl ;
    TFile fdbg("RooUnitTest_DEBUG.root","UPDATE") ;
    iter6->first->SetName(Form("%s_test",iter6->first->GetName())) ;
    iter6->first->Write() ;
@@ -536,7 +523,7 @@ bool RooUnitTest::runCompTests()
 
       // Writing mode
 
-      std::cout <<"RooUnitTest: Writing reference TH1 " << iter6->first << " as benchmark " << iter6->second << endl ;
+      std::cout <<"RooUnitTest: Writing reference TH1 " << iter6->first << " as benchmark " << iter6->second << std::endl ;
       _refFile->cd() ;
       iter6->first->Write(iter6->second.c_str()) ;
       gMemDir->cd() ;
@@ -586,7 +573,7 @@ bool RooUnitTest::runTest()
   if (_verb<2) {
     setSilentMode() ;
   } else {
-    std::cout << "*** Begin of output of Unit Test at normal verbosity *************" << endl ;
+    std::cout << "*** Begin of output of Unit Test at normal verbosity *************" << std::endl ;
   }
 
   RooMsgService::instance().clearErrorCount() ;
@@ -602,11 +589,11 @@ bool RooUnitTest::runTest()
   if (_verb<2) {
     clearSilentMode() ;
   } else {
-    std::cout << "*** End of output of Unit Test at normal verbosity ***************" << endl ;
+    std::cout << "*** End of output of Unit Test at normal verbosity ***************" << std::endl ;
   }
 
   if (RooMsgService::instance().errorCount()>0) {
-    if(_verb >= 0) std::cout << "RooUnitTest: ERROR messages were logged, failing test" << endl ;
+    if(_verb >= 0) std::cout << "RooUnitTest: ERROR messages were logged, failing test" << std::endl ;
     return false ;
   }
 
@@ -620,3 +607,5 @@ bool RooUnitTest::runTest()
 void RooUnitTest::setMemDir(TDirectory* memDir) {
    gMemDir = memDir ;
 }
+
+/// \endcond
