@@ -2,12 +2,11 @@
 // Authors: Stephan Hageboeck, CERN  05/2020
 //          Jonas Rembser, CERN  08/2023
 
-#include "RooRealBinding.h"
-#include "RooRealVar.h"
-#include "RooFormulaVar.h"
-#include "RooRombergIntegrator.h"
-#include "RooNumIntConfig.h"
-#include "RooHelpers.h"
+#include <RooRealBinding.h>
+#include <RooRealVar.h>
+#include <RooFormulaVar.h>
+#include <RooNumIntConfig.h>
+#include <RooHelpers.h>
 #include <Math/ProbFuncMathCore.h>
 #include <Math/SpecFuncMathCore.h>
 #include <Math/Integrator.h>
@@ -18,8 +17,11 @@
 
 #include <numeric>
 #include <algorithm>
+#include <iomanip>
 
-TEST(RooIntegrator1D, RunFormulaVar_Trapezoid)
+#include "../src/RooRombergIntegrator.h"
+
+TEST(Roo1DIntegrator, RunFormulaVar_Trapezoid)
 {
 
    RooRealVar x("x", "x", -100, 100);
@@ -110,7 +112,9 @@ void testConvergenceSettings(const RooFormulaVar &formula, const RooArgSet &obse
    SCOPED_TRACE(std::string("Function to integrate: ") + name + "\t" + formula.GetTitle());
 
    for (double relEps : {1.E-3, 1.E-6, 1.E-8}) {
-      SCOPED_TRACE(std::string("relEps=" + std::to_string(relEps)));
+      std::stringstream traceName;
+      traceName << "relEps=" << relEps;
+      SCOPED_TRACE(traceName.str());
       TStopwatch stopNew;
       TStopwatch stopRoot;
 
@@ -298,9 +302,9 @@ TEST(RooIntegrator1D, RunErf_NStep)
       const double min = limits.first;
       const double max = limits.second;
       RooRealVar theX("theX", "x", min, max);
-      std::string funcStr =
-         std::string("ROOT::Math::gaussian_pdf(theX, ") + std::to_string(sigma) + ", " + std::to_string(mean) + ")";
-      RooFormulaVar gaus("gaus", funcStr.c_str(), theX);
+      std::stringstream funcStr;
+      funcStr << "ROOT::Math::gaussian_pdf(theX, " << sigma << ", " << mean << ")";
+      RooFormulaVar gaus("gaus", funcStr.str().c_str(), theX);
       RooRealBinding binding(gaus, theX);
 
       double targetError = 0.;
@@ -335,7 +339,8 @@ TEST(RooIntegrator1D, RunErf_NStep)
 
 TEST(RooIntegrator1D, RunErf_Midpoint)
 {
-   const double min = 0, max = 1;
+   const double min = 0;
+   const double max = 1;
    RooRealVar theX("theX", "x", min, max);
    RooFormulaVar gaus("gaus", "ROOT::Math::gaussian_pdf(theX, 1, 0)", theX);
    RooRealBinding binding(gaus, theX);
