@@ -62,16 +62,13 @@ call HypoTestInverter::UseCLs().
 #include "TCanvas.h"
 #include "TGraphErrors.h"
 
-#include "RooStats/ProofConfig.h"
-
 #include <cassert>
 #include <cmath>
 #include <memory>
 
-ClassImp(RooStats::HypoTestInverter);
 
 using namespace RooStats;
-using namespace std;
+using std::endl;
 
 // static variable definitions
 double HypoTestInverter::fgCLAccuracy = 0.005;
@@ -81,8 +78,6 @@ double HypoTestInverter::fgAbsAccuracy = 0.05;
 double HypoTestInverter::fgRelAccuracy = 0.05;
 std::string HypoTestInverter::fgAlgo = "logSecant";
 
-bool HypoTestInverter::fgCloseProof = false;
-
 // helper class to wrap the functionality of the various HypoTestCalculators
 
 template<class HypoTestType>
@@ -92,13 +87,6 @@ struct HypoTestWrapper {
 
 };
 
-////////////////////////////////////////////////////////////////////////////////
-/// set flag to close proof for every new run
-
-void HypoTestInverter::SetCloseProof(bool flag) {
-   fgCloseProof  = flag;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 /// get  the variable to scan
 /// try first with null model if not go to alternate model
@@ -151,11 +139,12 @@ void HypoTestInverter::CheckInputModels(const HypoTestCalculatorGeneric &hc,cons
    }
    if (bParams->find(scanVariable.GetName() ) ) {
       const RooArgSet * poiB  = modelB->GetSnapshot();
-      if (!poiB ||  !poiB->find(scanVariable.GetName()) ||
-          ( (RooRealVar*)  poiB->find(scanVariable.GetName()) )->getVal() != 0 )
-         oocoutW(nullptr,InputArguments) << "HypoTestInverter - using a B model  with POI "
-                                             <<    scanVariable.GetName()  << " not equal to zero "
-                                             << " user must check input model configurations " << endl;
+      if (!poiB || !poiB->find(scanVariable.GetName()) ||
+          (static_cast<RooRealVar *>(poiB->find(scanVariable.GetName())))->getVal() != 0) {
+         oocoutW(nullptr, InputArguments)
+            << "HypoTestInverter - using a B model  with POI " << scanVariable.GetName() << " not equal to zero "
+            << " user must check input model configurations " << std::endl;
+      }
       if (poiB) delete poiB;
    }
 }
@@ -207,10 +196,11 @@ HypoTestInverter::HypoTestInverter( HypoTestCalculatorGeneric& hc,
    if (!fScannedVariable) {
       fScannedVariable = HypoTestInverter::GetVariableToScan(hc);
    }
-   if (!fScannedVariable)
+   if (!fScannedVariable) {
       oocoutE(nullptr,InputArguments) << "HypoTestInverter - Cannot guess the variable to scan " << std::endl;
-   else
-      CheckInputModels(hc,*fScannedVariable);
+   } else {
+      CheckInputModels(hc, *fScannedVariable);
+   }
 
    HybridCalculator * hybCalc = dynamic_cast<HybridCalculator*>(&hc);
    if (hybCalc) {
@@ -260,11 +250,11 @@ HypoTestInverter::HypoTestInverter( HybridCalculator& hc,
    if (!fScannedVariable) {
       fScannedVariable = GetVariableToScan(hc);
    }
-   if (!fScannedVariable)
+   if (!fScannedVariable) {
       oocoutE(nullptr,InputArguments) << "HypoTestInverter - Cannot guess the variable to scan " << std::endl;
-   else
-      CheckInputModels(hc,*fScannedVariable);
-
+   } else {
+      CheckInputModels(hc, *fScannedVariable);
+   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -293,10 +283,11 @@ HypoTestInverter::HypoTestInverter( FrequentistCalculator& hc,
    if (!fScannedVariable) {
       fScannedVariable = GetVariableToScan(hc);
    }
-   if (!fScannedVariable)
+   if (!fScannedVariable) {
       oocoutE(nullptr,InputArguments) << "HypoTestInverter - Cannot guess the variable to scan " << std::endl;
-   else
-      CheckInputModels(hc,*fScannedVariable);
+   } else {
+      CheckInputModels(hc, *fScannedVariable);
+   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -325,11 +316,11 @@ HypoTestInverter::HypoTestInverter( AsymptoticCalculator& hc,
    if (!fScannedVariable) {
       fScannedVariable = GetVariableToScan(hc);
    }
-   if (!fScannedVariable)
+   if (!fScannedVariable) {
       oocoutE(nullptr,InputArguments) << "HypoTestInverter - Cannot guess the variable to scan " << std::endl;
-   else
-      CheckInputModels(hc,*fScannedVariable);
-
+   } else {
+      CheckInputModels(hc, *fScannedVariable);
+   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -362,11 +353,11 @@ HypoTestInverter::HypoTestInverter( RooAbsData& data, ModelConfig &sbModel, Mode
    if (!fScannedVariable) {
       fScannedVariable = GetVariableToScan(*fCalculator0);
    }
-   if (!fScannedVariable)
+   if (!fScannedVariable) {
       oocoutE(nullptr,InputArguments) << "HypoTestInverter - Cannot guess the variable to scan " << std::endl;
-   else
-      CheckInputModels(*fCalculator0,*fScannedVariable);
-
+   } else {
+      CheckInputModels(*fCalculator0, *fScannedVariable);
+   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -469,9 +460,9 @@ void  HypoTestInverter::CreateResults() const {
    if (fCalculator0) {
       // if asymptotic calculator
       AsymptoticCalculator * ac = dynamic_cast<AsymptoticCalculator*>(fCalculator0);
-      if (ac)
+      if (ac) {
          fResults->fIsTwoSided = ac->IsTwoSided();
-      else {
+      } else {
          // in case of the other calculators
          TestStatSampler * sampler = fCalculator0->GetTestStatSampler();
          if (sampler) {
@@ -491,7 +482,7 @@ HypoTestInverterResult* HypoTestInverter::GetInterval() const {
    // if having a result with at least  one point return it
    if (fResults && fResults->ArraySize() >= 1) {
       oocoutI(nullptr,Eval) << "HypoTestInverter::GetInterval - return an already existing interval " << std::endl;
-      return  (HypoTestInverterResult*)(fResults->Clone());
+      return  static_cast<HypoTestInverterResult*>(fResults->Clone());
    }
 
    if (fNBins > 0) {
@@ -502,15 +493,14 @@ HypoTestInverterResult* HypoTestInverter::GetInterval() const {
    }
    else {
       oocoutI(nullptr,Eval) << "HypoTestInverter::GetInterval - run an automatic scan" << std::endl;
-      double limit(0),err(0);
+      double limit(0);
+      double err(0);
       bool ret = RunLimit(limit,err);
       if (!ret)
          oocoutE(nullptr,Eval) << "HypoTestInverter::GetInterval - error running an auto scan " << std::endl;
    }
 
-   if (fgCloseProof) ProofConfig::CloseProof();
-
-   return (HypoTestInverterResult*) (fResults->Clone());
+   return static_cast<HypoTestInverterResult*> (fResults->Clone());
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -544,10 +534,12 @@ HypoTestResult * HypoTestInverter::Eval(HypoTestCalculatorGeneric &hc, bool adap
    //       hcResult->SetPValueIsRightTail(!hcResult->GetPValueIsRightTail());
 
    // adjust for some numerical error in discrete models and == is not anymore
-   if (hcResult->GetPValueIsRightTail() )
+   if (hcResult->GetPValueIsRightTail()) {
       hcResult->SetTestStatisticData(hcResult->GetTestStatisticData()-fNumErr); // issue with < vs <= in discrete models
-   else
-      hcResult->SetTestStatisticData(hcResult->GetTestStatisticData()+fNumErr); // issue with < vs <= in discrete models
+   } else {
+      hcResult->SetTestStatisticData(hcResult->GetTestStatisticData() +
+                                     fNumErr); // issue with < vs <= in discrete models
+   }
 
    double clsMid    = (fUseCLs ? hcResult->CLs()      : hcResult->CLsplusb());
    double clsMidErr = (fUseCLs ? hcResult->CLsError() : hcResult->CLsplusbError());
@@ -556,8 +548,8 @@ HypoTestResult * HypoTestInverter::Eval(HypoTestCalculatorGeneric &hc, bool adap
 
    if (adaptive) {
 
-      if (fCalcType == kHybrid) HypoTestWrapper<HybridCalculator>::SetToys((HybridCalculator*)&hc, fUseCLs ? fgNToys : 1, 4*fgNToys);
-      if (fCalcType == kFrequentist) HypoTestWrapper<FrequentistCalculator>::SetToys((FrequentistCalculator*)&hc, fUseCLs ? fgNToys : 1, 4*fgNToys);
+      if (fCalcType == kHybrid) HypoTestWrapper<HybridCalculator>::SetToys(static_cast<HybridCalculator*>(&hc), fUseCLs ? fgNToys : 1, 4*fgNToys);
+      if (fCalcType == kFrequentist) HypoTestWrapper<FrequentistCalculator>::SetToys(static_cast<FrequentistCalculator*>(&hc), fUseCLs ? fgNToys : 1, 4*fgNToys);
 
    while (clsMidErr >= fgCLAccuracy && (clsTarget == -1 || std::abs(clsMid-clsTarget) < 3*clsMidErr) ) {
       std::unique_ptr<HypoTestResult> more(hc.GetHypoTest());
@@ -646,10 +638,11 @@ bool HypoTestInverter::RunFixedScan( int nBins, double xMin, double xMax, bool s
    for (int i=0; i<nBins; i++) {
 
       if (i > 0) { // avoids case of nBins = 1
-         if (scanLog)
+      if (scanLog) {
             thisX = exp(  log(xMin) +  i*(log(xMax)-log(xMin))/(nBins-1)  );  // scan in log x
-         else
-            thisX = xMin + i*(xMax-xMin)/(nBins-1);          // linear scan in x
+      } else {
+            thisX = xMin + i * (xMax - xMin) / (nBins - 1); // linear scan in x
+      }
       }
 
       const bool status = RunOnePoint(thisX);
@@ -680,10 +673,11 @@ bool HypoTestInverter::RunOnePoint( double rVal, bool adaptive, double clTarget)
    }
    if ( rVal > fScannedVariable->getMax() ) {
       // print a message when you have a significative difference since rval is computed
-      if ( rVal > fScannedVariable->getMax()*(1.+1.E-12) )
-         oocoutE(nullptr,InputArguments) << "HypoTestInverter::RunOnePoint - Out of range: using the upper bound "
-                                             << fScannedVariable->getMax()
-                                             << " on the scanned variable rather than " << rVal<< "\n";
+     if (rVal > fScannedVariable->getMax() * (1. + 1.E-12)) {
+        oocoutE(nullptr, InputArguments) << "HypoTestInverter::RunOnePoint - Out of range: using the upper bound "
+                                         << fScannedVariable->getMax() << " on the scanned variable rather than "
+                                         << rVal << "\n";
+     }
      rVal = fScannedVariable->getMax();
    }
 
@@ -701,13 +695,13 @@ bool HypoTestInverter::RunOnePoint( double rVal, bool adaptive, double clTarget)
    const_cast<ModelConfig*>(sbModel)->SetSnapshot(poi);
 
    if (fVerbose > 0)
-      oocoutP(nullptr,Eval) << "Running for " << fScannedVariable->GetName() << " = " << fScannedVariable->getVal() << endl;
+      oocoutP(nullptr,Eval) << "Running for " << fScannedVariable->GetName() << " = " << fScannedVariable->getVal() << std::endl;
 
    // compute the results
    std::unique_ptr<HypoTestResult> result( Eval(*fCalculator0,adaptive,clTarget) );
    if (!result) {
       oocoutE(nullptr,Eval) << "HypoTestInverter - Error running point " << fScannedVariable->GetName() << " = " <<
-   fScannedVariable->getVal() << endl;
+   fScannedVariable->getVal() << std::endl;
       return false;
    }
    // in case of a dummy result
@@ -715,7 +709,7 @@ bool HypoTestInverter::RunOnePoint( double rVal, bool adaptive, double clTarget)
    const double altPV = result->AlternatePValue();
    if (!std::isfinite(nullPV) || nullPV < 0. || nullPV > 1. || !std::isfinite(altPV) || altPV < 0. || altPV > 1.) {
       oocoutW(nullptr,Eval) << "HypoTestInverter - Skipping invalid result for  point " << fScannedVariable->GetName() << " = " <<
-         fScannedVariable->getVal() << ". null p-value=" << nullPV << ", alternate p-value=" << altPV << endl;
+         fScannedVariable->getVal() << ". null p-value=" << nullPV << ", alternate p-value=" << altPV << std::endl;
       return false;
    }
 
@@ -786,8 +780,11 @@ bool HypoTestInverter::RunLimit(double &limit, double &limitErr, double absAccur
 
   typedef std::pair<double,double> CLs_t;
   double clsTarget = fSize;
-  CLs_t clsMin(1,0), clsMax(0,0), clsMid(0,0);
-  double rMin = r->getMin(), rMax = r->getMax();
+  CLs_t clsMin(1, 0);
+  CLs_t clsMax(0, 0);
+  CLs_t clsMid(0, 0);
+  double rMin = r->getMin();
+  double rMax = r->getMax();
   limit    = 0.5*(rMax + rMin);
   limitErr = 0.5*(rMax - rMin);
   bool done = false;
@@ -866,7 +863,9 @@ bool HypoTestInverter::RunLimit(double &limit, double &limitErr, double absAccur
      // determine point by bisection or interpolation
      limit = 0.5*(rMin+rMax); limitErr = 0.5*(rMax-rMin);
      if (fgAlgo == "logSecant" && clsMax.first != 0) {
-        double logMin = log(clsMin.first), logMax = log(clsMax.first), logTarget = log(clsTarget);
+        double logMin = log(clsMin.first);
+        double logMax = log(clsMax.first);
+        double logTarget = log(clsTarget);
         limit = rMin + (rMax-rMin) * (logTarget - logMin)/(logMax - logMin);
         if (clsMax.second != 0 && clsMin.second != 0) {
            limitErr = hypot((logTarget-logMax) * (clsMin.second/clsMin.first), (logTarget-logMin) * (clsMax.second/clsMax.first));
@@ -877,9 +876,10 @@ bool HypoTestInverter::RunLimit(double &limit, double &limitErr, double absAccur
 
      // exit if reached accuracy on r
      if (limitErr < std::max(absAccuracy, relAccuracy * limit)) {
-        if (fVerbose > 1)
-            oocoutI(nullptr,Eval) << "HypoTestInverter::RunLimit - reached accuracy " << limitErr
-                                      << " below " << std::max(absAccuracy, relAccuracy * limit)  << std::endl;
+        if (fVerbose > 1) {
+           oocoutI(nullptr, Eval) << "HypoTestInverter::RunLimit - reached accuracy " << limitErr << " below "
+                                  << std::max(absAccuracy, relAccuracy * limit) << std::endl;
+        }
         done = true; break;
      }
 
@@ -904,7 +904,8 @@ bool HypoTestInverter::RunLimit(double &limit, double &limitErr, double absAccur
        }
      } else {
        if (fVerbose > 0) std::cout << "Trying to move the interval edges closer" << std::endl;
-       double rMinBound = rMin, rMaxBound = rMax;
+       double rMinBound = rMin;
+       double rMaxBound = rMax;
        // try to reduce the size of the interval
        while (clsMin.second == 0 || std::abs(rMin-limit) > std::max(absAccuracy, relAccuracy * limit)) {
          rMin = 0.5*(rMin+limit);
@@ -940,7 +941,9 @@ bool HypoTestInverter::RunLimit(double &limit, double &limitErr, double absAccur
       expoFit.FixParameter(0,clsTarget);
       expoFit.SetParameter(1,log(clsMax.first/clsMin.first)/(rMax-rMin));
       expoFit.SetParameter(2,limit);
-      double rMinBound, rMaxBound; expoFit.GetRange(rMinBound, rMaxBound);
+      double rMinBound;
+      double rMaxBound;
+      expoFit.GetRange(rMinBound, rMaxBound);
       limitErr = std::max(std::abs(rMinBound-limit), std::abs(rMaxBound-limit));
       int npoints = 0;
 
@@ -978,7 +981,8 @@ bool HypoTestInverter::RunLimit(double &limit, double &limitErr, double absAccur
        //new TCanvas("c1","c1");
       fLimitPlot->Sort();
       fLimitPlot->SetLineWidth(2);
-      double xmin = r->getMin(), xmax = r->getMax();
+      double xmin = r->getMin();
+      double xmax = r->getMax();
       for (int j = 0; j < fLimitPlot->GetN(); ++j) {
         if (fLimitPlot->GetY()[j] > 1.4*clsTarget || fLimitPlot->GetY()[j] < 0.6*clsTarget) continue;
         xmin = std::min(fLimitPlot->GetX()[j], xmin);
@@ -1200,7 +1204,7 @@ SamplingDistribution * HypoTestInverter::RebuildDistributions(bool isUpper, int
                                        << nToys << std::endl;
 
 
-      printf("\n\nshnapshot of s+b model \n");
+      std::cout << "\n\nshnapshot of s+b model \n";
       sbModel->GetSnapshot()->Print("v");
 
       // reset parameters to initial values to be sure in case they are not reset
@@ -1222,7 +1226,7 @@ SamplingDistribution * HypoTestInverter::RebuildDistributions(bool isUpper, int
          RooArgList  genObs(*bkgdata->get(0));
          RooStats::PrintListContent(genObs, oocoutP(nullptr,Generation) );
          nObs = 0;
-         for (int i = 0; i < genObs.getSize(); ++i) {
+         for (std::size_t i = 0; i < genObs.size(); ++i) {
             RooRealVar * x = dynamic_cast<RooRealVar*>(&genObs[i]);
             if (x) nObs += x->getVal();
          }
