@@ -25,9 +25,8 @@
 
 #include <sqlite3.h>
 
-#include <stdlib.h>
+#include <cstdlib>
 
-ClassImp(TSQLiteStatement);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Normal constructor.
