@@ -24,8 +24,6 @@
 // MyEvent class implementation
 //______________________________________________________________________________
 
-ClassImp(EventHeader);
-ClassImp(MyEvent);
 
 TClonesArray *MyEvent::fgParticles = 0;
 
