@@ -50,7 +50,6 @@ Counter hitCount("THit");
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(THit);
 //-------------------------------------------------------------
 THit::THit() {
    fPulses = 0;
@@ -129,7 +128,6 @@ TBuffer &operator<<(TBuffer &buf, const THit *obj)
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TObjHit);
 //-------------------------------------------------------------
 
 TObjHit::TObjHit() :THit() {}
@@ -138,7 +136,6 @@ TObjHit::TObjHit(int t) :THit(t) {}
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhit);
 //-------------------------------------------------------------
 TSTLhit::TSTLhit()
 {
@@ -213,7 +210,6 @@ Int_t TSTLhit::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitList);
 //-------------------------------------------------------------
 TSTLhitList::TSTLhitList()
 {
@@ -286,7 +282,6 @@ Int_t TSTLhitList::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitDeque);
 //-------------------------------------------------------------
 TSTLhitDeque::TSTLhitDeque()
 {
@@ -360,7 +355,6 @@ Int_t TSTLhitDeque::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitSet);
 //-------------------------------------------------------------
 TSTLhitSet::TSTLhitSet()
 {
@@ -434,7 +428,6 @@ Int_t TSTLhitSet::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitMultiset);
 //-------------------------------------------------------------
 TSTLhitMultiset::TSTLhitMultiset()
 {
@@ -508,7 +501,6 @@ Int_t TSTLhitMultiset::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitMap);
 //-------------------------------------------------------------
 TSTLhitMap::TSTLhitMap()
 {
@@ -582,7 +574,6 @@ Int_t TSTLhitMap::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitMultiMap);
 //-------------------------------------------------------------
 TSTLhitMultiMap::TSTLhitMultiMap()
 {
@@ -657,7 +648,6 @@ Int_t TSTLhitMultiMap::ReadTree()
 #if 0
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitHashSet);
 //-------------------------------------------------------------
 TSTLhitHashSet::TSTLhitHashSet()
 {
@@ -731,7 +721,6 @@ Int_t TSTLhitHashSet::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitHashMultiSet);
 //-------------------------------------------------------------
 TSTLhitHashMultiSet::TSTLhitHashMultiSet()
 {
@@ -805,7 +794,6 @@ Int_t TSTLhitHashMultiSet::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitStar);
 //-------------------------------------------------------------
 TSTLhitStar::TSTLhitStar()
 {
@@ -882,7 +870,6 @@ Int_t TSTLhitStar::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitStarList);
 //-------------------------------------------------------------
 TSTLhitStarList::TSTLhitStarList()
 {
@@ -957,7 +944,6 @@ Int_t TSTLhitStarList::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitStarDeque);
 //-------------------------------------------------------------
 TSTLhitStarDeque::TSTLhitStarDeque()
 {
@@ -1032,7 +1018,6 @@ Int_t TSTLhitStarDeque::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitStarSet);
 //-------------------------------------------------------------
 TSTLhitStarSet::TSTLhitStarSet()
 {
@@ -1108,7 +1093,6 @@ Int_t TSTLhitStarSet::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitStarMultiSet);
 //-------------------------------------------------------------
 TSTLhitStarMultiSet::TSTLhitStarMultiSet()
 {
@@ -1184,7 +1168,6 @@ Int_t TSTLhitStarMultiSet::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitStarMap);
 //-------------------------------------------------------------
 TSTLhitStarMap::TSTLhitStarMap()
 {
@@ -1259,7 +1242,6 @@ Int_t TSTLhitStarMap::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TSTLhitStarMultiMap);
 //-------------------------------------------------------------
 TSTLhitStarMultiMap::TSTLhitStarMultiMap()
 {
@@ -1335,7 +1317,6 @@ Int_t TSTLhitStarMultiMap::ReadTree()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-ClassImp(TCloneshit);
 //-------------------------------------------------------------
 TCloneshit::TCloneshit()
 {
