@@ -39,7 +39,7 @@
 #include <list>
 #include <array>
 #include <functional>
-#include <stdlib.h>
+#include <cstdlib>
 #include "TApplication.h"
 #include "TEntryList.h"
 #include "TEventList.h"
@@ -95,7 +95,7 @@ Bool_t Test1(bool fixedCut)
    smallchain->Draw("x >> hcheck", "", "goff");
    wrongentries1 = 0;
    for (Int_t i=1; i<=range; i++){
-      if (TMath::Abs(hx->GetBinContent(i)-hcheck->GetBinContent(i)) > 0.1){
+      if (std::abs(hx->GetBinContent(i)-hcheck->GetBinContent(i)) > 0.1){
          wrongentries1++;
       }
    }
@@ -107,7 +107,7 @@ Bool_t Test1(bool fixedCut)
    bigchain->Draw("x >> hcheck_", "", "goff");
    wrongentries2 = 0;
    for (Int_t i=1; i<=range; i++){
-      if (TMath::Abs(hx->GetBinContent(i)-hcheck->GetBinContent(i)) > 0.1){
+      if (std::abs(hx->GetBinContent(i)-hcheck->GetBinContent(i)) > 0.1){
          wrongentries2++;
       }
    }
@@ -163,7 +163,7 @@ Bool_t Test1(bool fixedCut)
    smallchain->Draw("x>>hcheck", "", "goff");
    wrongentries4 = 0;
    for (Int_t i=1; i<=range; i++){
-      if (TMath::Abs(hx->GetBinContent(i)-hcheck->GetBinContent(i)) > 0.1){
+      if (std::abs(hx->GetBinContent(i)-hcheck->GetBinContent(i)) > 0.1){
          //printf("%d hx: %f hcheck %f\n", i, hx->GetBinContent(i), hcheck->GetBinContent(i));
          wrongentries4++;
       }
@@ -177,7 +177,7 @@ Bool_t Test1(bool fixedCut)
    smallchain->Draw("x >> hcheck", "", "goff");
    wrongentries5 = 0;
    for (Int_t i=1; i<=range; i++){
-      if (TMath::Abs(hx->GetBinContent(i)-hcheck->GetBinContent(i)) > 0.1){
+      if (std::abs(hx->GetBinContent(i)-hcheck->GetBinContent(i)) > 0.1){
          //printf("i=%d hx(i)=%f, hcheck(i)=%f\n", i, hx->GetBinContent(i), hcheck->GetBinContent(i));
          wrongentries5++;
       }
@@ -360,7 +360,7 @@ Bool_t Test3()
       bin1 = h1->GetBinContent(i);
       bin2 = h2->GetBinContent(i);
       bin3 = h3->GetBinContent(i);
-      if (TMath::Abs(bin1-bin2) > 0.1 || TMath::Abs(bin1-bin3) || TMath::Abs(bin2-bin3) > 0.1) {
+      if (std::abs(bin1-bin2) > 0.1 || std::abs(bin1-bin3) || std::abs(bin2-bin3) > 0.1) {
          //printf("bin1=%f, bin2=%f, bin3=%f\n", bin1, bin2, bin3);
          wrongbins++;
       }
@@ -408,7 +408,7 @@ Bool_t Test4()
       bin1 = h1->GetBinContent(i);
       bin2 = h2->GetBinContent(i);
       bin3 = h3->GetBinContent(i);
-      if (TMath::Abs(bin1-bin2) > 0.1 || TMath::Abs(bin1-bin3) || TMath::Abs(bin2-bin3) > 0.1) {
+      if (std::abs(bin1-bin2) > 0.1 || std::abs(bin1-bin3) || std::abs(bin2-bin3) > 0.1) {
          //printf("bin1=%f, bin2=%f, bin3=%f\n", bin1, bin2, bin3);
          wrongbins++;
       }
@@ -456,7 +456,7 @@ Bool_t Test5And6(const std::list<const char*>& treeNamesForChain )
       for (Int_t i=offset[itree]; i<offset[itree+1]; i++){
          real = i-offset[itree];
          cur = elfull->GetEntry(i);
-         if (TMath::Abs(real-cur)>0.1){
+         if (std::abs(real-cur)>0.1){
             //printf("real=%lld, cur=%lld\n", real, cur);
             wrongentries1++;
          }
@@ -468,7 +468,7 @@ Bool_t Test5And6(const std::list<const char*>& treeNamesForChain )
       for (Int_t i=offset[itree]; i<offset[itree+1]; i+=2){
          real = i-offset[itree];
          cur = elfull->GetEntry(i);
-         if (TMath::Abs(real-cur)>0.1){
+         if (std::abs(real-cur)>0.1){
             //printf("real=%lld, cur=%lld\n", real, cur);
             wrongentries2++;
          }
@@ -482,7 +482,7 @@ Bool_t Test5And6(const std::list<const char*>& treeNamesForChain )
    TEntryList *elempty = (TEntryList*)gDirectory->Get("elempty");
    //just a check
    Long64_t temp = elempty->GetEntry(3);
-   if (TMath::Abs(temp+1)>0.1)
+   if (std::abs(temp+1)>0.1)
    wrongentries5++;
 
    //Merge the almost full list with the almost empty list
@@ -499,7 +499,7 @@ Bool_t Test5And6(const std::list<const char*>& treeNamesForChain )
       for (Int_t i=offset[itree]; i<offset[itree+1]; i++){
          real = i-offset[itree];
          cur = elfull->GetEntry(i);
-         if (TMath::Abs(real-cur)>0.1){
+         if (std::abs(real-cur)>0.1){
             //printf("real=%lld, cur=%lld\n", real, cur);
             wrongentries3++;
          }
@@ -512,7 +512,7 @@ Bool_t Test5And6(const std::list<const char*>& treeNamesForChain )
       for (Int_t i=offset[itree]; i<offset[itree+1]; i+=2){
          real = i-offset[itree];
          cur = elfull->GetEntry(i);
-         if (TMath::Abs(real-cur)>0.1){
+         if (std::abs(real-cur)>0.1){
             //printf("real=%lld, cur=%lld\n", real, cur);
             wrongentries4++;
          }
@@ -524,7 +524,7 @@ Bool_t Test5And6(const std::list<const char*>& treeNamesForChain )
    chain->SetEntryList(elfull);
    chain->Draw("x>>hx", "", "goff");
    TH1F *hx = (TH1F*)gDirectory->Get("hx");
-   if (TMath::Abs(hx->GetEntries()-chain->GetEntries())>0.1){
+   if (std::abs(hx->GetEntries()-chain->GetEntries())>0.1){
       wrongentries5++;
       //printf("entries in chain: %lld, entries in histo: %f\n", chain->GetEntries(), hx->GetEntries());
    }
