@@ -2199,7 +2199,7 @@ void guitest_playback()
    guitest_ref[10] = 68657;
    for (i=0;i<11;++i) {
       printf("guitest %02d: output............................................", i+1);
-      if (TMath::Abs(guitest_ref[i] - guitest_size[i]) <= guitest_err[i]) {
+      if (std::abs(guitest_ref[i] - guitest_size[i]) <= guitest_err[i]) {
          printf("..... OK\n");
          // delete successful tests, keep only the failing ones (for verification)
          gSystem->Unlink(TString::Format("%s/guitest%03d.C", dir.Data(), i+1));
@@ -2579,4 +2579,4 @@ int main(int argc, char *argv[])
    return 0;
 }
 
-#endif
\ No newline at end of file
+#endif
