@@ -7,11 +7,8 @@ namespace Experimental {
 namespace SOFIE {
 
 ParserFuncSignature ParseCast = [](RModelParser_ONNX &parser, const onnx::NodeProto &nodeproto) {
-   ETensorType input_type;
    auto input_name = nodeproto.input(0);
-   if (parser.IsRegisteredTensorType(input_name)) {
-      input_type = parser.GetTensorType(input_name);
-   } else {
+   if (!parser.IsRegisteredTensorType(input_name)) {
       throw std::runtime_error("TMVA::SOFIE ONNX Parser Cast op has input tensor" + input_name +
                                "  but its type is not yet registered");
    }
@@ -26,15 +23,7 @@ ParserFuncSignature ParseCast = [](RModelParser_ONNX &parser, const onnx::NodePr
    }
 
    std::string output_name = nodeproto.output(0);
-   switch (input_type) {
-   case ETensorType::INT64:
-   case ETensorType::DOUBLE:
-   case ETensorType::FLOAT:
-   case ETensorType::INT32: op.reset(new ROperator_Cast<float>(attr_type, nodeproto.input(0), output_name)); break;
-   default:
-      throw std::runtime_error("TMVA::SOFIE - Unsupported - Operator Cast does not yet support input type " +
-                               std::to_string(static_cast<int>(input_type)));
-   }
+   op.reset(new ROperator_Cast(attr_type, nodeproto.input(0), output_name));
 
    if (!parser.IsRegisteredTensorType(output_name)) {
       ETensorType output_type = ConvertStringToType(attr_type);
