@@ -21,8 +21,8 @@ ParserFuncSignature ParseReshape = [](RModelParser_ONNX &parser, const onnx::Nod
    // reshape has as extra input shape tensor (int64) but
    // it is not present for Flatten, Squeeze and Unsquueze
    auto input_name = nodeproto.input(0);
-   // for squeeze is optional ?
-   auto shape_name = ((nodeproto.input_size() > 1) && ( opMode == Reshape || opMode == Unsqueeze || opMode == Squeeze) )
+   // second input is a tensor indicating shape (ReShape)  or (for Squeeze/Unsqueeze) axes (it is optional)
+   auto input2_name = ((nodeproto.input_size() > 1) && ( opMode == Reshape || opMode == Unsqueeze || opMode == Squeeze) )
       ? nodeproto.input(1) : "";
    if (parser.IsRegisteredTensorType(input_name)) {
       input_type = parser.GetTensorType(input_name);
@@ -36,7 +36,7 @@ ParserFuncSignature ParseReshape = [](RModelParser_ONNX &parser, const onnx::Nod
    // old version of reshape and squeeze have axes as attributes
    std::unique_ptr<ROperator> op;
    int attr_value = (opMode == Reshape) ? 0 : 1;
-   if (opMode == Reshape && nodeproto.attribute_size() > 0)
+   if ((opMode == Reshape || opMode == Flatten) && nodeproto.attribute_size() > 0)
       attr_value = nodeproto.attribute(0).i();
 
    std::vector<int64_t> attr_axes = {};
@@ -50,7 +50,8 @@ ParserFuncSignature ParseReshape = [](RModelParser_ONNX &parser, const onnx::Nod
    std::string output_name = nodeproto.output(0);
 
    if (attr_axes.empty())
-      op.reset(new ROperator_Reshape(opMode, attr_value, input_name, shape_name, output_name));
+      // for Reshape and new version of Squeeze and Unsqueeze
+      op.reset(new ROperator_Reshape(opMode, attr_value, input_name, input2_name, output_name));
    else // for old Squeeze and Unsqueeze
       op.reset(new ROperator_Reshape(opMode, attr_axes, input_name, output_name));
 
