@@ -8,7 +8,7 @@
 
 using namespace TMVA::Experimental;
 
-static const std::string filename_ = std::string(gROOT->GetTutorialDir()) + "/tmva/data/tmva_class_example.root";
+static const std::string filename_ = std::string(gROOT->GetTutorialDir()) + "/machine_learning/data/tmva_class_example.root";
 static const std::vector<std::string> variables = {"var1", "var2", "var3", "var4"};
 
 TEST(RStandardScaler, TensorOutputShape)
