@@ -34,6 +34,7 @@
 #include <string>
 #include <cstring>
 #include <typeinfo>
+#include <cstdint>
 
 using namespace ROOT::Detail::RDF;
 using namespace ROOT::RDF;
@@ -59,47 +60,101 @@ namespace ROOT {
 namespace Internal {
 namespace RDF {
 
+unsigned int &NThreadPerTH3()
+{
+   static unsigned int nThread = 1;
+   return nThread;
+}
+
 /// Return the type_info associated to a name. If the association fails, an
 /// exception is thrown.
 /// References and pointers are not supported since those cannot be stored in
 /// columns.
 const std::type_info &TypeName2TypeID(const std::string &name)
 {
-
+   // This map includes all relevant C++ fundamental types found at
+   // https://en.cppreference.com/w/cpp/language/types.html and the associated
+   // ROOT portable types when available.
    const static std::unordered_map<std::string, TypeInfoRef> typeName2TypeIDMap{
+      // Integral types
+      // Standard integer types
+      {"short", typeid(short)},
+      {"short int", typeid(short int)},
+      {"signed short", typeid(signed short)},
+      {"signed short int", typeid(signed short int)},
+      {"unsigned short", typeid(unsigned short)},
+      {"unsigned short int", typeid(unsigned short int)},
+      {"int", typeid(int)},
+      {"signed", typeid(signed)},
+      {"signed int", typeid(signed int)},
+      {"unsigned", typeid(unsigned)},
+      {"unsigned int", typeid(unsigned int)},
+      {"long", typeid(long)},
+      {"long int", typeid(long int)},
+      {"signed long", typeid(signed long)},
+      {"signed long int", typeid(signed long int)},
+      {"unsigned long", typeid(unsigned long)},
+      {"unsigned long int", typeid(unsigned long int)},
+      {"long long", typeid(long long)},
+      {"long long int", typeid(long long int)},
+      {"signed long long", typeid(signed long long)},
+      {"signed long long int", typeid(signed long long int)},
+      {"unsigned long long", typeid(unsigned long long)},
+      {"unsigned long long int", typeid(unsigned long long int)},
+      {"std::size_t", typeid(std::size_t)},
+   // Extended standard integer types
+#ifdef INT8_MAX
+      {"std::int8_t", typeid(std::int8_t)},
+#endif
+#ifdef INT16_MAX
+      {"std::int16_t", typeid(std::int16_t)},
+#endif
+#ifdef INT32_MAX
+      {"std::int32_t", typeid(std::int32_t)},
+#endif
+#ifdef INT64_MAX
+      {"std::int64_t", typeid(std::int64_t)},
+#endif
+#ifdef UINT8_MAX
+      {"std::uint8_t", typeid(std::uint8_t)},
+#endif
+#ifdef UINT16_MAX
+      {"std::uint16_t", typeid(std::uint16_t)},
+#endif
+#ifdef UINT32_MAX
+      {"std::uint32_t", typeid(std::uint32_t)},
+#endif
+#ifdef UINT64_MAX
+      {"std::uint64_t", typeid(std::uint64_t)},
+#endif
+      // ROOT integer types
+      {"Int_t", typeid(Int_t)},
+      {"UInt_t", typeid(UInt_t)},
+      {"Short_t", typeid(Short_t)},
+      {"UShort_t", typeid(UShort_t)},
+      {"Long_t", typeid(Long_t)},
+      {"ULong_t", typeid(ULong_t)},
+      {"Long64_t", typeid(Long64_t)},
+      {"ULong64_t", typeid(ULong64_t)},
+      // Boolean type
+      {"bool", typeid(bool)},
+      {"Bool_t", typeid(bool)},
+      // Character types
       {"char", typeid(char)},
       {"Char_t", typeid(char)},
+      {"signed char", typeid(signed char)},
       {"unsigned char", typeid(unsigned char)},
       {"UChar_t", typeid(unsigned char)},
-      {"int", typeid(int)},
-      {"Int_t", typeid(int)},
-      {"unsigned", typeid(unsigned int)},
-      {"unsigned int", typeid(unsigned int)},
-      {"UInt_t", typeid(unsigned int)},
-      {"short", typeid(short)},
-      {"short int", typeid(short)},
-      {"Short_t", typeid(short)},
-      {"unsigned short", typeid(unsigned short)},
-      {"unsigned short int", typeid(unsigned short)},
-      {"UShort_t", typeid(unsigned short)},
-      {"long", typeid(long)},
-      {"long int", typeid(long)},
-      {"Long_t", typeid(long)},
-      {"unsigned long", typeid(unsigned long)},
-      {"unsigned long int", typeid(unsigned long)},
-      {"ULong_t", typeid(unsigned long)},
-      {"double", typeid(double)},
-      {"Double_t", typeid(double)},
+      {"char16_t", typeid(char16_t)},
+      {"char32_t", typeid(char32_t)},
+      // Floating-point types
+      // Standard floating-point types
       {"float", typeid(float)},
+      {"double", typeid(double)},
+      {"long double", typeid(long double)},
+      // ROOT floating-point types
       {"Float_t", typeid(float)},
-      {"long long", typeid(long long)},
-      {"long long int", typeid(long long)},
-      {"Long64_t", typeid(long long)},
-      {"unsigned long long", typeid(unsigned long long)},
-      {"unsigned long long int", typeid(unsigned long long)},
-      {"ULong64_t", typeid(unsigned long long)},
-      {"bool", typeid(bool)},
-      {"Bool_t", typeid(bool)}};
+      {"Double_t", typeid(double)}};
 
    if (auto it = typeName2TypeIDMap.find(name); it != typeName2TypeIDMap.end())
       return it->second.get();
@@ -123,12 +178,19 @@ const std::type_info &TypeName2TypeID(const std::string &name)
 std::string TypeID2TypeName(const std::type_info &id)
 {
    const static std::unordered_map<TypeInfoRef, std::string, TypeInfoRefHash, TypeInfoRefEqualComp> typeID2TypeNameMap{
-      {typeid(char), "char"},         {typeid(unsigned char), "unsigned char"},
-      {typeid(int), "int"},           {typeid(unsigned int), "unsigned int"},
-      {typeid(short), "short"},       {typeid(unsigned short), "unsigned short"},
-      {typeid(long), "long"},         {typeid(unsigned long), "unsigned long"},
-      {typeid(double), "double"},     {typeid(float), "float"},
-      {typeid(Long64_t), "Long64_t"}, {typeid(ULong64_t), "ULong64_t"},
+      {typeid(char), "char"},
+      {typeid(unsigned char), "unsigned char"},
+      {typeid(signed char), "signed char"},
+      {typeid(int), "int"},
+      {typeid(unsigned int), "unsigned int"},
+      {typeid(short), "short"},
+      {typeid(unsigned short), "unsigned short"},
+      {typeid(long), "long"},
+      {typeid(unsigned long), "unsigned long"},
+      {typeid(double), "double"},
+      {typeid(float), "float"},
+      {typeid(Long64_t), "Long64_t"},
+      {typeid(ULong64_t), "ULong64_t"},
       {typeid(bool), "bool"}};
 
    if (auto it = typeID2TypeNameMap.find(id); it != typeID2TypeNameMap.end())
@@ -141,6 +203,23 @@ std::string TypeID2TypeName(const std::type_info &id)
    return "";
 }
 
+char TypeID2ROOTTypeName(const std::type_info &tid)
+{
+   const static std::unordered_map<TypeInfoRef, char, TypeInfoRefHash, TypeInfoRefEqualComp> typeID2ROOTTypeNameMap{
+      {typeid(char), 'B'},      {typeid(Char_t), 'B'},   {typeid(unsigned char), 'b'},      {typeid(UChar_t), 'b'},
+      {typeid(int), 'I'},       {typeid(Int_t), 'I'},    {typeid(unsigned int), 'i'},       {typeid(UInt_t), 'i'},
+      {typeid(short), 'S'},     {typeid(Short_t), 'S'},  {typeid(unsigned short), 's'},     {typeid(UShort_t), 's'},
+      {typeid(long), 'G'},      {typeid(Long_t), 'G'},   {typeid(unsigned long), 'g'},      {typeid(ULong_t), 'g'},
+      {typeid(long long), 'L'}, {typeid(Long64_t), 'L'}, {typeid(unsigned long long), 'l'}, {typeid(ULong64_t), 'l'},
+      {typeid(float), 'F'},     {typeid(Float_t), 'F'},  {typeid(Double_t), 'D'},           {typeid(double), 'D'},
+      {typeid(bool), 'O'},      {typeid(Bool_t), 'O'}};
+
+   if (auto it = typeID2ROOTTypeNameMap.find(tid); it != typeID2ROOTTypeNameMap.end())
+      return it->second;
+
+   return ' ';
+}
+
 std::string ComposeRVecTypeName(const std::string &valueType)
 {
    return "ROOT::VecOps::RVec<" + valueType + ">";
@@ -572,10 +651,10 @@ std::string ROOT::Internal::RDF::DescribeDataset(ROOT::RDF::RDataSource &ds)
 }
 
 ROOT::RDF::RSampleInfo ROOT::Internal::RDF::CreateSampleInfo(
-   const ROOT::RDF::RDataSource &ds,
+   const ROOT::RDF::RDataSource &ds, unsigned int slot,
    const std::unordered_map<std::string, ROOT::RDF::Experimental::RSample *> &sampleMap)
 {
-   return ds.CreateSampleInfo(sampleMap);
+   return ds.CreateSampleInfo(slot, sampleMap);
 }
 
 void ROOT::Internal::RDF::RunFinalChecks(const ROOT::RDF::RDataSource &ds, bool nodesLeftNotRun)
@@ -598,4 +677,4 @@ ROOT::Internal::RDF::CreateColumnReader(ROOT::RDF::RDataSource &ds, unsigned int
 std::vector<ROOT::RDF::Experimental::RSample> ROOT::Internal::RDF::MoveOutSamples(ROOT::RDF::Experimental::RDatasetSpec &spec)
 {
    return std::move(spec.fSamples);
-}
\ No newline at end of file
+}
