@@ -48,11 +48,14 @@ void ROOT::Internal::RDF::TriggerRun(ROOT::RDF::RNode node)
 
 std::string ROOT::Internal::RDF::GetDataSourceLabel(const ROOT::RDF::RNode &node)
 {
-   if (node.fLoopManager->GetTree()) {
-      return "TTreeDS";
-   } else if (node.fDataSource) {
-      return node.fDataSource->GetLabel();
+   if (auto ds = node.GetDataSource()) {
+      return ds->GetLabel();
    } else {
       return "EmptyDS";
    }
 }
+
+void ROOT::Internal::RDF::SetTTreeLifeline(ROOT::RDF::RNode &node, std::any lifeline)
+{
+   node.GetLoopManager()->SetTTreeLifeline(std::move(lifeline));
+}
