@@ -1,10 +1,5 @@
 #include <gtest/gtest.h>
 
-// Backward compatibility for gtest version < 1.10.0
-#ifndef INSTANTIATE_TEST_SUITE_P
-#define INSTANTIATE_TEST_SUITE_P INSTANTIATE_TEST_CASE_P
-#endif
-
 #include <ROOT/RDataFrame.hxx>
 #include <ROOT/RVec.hxx>
 #include <ROOT/RDFHelpers.hxx>
@@ -12,11 +7,17 @@
 #include <ROOT/RDF/RSample.hxx>
 #include <ROOT/RDF/RDatasetSpec.hxx>
 #include <ROOT/RDF/RMetaData.hxx>
-#include <nlohmann/json.hpp>
 #include <TSystem.h>
 
+#include <ROOT/RNTupleModel.hxx>
+#include <ROOT/RNTupleReader.hxx>
+#include <ROOT/RNTupleWriter.hxx>
+
 #include <thread> // std::thread::hardware_concurrency
 
+#include <TFile.h>
+#include <TTree.h>
+
 using namespace ROOT;
 using namespace ROOT::RDF::Experimental;
 
@@ -93,7 +94,7 @@ protected:
          ROOT::EnableImplicitMT(NSLOTS);
    }
 
-   ~RDatasetSpecTest()
+   ~RDatasetSpecTest() override
    {
       if (GetParam())
          ROOT::DisableImplicitMT();
@@ -108,7 +109,7 @@ public:
       for (const auto &d : data)
          for (const auto &trees : d.trees)
             for (const auto &t : trees)
-               dfWriter.Range(t.start, t.end).Snapshot<ULong64_t>(t.tree, t.file, {"x"});
+               dfWriter.Range(t.start, t.end).Snapshot(t.tree, t.file, {"x"});
    }
 
    static void TearDownTestCase()
@@ -262,8 +263,14 @@ TEST_P(RDatasetSpecTest, SimpleMetaDataHandling)
 // start > end -> error in the spec directly
 TEST_P(RDatasetSpecTest, Ranges)
 {
-   std::vector<RDatasetSpec::REntryRange> ranges = {{1, 4},     {2, 4},    {100},     {1, 100}, {2, 2},
-                                                    {100, 100}, {85, 100}, {86, 100}, {92, 100}};
+   // Logically correct ranges
+   std::vector<RDatasetSpec::REntryRange> goodRanges = {{1, 4}, {2, 4}, {75}, {1, 75}, {2, 2}, {100, 100}};
+
+   // Problematic ranges
+   RDatasetSpec::REntryRange endEntryBeyondEndRange{1, 100};
+   RDatasetSpec::REntryRange offByOneBeginEntryRange{85, 86};
+   RDatasetSpec::REntryRange offByTwoBeginEntryRange{86, 100};
+   RDatasetSpec::REntryRange ReallyBeyondEndRange{92, 100};
 
    std::vector<RDatasetSpec> specs(2);
    for (const auto &d : data) {
@@ -283,48 +290,92 @@ TEST_P(RDatasetSpecTest, Ranges)
       specs[1].AddSample({d.name, treeNamesExpanded, fileGlobsExpanded, d.meta});
    }
    for (auto &spec : specs) {
-      for (auto i = 0u; i < ranges.size(); ++i) {
-         auto df = RDataFrame(spec.WithGlobalRange(ranges[i]));
+
+      // Good ranges
+      for (auto i = 0u; i < goodRanges.size(); ++i) {
+         auto df = RDataFrame(spec.WithGlobalRange(goodRanges[i]));
          auto takeRes = df.Take<ULong64_t>("x"); // lazy action
-         if (i < 6u) {                           // the first 6 ranges, are logically correct
-            auto &res = *takeRes;
-            std::sort(res.begin(), res.end());
-            if (ranges[i].fBegin != ranges[i].fEnd)
-               EXPECT_VEC_SEQ_EQ(res, ROOT::TSeq<ULong64_t>(ranges[i].fBegin, std::min(85ll, ranges[i].fEnd)));
-            else
-               EXPECT_EQ((*takeRes).size(), 0u);
-         } else {
-            if (GetParam()) { // MT case, error coming from the TTreeProcessorMT
-               EXPECT_THROW(
-                  try { *takeRes; } catch (const std::logic_error &err) {
-                     const auto msg =
-                        std::string("A range of entries was passed in the creation of the TTreeProcessorMT, "
-                                    "but the starting entry (") +
-                        ranges[i].fBegin +
-                        ") is larger "
-                        "than the total number of entries (85) in the dataset.";
-                     EXPECT_EQ(std::string(err.what()), msg);
-                     throw;
-                  },
-                  std::logic_error);
-            } else {
-               if (i == 6u) // Single-threaded case undesired behaviour, see
-                            // https://github.com/root-project/root/issues/10774
-                  EXPECT_EQ((*takeRes).size(), 0u);
-               else {
-                  EXPECT_THROW(
-                     try {
-                        ROOT_EXPECT_ERROR(*takeRes, "TTreeReader::SetEntriesRange()",
-                                          (std::string("Error setting first entry ") + ranges[i].fBegin +
-                                           ": one of the readers was not successfully initialized")
-                                             .Data());
-                     } catch (const std::logic_error &err) { // error coming from the RLoopManager
-                        EXPECT_STREQ(err.what(), "Something went wrong in initializing the TTreeReader.");
-                        throw;
-                     },
-                     std::logic_error);
-               }
-            }
+         auto &res = *takeRes;
+         std::sort(res.begin(), res.end());
+         if (goodRanges[i].fBegin != goodRanges[i].fEnd)
+            EXPECT_VEC_SEQ_EQ(res, ROOT::TSeq<ULong64_t>(goodRanges[i].fBegin, std::min(85ll, goodRanges[i].fEnd)));
+         else
+            EXPECT_EQ((*takeRes).size(), 0u);
+      }
+
+      // Bad ranges
+
+      {
+         // End entry out of bounds
+         auto df = RDataFrame(spec.WithGlobalRange(endEntryBeyondEndRange));
+         auto takeRes = df.Take<ULong64_t>("x");
+         ROOT_EXPECT_WARNING(*takeRes, "RDataFrame::Run",
+                             "RDataFrame stopped processing after 84 entries, whereas an entry range (begin=1,end=100) "
+                             "was requested. Consider adjusting the end value of the entry range to a maximum of 85.");
+      }
+
+      if (GetParam()) {
+         // Begin entry beyond maximum number of entries, in MT case the error
+         // comes from TTreeProcessorMT
+         for (auto &&range : {offByOneBeginEntryRange, offByTwoBeginEntryRange, ReallyBeyondEndRange}) {
+            auto df = RDataFrame(spec.WithGlobalRange(range));
+            auto takeRes = df.Take<ULong64_t>("x");
+            EXPECT_THROW(
+               try { *takeRes; } catch (const std::logic_error &err) {
+                  const auto msg = std::string("A range of entries was passed in the creation of the TTreeProcessorMT, "
+                                               "but the starting entry (") +
+                                   range.fBegin +
+                                   ") is larger "
+                                   "than the total number of entries (85) in the dataset.";
+                  EXPECT_EQ(std::string(err.what()), msg);
+                  throw;
+               },
+               std::logic_error);
+         }
+      } else {
+         // Begin entry beyond maximum number of entries, in single-threaded
+         // case we rely on TTreeReader status codes
+         {
+            // Off by one case, SetEntriesRange does not error out immediately.
+            // The first call to TTreeReader::Next will detect the issue.
+            auto df = RDataFrame(spec.WithGlobalRange(offByOneBeginEntryRange));
+            auto takeRes = df.Take<ULong64_t>("x");
+            EXPECT_THROW(
+               try {
+                  ROOT_EXPECT_ERROR(*takeRes, "TTreeReader::SetEntryBase()",
+                                    "The beginning entry specified via SetEntriesRange (85) is equal to or beyond "
+                                    "the total number of entries in the dataset (85). Make sure to specify a "
+                                    "beginning entry lower than the number of available entries.");
+               } catch (const std::runtime_error &err) { // error coming from the RLoopManager
+                  EXPECT_STREQ(err.what(),
+                               "An error was encountered while processing the data. TTreeReader status code is: 3");
+                  throw;
+               },
+               std::runtime_error);
+         }
+
+         for (auto &&range : {offByTwoBeginEntryRange, ReallyBeyondEndRange}) {
+            // Other cases with begin entry further beyond are caught directly by SetEntriesRange
+            auto df = RDataFrame(spec.WithGlobalRange(range));
+            auto takeRes = df.Take<ULong64_t>("x");
+            EXPECT_THROW(
+               try {
+                  ROOT::TestSupport::CheckDiagsRAII diagRAII;
+                  diagRAII.requiredDiag(kError, "TTreeReader::SetEntryBase()",
+                                        "The beginning entry specified via SetEntriesRange (" +
+                                           std::to_string(range.fBegin) +
+                                           ") is equal to or beyond "
+                                           "the total number of entries in the dataset (85). Make sure to specify a "
+                                           "beginning entry lower than the number of available entries.");
+                  diagRAII.requiredDiag(kError, "TTreeReader::SetEntriesRange()",
+                                        "Error setting first entry " + std::to_string(range.fBegin) +
+                                           ": cannot access chain element");
+                  *takeRes;
+               } catch (const std::logic_error &err) { // error coming from the RLoopManager
+                  EXPECT_STREQ(err.what(), "Something went wrong in initializing the TTreeReader.");
+                  throw;
+               },
+               std::logic_error);
          }
       }
    }
@@ -339,46 +390,83 @@ TEST_P(RDatasetSpecTest, Ranges)
       std::logic_error);
 }
 
-// reuse the possible ranges from above
-TEST_P(RDatasetSpecTest, Friends)
+TEST_P(RDatasetSpecTest, FriendsEqualSize)
 {
-
+   // Test the canonical case: main tree and friend trees have all equal size.
    RDatasetSpec spec;
-   // pick the second sample as the main chain, so that can test shorter, equal-sized, longer friends
    spec.AddSample({data[1].name, data[1].trees[0][0].tree, data[1].fileGlobs});
-   for (const auto &d : data) {
-      std::vector<std::string> treeNames{};
-      std::vector<std::string> fileGlobs{};
-      std::vector<std::string> treeNamesExpanded{};
-      std::vector<std::string> fileGlobsExpanded{};
-      for (auto i = 0u; i < d.fileGlobs.size(); ++i) {
-         for (const auto &t : d.trees[i]) {
-            treeNamesExpanded.emplace_back(t.tree);
-            fileGlobsExpanded.emplace_back(t.file);
-         }
-         treeNames.emplace_back(d.trees[i][0].tree);
-         fileGlobs.emplace_back(d.fileGlobs[i]);
+   const auto &d = data[1];
+   std::vector<std::string> treeNames{};
+   std::vector<std::string> fileGlobs{};
+   std::vector<std::string> treeNamesExpanded{};
+   std::vector<std::string> fileGlobsExpanded{};
+   for (auto i = 0u; i < d.fileGlobs.size(); ++i) {
+      for (const auto &t : d.trees[i]) {
+         treeNamesExpanded.emplace_back(t.tree);
+         fileGlobsExpanded.emplace_back(t.file);
       }
-      spec.WithGlobalFriends(treeNames, fileGlobs, "friend_glob_" + d.name);
-      spec.WithGlobalFriends(treeNamesExpanded, fileGlobsExpanded, "friend_expanded_" + d.name);
-   }
-   auto df = RDataFrame(spec);
-   std::unordered_map<std::string, ROOT::RDF::RResultPtr<std::vector<ULong64_t>>> res;
-   for (const auto &d : data) {
-      res["friend_glob_" + d.name + "x"] = df.Take<ULong64_t>("friend_glob_" + d.name + ".x");
-      res["friend_expanded_" + d.name + "x"] = df.Take<ULong64_t>("friend_expanded_" + d.name + ".x");
+      treeNames.emplace_back(d.trees[i][0].tree);
+      fileGlobs.emplace_back(d.fileGlobs[i]);
    }
+   spec.WithGlobalFriends(treeNames, fileGlobs, "friend_glob_" + d.name);
+   spec.WithGlobalFriends(treeNamesExpanded, fileGlobsExpanded, "friend_expanded_" + d.name);
 
-   for (auto i = 0u; i < data.size(); ++i) {
-      std::sort(res["friend_glob_" + data[i].name + "x"]->begin(), res["friend_glob_" + data[i].name + "x"]->end());
-      std::sort(res["friend_expanded_" + data[i].name + "x"]->begin(),
-                res["friend_expanded_" + data[i].name + "x"]->end());
-      // case i = 0 is shorter friend, which currently leads to undesired behaviour
-      // see: https://github.com/root-project/root/issues/9137
-      if (i > 0) {
-         EXPECT_VEC_SEQ_EQ(*res["friend_glob_" + data[i].name + "x"],
-                           ROOT::TSeq<ULong64_t>(data[i].sampleStart, data[i].sampleStart + 24));
-      }
+   auto df = RDataFrame(spec);
+   auto take_glob = df.Take<ULong64_t>("friend_glob_" + d.name + ".x");
+   auto take_expanded = df.Take<ULong64_t>("friend_expanded_" + d.name + ".x");
+   std::sort(take_glob->begin(), take_glob->end());
+   std::sort(take_expanded->begin(), take_expanded->end());
+   EXPECT_VEC_SEQ_EQ(*take_glob, ROOT::TSeq<ULong64_t>(d.sampleStart, d.sampleStart + 24));
+   EXPECT_VEC_SEQ_EQ(*take_expanded, ROOT::TSeq<ULong64_t>(d.sampleStart, d.sampleStart + 24));
+}
+
+TEST_P(RDatasetSpecTest, FriendsLonger)
+{
+   // Test the case where there are still entries in the friend trees after
+   // processing of the main tree finishes. This should issue a warning
+   RDatasetSpec spec;
+   spec.AddSample({data[0].name, data[0].trees[0][0].tree, data[0].fileGlobs});
+   const auto &d = data[1];
+   std::vector<std::string> treeNames{};
+   std::vector<std::string> fileGlobs{};
+   for (auto i = 0u; i < d.fileGlobs.size(); ++i) {
+      treeNames.emplace_back(d.trees[i][0].tree);
+      fileGlobs.emplace_back(d.fileGlobs[i]);
+   }
+   spec.WithGlobalFriends(treeNames, fileGlobs, "friend_glob_" + d.name);
+
+   auto df = RDataFrame(spec);
+   auto take_glob = df.Take<ULong64_t>("friend_glob_" + d.name + ".x");
+   // The warning about longer friend only makes sense for the single-threaded case
+   if (!GetParam())
+      ROOT_EXPECT_WARNING(*take_glob, "SetEntryBase",
+                          "Last entry available from main tree '' was 14 but friend tree 'subTree' has more entries "
+                          "beyond the end of the main tree.");
+}
+
+TEST_P(RDatasetSpecTest, FriendsShorter)
+{
+   // Test the case where the friend trees are shorter than the main one.
+   // This should throw an exception.
+   RDatasetSpec spec;
+   spec.AddSample({data[1].name, data[1].trees[0][0].tree, data[1].fileGlobs});
+   const auto &d = data[0];
+   std::vector<std::string> treeNames{};
+   std::vector<std::string> fileGlobs{};
+   for (auto i = 0u; i < d.fileGlobs.size(); ++i) {
+      treeNames.emplace_back(d.trees[i][0].tree);
+      fileGlobs.emplace_back(d.fileGlobs[i]);
+   }
+   spec.WithGlobalFriends(treeNames, fileGlobs, "friend_glob_" + d.name);
+
+   auto df = RDataFrame(spec);
+   auto take_glob = df.Take<ULong64_t>("friend_glob_" + d.name + ".x");
+   try {
+      *take_glob;
+   } catch (const std::runtime_error &err) {
+      const std::string msg = "Cannot read entry 15 from friend tree 'tree'. The friend tree has less entries than the "
+                              "main tree. Make sure all trees of the dataset have the same number of entries.";
+      EXPECT_STREQ(err.what(), msg.c_str());
    }
 }
 
@@ -387,8 +475,7 @@ TEST_P(RDatasetSpecTest, Histo1D)
    RDatasetSpec spec;
    spec.AddSample({"real0", "tree"s, {"specTestFile0.root"s}});
    spec.AddSample({"real1", {{"tree"s, "specTestFile00*.root"s}}});
-   // 1 friend with entries from 15 up to 39 -> shortened to have the size of the main chain
-   spec.WithGlobalFriends({{"subTree"s, "specTestFile1*.root"s}}, "friend"s);
+   spec.WithGlobalFriends("subTree", std::vector<std::string>{"specTestFile1.root", "specTestFile11.root"}, "friend");
    ROOT::RDataFrame d(spec);
 
    auto h1 = d.Histo1D(::TH1D("h1", "h1", 10, 0, 10), "x");
@@ -434,8 +521,7 @@ TEST_P(RDatasetSpecTest, FilterDependingOnVariation)
    RDatasetSpec spec;
    spec.AddSample({"real0", "tree"s, {"specTestFile0.root"s}});
    spec.AddSample({"real1", {{"tree"s, "specTestFile00*.root"s}}});
-   // 1 friend with entries from 15 up to 39 -> shortened to have the size of the main chain
-   spec.WithGlobalFriends({{"subTree"s, "specTestFile1*.root"s}}, "friend"s);
+   spec.WithGlobalFriends("subTree", std::vector<std::string>{"specTestFile1.root", "specTestFile11.root"}, "friend");
    ROOT::RDataFrame df(spec);
 
    auto sum = df.Vary(
@@ -486,7 +572,7 @@ TEST_P(RDatasetSpecTest, SaveGraph)
    static const std::string expectedGraph(
       "digraph {\n"
       "\t1 [label=\"Sum\", style=\"filled\", fillcolor=\"#e47c7e\", shape=\"box\"];\n"
-      "\t0 [label=\"\", style=\"filled\", fillcolor=\"#f4b400\", shape=\"ellipse\"];\n"
+      "\t0 [label=\"TTreeDS\", style=\"filled\", fillcolor=\"#f4b400\", shape=\"ellipse\"];\n"
       "\t2 [label=\"Sum\", style=\"filled\", fillcolor=\"#e47c7e\", shape=\"box\"];\n"
       "\t0 -> 1;\n"
       "\t0 -> 2;\n"
@@ -499,10 +585,10 @@ TEST(RDatasetSpecTest, Describe)
    auto dfWriter1 = RDataFrame(10)
                        .Define("x", [](ULong64_t e) { return double(e); }, {"rdfentry_"})
                        .Define("w", [](ULong64_t e) { return e + 1.; }, {"rdfentry_"});
-   dfWriter1.Range(5).Snapshot<double>("subTree0", "specTestDescribe1.root", {"x"});
-   dfWriter1.Range(5, 10).Snapshot<double>("subTree1", "specTestDescribe2.root", {"x"});
-   dfWriter1.Range(5).Snapshot<double>("subTree2", "specTestDescribe3.root", {"w"});
-   dfWriter1.Range(5, 10).Snapshot<double>("subTree3", "specTestDescribe4.root", {"w"});
+   dfWriter1.Range(5).Snapshot("subTree0", "specTestDescribe1.root", {"x"});
+   dfWriter1.Range(5, 10).Snapshot("subTree1", "specTestDescribe2.root", {"x"});
+   dfWriter1.Range(5).Snapshot("subTree2", "specTestDescribe3.root", {"w"});
+   dfWriter1.Range(5, 10).Snapshot("subTree3", "specTestDescribe4.root", {"w"});
 
    RDatasetSpec spec;
    spec.AddSample({"sampleA", "subTree0"s, "specTestDescribe1.root"s});
@@ -512,7 +598,7 @@ TEST(RDatasetSpecTest, Describe)
    auto res0 = df.Sum<double>("x");
    auto res1 = df.Sum<double>("w");
 
-   static const std::string expectedDescribe("Dataframe from TChain  in files\n"
+   static const std::string expectedDescribe("Dataframe from TChain in files\n"
                                              "  specTestDescribe1.root\n"
                                              "  specTestDescribe2.root\n"
                                              "with friend\n"
@@ -530,7 +616,7 @@ TEST(RDatasetSpecTest, Describe)
                                              "Column  Type            Origin\n"
                                              "------  ----            ------\n"
                                              "w       Double_t        Dataset\n"
-                                             "x       Double_t        Dataset");
+                                             "x       Double_t        Dataset\n");
    EXPECT_EQ(expectedDescribe, df.Describe().AsString());
    for (auto i = 12u; i < 16; ++i)
       gSystem->Unlink(("specTestFile" + std::to_string(i) + ".root").c_str());
@@ -539,12 +625,12 @@ TEST(RDatasetSpecTest, Describe)
 TEST(RDatasetSpecTest, FromSpec)
 {
    auto dfWriter0 = ROOT::RDataFrame(5).Define("z", [](ULong64_t e) { return e + 100; }, {"rdfentry_"});
-   dfWriter0.Range(0, 2).Snapshot<ULong64_t>("subTree", "PYspecTestFile2.root", {"z"});
-   dfWriter0.Range(2, 4).Snapshot<ULong64_t>("subTree", "PYspecTestFile3.root", {"z"});
-   dfWriter0.Range(4, 5).Snapshot<ULong64_t>("subTree", "PYspecTestFile4.root", {"z"});
-   dfWriter0.Range(0, 2).Snapshot<ULong64_t>("subTree1", "PYspecTestFile5.root", {"z"});
-   dfWriter0.Range(2, 4).Snapshot<ULong64_t>("subTree2", "PYspecTestFile6.root", {"z"});
-   dfWriter0.Snapshot<ULong64_t>("anotherTree", "PYspecTestFile7.root", {"z"});
+   dfWriter0.Range(0, 2).Snapshot("subTree", "CPPspecTestFile2.root", {"z"});
+   dfWriter0.Range(2, 4).Snapshot("subTree", "CPPspecTestFile3.root", {"z"});
+   dfWriter0.Range(4, 5).Snapshot("subTree", "CPPspecTestFile4.root", {"z"});
+   dfWriter0.Range(0, 2).Snapshot("subTree1", "CPPspecTestFile5.root", {"z"});
+   dfWriter0.Range(2, 4).Snapshot("subTree2", "CPPspecTestFile6.root", {"z"});
+   dfWriter0.Snapshot("anotherTree", "CPPspecTestFile7.root", {"z"});
 
    auto rdf =
       FromSpec("spec.json")
@@ -575,7 +661,42 @@ TEST(RDatasetSpecTest, FromSpec)
    }
 
    for (auto i = 2u; i < 8u; ++i)
-      gSystem->Unlink(("PYspecTestFile" + std::to_string(i) + ".root").c_str());
+      gSystem->Unlink(("CPPspecTestFile" + std::to_string(i) + ".root").c_str());
+}
+
+TEST(RDatasetSpecTest, FromSpec_ordering_samplesAndFriends)
+{
+   auto dfWriter0 = ROOT::RDataFrame(1).Define("z", [](ULong64_t e) { return e + 100; }, {"rdfentry_"});
+   dfWriter0.Snapshot("subTree", "FromSpecTestFile1.root", {"z"});
+   dfWriter0.Snapshot("subTree", "FromSpecTestFile2.root", {"z"});
+   dfWriter0.Snapshot("anotherTree", "FromSpecTestFile4.root", {"z"});
+   dfWriter0.Snapshot("anotherTree", "FromSpecTestFile3.root", {"z"});
+
+   auto rdf_1 = FromSpec("spec_ordering_samples_withFriends.json");
+
+   static const std::string expectedDescribe("Dataframe from TChain in files\n"
+                                             "  FromSpecTestFile2.root\n"
+                                             "  FromSpecTestFile1.root\n"
+                                             "with friends\n"
+                                             "   (friendTree2) FromSpecTestFile4.root\n"
+                                             "   (friendTree1) FromSpecTestFile3.root\n"
+                                             "\n"
+                                             "Property                Value\n"
+                                             "--------                -----\n"
+                                             "Columns in total            3\n"
+                                             "Columns from defines        0\n"
+                                             "Event loops run             0\n"
+                                             "Processing slots            1\n"
+                                             "\n"
+                                             "Column          Type            Origin\n"
+                                             "------          ----            ------\n"
+                                             "friendTree1.z   ULong64_t       Dataset\n"
+                                             "friendTree2.z   ULong64_t       Dataset\n"
+                                             "z               ULong64_t       Dataset\n");
+   EXPECT_EQ(expectedDescribe, rdf_1.Describe().AsString());
+
+   for (auto i = 1u; i < 5; ++i)
+      gSystem->Unlink(("FromSpecTestFile" + std::to_string(i) + ".root").c_str());
 }
 
 TEST(RMetaData, SimpleOperations)
@@ -637,12 +758,12 @@ TEST(RDatasetSpecTest, Clusters)
    opts[2].fAutoFlush = 7;  // last split is always shorter
    opts[3].fAutoFlush = 1;  // each entry is a split
    for (auto &opt : opts) {
-      dfWriter2.Range(40).Snapshot<ULong64_t>("mainA", "CspecTestFile0.root", {"x"}, opt);
-      dfWriter2.Range(40, 120).Snapshot<ULong64_t>("mainB", "CspecTestFile1.root", {"x"}, opt);
-      dfWriter2.Range(120, 200).Snapshot<ULong64_t>("mainC", "CspecTestFile2.root", {"x"}, opt);
-      dfWriter2.Range(50).Snapshot<ULong64_t>("friendA", "CspecTestFile3.root", {"x"}, opt);
-      dfWriter2.Range(50, 150).Snapshot<ULong64_t>("friendB", "CspecTestFile4.root", {"x"}, opt);
-      dfWriter2.Range(150, 200).Snapshot<ULong64_t>("friendC", "CspecTestFile5.root", {"x"}, opt);
+      dfWriter2.Range(40).Snapshot("mainA", "CspecTestFile0.root", {"x"}, opt);
+      dfWriter2.Range(40, 120).Snapshot("mainB", "CspecTestFile1.root", {"x"}, opt);
+      dfWriter2.Range(120, 200).Snapshot("mainC", "CspecTestFile2.root", {"x"}, opt);
+      dfWriter2.Range(50).Snapshot("friendA", "CspecTestFile3.root", {"x"}, opt);
+      dfWriter2.Range(50, 150).Snapshot("friendB", "CspecTestFile4.root", {"x"}, opt);
+      dfWriter2.Range(150, 200).Snapshot("friendC", "CspecTestFile5.root", {"x"}, opt);
 
       ROOT::EnableImplicitMT(std::min(4u, std::thread::hardware_concurrency()));
 
@@ -678,6 +799,266 @@ TEST(RDatasetSpecTest, Clusters)
 }
 #endif
 
+TEST_P(RDatasetSpecTest, TreeInSubdir)
+{
+   class FileRAII {
+   private:
+      std::string fPath;
+
+   public:
+      explicit FileRAII(const std::string &path) : fPath(path) {}
+      FileRAII(const FileRAII &) = delete;
+      FileRAII &operator=(const FileRAII &) = delete;
+      ~FileRAII() { std::remove(fPath.c_str()); }
+      auto GetPath() const { return fPath.c_str(); }
+   };
+   FileRAII fraii("definepersample_treeinsubdir.root");
+   // Write TTree in TFile subdirectory
+   {
+      TFile f{fraii.GetPath(), "recreate"};
+      auto *outDir = f.mkdir("subdir");
+
+      outDir->cd();
+      int event{42};
+      TTree tree("T", "T");
+      tree.Branch("event", &event, "event/I");
+      tree.Fill();
+      tree.SetDirectory(outDir);
+      f.Write();
+   }
+
+   RDatasetSpec spec;
+   spec.AddSample({"mysample", {{"subdir/T", fraii.GetPath()}}});
+   ROOT::RDataFrame df{spec};
+   auto df_sample =
+      df.DefinePerSample("sample_id", [](unsigned int, const ROOT::RDF::RSampleInfo &id) { return id.AsString(); });
+   auto sample_ids = df_sample.Take<std::string>("sample_id");
+   EXPECT_EQ(sample_ids->size(), 1);
+   EXPECT_EQ(sample_ids->at(0), fraii.GetPath() + std::string("/subdir/T"));
+}
+
+// // Tests with RNTuple
+struct InputRNTuplesRAII {
+   unsigned int fNFiles = 0;
+   std::string fPrefix;
+
+   InputRNTuplesRAII(unsigned int nFiles, std::string prefix) : fNFiles(nFiles), fPrefix(std::move(prefix))
+   {
+      unsigned int fNEntries{5};
+      for (auto i = 0u; i < fNFiles; ++i) {
+         auto model = ROOT::RNTupleModel::Create();
+         auto fldX = model->MakeField<int>("x");
+         auto fn = fPrefix + std::to_string(i) + ".root";
+         auto ntpl = ROOT::RNTupleWriter::Recreate(std::move(model), "ntuple", fn);
+         for (ULong64_t entry = 0; entry < fNEntries; entry++) {
+            *fldX = entry;
+            ntpl->Fill();
+         }
+      }
+   }
+   ~InputRNTuplesRAII()
+   {
+      for (auto i = 0u; i < fNFiles; ++i)
+         std::remove((fPrefix + std::to_string(i) + ".root").c_str());
+   }
+};
+
+struct InputRNTuplesRAIIRanges {
+   unsigned int fNFiles = 0;
+   std::string fPrefix;
+   InputRNTuplesRAIIRanges(unsigned int nFiles, std::string prefix) : fNFiles(nFiles), fPrefix(std::move(prefix))
+   {
+      unsigned int fNEntries{5};
+      for (auto i = 0u; i < fNFiles; ++i) {
+         auto model = ROOT::RNTupleModel::Create();
+         auto fldX = model->MakeField<int>("x");
+         auto fn = fPrefix + std::to_string(i) + ".root";
+         auto ntpl = ROOT::RNTupleWriter::Recreate(std::move(model), "ntuple", fn);
+         for (ULong64_t entry = 0; entry < fNEntries; entry++) {
+            *fldX = i * entry;
+            ntpl->Fill();
+         }
+      }
+   }
+   ~InputRNTuplesRAIIRanges()
+   {
+      for (auto i = 0u; i < fNFiles; ++i)
+         std::remove((fPrefix + std::to_string(i) + ".root").c_str());
+   }
+};
+
+TEST_P(RDatasetSpecTest, RNTupleSingle)
+{
+   const std::string prefix = "rdatasetspec_rntuple";
+   InputRNTuplesRAII file(1u, prefix);
+   auto samp = ROOT::RDF::Experimental::RSample("mysample", "ntuple", prefix + "0.root");
+   RDatasetSpec spec;
+   spec.AddSample(samp);
+   auto df1 = ROOT::RDataFrame(spec);
+   auto count = df1.Filter("x > 3").Count();
+   EXPECT_EQ(count.GetValue(), 1);
+}
+
+TEST_P(RDatasetSpecTest, RNTupleMultiple)
+{
+   const std::string prefix = "rdatasetspec_rntuple";
+   InputRNTuplesRAII file(4u, prefix);
+   ROOT::RDF::Experimental::RMetaData meta, meta1, meta2;
+   meta.Add("lum", 10.0);
+   meta1.Add("lum", 20.0);
+   meta2.Add("lum", 30.0);
+   auto samp = ROOT::RDF::Experimental::RSample("mysample", "ntuple",
+                                                std::vector<std::string>{prefix + "0.root", prefix + "3.root"}, meta);
+   auto samp1 = ROOT::RDF::Experimental::RSample("mysample1", "ntuple", prefix + "1.root", meta1);
+   auto samp2 = ROOT::RDF::Experimental::RSample("mysample2", "ntuple", prefix + "2.root", meta2);
+   RDatasetSpec spec;
+   spec.AddSample(samp);
+   spec.AddSample(samp1);
+   spec.AddSample(samp2);
+   auto df1 = ROOT::RDataFrame(spec);
+
+   auto df_final = df1.Filter("x > 3").Count();
+
+   auto definepersamp =
+      df1.DefinePerSample("lum", [](unsigned int, const ROOT::RDF::RSampleInfo &id) { return id.GetD("lum"); });
+   auto df_filtered = definepersamp.Filter("lum == 10.").Count();
+
+   EXPECT_EQ(df_final.GetValue(), 4);
+   EXPECT_EQ(df_filtered.GetValue(), 10);
+}
+
+TEST_P(RDatasetSpecTest, RNTupleWithGlobalRanges)
+{
+   const std::string prefix = "rdatasetspec_ranges_rntuple";
+   InputRNTuplesRAIIRanges file(5u, prefix);
+   ROOT::RDF::Experimental::RMetaData meta, meta1, meta2;
+   meta.Add("lum", 10.0);
+   meta1.Add("lum", 20.0);
+   meta2.Add("lum", 30.0);
+   auto samp = ROOT::RDF::Experimental::RSample(
+      "mysample", "ntuple",
+      std::vector<std::string>{"rdatasetspec_ranges_rntuple1.root", "rdatasetspec_ranges_rntuple4.root"}, meta);
+   auto samp1 = ROOT::RDF::Experimental::RSample("mysample1", "ntuple", "rdatasetspec_ranges_rntuple2.root", meta1);
+   auto samp2 = ROOT::RDF::Experimental::RSample("mysample2", "ntuple", "rdatasetspec_ranges_rntuple3.root", meta2);
+   RDatasetSpec spec;
+   spec.AddSample(samp);
+   spec.AddSample(samp1);
+   spec.AddSample(samp2);
+   auto df1 = ROOT::RDataFrame(spec);
+
+   std::vector<RDatasetSpec::REntryRange> goodRanges = {{1, 4}, {2, 7}, {6, 19}, {16, 20}};
+
+   auto df_final = df1.Filter("x > 3").Count();
+
+   auto definepersamp =
+      df1.DefinePerSample("lum", [](unsigned int, const ROOT::RDF::RSampleInfo &id) { return id.GetD("lum"); });
+   auto df_filtered = definepersamp.Filter("lum == 10.").Count();
+
+   auto df = RDataFrame(spec.WithGlobalRange(goodRanges[0]));
+   auto filt = df.Filter("rdfentry_ == 2");
+   auto result = filt.Take<ULong64_t>("x");
+   auto res = result.GetValue();
+   auto count_entries = df.Count().GetValue();
+   EXPECT_EQ(res[0], 2);
+   EXPECT_EQ(count_entries, 3);
+
+   auto df2 = RDataFrame(spec.WithGlobalRange(goodRanges[1]));
+   auto filt2 = df2.Filter("rdfentry_ == 3");
+   auto result2 = filt2.Take<ULong64_t>("x");
+   auto res2 = result2.GetValue();
+   auto count_entries_2 = df2.Count().GetValue();
+   EXPECT_EQ(res2[0], 3);
+   EXPECT_EQ(count_entries_2, 5);
+
+   auto df3 = RDataFrame(spec.WithGlobalRange(goodRanges[2]));
+   auto filt3 = df3.Filter("rdfentry_ == 8");
+   auto result3 = filt3.Take<ULong64_t>("x");
+   auto res3 = result3.GetValue();
+   auto count_entries_3 = df3.Count().GetValue();
+   EXPECT_EQ(res3[0], 12);
+   EXPECT_EQ(count_entries_3, 13);
+
+   auto df4 = RDataFrame(spec.WithGlobalRange(goodRanges[3]));
+   auto filt4 = df4.Filter("rdfentry_ == 19");
+   auto result4 = filt4.Take<ULong64_t>("x");
+   auto res4 = result4.GetValue();
+   auto count_entries_4 = df4.Count().GetValue();
+   EXPECT_EQ(res4[0], 12);
+   EXPECT_EQ(count_entries_4, 4);
+
+   EXPECT_EQ(df_final.GetValue(), 11);
+   EXPECT_EQ(df_filtered.GetValue(), 10);
+}
+
+TEST_P(RDatasetSpecTest, FromSpecRNTuple)
+{
+   const std::string prefix = "rdatasetspec_rntuple";
+   InputRNTuplesRAII file(3u, prefix);
+   auto df_fromspec = ROOT::RDF::Experimental::FromSpec("spec_rntuple.json");
+   auto df_final = df_fromspec.Filter("x > 3").Count();
+   auto definepersamp =
+      df_fromspec.DefinePerSample("lum", [](unsigned int, const ROOT::RDF::RSampleInfo &id) { return id.GetD("lum"); });
+   auto df_filtered = definepersamp.Filter("lum == 20.").Count();
+
+   EXPECT_EQ(df_final.GetValue(), 3);
+   EXPECT_EQ(df_filtered.GetValue(), 5);
+}
+
+TEST_P(RDatasetSpecTest, RNTupleWrong)
+{
+   const std::string prefix = "rdatasetspec_rntuple";
+   InputRNTuplesRAII file(1u, prefix);
+
+   auto model = ROOT::RNTupleModel::Create();
+   auto fldX = model->MakeField<int>("x");
+   auto ntpl = ROOT::RNTupleWriter::Recreate(std::move(model), "mytuple", "rntuple_wrong.root");
+   *fldX = 2;
+   ntpl->Fill();
+
+   EXPECT_THROW(
+      try {
+         auto samp = ROOT::RDF::Experimental::RSample("mysample", "ntuple", prefix + "0.root");
+         auto samp1 = ROOT::RDF::Experimental::RSample("mysample1", "mytuple", "rntuple_wrong.root");
+
+         RDatasetSpec spec;
+         spec.AddSample(samp);
+         spec.AddSample(samp1);
+         auto df_error = ROOT::RDataFrame(spec);
+      }
+
+      catch (const std::runtime_error &err) {
+         EXPECT_EQ(std::string(err.what()),
+                   "More than one RNTuple name was found, please make sure to use RNTuples with the same name.");
+         throw;
+      },
+      std::runtime_error);
+}
+
+TEST_P(RDatasetSpecTest, CompareWithRNTupleReader)
+{
+   const std::string prefix = "rdatasetspec_rntuple";
+   InputRNTuplesRAII file(1u, prefix);
+
+   auto model = ROOT::RNTupleModel::Create();
+   std::shared_ptr<int> x = model->MakeField<int>("x");
+   auto reader = ROOT::RNTupleReader::Open(std::move(model), "ntuple", "rdatasetspec_rntuple0.root");
+
+   TH1I h("h", "x", 5, 0, 5);
+   for (auto i = 0u; i < 5; ++i) {
+      reader->LoadEntry(i);
+      h.Fill(*x);
+   }
+
+   auto samp = ROOT::RDF::Experimental::RSample("mysample", "ntuple", "rdatasetspec_rntuple0.root");
+   RDatasetSpec spec;
+   spec.AddSample(samp);
+   auto df1 = ROOT::RDataFrame(spec);
+
+   auto mean = df1.Mean("x");
+
+   EXPECT_FLOAT_EQ(h.GetMean(), mean.GetValue());
+}
+
 // instantiate single-thread tests
 INSTANTIATE_TEST_SUITE_P(Seq, RDatasetSpecTest, ::testing::Values(false));
 
