@@ -13,6 +13,10 @@
 #include <string>
 
 #include "gtest/gtest.h"
+
+#include <TFile.h>
+#include <TTree.h>
+
 using namespace ROOT;
 using namespace ROOT::RDF;
 using namespace ROOT::VecOps;
@@ -753,8 +757,8 @@ TEST(RDFHelpers, ProgressHelper_Existence_ST)
    std::streambuf *oldCoutStreamBuf = std::cout.rdbuf();
    std::ostringstream strCout;
    std::cout.rdbuf(strCout.rdbuf());
-   auto d_write_1 = ROOT::RDataFrame(10000).Define("x", ret42).Snapshot<int>("tree", "fh1.root", {"x"});
-   auto d_write_2 = ROOT::RDataFrame(10000).Define("y", ret1).Snapshot<int>("tree", "fh2.root", {"y"});
+   auto d_write_1 = ROOT::RDataFrame(10000).Define("x", ret42).Snapshot("tree", "fh1.root", {"x"});
+   auto d_write_2 = ROOT::RDataFrame(10000).Define("y", ret1).Snapshot("tree", "fh2.root", {"y"});
    ROOT::RDF::RNode d = ROOT::RDataFrame("tree", {"fh1.root", "fh2.root"});
    ROOT::RDF::Experimental::AddProgressBar(d);
    d.Count().GetValue();
@@ -819,8 +823,8 @@ TEST(RDFHelpers, ProgressHelper_Existence_MT)
    std::streambuf *oldCoutStreamBuf = std::cout.rdbuf();
    std::ostringstream strCout;
    std::cout.rdbuf(strCout.rdbuf());
-   auto d_write_1 = ROOT::RDataFrame(10000).Define("x", ret42).Snapshot<int>("tree", "fh1.root", {"x"});
-   auto d_write_2 = ROOT::RDataFrame(10000).Define("y", ret1).Snapshot<int>("tree", "fh2.root", {"y"});
+   auto d_write_1 = ROOT::RDataFrame(10000).Define("x", ret42).Snapshot("tree", "fh1.root", {"x"});
+   auto d_write_2 = ROOT::RDataFrame(10000).Define("y", ret1).Snapshot("tree", "fh2.root", {"y"});
    ROOT::RDF::RNode d = ROOT::RDataFrame("tree", {"fh1.root", "fh2.root"});
    ROOT::RDF::Experimental::AddProgressBar(d);
    d.Count().GetValue();
@@ -837,7 +841,7 @@ TEST(RDFHelpers, ProgressHelper_existence_singleTTreeInput)
    std::streambuf *oldCoutStreamBuf = std::cout.rdbuf();
    std::ostringstream strCout;
    std::cout.rdbuf(strCout.rdbuf());
-   auto d_write_1 = ROOT::RDataFrame(10000).Define("x", ret42).Snapshot<int>("tree", "fh1.root", {"x"});
+   auto d_write_1 = ROOT::RDataFrame(10000).Define("x", ret42).Snapshot("tree", "fh1.root", {"x"});
    ROOT::RDF::RNode d = ROOT::RDataFrame("tree", {"fh1.root"});
    ROOT::RDF::Experimental::AddProgressBar(d);
    d.Count().GetValue();
@@ -846,4 +850,64 @@ TEST(RDFHelpers, ProgressHelper_existence_singleTTreeInput)
 
    EXPECT_FALSE(strCout.str().empty());
 }
+
+TEST(RDFHelpers, Histo3DThreadSafe)
+{
+   constexpr unsigned int N = 100;
+   // target mean value generated by the x sequence below:
+   constexpr double xMean = ((N - 1) / 2.) / N * 0.5;
+   constexpr double yMean = xMean + 0.1;
+   constexpr double zMean = xMean + 0.2;
+   auto addDefines = [=](auto rdf) {
+      return rdf.Define("x", [=](ULong64_t x) { return ((double)x) / N * 0.5; }, {"rdfentry_"})
+         .Define("y", [](double x) { return x + .1; }, {"x"})
+         .Define("z", [](double x) { return x + .2; }, {"x"})
+         .Define("w", []() { return 0.1; });
+   };
+
+   ROOT::RDataFrame rdf1(N);
+   auto d1 = addDefines(rdf1);
+   auto h1 = d1.Histo3D(::TH3D("h1", "h1", 10, 0, 1, 5, 0, 1, 2, 0, 1), "x", "y", "z");
+   auto h1w = d1.Histo3D(::TH3D("h1w", "h1w", 10, 0, 1, 5, 0, 1, 2, 0, 1), "x", "y", "z", "w");
+
+   // Trigger event loop before going MT:
+   h1->GetEntries();
+
+   ROOT::EnableImplicitMT(16);
+   for (unsigned int threadsPerTH3 : {1, 8, 16}) {
+      ROOT::RDF::Experimental::ThreadsPerTH3(threadsPerTH3);
+
+      ROOT::RDataFrame rdf2(N);
+      auto d2 = addDefines(rdf2);
+      auto h2 = d2.Histo3D(::TH3D("h1", "h1", 10, 0, 1, 5, 0, 1, 2, 0, 1), "x", "y", "z");
+      auto h2w = d2.Histo3D(::TH3D("h1w", "h1w", 10, 0, 1, 5, 0, 1, 2, 0, 1), "x", "y", "z", "w");
+
+      for (auto h : {h1, h2}) {
+         EXPECT_EQ(h->GetEntries(), N);
+         EXPECT_DOUBLE_EQ(h->GetSumOfWeights(), N) << "ThreadsPerTH3: " << threadsPerTH3;
+         EXPECT_NEAR(h->GetMean(1), xMean, 1.E-14);
+         EXPECT_NEAR(h->GetMean(2), yMean, 1.E-14);
+         EXPECT_NEAR(h->GetMean(3), zMean, 1.E-14);
+      }
+
+      for (auto h : {h1w, h2w}) {
+         EXPECT_EQ(h->GetEntries(), N);
+         EXPECT_NEAR(h->GetSumOfWeights(), N * 0.1, 1.E-14);
+         EXPECT_NEAR(h->GetMean(1), xMean, 1.E-14);
+         EXPECT_NEAR(h->GetMean(2), yMean, 1.E-14);
+         EXPECT_NEAR(h->GetMean(3), zMean, 1.E-14);
+      }
+
+      for (auto axis : {1, 2, 3, 11, 12, 13}) {
+         EXPECT_NEAR(h1->GetMean(axis), h2->GetMean(axis), h1->GetMean() * 1.E-14);
+         EXPECT_NEAR(h1w->GetMean(axis), h2w->GetMean(axis), h1w->GetMean() * 1.E-14);
+      }
+
+      const auto nbin = (h1->GetNbinsX() + 2) * (h1->GetNbinsY() + 2) + (h1->GetNbinsZ() + 2);
+      for (int i = 0; i < nbin; ++i) {
+         EXPECT_NEAR(h1->GetBinContent(i), h2->GetBinContent(i), fabs(h1->GetBinContent(i)) * 1.E-13);
+         EXPECT_NEAR(h1w->GetBinContent(i), h2w->GetBinContent(i), fabs(h1w->GetBinContent(i)) * 1.E-13);
+      }
+   }
+}
 #endif // R__USE_IMT
