@@ -9,6 +9,7 @@
 #include "TMemFile.h"
 #include "TSystem.h"
 #include "TTree.h"
+#include "TChain.h"
 
 #include "gtest/gtest.h"
 
@@ -92,6 +93,14 @@ TEST(RDataFrameInterface, CreateFromNonExistingTree)
    EXPECT_ANY_THROW(RDataFrame("theTreeWhichDoesNotExist", gDirectory));
 }
 
+TEST(RDataFrameInterface, CreateFromGlob)
+{
+   EXPECT_THROW(RDataFrame("t", "globTest_invalid_[0-9].root"), std::invalid_argument);
+   TreeInFileRAII f1("globTest_1.root");
+   TreeInFileRAII f2("globTest_2.root");
+   RDataFrame("t", "globTest_[1-9].root");
+}
+
 TEST(RDataFrameInterface, CreateFromTree)
 {
    TMemFile f("dataframe_interfaceAndUtils_0.root", "RECREATE");
@@ -475,9 +484,9 @@ TEST(RDataFrameInterface, TypeUnknownToInterpreter)
    auto df = ROOT::RDataFrame(1).Define("res", make_s);
    bool hasThrown = false;
    std::stringstream ss;
-   ss << "The type of custom column \"res\" (" << symbol << ") is not known to the interpreter, " <<
-         "but a just-in-time-compiled Snapshot call requires this column. Make sure to create " <<
-         "and load ROOT dictionaries for this column's class.";
+   ss << "No runtime type information is available for column \"res\" with type name \"" << symbol
+      << "\". Thus, it cannot be written to disk with Snapshot. Make sure to generate and load ROOT dictionaries for "
+         "the type of this column.";
    EXPECT_RUNTIME_ERROR_WITH_MSG(
       df.Snapshot("result", "RESULT2.root"),
       ss.str().c_str());
@@ -852,9 +861,7 @@ TEST(RDataFrameInterface, BookWithoutColumns)
 
 TEST(RDataFrameInterface, SnapshotWithDuplicateColumns)
 {
-   EXPECT_THROW(
-      (ROOT::RDataFrame(1).Snapshot<ULong64_t, ULong64_t>("t", "neverwritten.root", {"rdfentry_", "rdfentry_"})),
-      std::logic_error);
+   EXPECT_THROW((ROOT::RDataFrame(1).Snapshot("t", "neverwritten.root", {"rdfentry_", "rdfentry_"})), std::logic_error);
    EXPECT_THROW((ROOT::RDataFrame(1).Snapshot("t", "neverwritten.root", {"rdfentry_", "rdfentry_"})), std::logic_error);
 }
 
