@@ -11,6 +11,8 @@
 
 #include "gtest/gtest.h"
 
+#include <TFile.h>
+
 // Fixture for all tests in this file. If parameter is true, run with implicit MT, else run sequentially
 class RDFRegressionTests : public ::testing::TestWithParam<bool> {
 protected:
@@ -77,7 +79,7 @@ TEST_P(RDFRegressionTests, MultipleTriggerRun)
    const auto treeName = "t";
    {
       ROOT::RDataFrame tdf(1);
-      tdf.Define("b1", []() { return 1U; }).Snapshot<unsigned int>(treeName, fileName, {"b1"});
+      tdf.Define("b1", []() { return 1U; }).Snapshot(treeName, fileName, {"b1"});
    }
 
    ROOT::RDataFrame d(treeName, fileName, {"b1"});
@@ -141,7 +143,7 @@ TEST_P(RDFRegressionTests, UniqueEntryNumbers)
 {
    const auto treename = "t";
    const auto fname = "df_uniqueentrynumbers.root";
-   ROOT::RDataFrame(10).Snapshot<unsigned int>(treename, fname, {"rdfslot_"}); // does not matter what column we write
+   ROOT::RDataFrame(10).Snapshot(treename, fname, {"rdfslot_"}); // does not matter what column we write
 
    ROOT::RDataFrame df(treename, {fname, fname});
    auto entries = *df.Take<ULong64_t>("rdfentry_");
@@ -181,7 +183,7 @@ TEST_P(RDFRegressionTests, ReadWriteVector3)
    EXPECT_EQ(ha->GetMean(), 4.5);
    EXPECT_EQ(ha->GetMean(), hb->GetMean());
 
-   auto out_df = rdf.Snapshot<TVector3, TVector3>("t", snap_fname, {"a", "b"});
+   auto out_df = rdf.Snapshot("t", snap_fname, {"a", "b"});
 
    auto ha_snap = out_df->Define("aval", "a.X()").Histo1D("aval");
    auto hb_snap = out_df->Define("bval", "b.X()").Histo1D("bval");
@@ -238,7 +240,7 @@ TEST_P(RDFRegressionTests, PolymorphicTBranchObject)
    ASSERT_EQ(rdf.Count().GetValue(), 4ull);
    rdf.Foreach(checkEntries, {"o", "rdfentry_"});
 
-   auto out_df = rdf.Snapshot<TObject>("t", snap_fname, {"o"});
+   auto out_df = rdf.Snapshot("t", snap_fname, {"o"});
    out_df->Foreach(checkEntries, {"o", "rdfentry_"});
 
    TFile f(snap_fname.c_str());
