@@ -519,6 +519,15 @@ TEST_P(RDFSimpleTests, TakeCarrays)
    gSystem->Unlink(fileName);
 }
 
+TEST_P(RDFSimpleTests, HistoExtend)
+{
+   auto d = RDataFrame(10000).Define("x", "rdfentry_");
+   auto hist = d.Histo1D("x");
+   EXPECT_GE(hist->GetXaxis()->GetXmax(), 10000);
+   auto histWeighted = d.Define("w", "1").Histo1D("x", "w");
+   EXPECT_GE(histWeighted->GetXaxis()->GetXmax(), 10000);
+}
+
 TEST_P(RDFSimpleTests, Reduce)
 {
    auto d = RDataFrame(5).DefineSlotEntry("x", [](unsigned int, ULong64_t e) { return static_cast<int>(e) + 1; });
@@ -823,8 +832,8 @@ TEST_P(RDFSimpleTests, DifferentTreesInDifferentThreads)
       auto df = RDataFrame(64)
                    .Define("x", []() { return 1; })
                    .Define("y", []() { return 1; })
-                   .Snapshot<int, int>(treename, filename, {"x", "y"},
-                                       {"RECREATE", ROOT::RCompressionSetting::EAlgorithm::kZLIB, 4, 2, 99, false});
+                   .Snapshot(treename, filename, {"x", "y"},
+                             {"RECREATE", ROOT::RCompressionSetting::EAlgorithm::kZLIB, 4, 2, 99, false});
    }
 
    TFile f(filename);
@@ -862,8 +871,7 @@ TEST_P(RDFSimpleTests, ManyRangesPerWorker)
    {
       ROOT::RDataFrame(184)
          .Define("i", []() { return 0; })
-         .Snapshot<int>("t", filename, {"i"},
-                        {"RECREATE", ROOT::RCompressionSetting::EAlgorithm::kZLIB, 1, 1, 99, false});
+         .Snapshot("t", filename, {"i"}, {"RECREATE", ROOT::RCompressionSetting::EAlgorithm::kZLIB, 1, 1, 99, false});
    }
    ROOT::RDataFrame("t", filename).Mean<int>("i");
    gSystem->Unlink(filename);
@@ -884,7 +892,7 @@ TEST_P(RDFSimpleTests, NonExistingFile)
 TEST_P(RDFSimpleTests, NonExistingFileInChain)
 {
    const auto filename = "rdf_nonexistingfileinchain.root";
-   ROOT::RDataFrame(1).Define("x", [] { return 10; }).Snapshot<int>("t", filename, {"x"});
+   ROOT::RDataFrame(1).Define("x", [] { return 10; }).Snapshot("t", filename, {"x"});
 
    ROOT::RDataFrame df("t", {filename, "doesnotexist.root"});
 
@@ -1003,8 +1011,8 @@ TEST_P(RDFSimpleTests, ChainWithDifferentTreeNames)
    const auto fname1 = "test_chainwithdifferenttreenames_1.root";
    const auto fname2 = "test_chainwithdifferenttreenames_2.root";
    {
-      ROOT::RDataFrame(10).Define("x", [] { return 1; }).Snapshot<int>("t1", fname1, {"x"});
-      ROOT::RDataFrame(10).Define("x", [] { return 3; }).Snapshot<int>("t2", fname2, {"x"});
+      ROOT::RDataFrame(10).Define("x", [] { return 1; }).Snapshot("t1", fname1, {"x"});
+      ROOT::RDataFrame(10).Define("x", [] { return 3; }).Snapshot("t2", fname2, {"x"});
    }
 
    // add trees to chain
