@@ -16,6 +16,7 @@
 #include <TTreeReaderArray.h>
 
 #include "DummyHeader.hxx"
+#include "DataFrameSnapshotUtils.hxx"
 
 using namespace ROOT;              // RDataFrame
 using namespace ROOT::RDF;         // RInterface
@@ -139,11 +140,9 @@ void TestCustomBasketSize()
    ROOT::RDF::RSnapshotOptions options;
    options.fBasketSize = 2048;
 
-   df_with_new_columns.Snapshot<float, float>("tree", raii.GetOutputFileCustom(), {"branch_x", "branch_x_new"},
-                                              options);
+   df_with_new_columns.Snapshot("tree", raii.GetOutputFileCustom(), {"branch_x", "branch_x_new"}, options);
 
-   df_with_new_columns.Snapshot<std::vector<float>, std::vector<float>>("tree", raii.GetOutputFileCollection(),
-                                                                        {"branch_vec", "branch_vec_new"}, options);
+   df_with_new_columns.Snapshot("tree", raii.GetOutputFileCollection(), {"branch_vec", "branch_vec_new"}, options);
 
    TFile output_file_custom(raii.GetOutputFileCustom().c_str());
    auto output_tree_custom = output_file_custom.Get<TTree>("tree");
@@ -198,13 +197,13 @@ void TestBasketSizePreservation()
       ROOT::RDataFrame df("tree", helper.GetInputFile());
       ROOT::RDF::RSnapshotOptions options;
       options.fBasketSize = 64000; // 64KB
-      df.Snapshot<float, std::vector<float>>("tree", helper.GetOutputFileCustom(), columns, options);
+      df.Snapshot("tree", helper.GetOutputFileCustom(), columns, options);
    }
 
    // Now read that tree and create new snapshot without specifying basket size
    {
       ROOT::RDataFrame df("tree", helper.GetOutputFileCustom());
-      df.Snapshot<float, std::vector<float>>("tree", helper.GetOutputFileCollection(), columns);
+      df.Snapshot("tree", helper.GetOutputFileCollection(), columns);
    }
 
    // Open both files and compare basket sizes
@@ -251,8 +250,9 @@ TEST(RDFSnapshotMore, BasketSizePreservation)
 // fixture that provides fixed and variable sized arrays as RDF columns
 class RDFSnapshotArrays : public ::testing::Test {
 protected:
-   const static unsigned int kNEvents = 10u;
-   static const std::vector<std::string> kFileNames;
+   static constexpr unsigned int kNEvents = 10u;
+   static constexpr unsigned int kFixedSize = 4u;
+   static inline const std::vector<std::string> kFileNames = {"test_snapshotarray1.root", "test_snapshotarray2.root"};
 
    static void SetUpTestCase()
    {
@@ -264,18 +264,17 @@ protected:
          TTree t("arrayTree", "arrayTree");
 
          // doubles, floats
-         const unsigned int fixedSize = 4u;
-         float fixedSizeArr[fixedSize];
-         t.Branch("fixedSizeArr", fixedSizeArr, ("fixedSizeArr[" + std::to_string(fixedSize) + "]/F").c_str());
+         float fixedSizeArr[kFixedSize];
+         t.Branch("fixedSizeArr", fixedSizeArr, ("fixedSizeArr[" + std::to_string(kFixedSize) + "]/F").c_str());
          unsigned int size = 0u;
          t.Branch("size", &size);
          double *varSizeArr = new double[eventsPerFile * 100u];
          t.Branch("varSizeArr", varSizeArr, "varSizeArr[size]/D");
 
          // bools. std::vector<bool> makes bool treatment in RDF special
-         bool fixedSizeBoolArr[fixedSize];
+         bool fixedSizeBoolArr[kFixedSize];
          t.Branch("fixedSizeBoolArr", fixedSizeBoolArr,
-                  ("fixedSizeBoolArr[" + std::to_string(fixedSize) + "]/O").c_str());
+                  ("fixedSizeBoolArr[" + std::to_string(kFixedSize) + "]/O").c_str());
          bool *varSizeBoolArr = new bool[eventsPerFile * 100u];
          t.Branch("varSizeBoolArr", varSizeBoolArr, "varSizeBoolArr[size]/O");
 
@@ -306,7 +305,6 @@ protected:
          gSystem->Unlink(fname.c_str());
    }
 };
-const std::vector<std::string> RDFSnapshotArrays::kFileNames = {"test_snapshotarray1.root", "test_snapshotarray2.root"};
 
 /********* SINGLE THREAD TESTS ***********/
 
@@ -329,7 +327,7 @@ TEST_F(RDFSnapshot, Snapshot_aliases)
    auto tdfa = tdf.Alias(alias0, "ans");
    auto tdfb = tdfa.Define("vec", [] { return RVec<int>{1, 2, 3}; }).Alias(alias1, "vec");
    testing::internal::CaptureStderr();
-   auto snap = tdfb.Snapshot<int, RVec<int>>("mytree", "Snapshot_aliases.root", {alias0, alias1});
+   auto snap = tdfb.Snapshot("mytree", "Snapshot_aliases.root", {alias0, alias1});
    std::string err = testing::internal::GetCapturedStderr();
    EXPECT_TRUE(err.empty()) << err;
    auto names = snap->GetColumnNames();
@@ -357,7 +355,7 @@ void TestSnapshotUpdate(RInterface<RLoopManager> &tdf, const std::string &outfil
 {
    // test snapshotting two trees to the same file opened in "UPDATE" mode
    auto df = tdf.Define("x", [] { return 10; });
-   auto s1 = df.Snapshot<int>(tree1, outfile, {"x"});
+   auto s1 = df.Snapshot(tree1, outfile, {"x"});
 
    auto c1 = s1->Count();
    auto mean1 = s1->Mean<int>("x");
@@ -367,7 +365,7 @@ void TestSnapshotUpdate(RInterface<RLoopManager> &tdf, const std::string &outfil
    RSnapshotOptions opts;
    opts.fMode = "UPDATE";
    opts.fOverwriteIfExists = overwriteIfExists;
-   auto s2 = ROOT::RDataFrame(50ull).Define("x", [] { return 10; }).Snapshot<int>(tree2, outfile, {"x"}, opts);
+   auto s2 = ROOT::RDataFrame(50ull).Define("x", [] { return 10; }).Snapshot(tree2, outfile, {"x"}, opts);
 
    auto c2 = s2->Count();
    auto mean2 = s2->Mean<int>("x");
@@ -423,7 +421,7 @@ void test_snapshot_options(RInterface<RLoopManager> &tdf)
    for (auto algorithm : {RCAlgo::kZLIB, RCAlgo::kLZMA, RCAlgo::kLZ4, RCAlgo::kZSTD}) {
       opts.fCompressionAlgorithm = algorithm;
 
-      auto s = tdf.Snapshot<int>("t", outfile, {"ans"}, opts);
+      auto s = tdf.Snapshot("t", outfile, {"ans"}, opts);
 
       auto c = s->Count();
       auto min = s->Min<int>("ans");
@@ -449,6 +447,18 @@ TEST_F(RDFSnapshot, Snapshot_action_with_options)
    test_snapshot_options(tdf);
 }
 
+TEST_F(RDFSnapshot, Snapshot_action_warn_on_rntuple_specific_opts)
+{
+   RSnapshotOptions opts;
+   opts.fApproxZippedClusterSize = 64;
+
+   ROOT_EXPECT_WARNING(
+      tdf.Snapshot("ntuple", "snapshot_test_warn_on_rntuple_specific_opts.root", "", opts), "Snapshot",
+      "The RNTuple-specific fApproxZippedClusterSize option in RSnapshotOptions has been set, but the output format is "
+      "set to TTree, so this option won't have any effect. Use the other options available in RSnapshotOptions to "
+      "configure the output TTree. Alternatively, change fOutputFormat to snapshot to RNTuple instead.");
+}
+
 void checkSnapshotArrayFile(RResultPtr<RInterface<RLoopManager>> &df, unsigned int kNEvents)
 {
    // fixedSizeArr and varSizeArr are RResultPtr<vector<vector<T>>>
@@ -477,18 +487,12 @@ void checkSnapshotArrayFile(RResultPtr<RInterface<RLoopManager>> &df, unsigned i
       const auto &bv = varSizeBoolArr->at(i);
       EXPECT_EQ(thisSize, dv.size());
       EXPECT_EQ(thisSize, bv.size());
-      std::cout << "bv: ";
-      for (auto j = 0u; j < thisSize; ++j)
-         std::cout << bv[j] << ' ';
-      std::cout << "\nexpected: ";
       for (auto j = 0u; j < thisSize; ++j) {
          EXPECT_DOUBLE_EQ(dv[j], i * j);
          const bool value = bv[j];
          const bool expected = j % 2 == 0;
-         std::cout << expected << ' ';
          EXPECT_EQ(value, expected);
       }
-      std::cout << '\n';
    }
 }
 
@@ -497,9 +501,8 @@ TEST_F(RDFSnapshotArrays, SingleThread)
    RDataFrame tdf("arrayTree", kFileNames);
    // template Snapshot
    // "size" _must_ be listed before "varSizeArr"!
-   auto dt = tdf.Snapshot<RVec<float>, unsigned int, RVec<double>, RVec<bool>, RVec<bool>>(
-      "outTree", "test_snapshotRVecoutST.root",
-      {"fixedSizeArr", "size", "varSizeArr", "varSizeBoolArr", "fixedSizeBoolArr"});
+   auto dt = tdf.Snapshot("outTree", "test_snapshotRVecoutST.root",
+                          {"fixedSizeArr", "size", "varSizeArr", "varSizeBoolArr", "fixedSizeBoolArr"});
 
    checkSnapshotArrayFile(dt, kNEvents);
 }
@@ -515,20 +518,56 @@ TEST_F(RDFSnapshotArrays, SingleThreadJitted)
    checkSnapshotArrayFile(dj, kNEvents);
 }
 
-TEST_F(RDFSnapshotArrays, RedefineArray)
+// Run a define across a TTree boundary to trigger an update of branch addresses
+TEST_F(RDFSnapshotArrays, AddDefine)
 {
    RDataFrame df("arrayTree", kFileNames);
-   auto df2 = df.Redefine("fixedSizeArr",
-                          [] {
-                             return ROOT::RVecF{42.f, 42.f};
-                          })
-                 .Snapshot<ROOT::RVec<float>>("t", "test_snapshotRVecRedefineArray.root", {"fixedSizeArr"});
+   auto df2 = df.Define("RVecColumn",
+                        [](ULong64_t entry) {
+                           return ROOT::RVec<ULong64_t>{entry, entry + 1, entry + 2, entry + 3, entry + 4, entry + 5};
+                        },
+                        {"rdfentry_"})
+                 .Snapshot("t", "test_snapshotDefineRVec.root", {"RVecColumn"});
    df2->Foreach(
-      [](const ROOT::RVecF &v) {
-         EXPECT_EQ(v.size(), 2u); // not 4 as it was in the original input
-         EXPECT_TRUE(All(v == ROOT::RVecF{42.f, 42.f}));
+      [](const ROOT::RVec<ULong64_t> &v, ULong64_t entry) {
+         EXPECT_EQ(v.size(), 6);
+         EXPECT_FLOAT_EQ(v.front(), entry);
+
+         std::vector<ULong64_t> column(v.begin(), v.end());
+         std::vector<ULong64_t> target(6);
+         std::generate(target.begin(), target.end(), [i = entry]() mutable { return i++; });
+         EXPECT_EQ(column, target);
       },
-      {"fixedSizeArr"});
+      {"RVecColumn", "rdfentry_"});
+
+   gSystem->Unlink("test_snapshotDefineRVec.root.root");
+}
+
+TEST_F(RDFSnapshotArrays, RedefineArray)
+{
+   static constexpr unsigned int newArraySize = 6u;
+   static_assert(kFixedSize != newArraySize);
+
+   RDataFrame df("arrayTree", kFileNames);
+   auto df2 =
+      df.Redefine("fixedSizeArr",
+                  [](ULong64_t entry) {
+                     return ROOT::RVecF{float(entry), entry + 1.f, entry + 2.f, entry + 3.f, entry + 4.f, entry + 5.f};
+                  },
+                  {"rdfentry_"})
+         .Snapshot("t", "test_snapshotRVecRedefineArray.root", {"fixedSizeArr"});
+   df2->Foreach(
+      [](const ROOT::RVecF &v, ULong64_t entry) {
+         EXPECT_EQ(v.size(), newArraySize); // not 4 as it was in the original input
+         EXPECT_FLOAT_EQ(v.front(), entry);
+         std::vector<float> diffs(newArraySize);
+         std::adjacent_difference(v.begin(), v.end(), diffs.begin());
+
+         std::vector<float> target(6, 1.f);
+         target.front() = entry;
+         EXPECT_EQ(diffs, target);
+      },
+      {"fixedSizeArr", "rdfentry_"});
 
    gSystem->Unlink("test_snapshotRVecRedefineArray.root");
 }
@@ -597,8 +636,7 @@ TEST(RDFSnapshotMore, ColsWithCustomTitles)
    // read and write test tree with RDF
    RDataFrame d(tname, fname);
    const std::string prefix = "snapshotted_";
-   auto res_tdf =
-      d.Snapshot<int, float, RVec<int>, RVec<int>>(tname, prefix + fname, {"i", "float", "arrint", "vararrint"});
+   auto res_tdf = d.Snapshot(tname, prefix + fname, {"i", "float", "arrint", "vararrint"});
 
    // check correct results have been written out
    res_tdf->Foreach(CheckColsWithCustomTitles, {"tdfentry_", "i", "arrint", "vararrint", "float"});
@@ -652,7 +690,7 @@ TEST(RDFSnapshotMore, ReadWriteStdVec)
    // read and write using RDataFrame
 
    const auto outfname1 = "out_readwritestdvec1.root";
-   RDataFrame(treename, fname).Snapshot<std::vector<int>, std::vector<bool>>(treename, outfname1, {"v", "vb"});
+   RDataFrame(treename, fname).Snapshot(treename, outfname1, {"v", "vb"});
    outputChecker(outfname1);
 
    const auto outfname2 = "out_readwritestdvec2.root";
@@ -660,7 +698,7 @@ TEST(RDFSnapshotMore, ReadWriteStdVec)
    outputChecker(outfname2);
 
    const auto outfname3 = "out_readwritestdvec3.root";
-   RDataFrame(treename, fname).Snapshot<RVec<int>, RVec<bool>>(treename, outfname3, {"v", "vb"});
+   RDataFrame(treename, fname).Snapshot(treename, outfname3, {"v", "vb"});
    outputChecker(outfname3);
 
    gSystem->Unlink(fname);
@@ -753,12 +791,13 @@ void ReadWriteCarray(const char *outFileNameBase)
       EXPECT_TRUE(r.Next());
       EXPECT_EQ(rv.GetSize(), 100000u);
       EXPECT_EQ(rvb.GetSize(), 100000u);
-      for (auto e : rv)
-         EXPECT_EQ(e, 84);
-      for (auto e : rvb)
-         EXPECT_TRUE(e);
-      for (auto e : rvl)
-         EXPECT_EQ(e, 42);
+      EXPECT_EQ(rvl.GetSize(), 100000u);
+      EXPECT_TRUE(std::all_of(rv.begin(), rv.end(), [](auto elm) { return elm == 84; }))
+         << rv[0] << " " << rv[1] << " " << rv[2];
+      EXPECT_TRUE(std::all_of(rvb.begin(), rvb.end(), [](auto elm) { return elm; }))
+         << rvb[0] << " " << rvb[1] << " " << rvb[2];
+      EXPECT_TRUE(std::all_of(rvl.begin(), rvl.end(), [](auto elm) { return elm == 42; }))
+         << rvl[0] << " " << rvl[1] << " " << rvl[2];
 
       // Size 3
       EXPECT_TRUE(r.Next());
@@ -784,8 +823,7 @@ void ReadWriteCarray(const char *outFileNameBase)
    outputChecker(outfname1.c_str());
 
    const auto outfname2 = outFileNameBaseStr + "_out2.root";
-   RDataFrame(treename, fname)
-      .Snapshot<int, RVec<int>, RVec<bool>, RVec<long int>>(treename, outfname2, {"size", "v", "vb", "vl"});
+   RDataFrame(treename, fname).Snapshot(treename, outfname2, {"size", "v", "vb", "vl"});
    outputChecker(outfname2.c_str());
 
    gSystem->Unlink(fname.c_str());
@@ -829,7 +867,7 @@ TEST(RDFSnapshotMore, ReadWriteNestedLeaves)
       ROOT::TestSupport::CheckDiagsRAII diagRAII;
       diagRAII.requiredDiag(kInfo, "Snapshot", "Column v.a will be saved as v_a");
       diagRAII.requiredDiag(kInfo, "Snapshot", "Column v.b will be saved as v_b");
-      d2 = *d.Snapshot<int, int>(treename, outfname, {"v.a", "v.b"});
+      d2 = *d.Snapshot(treename, outfname, {"v.a", "v.b"});
    }
    EXPECT_EQ(d2.GetColumnNames(), std::vector<std::string>({"v_a", "v_b"}));
    auto check_a_b = [](int a, int b) {
@@ -841,7 +879,7 @@ TEST(RDFSnapshotMore, ReadWriteNestedLeaves)
    gSystem->Unlink(outfname);
 
    try {
-      d.Define("v_a", [] { return 0; }).Snapshot<int, int>(treename, outfname, {"v.a", "v_a"});
+      d.Define("v_a", [] { return 0; }).Snapshot(treename, outfname, {"v.a", "v_a"});
    } catch (std::runtime_error &e) {
       const auto error_msg = "Column v.a would be written as v_a but this column already exists. Please use Alias to "
                              "select a new name for v.a";
@@ -856,6 +894,7 @@ TEST(RDFSnapshotMore, Lazy)
    const auto fname1 = "lazy1.root";
    // make sure the file is not here beforehand
    gSystem->Unlink(fname0);
+   gSystem->Unlink(fname1);
    RDataFrame d(1);
    auto v = 0U;
    auto genf = [&v]() {
@@ -863,10 +902,10 @@ TEST(RDFSnapshotMore, Lazy)
       return 42;
    };
    RSnapshotOptions opts = {"RECREATE", ROOT::RCompressionSetting::EAlgorithm::kZLIB, 0, 0, 99, true};
-   auto ds = d.Define("c0", genf).Snapshot<int>(treename, fname0, {"c0"}, opts);
+   auto ds = d.Define("c0", genf).Snapshot(treename, fname0, {"c0"}, opts);
    EXPECT_EQ(v, 0U);
    EXPECT_TRUE(gSystem->AccessPathName(fname0)); // This returns FALSE if the file IS there
-   auto ds2 = ds->Define("c1", genf).Snapshot<int>(treename, fname1, {"c1"}, opts);
+   auto ds2 = ds->Define("c1", genf).Snapshot(treename, fname1, {"c1"}, opts);
    EXPECT_EQ(v, 1U);
    EXPECT_FALSE(gSystem->AccessPathName(fname0));
    EXPECT_TRUE(gSystem->AccessPathName(fname1));
@@ -897,7 +936,7 @@ void BookLazySnapshot()
    auto d = ROOT::RDataFrame(1);
    ROOT::RDF::RSnapshotOptions opts;
    opts.fLazy = true;
-   d.Snapshot<ULong64_t>("t", "lazysnapshotnottriggered_shouldnotbecreated.root", {"rdfentry_"}, opts);
+   d.Snapshot("t", "lazysnapshotnottriggered_shouldnotbecreated.root", {"rdfentry_"}, opts);
 }
 
 TEST(RDFSnapshotMore, LazyNotTriggered)
@@ -914,7 +953,7 @@ RResultPtr<RInterface<RLoopManager, void>> ReturnLazySnapshot(const char *fname)
    auto d = ROOT::RDataFrame(1);
    ROOT::RDF::RSnapshotOptions opts;
    opts.fLazy = true;
-   auto res = d.Snapshot<ULong64_t>("t", fname, {"rdfentry_"}, opts);
+   auto res = d.Snapshot("t", fname, {"rdfentry_"}, opts);
    RResultPtr<RInterface<RLoopManager, void>> res2 = res;
    return res;
 }
@@ -954,8 +993,8 @@ void ReadWriteTClonesArray()
 
    {
       // write as TClonesArray
-      auto out_df = ROOT::RDataFrame("t", "df_readwritetclonesarray.root")
-                       .Snapshot<TClonesArray>("t", "df_readwriteclonesarray1.root", {"arr"});
+      auto out_df =
+         ROOT::RDataFrame("t", "df_readwritetclonesarray.root").Snapshot("t", "df_readwriteclonesarray1.root", {"arr"});
       RVec<TH1D> hvec;
 
 #ifndef NDEBUG
@@ -974,7 +1013,7 @@ void ReadWriteTClonesArray()
    //                                    "vector;TH1D.h;ROOT/RVec.hxx");
    //   // write as RVecs
    //   auto out_df = ROOT::RDataFrame("t", "df_readwritetclonesarray.root")
-   //                    .Snapshot<RVec<TH1D>>("t", "df_readwriteclonesarray2.root", {"arr"});
+   //                    .Snapshot("t", "df_readwriteclonesarray2.root", {"arr"});
    //   const auto hvec = out_df->Take<RVec<TH1D>>("arr")->at(0);
    //   CheckTClonesArrayOutput(hvec);
    //}
@@ -1005,17 +1044,17 @@ TEST(RDFSnapshotMore, TClonesArray)
    ReadWriteTClonesArray();
 }
 
-// ROOT-10702
+// https://its.cern.ch/jira/browse/ROOT-10702
 TEST(RDFSnapshotMore, CompositeTypeWithNameClash)
 {
-   const auto fname = "snap_compositetypewithnameclash.root";
-   gInterpreter->Declare("struct Int { int x; };");
-   ROOT::RDataFrame df(3);
-   auto snap_df = df.Define("i", "Int{-1};").Define("x", [] { return 1; }).Snapshot("t", fname);
+   constexpr auto fName{"snap_compositetypewithnameclash.root"};
+   struct FileGuard {
+      ~FileGuard() { std::remove("snap_compositetypewithnameclash.root"); }
+   } _;
+   ROOT::RDataFrame df{3};
+   auto snap_df = df.Define("i", [] { return Int{-1}; }).Define("x", [] { return 1; }).Snapshot("t", fName);
    EXPECT_EQ(snap_df->Sum<int>("x").GetValue(), 3); // prints -3 if the wrong "x" is written out
    EXPECT_EQ(snap_df->Sum<int>("i.x").GetValue(), -3);
-
-   gSystem->Unlink(fname);
 }
 
 // Test that we error out gracefully in case the output file specified for a Snapshot cannot be opened
@@ -1026,7 +1065,7 @@ TEST(RDFSnapshotMore, ForbiddenOutputFilename)
 
    // Compiled
    try {
-      ROOT_EXPECT_SYSERROR(df.Snapshot<unsigned int>("t", out_fname, {"rdfslot_"}), "TFile::TFile",
+      ROOT_EXPECT_SYSERROR(df.Snapshot("t", out_fname, {"rdfslot_"}), "TFile::TFile",
                            "file /definitely/not/a/valid/path/f.root can not be opened No such file or directory")
    } catch (const std::runtime_error &e) {
       EXPECT_STREQ(e.what(), "Snapshot: could not create output file /definitely/not/a/valid/path/f.root");
@@ -1046,7 +1085,7 @@ TEST(RDFSnapshotMore, ForbiddenOutputFilename)
 TEST(RDFSnapshotMore, ZeroOutputEntries)
 {
    const auto fname = "snapshot_zerooutputentries.root";
-   ROOT::RDataFrame(10).Alias("c", "rdfentry_").Filter([] { return false; }).Snapshot<ULong64_t>("t", fname, {"c"});
+   ROOT::RDataFrame(10).Alias("c", "rdfentry_").Filter([] { return false; }).Snapshot("t", fname, {"c"});
    EXPECT_EQ(gSystem->AccessPathName(fname), 0); // This returns 0 if the file IS there
 
    TFile f(fname);
@@ -1090,10 +1129,6 @@ TEST(RDFSnapshotMore, MissingSizeBranch)
 
    ROOT::RDataFrame df("t", inFile);
 
-   // fully typed Snapshot call throws
-   EXPECT_THROW(df.Snapshot<ROOT::RVecF>("t", outFile, {"vec"}), std::runtime_error);
-
-   // jitted Snapshot works anyway
    auto out = df.Snapshot("t", outFile, {"vec"});
 
    auto sizes = out->Take<int>("sz");
@@ -1131,31 +1166,17 @@ TEST(RDFSnapshotMore, OutOfOrderSizeBranch)
       t.Write();
    }
 
-   auto check = [](const std::vector<int> &sizes, const std::vector<ROOT::RVecF> &vecs) {
-      EXPECT_EQ(sizes.at(0), 1);
-      EXPECT_EQ(sizes.at(1), 2);
-      EXPECT_EQ(sizes.at(2), 3);
-      EXPECT_TRUE(All(vecs.at(0) == ROOT::RVecF{1}));
-      EXPECT_TRUE(All(vecs.at(1) == ROOT::RVecF{1, 2}));
-      EXPECT_TRUE(All(vecs.at(2) == ROOT::RVecF{1, 2, 3}));
-   };
-
    {
-      // fully typed Snapshot
-      auto out = ROOT::RDataFrame("t", inFile).Snapshot<ROOT::RVecF, int>("t", outFile, {"vec", "sz"});
-      auto sizes = out->Take<int>("sz");
-      auto vecs = out->Take<ROOT::RVecF>("vec");
-
-      check(*sizes, *vecs);
-   }
-
-   {
-      // jitted Snapshot
       auto out = ROOT::RDataFrame("t", inFile).Snapshot("t", outFile, {"vec", "sz"});
       auto sizes = out->Take<int>("sz");
       auto vecs = out->Take<ROOT::RVecF>("vec");
 
-      check(*sizes, *vecs);
+      EXPECT_EQ(sizes->at(0), 1);
+      EXPECT_EQ(sizes->at(1), 2);
+      EXPECT_EQ(sizes->at(2), 3);
+      EXPECT_TRUE(All(vecs->at(0) == ROOT::RVecF{1}));
+      EXPECT_TRUE(All(vecs->at(1) == ROOT::RVecF{1, 2}));
+      EXPECT_TRUE(All(vecs->at(2) == ROOT::RVecF{1, 2, 3}));
    }
 
    gSystem->Unlink(inFile);
@@ -1307,7 +1328,7 @@ TEST_F(RDFSnapshotMT, Reshuffled_friends)
 TEST(RDFSnapshotMore, ManyTasksPerThread)
 {
    const auto nSlots = 4u;
-   ROOT::EnableImplicitMT(nSlots);
+   TIMTEnabler imt(nSlots);
    // make sure the file is not here beforehand
    gSystem->Unlink("snapshot_manytasks_out.root");
 
@@ -1318,12 +1339,12 @@ TEST(RDFSnapshotMore, ManyTasksPerThread)
    ROOT::RDataFrame d(1);
    auto dd = d.Define("x", []() { return 42; });
    for (auto i = 0u; i < nInputFiles; ++i)
-      dd.Snapshot<int>("t", inputFilePrefix + std::to_string(i) + ".root", {"x"});
+      dd.Snapshot("t", inputFilePrefix + std::to_string(i) + ".root", {"x"});
 
    // test multi-thread Snapshotting from many tasks per worker thread
    const auto outputFile = "snapshot_manytasks_out.root";
    ROOT::RDataFrame tdf("t", inputFilePrefix + "*.root");
-   tdf.Snapshot<int>("t", outputFile, {"x"});
+   tdf.Snapshot("t", outputFile, {"x"});
 
    // check output contents
    ROOT::RDataFrame checkTdf("t", outputFile);
@@ -1337,8 +1358,6 @@ TEST(RDFSnapshotMore, ManyTasksPerThread)
    for (auto i = 0u; i < nInputFiles; ++i)
       gSystem->Unlink((inputFilePrefix + std::to_string(i) + ".root").c_str());
    gSystem->Unlink(outputFile);
-
-   ROOT::DisableImplicitMT();
 }
 
 void checkSnapshotArrayFileMT(RResultPtr<RInterface<RLoopManager>> &df, unsigned int kNEvents)
@@ -1356,29 +1375,58 @@ void checkSnapshotArrayFileMT(RResultPtr<RInterface<RLoopManager>> &df, unsigned
 
 TEST_F(RDFSnapshotArrays, MultiThread)
 {
-   ROOT::EnableImplicitMT(4);
+   TIMTEnabler imt(4);
 
    RDataFrame tdf("arrayTree", kFileNames);
-   auto dt = tdf.Snapshot<RVec<float>, unsigned int, RVec<double>, RVec<bool>, RVec<bool>>(
-      "outTree", "test_snapshotRVecoutMT.root",
-      {"fixedSizeArr", "size", "varSizeArr", "varSizeBoolArr", "fixedSizeBoolArr"});
+   auto dt = tdf.Snapshot("outTree", "test_snapshotRVecoutMT.root",
+                          {"fixedSizeArr", "size", "varSizeArr", "varSizeBoolArr", "fixedSizeBoolArr"});
 
    checkSnapshotArrayFileMT(dt, kNEvents);
-
-   ROOT::DisableImplicitMT();
 }
 
 TEST_F(RDFSnapshotArrays, MultiThreadJitted)
 {
-   ROOT::EnableImplicitMT(4);
+   TIMTEnabler imt(4);
 
    RDataFrame tdf("arrayTree", kFileNames);
    auto dj = tdf.Snapshot("outTree", "test_snapshotRVecoutMTJitted.root",
                           {"fixedSizeArr", "size", "varSizeArr", "varSizeBoolArr", "fixedSizeBoolArr"});
 
    checkSnapshotArrayFileMT(dj, kNEvents);
+}
 
-   ROOT::DisableImplicitMT();
+TEST_F(RDFSnapshotArrays, RedefineArrayMT)
+{
+   static constexpr unsigned int newArraySize = 6u;
+   static_assert(kFixedSize != newArraySize);
+   TIMTEnabler(3);
+
+   // More input files than threads, so output branches need to be bound to new inputs
+   std::vector<std::string> fileNames(kFileNames);
+   std::copy(kFileNames.begin(), kFileNames.end(), std::back_inserter(fileNames));
+   std::copy(kFileNames.begin(), kFileNames.end(), std::back_inserter(fileNames));
+
+   RDataFrame df("arrayTree", fileNames);
+   auto df2 =
+      df.Redefine("fixedSizeArr",
+                  [](ULong64_t entry) {
+                     return ROOT::RVecF{float(entry), entry + 1.f, entry + 2.f, entry + 3.f, entry + 4.f, entry + 5.f};
+                  },
+                  {"rdfentry_"})
+         .Snapshot("t", "test_snapshotRVecRedefineArray.root", {"fixedSizeArr"});
+   EXPECT_EQ(df2->Count().GetValue(), 3 * kNEvents);
+   df2->Foreach(
+      [](const ROOT::RVecF &v) {
+         EXPECT_EQ(v.size(), newArraySize); // not 4 as it was in the original input
+         EXPECT_LT(v.front(), 3 * kNEvents);
+
+         std::vector<float> target(newArraySize, 0.f);
+         for (unsigned int i = 0; i < newArraySize; ++i)
+            EXPECT_FLOAT_EQ(v[i], v.front() + i);
+      },
+      {"fixedSizeArr"});
+
+   gSystem->Unlink("test_snapshotRVecRedefineArray.root");
 }
 
 // See also https://github.com/root-project/root/issues/10225
@@ -1386,11 +1434,11 @@ TEST_F(RDFSnapshotArrays, WriteRVecFromFile)
 {
    {
       auto df = ROOT::RDataFrame(3).Define("x", [](ULong64_t e) { return ROOT::RVecD(e, double(e)); }, {"rdfentry_"});
-      df.Snapshot<ROOT::RVecD>("t", "test_snapshotRVecWriteRVecFromFile.root", {"x"});
+      df.Snapshot("t", "test_snapshotRVecWriteRVecFromFile.root", {"x"});
    }
 
    ROOT::RDataFrame df("t", "test_snapshotRVecWriteRVecFromFile.root");
-   auto outdf = df.Snapshot<ROOT::RVecD>("t", "test_snapshotRVecWriteRVecFromFile2.root", {"x"});
+   auto outdf = df.Snapshot("t", "test_snapshotRVecWriteRVecFromFile2.root", {"x"});
 
    const auto res = outdf->Take<ROOT::RVecD>("x").GetValue();
 
@@ -1412,11 +1460,10 @@ TEST(RDFSnapshotMore, ColsWithCustomTitlesMT)
    WriteColsWithCustomTitles(tname, fname);
 
    // read and write test tree with RDF (in parallel)
-   ROOT::EnableImplicitMT(4);
+   TIMTEnabler imt(4);
    RDataFrame d(tname, fname);
    const std::string prefix = "snapshotted_";
-   auto res_tdf =
-      d.Snapshot<int, float, RVec<int>, RVec<int>>(tname, prefix + fname, {"i", "float", "arrint", "vararrint"});
+   auto res_tdf = d.Snapshot(tname, prefix + fname, {"i", "float", "arrint", "vararrint"});
 
    // check correct results have been written out
    res_tdf->Foreach(CheckColsWithCustomTitles, {"tdfentry_", "i", "arrint", "vararrint", "float"});
@@ -1425,17 +1472,16 @@ TEST(RDFSnapshotMore, ColsWithCustomTitlesMT)
    // clean-up
    gSystem->Unlink(fname);
    gSystem->Unlink((prefix + fname).c_str());
-   ROOT::DisableImplicitMT();
 }
 
 TEST(RDFSnapshotMore, TreeWithFriendsMT)
 {
    const auto fname1 = "treewithfriendsmt1.root";
    const auto fname2 = "treewithfriendsmt2.root";
-   RDataFrame(10).Define("x", []() { return 42; }).Snapshot<int>("t", fname1, {"x"});
-   RDataFrame(10).Define("x", []() { return 0; }).Snapshot<int>("t", fname2, {"x"});
+   RDataFrame(10).Define("x", []() { return 42; }).Snapshot("t", fname1, {"x"});
+   RDataFrame(10).Define("x", []() { return 0; }).Snapshot("t", fname2, {"x"});
 
-   ROOT::EnableImplicitMT();
+   TIMTEnabler imt(0);
 
    TFile file(fname1);
    auto tree = file.Get<TTree>("t");
@@ -1445,11 +1491,10 @@ TEST(RDFSnapshotMore, TreeWithFriendsMT)
 
    const auto outfname = "out_treewithfriendsmt.root";
    RDataFrame df(*tree);
-   auto df_out = df.Snapshot<int>("t", outfname, {"x"});
+   auto df_out = df.Snapshot("t", outfname, {"x"});
    EXPECT_EQ(df_out->Max<int>("x").GetValue(), 42);
    EXPECT_EQ(df_out->GetColumnNames(), std::vector<std::string>{"x"});
 
-   ROOT::DisableImplicitMT();
    gSystem->Unlink(fname1);
    gSystem->Unlink(fname2);
    gSystem->Unlink(outfname);
@@ -1476,13 +1521,12 @@ TEST(RDFSnapshotMore, JittedSnapshotAndAliasedColumns)
 
 TEST(RDFSnapshotMore, LazyNotTriggeredMT)
 {
-   ROOT::EnableImplicitMT(4);
+   TIMTEnabler imt(4);
    ROOT_EXPECT_WARNING(BookLazySnapshot(), "Snapshot",
                        "A lazy Snapshot action was booked but never triggered. The tree 't' in output file "
                        "'lazysnapshotnottriggered_shouldnotbecreated.root' was not created. "
                        "In case it was desired instead, remember to trigger the Snapshot operation, by "
                        "storing its result in a variable and for example calling the GetValue() method on it.");
-   ROOT::DisableImplicitMT();
 }
 
 TEST(RDFSnapshotMore, LazyTriggeredMT)
@@ -1499,7 +1543,7 @@ TEST(RDFSnapshotMore, EmptyBuffersMT)
    const auto fname = "emptybuffersmt.root";
    const auto treename = "t";
    const unsigned int nslots = std::min(4U, std::thread::hardware_concurrency());
-   ROOT::EnableImplicitMT(nslots);
+   TIMTEnabler imt(nslots);
    ROOT::RDataFrame d(10);
    std::atomic_bool firstWorker{true};
    auto dd = d.DefineSlot("x", [&](unsigned int) {
@@ -1510,7 +1554,7 @@ TEST(RDFSnapshotMore, EmptyBuffersMT)
                  return 1;
               }).Filter([](int x) { return x == 0; }, {"x"}, "f");
    auto r = dd.Report();
-   dd.Snapshot<int>(treename, fname, {"x"});
+   dd.Snapshot(treename, fname, {"x"});
 
    // check test sanity
    const auto passed = r->At("f").GetPass();
@@ -1522,15 +1566,13 @@ TEST(RDFSnapshotMore, EmptyBuffersMT)
    EXPECT_EQ(t->GetListOfBranches()->GetEntries(), 1);
    EXPECT_EQ(t->GetEntries(), Long64_t(passed));
 
-   ROOT::DisableImplicitMT();
    gSystem->Unlink(fname);
 }
 
 TEST(RDFSnapshotMore, ReadWriteCarrayMT)
 {
-   ROOT::EnableImplicitMT(4);
+   TIMTEnabler imt(4);
    ReadWriteCarray("ReadWriteCarrayMT");
-   ROOT::DisableImplicitMT();
 }
 
 TEST(RDFSnapshotMore, TClonesArrayMT)
@@ -1549,7 +1591,7 @@ TEST(RDFSnapshotMore, ForbiddenOutputFilenameMT)
    // Compiled
    try {
       const auto expected = "file /definitely/not/a/valid/path/f.root can not be opened No such file or directory";
-      ROOT_EXPECT_SYSERROR(df.Snapshot<unsigned int>("t", out_fname, {"rdfslot_"}), "TFile::TFile", expected);
+      ROOT_EXPECT_SYSERROR(df.Snapshot("t", out_fname, {"rdfslot_"}), "TFile::TFile", expected);
    } catch (const std::runtime_error &e) {
       EXPECT_STREQ(e.what(), "Snapshot: could not create output file /definitely/not/a/valid/path/f.root");
    }
@@ -1594,18 +1636,16 @@ TEST(RDFSnapshotMore, SetMaxTreeSizeMT)
       }
 
       ROOT::RDataFrame df{t};
-      df.Snapshot<Int_t>("T", "rdfsnapshot_ttree_sequential_setmaxtreesize.root", {"x"});
+      df.Snapshot("T", "rdfsnapshot_ttree_sequential_setmaxtreesize.root", {"x"});
    }
 
    // Create an RDF from the previously snapshotted file, then Snapshot again
    // with IMT enabled.
    {
-      ROOT::EnableImplicitMT();
+      TIMTEnabler imt(0);
 
       ROOT::RDataFrame df{"T", "rdfsnapshot_ttree_sequential_setmaxtreesize.root"};
-      df.Snapshot<Int_t>("T", "rdfsnapshot_imt_setmaxtreesize.root", {"x"});
-
-      ROOT::DisableImplicitMT();
+      df.Snapshot("T", "rdfsnapshot_imt_setmaxtreesize.root", {"x"});
    }
 
    // Check the file for data integrity.
@@ -1638,7 +1678,7 @@ TEST(RDFSnapshotMore, SetMaxTreeSizeMT)
 TEST(RDFSnapshotMore, ZeroOutputEntriesMT)
 {
    const auto fname = "snapshot_zerooutputentriesmt.root";
-   ROOT::RDataFrame(10).Alias("c", "rdfentry_").Filter([] { return false; }).Snapshot<ULong64_t>("t", fname, {"c"});
+   ROOT::RDataFrame(10).Alias("c", "rdfentry_").Filter([] { return false; }).Snapshot("t", fname, {"c"});
    EXPECT_EQ(gSystem->AccessPathName(fname), 0); // This returns 0 if the file IS there
 
    TFile f(fname);
@@ -1650,25 +1690,22 @@ TEST(RDFSnapshotMore, ZeroOutputEntriesMT)
 
 TEST(RDFSnapshotMore, CustomBasketSizeMT)
 {
-   ROOT::EnableImplicitMT();
+   TIMTEnabler imt(0);
    TestCustomBasketSize();
-   ROOT::DisableImplicitMT();
 }
 
 // Test for default basket size
 TEST(RDFSnapshotMore, DefaultBasketSizeMT)
 {
-   ROOT::EnableImplicitMT();
+   TIMTEnabler imt(0);
    TestDefaultBasketSize();
-   ROOT::DisableImplicitMT();
 }
 
 // Test for basket size preservation
 TEST(RDFSnapshotMore, BasketSizePreservationMT)
 {
-   ROOT::EnableImplicitMT();
+   TIMTEnabler imt(0);
    TestBasketSizePreservation();
-   ROOT::DisableImplicitMT();
 }
 
 #endif // R__USE_IMT
