@@ -19,6 +19,7 @@
 #include <ROOT/RNTupleMetrics.hxx>
 #include <ROOT/RNTupleModel.hxx>
 #include <ROOT/RNTupleSerialize.hxx>
+#include <ROOT/RNTupleUtils.hxx>
 #include <ROOT/RNTupleZip.hxx>
 #include <ROOT/RPageAllocator.hxx>
 #include <ROOT/RPageSinkBuf.hxx>
@@ -829,6 +830,11 @@ ROOT::Internal::RPagePersistentSink::AddColumn(ROOT::DescriptorId_t fieldId, RCo
 void ROOT::Internal::RPagePersistentSink::UpdateSchema(const ROOT::Internal::RNTupleModelChangeset &changeset,
                                                        ROOT::NTupleSize_t firstEntry)
 {
+   if (fIsInitialized)
+      for (const auto &field : changeset.fAddedFields)
+         if (field->GetStructure() == ENTupleStructure::kStreamer)
+            throw ROOT::RException(R__FAIL("a Model cannot be extended with Streamer fields"));
+
    const auto &descriptor = fDescriptorBuilder.GetDescriptor();
 
    if (descriptor.GetNLogicalColumns() > descriptor.GetNPhysicalColumns()) {
@@ -921,7 +927,7 @@ void ROOT::Internal::RPagePersistentSink::UpdateExtraTypeInfo(const ROOT::RExtra
    if (extraTypeInfo.GetContentId() != EExtraTypeInfoIds::kStreamerInfo)
       throw RException(R__FAIL("ROOT bug: unexpected type extra info in UpdateExtraTypeInfo()"));
 
-   fStreamerInfos.merge(RNTupleSerializer::DeserializeStreamerInfos(extraTypeInfo.GetContent()).Unwrap());
+   fInfosOfStreamerFields.merge(RNTupleSerializer::DeserializeStreamerInfos(extraTypeInfo.GetContent()).Unwrap());
 }
 
 void ROOT::Internal::RPagePersistentSink::InitImpl(ROOT::RNTupleModel &model)
@@ -1002,6 +1008,8 @@ ROOT::Internal::RPagePersistentSink::InitFromDescriptor(const ROOT::RNTupleDescr
    // Create model
    auto modelOpts = ROOT::RNTupleDescriptor::RCreateModelOptions();
    modelOpts.SetReconstructProjections(true);
+   // We want to emulate unknown types to allow merging RNTuples containing types that we lack dictionaries for.
+   modelOpts.SetEmulateUnknownTypes(true);
    auto model = descriptor.CreateModel(modelOpts);
    if (!copyClusters) {
       auto &projectedFields = ROOT::Internal::GetProjectedFieldsOfModel(*model);
@@ -1247,7 +1255,7 @@ void ROOT::Internal::RPagePersistentSink::CommitClusterGroup()
 
 void ROOT::Internal::RPagePersistentSink::CommitDatasetImpl()
 {
-   if (!fStreamerInfos.empty()) {
+   if (!fInfosOfStreamerFields.empty()) {
       // De-duplicate extra type infos before writing. Usually we won't have them already in the descriptor, but
       // this may happen when we are writing back an already-existing RNTuple, e.g. when doing incremental merging.
       for (const auto &etDesc : fDescriptorBuilder.GetDescriptor().GetExtraTypeInfoIterable()) {
@@ -1257,13 +1265,13 @@ void ROOT::Internal::RPagePersistentSink::CommitDatasetImpl()
             R__ASSERT(etDesc.GetTypeName().empty());
             R__ASSERT(etDesc.GetTypeVersion() == 0);
             auto etInfo = RNTupleSerializer::DeserializeStreamerInfos(etDesc.GetContent()).Unwrap();
-            fStreamerInfos.merge(etInfo);
+            fInfosOfStreamerFields.merge(etInfo);
          }
       }
 
       RExtraTypeInfoDescriptorBuilder extraInfoBuilder;
       extraInfoBuilder.ContentId(EExtraTypeInfoIds::kStreamerInfo)
-         .Content(RNTupleSerializer::SerializeStreamerInfos(fStreamerInfos));
+         .Content(RNTupleSerializer::SerializeStreamerInfos(fInfosOfStreamerFields));
       fDescriptorBuilder.ReplaceExtraTypeInfo(extraInfoBuilder.MoveDescriptor().Unwrap());
    }
 
