@@ -1,6 +1,8 @@
 #ifdef __CLING__
 
 #pragma link C++ enum CustomEnum;
+#pragma link C++ enum RenamedCustomEnum;
+
 #pragma link C++ enum CustomEnumBool;
 #pragma link C++ enum CustomEnumInt8;
 #pragma link C++ enum CustomEnumUInt8;
@@ -11,6 +13,8 @@
 #pragma link C++ enum CustomEnumInt64;
 #pragma link C++ enum CustomEnumUInt64;
 
+#pragma link C++ class CustomAtomicNotLockFree+;
+
 #pragma link C++ class CustomStruct+;
 #pragma link C++ class DerivedA+;
 #pragma link C++ class DerivedA2+;
@@ -26,12 +30,10 @@
 #pragma link C++ class std::map<int, CustomStruct>+ ;
 #pragma link C++ class std::map<int, float>+ ;
 
-#pragma link C++ class EdmContent<float, long long>+;
 #pragma link C++ class EdmWrapper<CustomStruct> +;
 #pragma link C++ class EdmHash < 1> + ;
 #pragma link C++ class EdmWrapper<long long>+;
-#pragma link C++ class EdmWrapper<std::map<int, EdmContent<float, long long>>>+;
-#pragma link C++ class EdmContainer+;
+#pragma link C++ class EdmContainer;
 
 #pragma link C++ class DataVector < int, double> + ;
 #pragma link C++ class DataVector < int, float> + ;
@@ -46,6 +48,7 @@
 #pragma link C++ class DataVector < int, float> ::Nested < int, double> + ;
 #pragma link C++ class DataVector < int, float> ::Nested < int, float> + ;
 #pragma link C++ class InnerCV < const int, const volatile int, volatile const int, volatile int> + ;
+#pragma link C++ class InnerCV < const std::vector<std::string[2]>, int, int, int> + ;
 #pragma link C++ class IntegerTemplates < 0, 0> + ;
 #pragma link C++ class IntegerTemplates < -1, 1> + ;
 #pragma link C++ class IntegerTemplates < -2147483650ll, 9223372036854775810ull> + ;
@@ -64,7 +67,6 @@
 #pragma link C++ class StructUsingCollectionProxy<CustomStruct> + ;
 #pragma link C++ class StructUsingCollectionProxy<StructUsingCollectionProxy<float>> + ;
 #pragma link C++ class StructUsingCollectionProxy<int> + ;
-#pragma link C++ class StructUsingCollectionProxy<Long64_t>+;
 
 #pragma link C++ class TrivialTraitsBase + ;
 #pragma link C++ class TrivialTraits + ;
@@ -154,6 +156,8 @@
 #pragma link C++ class DuplicateBaseB + ;
 #pragma link C++ class DuplicateBaseC + ;
 #pragma link C++ class DuplicateBaseD + ;
+#pragma link C++ class PolymorphicBase + ;
+#pragma link C++ class PolymorphicDerived + ;
 
 #pragma link C++ class Left + ;
 #pragma link C++ class DerivedFromLeftAndTObject+;
