@@ -8,7 +8,7 @@
 #include <ROOT/RNTupleModel.hxx>
 #include <ROOT/RNTupleReader.hxx>
 #include <ROOT/RNTupleReadOptions.hxx>
-#include <ROOT/RNTupleUtil.hxx>
+#include <ROOT/RNTupleTypes.hxx>
 #include <ROOT/RNTupleWriter.hxx>
 #include <ROOT/RPage.hxx>
 #include <ROOT/RPageStorage.hxx>
@@ -42,6 +42,7 @@ protected:
    RNTupleDescriptor AttachImpl(RNTupleSerializer::EDescriptorDeserializeMode) final { return RNTupleDescriptor(); }
    std::unique_ptr<RPageSource> CloneImpl() const final { return nullptr; }
    RPageRef LoadPageImpl(ColumnHandle_t, const RClusterInfo &, ROOT::NTupleSize_t) final { return RPageRef(); }
+   void LoadStreamerInfo() final {}
 
 public:
    /// Records the cluster IDs requests by LoadClusters() calls
@@ -51,6 +52,8 @@ public:
    RPageSourceMock() : RPageSource("test", RNTupleReadOptions())
    {
       ROOT::Internal::RNTupleDescriptorBuilder descBuilder;
+      // Should set the version from the anchor; here it is fine to use the same version as for writing.
+      descBuilder.SetVersionForWriting();
       descBuilder.SetNTuple("ntpl", "");
       for (unsigned i = 0; i <= 5; ++i) {
          descBuilder.AddCluster(ROOT::Internal::RClusterDescriptorBuilder()
