@@ -1,4 +1,5 @@
 #include "ntuple_test.hxx"
+#include "SimpleCollectionProxy.hxx"
 
 TEST(RNTuple, TypeNameBasics)
 {
@@ -16,7 +17,7 @@ TEST(RNTuple, TypeNameBasics)
                 (ROOT::RField<std::tuple<std::tuple<char, CustomStruct, char>, int>>::TypeName().c_str()));
 }
 
-TEST(RNTuple, TypeNameNormalization)
+TEST(RNTuple, TypeNameNormalizationByName)
 {
    EXPECT_EQ("CustomStruct", RFieldBase::Create("f", "class CustomStruct").Unwrap()->GetTypeName());
    EXPECT_EQ("", RFieldBase::Create("f", "class CustomStruct").Unwrap()->GetTypeAlias());
@@ -37,11 +38,33 @@ TEST(RNTuple, TypeNameNormalization)
    EXPECT_EQ("std::uint32_t", RFieldBase::Create("f", "SG::sgkey_t").Unwrap()->GetTypeName());
    EXPECT_EQ("SG::sgkey_t", RFieldBase::Create("f", "SG::sgkey_t").Unwrap()->GetTypeAlias());
 
+   EXPECT_EQ("std::array<std::array<std::uint32_t,3>,2>",
+             RFieldBase::Create("f", "SG::sgkey_t[2][3]").Unwrap()->GetTypeName());
+   EXPECT_EQ("std::array<SG::sgkey_t,2>", RFieldBase::Create("f", "SG::sgkey_t[2]").Unwrap()->GetTypeAlias());
+   EXPECT_EQ("std::vector<std::array<double,2>>",
+             RFieldBase::Create("f", "vector<Double32_t[2]>").Unwrap()->GetTypeName());
+   EXPECT_EQ("std::vector<std::array<Double32_t,2>>",
+             RFieldBase::Create("f", "vector<Double32_t[2]>").Unwrap()->GetTypeAlias());
+
+   EXPECT_EQ("std::array<std::vector<std::array<double,3>>,10>",
+             RFieldBase::Create("f", "std::vector<Double32_t[3]   > [10 ]  ").Unwrap()->GetTypeName());
+   EXPECT_EQ("std::array<std::vector<std::array<Double32_t,3>>,10>",
+             RFieldBase::Create("f", "std::vector<Double32_t[3]   > [10 ]  ").Unwrap()->GetTypeAlias());
+
+   // We keep CV qualifiers only in template arguments of user-defined types
+   EXPECT_EQ("std::unique_ptr<std::array<std::string,2>>",
+             RFieldBase::Create("f", "std::unique_ptr<const std::string[2]>").Unwrap()->GetTypeName());
    const std::string innerCV = "class InnerCV<const int, const volatile int, volatile const int, volatile int>";
    const std::string normInnerCV =
       "InnerCV<const std::int32_t,const volatile std::int32_t,const volatile std::int32_t,volatile std::int32_t>";
    EXPECT_EQ(normInnerCV, RFieldBase::Create("f", innerCV).Unwrap()->GetTypeName());
    EXPECT_EQ("", RFieldBase::Create("f", innerCV).Unwrap()->GetTypeAlias());
+   EXPECT_EQ(
+      "InnerCV<const std::vector<std::array<std::string,2>>,std::int32_t,std::int32_t,std::int32_t>",
+      RFieldBase::Create("f", "InnerCV<const std::vector<std::string[2]>, int, int, int>").Unwrap()->GetTypeName());
+   EXPECT_EQ(
+      "",
+      RFieldBase::Create("f", "InnerCV<const std::vector<std::string[2]>, int, int, int>").Unwrap()->GetTypeAlias());
 
    const std::string example = "const pair<size_t, array<class CustomStruct, 6>>";
    std::string normExample;
@@ -56,6 +79,77 @@ TEST(RNTuple, TypeNameNormalization)
    EXPECT_EQ("std::vector<CustomStruct>",
              RFieldBase::Create("f", "::std::vector<::CustomStruct>").Unwrap()->GetTypeName());
    EXPECT_EQ("", RFieldBase::Create("f", "::std::vector<::CustomStruct>").Unwrap()->GetTypeAlias());
+
+   SimpleCollectionProxy<StructUsingCollectionProxy<int>> proxy;
+   auto klass = TClass::GetClass("StructUsingCollectionProxy<int>");
+   klass->CopyCollectionProxy(proxy);
+
+   EXPECT_EQ("StructUsingCollectionProxy<std::int32_t>",
+             RFieldBase::Create("f", "StructUsingCollectionProxy<int>").Unwrap()->GetTypeName());
+   EXPECT_EQ("", RFieldBase::Create("f", "StructUsingCollectionProxy<int>").Unwrap()->GetTypeAlias());
+}
+
+TEST(RNTuple, TypeNameNormalizationById)
+{
+   using ROOT::Internal::GetRenormalizedTypeName;
+
+   EXPECT_EQ("std::int32_t", GetRenormalizedTypeName(typeid(signed)));
+   EXPECT_EQ("std::int32_t", GetRenormalizedTypeName(typeid(std::int32_t)));
+   EXPECT_EQ("std::int64_t", GetRenormalizedTypeName(typeid(std::int64_t)));
+   EXPECT_EQ("std::uint64_t", GetRenormalizedTypeName(typeid(std::uint64_t)));
+   EXPECT_EQ("double", GetRenormalizedTypeName(typeid(Double32_t)));
+   EXPECT_EQ("float", GetRenormalizedTypeName(typeid(const float)));
+
+   EXPECT_EQ("std::string", GetRenormalizedTypeName(typeid(std::string)));
+   EXPECT_EQ("std::map<std::string,std::string>", GetRenormalizedTypeName(typeid(std::map<std::string, std::string>)));
+   EXPECT_EQ("std::vector<std::array<std::string,2>>", GetRenormalizedTypeName(typeid(std::vector<std::string[2]>)));
+   EXPECT_EQ("char", GetRenormalizedTypeName(typeid(std::string::value_type)));
+
+   EXPECT_EQ("std::vector<bool>", GetRenormalizedTypeName(typeid(std::vector<Bool_t>)));
+   EXPECT_EQ("std::vector<char>", GetRenormalizedTypeName(typeid(std::vector<char>)));
+   EXPECT_EQ("ROOT::VecOps::RVec<float>", GetRenormalizedTypeName(typeid(ROOT::RVec<float>)));
+   EXPECT_EQ("std::array<float,2>", GetRenormalizedTypeName(typeid(std::array<float, 0x02>)));
+   EXPECT_EQ("std::variant<float,double>", GetRenormalizedTypeName(typeid(std::variant<float, double>)));
+   EXPECT_EQ("std::pair<float,double>", GetRenormalizedTypeName(typeid(std::pair<float, double>)));
+   EXPECT_EQ("std::tuple<float,float,float>", GetRenormalizedTypeName(typeid(std::tuple<float, float, float>)));
+   EXPECT_EQ("std::bitset<1000>", GetRenormalizedTypeName(typeid(std::bitset<1000lu>)));
+   EXPECT_EQ("std::atomic<std::int32_t>", GetRenormalizedTypeName(typeid(std::atomic<int>)));
+   EXPECT_EQ("std::unique_ptr<std::int32_t>", GetRenormalizedTypeName(typeid(std::unique_ptr<int>)));
+   EXPECT_EQ("std::optional<std::int32_t>", GetRenormalizedTypeName(typeid(std::optional<int>)));
+
+   EXPECT_EQ("std::array<std::array<std::uint32_t,3>,2>", GetRenormalizedTypeName(typeid(SG::sgkey_t[2][3])));
+   EXPECT_EQ("std::vector<std::array<double,2>>", GetRenormalizedTypeName(typeid(std::vector<Double32_t[2]>)));
+   EXPECT_EQ("std::array<std::vector<std::array<double,3>>,10>",
+             GetRenormalizedTypeName(typeid(std::vector<Double32_t[3]>[10])));
+
+   EXPECT_EQ("std::set<std::int32_t>", GetRenormalizedTypeName(typeid(std::set<int>)));
+   EXPECT_EQ("std::unordered_set<std::int32_t>", GetRenormalizedTypeName(typeid(std::unordered_set<int>)));
+   EXPECT_EQ("std::multiset<std::int32_t>", GetRenormalizedTypeName(typeid(std::multiset<int>)));
+   EXPECT_EQ("std::unordered_multiset<std::int32_t>", GetRenormalizedTypeName(typeid(std::unordered_multiset<int>)));
+
+   EXPECT_EQ("std::map<std::int32_t,std::int32_t>", GetRenormalizedTypeName(typeid(std::map<int, int>)));
+   EXPECT_EQ("std::unordered_map<std::int32_t,std::int32_t>",
+             GetRenormalizedTypeName(typeid(std::unordered_map<int, int>)));
+   EXPECT_EQ("std::multimap<std::int32_t,std::int32_t>", GetRenormalizedTypeName(typeid(std::multimap<int, int>)));
+   EXPECT_EQ("std::unordered_multimap<std::int32_t,std::int32_t>",
+             GetRenormalizedTypeName(typeid(std::unordered_multimap<int, int>)));
+
+   EXPECT_EQ("CustomStruct", ROOT::Internal::GetRenormalizedTypeName(typeid(CustomStruct)));
+
+   // We keep CV qualifiers only in template arguments of user-defined types
+   EXPECT_EQ("std::unique_ptr<std::array<std::string,2>>",
+             GetRenormalizedTypeName(typeid(std::unique_ptr<const std::string[2]>)));
+   const std::string normInnerCV =
+      "InnerCV<const std::int32_t,const volatile std::int32_t,const volatile std::int32_t,volatile std::int32_t>";
+   EXPECT_EQ(normInnerCV,
+             GetRenormalizedTypeName(typeid(InnerCV<const int, const volatile int, volatile const int, volatile int>)));
+   EXPECT_EQ("InnerCV<const std::vector<std::array<std::string,2>>,std::int32_t,std::int32_t,std::int32_t>",
+             GetRenormalizedTypeName(typeid(InnerCV<const std::vector<std::string[2]>, int, int, int>)));
+
+   EXPECT_EQ("std::map<std::int32_t,std::int32_t>", GetRenormalizedTypeName(typeid(std::map<int, int>)));
+
+   EXPECT_EQ("std::pair<std::uint32_t,std::array<CustomStruct,2>>",
+             GetRenormalizedTypeName(typeid(const std::pair<unsigned int, std::array<::CustomStruct, 2>>)));
 }
 
 TEST(RNTuple, TClassDefaultTemplateParameter)
