@@ -4,9 +4,14 @@
 #include <TObject.h>
 #include <TRef.h>
 #include <TRotation.h>
+#include <TVirtualStreamerInfo.h>
 
 #include <memory>
 #include <sstream>
+#include <string>
+#include <unordered_set>
+#include <utility>
+#include <vector>
 
 namespace {
 class RNoDictionary {};
@@ -344,3 +349,92 @@ TEST(RNTuple, TClassInnerReadRule)
    reader->LoadEntry(1);
    EXPECT_TRUE(f->empty());
 }
+
+TEST(RNTuple, PolymorphicPointer)
+{
+   FileRaii fileGuard("test_ntuple_polymorphic_pointer.root");
+
+   {
+      auto model = RNTupleModel::Create();
+      auto ptr = model->MakeField<std::unique_ptr<PolymorphicBase>>("ptr");
+      auto writer = RNTupleWriter::Recreate(std::move(model), "ntpl", fileGuard.GetPath());
+      ptr->reset(new PolymorphicBase);
+      writer->Fill();
+      ptr->reset(new PolymorphicDerived);
+      EXPECT_THROW(writer->Fill(), ROOT::RException);
+   }
+
+   {
+      auto model = RNTupleModel::Create();
+      auto ptrField = RFieldBase::Create("ptr", "std::unique_ptr<PolymorphicBase>").Unwrap();
+      model->AddField(std::move(ptrField));
+      auto ptr = model->GetDefaultEntry().GetPtr<std::unique_ptr<PolymorphicBase>>("ptr");
+      auto writer = RNTupleWriter::Recreate(std::move(model), "ntpl", fileGuard.GetPath());
+      ptr->reset(new PolymorphicBase);
+      writer->Fill();
+      ptr->reset(new PolymorphicDerived);
+      EXPECT_THROW(writer->Fill(), ROOT::RException);
+   }
+}
+
+TEST(RNTuple, TClassMetaName)
+{
+   auto f1 = ROOT::RClassField("f", "EdmWrapper<long long>");
+   EXPECT_STREQ("EdmWrapper<Long64_t>", f1.GetClass()->GetName());
+
+   auto f2 = std::make_unique<ROOT::RField<EdmWrapper<long long>>>("f");
+   EXPECT_STREQ("EdmWrapper<Long64_t>", static_cast<ROOT::RClassField *>(f2.get())->GetClass()->GetName());
+
+   auto f3 = RFieldBase::Create("f", "EdmWrapper<long long>").Unwrap();
+   EXPECT_STREQ("EdmWrapper<Long64_t>", static_cast<ROOT::RClassField *>(f3.get())->GetClass()->GetName());
+
+   auto f4 = RFieldBase::Create("f", "EdmContainer").Unwrap();
+   EXPECT_STREQ("EdmWrapper<Long64_t>",
+                static_cast<const ROOT::RClassField *>(f4->GetConstSubfields()[0])->GetClass()->GetName());
+}
+
+TEST(RNTuple, StreamerInfoRecords)
+{
+   // Every testee consists of the type stored on disk and the expected streamer info records
+   std::vector<std::pair<std::string, std::vector<std::string>>> testees{
+      {"float", {}},
+      {"std::vector<float>", {}},
+      {"std::pair<float, float>", {}},
+      {"std::map<int, float>", {}},
+      {"CustomStruct", {"CustomStruct"}},
+      {"std::vector<CustomStruct>", {"CustomStruct"}},
+      {"std::map<int, CustomStruct>", {"CustomStruct"}},
+      {"DerivedA", {"DerivedA", "CustomStruct"}},
+      {"std::pair<CustomStruct, DerivedA>", {"DerivedA", "CustomStruct"}},
+      {"EdmWrapper<long long>", {"EdmWrapper<Long64_t>"}},
+      {"TRotation", {"TRotation"}}};
+
+   for (const auto &t : testees) {
+      FileRaii fileGuard("test_ntuple_streamer_info_records.root");
+
+      {
+         auto model = ROOT::RNTupleModel::Create();
+         if (t.first == "TRotation") {
+            model->AddField(std::make_unique<ROOT::RStreamerField>("f", t.first));
+         } else {
+            model->AddField(ROOT::RFieldBase::Create("f", t.first).Unwrap());
+         }
+         auto writer = ROOT::RNTupleWriter::Recreate(std::move(model), "ntpl", fileGuard.GetPath());
+      }
+
+      auto f = std::unique_ptr<TFile>(TFile::Open(fileGuard.GetPath().c_str()));
+      ASSERT_TRUE(f && !f->IsZombie());
+
+      std::unordered_set<std::string> expectedInfos{t.second.begin(), t.second.end()};
+      expectedInfos.insert("ROOT::RNTuple");
+      for (const auto info : TRangeDynCast<TVirtualStreamerInfo>(*f->GetStreamerInfoList())) {
+         auto itr = expectedInfos.find(info->GetName());
+         if (itr == expectedInfos.end()) {
+            FAIL() << "unexpected streamer info: " << info->GetName();
+            continue;
+         }
+         expectedInfos.erase(itr);
+      }
+      EXPECT_TRUE(expectedInfos.empty());
+   }
+}
