@@ -66,14 +66,13 @@ then all keys with names = "uv*" in a second pass, etc.
 #include "TMath.h"
 #include "TVirtualPad.h"
 #include "TVirtualX.h"
-#include "TH1.h"
+#include "TH2.h"
 #include "TBox.h"
 #include "TKey.h"
 #include "TRegexp.h"
 #include "TSystem.h"
 #include "strlcpy.h"
 
-ClassImp(TFileDrawMap);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default TreeFileMap constructor.
@@ -90,13 +89,11 @@ TFileDrawMap::TFileDrawMap() :TNamed()
 /// TFileDrawMap normal constructor.
 /// see descriptions of arguments above
 
-TFileDrawMap::TFileDrawMap(const TFile *file, const char *keys, Option_t *option)
+TFileDrawMap::TFileDrawMap(const TFile *file, const char *keys, Option_t *)
          : TNamed("TFileDrawMap","")
 {
-   fFile     = (TFile*)file;
+   fFile     = (TFile*) file;
    fKeys     = keys;
-   fOption   = option;
-   fOption.ToLower();
    SetBit(kCanDelete);
 
    //create histogram used to draw the map frame
@@ -106,31 +103,27 @@ TFileDrawMap::TFileDrawMap(const TFile *file, const char *keys, Option_t *option
    } else {
       fXsize = 1000;
    }
-   fFrame = new TH1D("hmapframe","",1000,0,fXsize);
+   fYsize = 1 + Int_t(file->GetEND()/fXsize);
+
+   fFrame = new TH2D("hmapframe","",100,0,fXsize,100,0,fYsize);
    fFrame->SetDirectory(nullptr);
    fFrame->SetBit(TH1::kNoStats);
    fFrame->SetBit(kCanDelete);
-   fFrame->SetMinimum(0);
    if (fXsize > 1000) {
       fFrame->GetYaxis()->SetTitle("MBytes");
    } else {
       fFrame->GetYaxis()->SetTitle("KBytes");
    }
    fFrame->GetXaxis()->SetTitle("Bytes");
-   fYsize = 1 + Int_t(file->GetEND()/fXsize);
-   fFrame->SetMaximum(fYsize);
-   fFrame->GetYaxis()->SetLimits(0,fYsize);
 
-   //bool show = false;
-   if (gPad) {
+   if (gPad)
       gPad->Clear();
-      //show = gPad->GetCanvas()->GetShowEventStatus();
-   }
+
+   fFrame->Draw("axis");
    Draw();
-   if (gPad) {
-      //if (!show) gPad->GetCanvas()->ToggleEventStatus();
+
+   if (gPad)
       gPad->Update();
-   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -141,6 +134,25 @@ TFileDrawMap::~TFileDrawMap()
    //delete fFrame; //should not be deleted (kCanDelete set)
 }
 
+////////////////////////////////////////////////////////////////////////////////
+/// Returns info which corresponds to recent mouse position
+/// In case of normal graphics it is object name
+/// In case of web canvas use stored click event position
+
+TString TFileDrawMap::GetRecentInfo()
+{
+   TString info;
+   if (gPad && gPad->IsWeb()) {
+      // in case of web canvas one can try to use last click event
+      GetObjectInfoDir(fFile, gPad->GetEventX(), gPad->GetEventY(), info);
+   } else {
+      // last selected place stored as name
+      info = GetName();
+   }
+   return info;
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////
 /// Show sequence of baskets reads for the list of baskets involved
 /// in the list of branches (separated by ",")
@@ -152,27 +164,36 @@ TFileDrawMap::~TFileDrawMap()
 
 void  TFileDrawMap::AnimateTree(const char *branches)
 {
-   TString ourbranches( GetName() );
-   Ssiz_t pos = ourbranches.Index(", basket=");
-   if (pos == kNPOS) return;
-   ourbranches.Remove(pos);
-   pos = ourbranches.Index(", branch=");
-   if (pos == kNPOS) return;
-   ourbranches[pos] = 0;
+   TString info = GetRecentInfo();
 
-   TTree *tree = (TTree*)fFile->Get(ourbranches.Data());
-   if (!tree) return;
-   TString info;
-   if (strlen(branches) > 0) info = branches;
-   else                      info = ourbranches.Data()+pos+9;
-   printf("Animating tree, branches=%s\n",info.Data());
+   auto pos = info.Index(", basket=");
+   if (pos == kNPOS)
+      return;
+   info.Remove(pos);
+
+   pos = info.Index(", branch=");
+   if (pos == kNPOS)
+      return;
+   TString select_branches = info(pos + 9, info.Length() - pos - 9);
+
+   auto colon = info.Index("::");
+   if (colon == kNPOS)
+      return;
+
+   info.Resize(colon - 1);
+
+   auto tree = fFile->Get<TTree>(info);
+   if (!tree)
+      return;
+   if (branches && *branches)
+      select_branches = branches;
 
    // create list of branches
    Int_t nzip = 0;
    TBranch *branch;
    TObjArray list;
    char *comma;
-   while((comma = strrchr((char*)info.Data(),','))) {
+   while((comma = strrchr((char*)select_branches.Data(),','))) {
       *comma = 0;
       comma++;
       while (*comma == ' ') comma++;
@@ -183,7 +204,7 @@ void  TFileDrawMap::AnimateTree(const char *branches)
          list.Add(branch);
       }
    }
-   comma = (char*)info.Data();
+   comma = (char*)select_branches.Data();
    while (*comma == ' ') comma++;
    branch = tree->GetBranch(comma);
    if (branch) {
@@ -281,34 +302,40 @@ void TFileDrawMap::DrawMarker(Int_t marker, Long64_t eseek)
 void TFileDrawMap::DrawObject()
 {
    TVirtualPad *padsave = gROOT->GetSelectedPad();
-   if (padsave == gPad) {
+   if ((padsave == gPad) || (gPad && gPad->IsWeb())) {
       //must create a new canvas
       gROOT->MakeDefCanvas();
    } else if (padsave) {
       padsave->cd();
    }
 
+   TString info = GetRecentInfo();
+
    // case of a TTree
-   char *info = new char[fName.Length()+1];
-   strlcpy(info,fName.Data(),fName.Length()+1);
-   char *cbasket = (char*)strstr(info,", basket=");
-   if (cbasket) {
-      *cbasket = 0;
-      char *cbranch = (char*)strstr(info,", branch=");
-      if (!cbranch) { delete [] info; return; }
-      *cbranch = 0;
-      cbranch += 9;
-      TTree *tree = (TTree*)fFile->Get(info);
-      if (tree) tree->Draw(cbranch);
-      delete [] info;
-      return;
+   auto pbasket = info.Index(", basket=");
+   if (pbasket != kNPOS) {
+      info.Resize(pbasket);
+      auto pbranch = info.Index(", branch=");
+      if (pbranch == kNPOS)
+         return;
+
+      TString cbranch = info(pbranch + 9, info.Length() - pbranch - 9);
+
+      auto colon = info.Index("::");
+      if (colon == kNPOS)
+         return;
+
+      info.Resize(colon - 1);
+
+      auto tree = fFile->Get<TTree>(info);
+      if (tree)
+         tree->Draw(cbranch);
+   } else {
+      // other objects
+      auto obj = GetObject();
+      if (obj)
+         obj->Draw();
    }
-
-   delete [] info;
-
-   // other objects
-   TObject *obj = GetObject();
-   if (obj) obj->Draw();
 }
 
 
@@ -322,25 +349,25 @@ void TFileDrawMap::DumpObject()
       obj->Dump();
       return;
    }
-   char *centry = (char*)strstr(GetName(),"entry=");
-   if (!centry) return;
+
+   TString info = GetRecentInfo();
+
+   auto indx = info.Index("entry=");
+   if (indx == kNPOS)
+      return;
+
    Int_t entry = 0;
-   sscanf(centry+6,"%d",&entry);
-   TString info(GetName());
-   char *colon = (char*)strstr((char*)info.Data(),"::");
-   if (!colon) return;
-   colon--;
-   *colon = 0;
-   TTree *tree; fFile->GetObject(info.Data(),tree);
-   if (tree) tree->Show(entry);
-}
+   sscanf(info.Data() + indx + 6, "%d", &entry);
 
-////////////////////////////////////////////////////////////////////////////////
-/// Execute action corresponding to one event.
+   auto colon = info.Index("::");
+   if (colon == kNPOS)
+      return;
 
-void TFileDrawMap::ExecuteEvent(Int_t event, Int_t px, Int_t py)
-{
-   fFrame->ExecuteEvent(event,px,py);
+   info.Resize(colon - 1);
+
+   auto tree = fFile->Get<TTree>(info);
+   if (tree)
+      tree->Show(entry);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -348,19 +375,18 @@ void TFileDrawMap::ExecuteEvent(Int_t event, Int_t px, Int_t py)
 
 TObject *TFileDrawMap::GetObject()
 {
-   if (strstr(GetName(),"entry=")) return nullptr;
-   char *info = new char[fName.Length()+1];
-   strlcpy(info,fName.Data(),fName.Length()+1);
-   char *colon = strstr(info,"::");
-   if (!colon) {
-      delete [] info;
+   TString info = GetRecentInfo();
+
+   if (info.Contains("entry="))
       return nullptr;
-   }
-   colon--;
-   *colon = 0;
-   auto res = fFile->Get(info);
-   delete [] info;
-   return res;
+
+   auto colon = info.Index("::");
+   if (colon == kNPOS)
+      return nullptr;
+
+   info.Resize(colon - 1);
+
+   return fFile->Get(info);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -414,23 +440,23 @@ bool TFileDrawMap::GetObjectInfoDir(TDirectory *dir, Int_t px, Int_t py, TString
       if (cl && cl->InheritsFrom(TTree::Class())) {
          TTree *tree = (TTree*)gDirectory->Get(key->GetName());
          TIter nextb(tree->GetListOfLeaves());
-         TLeaf *leaf;
-         while ((leaf = (TLeaf*)nextb())) {
+         while (auto leaf = (TLeaf *)nextb()) {
             TBranch *branch = leaf->GetBranch();
             Int_t nbaskets = branch->GetMaxBaskets();
             Int_t offsets = branch->GetEntryOffsetLen();
             Int_t len = leaf->GetLen();
-            for (Int_t i=0;i<nbaskets;i++) {
+            for (Int_t i = 0; i < nbaskets; i++) {
                bseek = branch->GetBasketSeek(i);
-               if (!bseek) break;
+               if (!bseek)
+                  break;
                nbytes = branch->GetBasketBytes()[i];
-               if (pbyte >= bseek && pbyte < bseek+nbytes) {
+               if (pbyte >= bseek && pbyte < bseek + nbytes) {
                   Int_t entry = branch->GetBasketEntry()[i];
                   if (!offsets) entry += (pbyte-bseek)/len;
                   if (curdir == (TDirectory*)fFile) {
-                     info.Form("%s%s, branch=%s, basket=%d, entry=%d",curdir->GetPath(),key->GetName(),branch->GetName(),i,entry);
+                     info.Form("%s%s ::%s, branch=%s, basket=%d, entry=%d",curdir->GetPath(),key->GetName(),key->GetClassName(),branch->GetName(),i,entry);
                   } else {
-                     info.Form("%s/%s, branch=%s, basket=%d, entry=%d",curdir->GetPath(),key->GetName(),branch->GetName(),i,entry);
+                     info.Form("%s/%s ::%s, branch=%s, basket=%d, entry=%d",curdir->GetPath(),key->GetName(),key->GetClassName(),branch->GetName(),i,entry);
                   }
                   return true;
                }
@@ -488,22 +514,8 @@ void TFileDrawMap::InspectObject()
 
 void TFileDrawMap::Paint(Option_t *)
 {
-   // draw map frame
-   if (!fOption.Contains("same")) {
-      gPad->Clear();
-      //just in case axis Y has been unzoomed
-      if (fFrame->GetMaximumStored() < -1000) {
-         fFrame->SetMaximum(fYsize+1);
-         fFrame->SetMinimum(0);
-         fFrame->GetYaxis()->SetLimits(0,fYsize+1);
-      }
-      fFrame->Paint();
-   }
-
    //draw keys
    PaintDir(fFile, fKeys.Data());
-
-   fFrame->Draw("sameaxis");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -573,13 +585,15 @@ void TFileDrawMap::PaintDir(TDirectory *dir, const char *keys)
       if (cl && cl->InheritsFrom(TTree::Class())) {
          TTree *tree = (TTree*)gDirectory->Get(key->GetName());
          TIter nextb(tree->GetListOfLeaves());
-         TLeaf *leaf;
-         while ((leaf = (TLeaf*)nextb())) {
+         while (auto leaf = (TLeaf*)nextb()) {
             TBranch *branch = leaf->GetBranch();
             color = branch->GetFillColor();
             if (color == 0) {
-               gPad->IncrementPaletteColor(1, "pfc");
-               color = gPad->NextPaletteColor();
+               if (fBranchColors.find(branch) == fBranchColors.end()) {
+                  gPad->IncrementPaletteColor(1, "pfc");
+                  fBranchColors[branch] = gPad->NextPaletteColor();
+               }
+               color = fBranchColors[branch];
             }
             box.SetFillColor(color);
             Int_t nbaskets = branch->GetMaxBaskets();
@@ -592,6 +606,7 @@ void TFileDrawMap::PaintDir(TDirectory *dir, const char *keys)
          }
       }
    }
+
    // draw the box for Keys list
    box.SetFillColor(50);
    box.SetFillStyle(1001);
