@@ -12,6 +12,7 @@
 
 #include "gtest/gtest.h"
 #include "ROOT/TestSupport.hxx"
+#include "ROOT/InternalTreeUtils.hxx" // GetFileNamesFromTree
 #include <cstdlib>
 #include <memory>
 
@@ -468,12 +469,12 @@ TEST(TTreeReaderBasic, InfLoop)
 // ROOT-10019
 TEST(TTreeReaderBasic, DisappearingBranch)
 {
-   ROOT::TestSupport::CheckDiagsRAII diags{ kError, "TTreeReader::SetEntryBase()", "There was an error while notifying the proxies." };
-   diags.requiredDiag(kWarning, "TTreeReader::SetEntryBase()", "Unexpected error '-6' in TChain::LoadTree");
+   ROOT::TestSupport::CheckDiagsRAII diags{kError, "TTreeReader::SetEntryBase()",
+                                           "There was an error while notifying the proxies."};
 
    auto createFile = [](const char *fileName, int ncols) {
-      // auto r = ROOT::RDataFrame(1).Define("col0",[](){return 0;}).Snapshot<int>("t","f1.root",{"col0"});
-      // r->Define("col1",[](){return 0;}).Snapshot<int,int>("t","f0.root",{"col0","col1"});
+      // auto r = ROOT::RDataFrame(1).Define("col0",[](){return 0;}).Snapshot("t","f1.root",{"col0"});
+      // r->Define("col1",[](){return 0;}).Snapshot("t","f0.root",{"col0","col1"});
       TFile f(fileName, "RECREATE");
       TTree t("t", "t");
       int i = 42;
@@ -489,6 +490,10 @@ TEST(TTreeReaderBasic, DisappearingBranch)
 
    TChain c("t");
    c.Add("DisappearingBranch*.root");
+   diags.requiredDiag(kError, "TBranchProxy::Setup()",
+                      "Branch 'col1' is not available from tree 't' in file '" +
+                         ROOT::Internal::TreeUtils::GetFileNamesFromTree(c)[1] + "'.");
+
    TTreeReader r(&c);
    TTreeReaderValue<int> rv(r, "col1");
    r.Next();
@@ -529,3 +534,39 @@ TEST(TTreeReaderBasic, ZeroEntriesTree)
    EXPECT_TRUE(b++ == e);
    EXPECT_TRUE(++b_copy == e);
 }
+
+TEST(TTreeReaderBasic, ZeroEntriesTreeCheckValueStatus)
+{
+   // Regression test for https://github.com/root-project/root/issues/18955
+   struct Dataset {
+      Dataset()
+      {
+         auto f = std::make_unique<TFile>("TTreeReaderBasicZeroEntriesTreeCheckValueStatus.root", "recreate");
+         auto t = std::make_unique<TTree>("t", "t");
+         std::vector<int> b1;
+         t->Branch("b1", &b1);
+         f->Write();
+      }
+      ~Dataset() { std::remove("TTreeReaderBasicZeroEntriesTreeCheckValueStatus.root"); }
+   } dataset;
+
+   // Test with TTree
+   {
+      auto f = std::make_unique<TFile>("TTreeReaderBasicZeroEntriesTreeCheckValueStatus.root");
+      auto t = f->Get<TTree>("t");
+      TTreeReader tr{t};
+      TTreeReaderValue<std::vector<int>> v1{tr, "b1"};
+      tr.Next();
+      EXPECT_EQ(v1.GetSetupStatus(), ROOT::Internal::TTreeReaderValueBase::ESetupStatus::kSetupMatchButEntryBeyondEnd);
+   }
+
+   // Test with TChain
+   {
+      TChain c{"t"};
+      c.Add("TTreeReaderBasicZeroEntriesTreeCheckValueStatus.root");
+      TTreeReader tr{&c};
+      TTreeReaderValue<std::vector<int>> v1{tr, "b1"};
+      tr.Next();
+      EXPECT_EQ(v1.GetSetupStatus(), ROOT::Internal::TTreeReaderValueBase::ESetupStatus::kSetupMatchButEntryBeyondEnd);
+   }
+}
